@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2012
  *					All rights reserved
  *
  *  This file is part of GPAC / X3D Scene Graph Generator sub-project
@@ -10,15 +11,15 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
@@ -29,19 +30,19 @@
 #include <gpac/list.h>
 #include <time.h>
 
-#define COPYRIGHT "/*\n *			GPAC - Multimedia Framework C SDK\n *\n *			Copyright (c) Jean Le Feuvre 2000-2005\n *					All rights reserved\n *\n *  This file is part of GPAC / X3D Scene Graph sub-project\n *\n *  GPAC is free software; you can redistribute it and/or modify\n *  it under the terms of the GNU Lesser General Public License as published by\n *  the Free Software Foundation; either version 2, or (at your option)\n *  any later version.\n *\n *  GPAC is distributed in the hope that it will be useful,\n *  but WITHOUT ANY WARRANTY; without even the implied warranty of\n *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n *  GNU Lesser General Public License for more details.	\n *\n *  You should have received a copy of the GNU Lesser General Public\n *  License along with this library; see the file COPYING.  If not, write to\n *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.\n *\n */\n"
+#define COPYRIGHT "/*\n *			GPAC - Multimedia Framework C SDK\n *\n *			Authors: Jean Le Feuvre\n *			Copyright (c) Telecom ParisTech 2000-2012\n *					All rights reserved\n *\n *  This file is part of GPAC / X3D Scene Graph sub-project\n *\n *  GPAC is free software; you can redistribute it and/or modify\n *  it under the terms of the GNU Lesser General Public License as published by\n *  the Free Software Foundation; either version 2, or (at your option)\n *  any later version.\n *\n *  GPAC is distributed in the hope that it will be useful,\n *  but WITHOUT ANY WARRANTY; without even the implied warranty of\n *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n *  GNU Lesser General Public License for more details.	\n *\n *  You should have received a copy of the GNU Lesser General Public\n *  License along with this library; see the file COPYING.  If not, write to\n *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.\n *\n */\n"
 
 static char *CurrentLine;
 
 void PrintUsage()
 {
 	printf("X3DGen [skip_file]\n"
-			"\nGPAC X3D Scene Graph generator\n"
-			"\n"
-			"\nskip_file: txt file with list of nodes to leave unimplemented"
-			"Generated Files are directly updated in the GPAC distribution - do NOT try to change this\n\n"
-			"Written by Jean Le Feuvre - (c) 2000-2005\n"
-			);
+	       "\nGPAC X3D Scene Graph generator\n"
+	       "\n"
+	       "\nskip_file: txt file with list of nodes to leave unimplemented"
+	       "Generated Files are directly updated in the GPAC distribution - do NOT try to change this\n\n"
+	       "Written by Jean Le Feuvre - (c) 2000-2005\n"
+	      );
 }
 
 //a node field
@@ -92,7 +93,7 @@ u32 GetNextToken(char *token, char *sep)
 	u32 i , j = 0;
 
 	strcpy(token, "");
-	
+
 	//skip separaors
 	while (*CurrentLine && strchr(sep, *CurrentLine)) {
 		CurrentLine = CurrentLine + 1;
@@ -101,7 +102,7 @@ u32 GetNextToken(char *token, char *sep)
 		if (*CurrentLine == '\n') return 0;
 	}
 
-	//copy token untill next blank
+	//copy token until next blank
 	i=0;
 	while (1) {
 		//bad line
@@ -126,7 +127,7 @@ u32 GetNextToken(char *token, char *sep)
 
 X3DField *BlankField()
 {
-	X3DField *n = malloc(sizeof(X3DField));
+	X3DField *n = gf_malloc(sizeof(X3DField));
 	memset(n, 0, sizeof(X3DField));
 	return n;
 }
@@ -134,7 +135,7 @@ X3DField *BlankField()
 
 X3DNode *BlankNode()
 {
-	X3DNode *n = malloc(sizeof(X3DNode));
+	X3DNode *n = gf_malloc(sizeof(X3DNode));
 	memset(n, 0, sizeof(X3DNode));
 	n->NDT = gf_list_new();
 	n->Fields = gf_list_new();
@@ -144,9 +145,8 @@ X3DNode *BlankNode()
 u8 IsNDT(GF_List *NDTs, char *famName)
 {
 	u32 i;
-	char *ndtName;
 	for (i=0; i<gf_list_count(NDTs); i++) {
-		ndtName = gf_list_get(NDTs, i);
+		char *ndtName = gf_list_get(NDTs, i);
 		//remove SF / MF as we don't need that
 		if (!strcmp(ndtName+2, famName+2)) return 1;
 	}
@@ -161,7 +161,7 @@ void CheckInTable(char *token, GF_List *NDTs)
 		p = gf_list_get(NDTs, i);
 		if (!strcmp(p, token)) return;
 	}
-	p = malloc(strlen(token)+1);
+	p = gf_malloc(strlen(token)+1);
 	strcpy(p, token);
 	gf_list_add(NDTs, p);
 }
@@ -178,8 +178,8 @@ FILE *BeginFile(u32 type)
 	} else {
 		sprintf(sPath, "..%c..%c..%csrc%cscenegraph%cx3d_nodes.c", GF_PATH_SEPARATOR, GF_PATH_SEPARATOR, GF_PATH_SEPARATOR, GF_PATH_SEPARATOR, GF_PATH_SEPARATOR);
 	}
-	
-	f = fopen(sPath, "wt");
+
+	f = gf_fopen(sPath, "wt");
 	fprintf(f, "%s\n", COPYRIGHT);
 
 	{
@@ -202,7 +202,7 @@ void EndFile(FILE *f, u32 type)
 		fprintf(f, "#ifdef __cplusplus\n}\n#endif\n\n");
 		fprintf(f, "\n\n#endif\t\t/*_GF_X3D_NODES_H*/\n\n");
 	}
-	fclose(f);
+	gf_fclose(f);
 }
 
 void TranslateToken(char *token)
@@ -226,10 +226,11 @@ void WriteNodesFile(GF_List *BNodes, GF_List *NDTs)
 	f = BeginFile(0);
 
 	fprintf(f, "#include <gpac/scenegraph_vrml.h>\n\n");
+	fprintf(f, "#ifndef GPAC_DISABLE_X3D\n\n");
 
 	//write all tags
 	fprintf(f, "\n\nenum {\n");
-	
+
 	for (i=0; i<gf_list_count(BNodes); i++) {
 		n = gf_list_get(BNodes, i);
 		if (i)
@@ -256,12 +257,12 @@ void WriteNodesFile(GF_List *BNodes, GF_List *NDTs)
 		}
 		for (j=0; j<gf_list_count(n->Fields); j++) {
 			bf = gf_list_get(n->Fields, j);
-			
+
 			if (!strcmp(bf->name, "addChildren") || !strcmp(bf->name, "removeChildren")) continue;
 			if (strcmp(bf->type, "eventOut") && !strcmp(bf->name, "children")) continue;
 
-			if (strstr(bf->familly, "Node")) {		
-				//this is a POINTER to a node 
+			if (strstr(bf->familly, "Node")) {
+				//this is a POINTER to a node
 				if (strstr(bf->familly, "SF")) {
 					fprintf(f, "\tGF_Node *%s;\t/*%s*/\n", bf->name, bf->type);
 				} else {
@@ -271,12 +272,13 @@ void WriteNodesFile(GF_List *BNodes, GF_List *NDTs)
 			} else {
 				fprintf(f, "\t%s %s;\t/*%s*/\n", bf->familly, bf->name, bf->type);
 			}
-			if (!strcmp(bf->type, "eventIn")) 
-				fprintf(f, "\tvoid (*on_%s)(GF_Node *pThis);\t/*eventInHandler*/\n", bf->name);
+			if (!strcmp(bf->type, "eventIn"))
+				fprintf(f, "\tvoid (*on_%s)(GF_Node *pThis, struct _route *route);\t/*eventInHandler*/\n", bf->name);
 		}
 		fprintf(f, "} X_%s;\n\n\n", n->name);
 	}
 
+	fprintf(f, "#endif /*GPAC_DISABLE_X3D*/\n\n");
 	EndFile(f, 0);
 
 }
@@ -288,11 +290,11 @@ void WriteNodeFields(FILE *f, X3DNode *n)
 
 	fprintf(f, "\nstatic u32 %s_get_field_count(GF_Node *node, u8 dummy)\n{\n\treturn %d;\n}\n\n", n->name, gf_list_count(n->Fields));
 	fprintf(f, "static GF_Err %s_get_field(GF_Node *node, GF_FieldInfo *info)\n{\n\tswitch (info->fieldIndex) {\n", n->name);
-	for (i=0;i<gf_list_count(n->Fields); i++) {
+	for (i=0; i<gf_list_count(n->Fields); i++) {
 		bf = gf_list_get(n->Fields, i);
 
 		fprintf(f, "\tcase %d:\n", i);
-		
+
 		fprintf(f, "\t\tinfo->name = \"%s\";\n", bf->name);
 
 		//skip all eventIn
@@ -332,7 +334,7 @@ void WriteNodeFields(FILE *f, X3DNode *n)
 	fprintf(f, "\tdefault:\n\t\treturn GF_BAD_PARAM;\n\t}\n}\n\n");
 
 	fprintf(f, "\nstatic s32 %s_get_field_index_by_name(char *name)\n{\n", n->name);
-	for (i=0;i<gf_list_count(n->Fields); i++) {
+	for (i=0; i<gf_list_count(n->Fields); i++) {
 		bf = gf_list_get(n->Fields, i);
 		fprintf(f, "\tif (!strcmp(\"%s\", name)) return %d;\n", bf->name, i);
 	}
@@ -350,6 +352,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 	fprintf(vrml_code, "\n#include <gpac/nodes_x3d.h>\n");
 	fprintf(vrml_code, "\n#include <gpac/internal/scenegraph_dev.h>\n");
 	fprintf(vrml_code, "\n/*for NDT tag definitions*/\n#include <gpac/nodes_mpeg4.h>\n");
+	fprintf(vrml_code, "#ifndef GPAC_DISABLE_X3D\n\n");
 
 	for (k=0; k<gf_list_count(BNodes); k++) {
 		Bool is_parent = 0;
@@ -363,42 +366,43 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 			//nothing on child events
 			if (!strcmp(bf->name, "addChildren")) continue;
 			if (!strcmp(bf->name, "removeChildren")) continue;
-			
+
 			//delete all children node
 			if (strcmp(bf->type, "eventOut") && !strcmp(bf->name, "children")) {
 				is_parent = 1;
 				continue;
 			}
 
-			//delete ALL fields that must be deleted: this includes eventIn and out since 
+			//delete ALL fields that must be deleted: this includes eventIn and out since
 			//all fields are defined in the node
 			if (!strcmp(bf->familly, "MFInt")
-				|| !strcmp(bf->familly, "MFFloat")
-				|| !strcmp(bf->familly, "MFDouble")
-				|| !strcmp(bf->familly, "MFBool")
-				|| !strcmp(bf->familly, "MFInt32")
-				|| !strcmp(bf->familly, "MFColor")
-				|| !strcmp(bf->familly, "MFRotation")
-				|| !strcmp(bf->familly, "MFString")
-				|| !strcmp(bf->familly, "MFTime")
-				|| !strcmp(bf->familly, "MFVec2f")
-				|| !strcmp(bf->familly, "MFVec3f")
-				|| !strcmp(bf->familly, "MFVec4f")
-				|| !strcmp(bf->familly, "MFVec2d")
-				|| !strcmp(bf->familly, "MFVec3d")
-				|| !strcmp(bf->familly, "MFURL")
-				|| !strcmp(bf->familly, "MFScript")
-				|| !strcmp(bf->familly, "SFString")
-				|| !strcmp(bf->familly, "SFURL")
-				|| !strcmp(bf->familly, "SFImage")
-				
-				) {
+			        || !strcmp(bf->familly, "MFFloat")
+			        || !strcmp(bf->familly, "MFDouble")
+			        || !strcmp(bf->familly, "MFBool")
+			        || !strcmp(bf->familly, "MFInt32")
+			        || !strcmp(bf->familly, "MFColor")
+			        || !strcmp(bf->familly, "MFRotation")
+			        || !strcmp(bf->familly, "MFString")
+			        || !strcmp(bf->familly, "MFTime")
+			        || !strcmp(bf->familly, "MFVec2f")
+			        || !strcmp(bf->familly, "MFVec3f")
+			        || !strcmp(bf->familly, "MFVec4f")
+			        || !strcmp(bf->familly, "MFVec2d")
+			        || !strcmp(bf->familly, "MFVec3d")
+			        || !strcmp(bf->familly, "MFURL")
+			        || !strcmp(bf->familly, "MFScript")
+			        || !strcmp(bf->familly, "SFString")
+			        || !strcmp(bf->familly, "SFURL")
+			        || !strcmp(bf->familly, "SFImage")
+					|| !strcmp(bf->familly, "MFColorRGBA")
+
+			   ) {
 				char szName[500];
 				strcpy(szName, bf->familly);
 				strlwr(szName);
 				fprintf(vrml_code, "\tgf_sg_%s_del(p->%s);\n", szName, bf->name);
-			} else if (strstr(bf->familly, "Node")) {		
-				//this is a POINTER to a node 
+			} else if (strstr(bf->familly, "Node")) {
+				//this is a POINTER to a node
 				if (strstr(bf->familly, "SF")) {
 					fprintf(vrml_code, "\tgf_node_unregister((GF_Node *) p->%s, node);\t\n", bf->name);
 				} else {
@@ -430,8 +434,8 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 				fprintf(vrml_code, "\tgf_sg_vrml_parent_setup((GF_Node *) p);\n");
 				break;
 			}
-			else if ( strstr(bf->familly, "Node") && strncmp(bf->type, "event", 5) ) {		
-				//this is a POINTER to a node 
+			else if ( strstr(bf->familly, "Node") && strncmp(bf->type, "event", 5) ) {
+				//this is a POINTER to a node
 				if (strstr(bf->familly, "MF")) {
 					//this is a POINTER to a chain
 					//fprintf(vrml_code, "\tp->%s = gf_list_new();\t\n", bf->name);
@@ -444,7 +448,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 		}
 
 		fprintf(vrml_code, "\n\t/*default field values*/\n");
-		
+
 		for (i=0; i<gf_list_count(n->Fields); i++) {
 			bf = gf_list_get(n->Fields, i);
 
@@ -457,14 +461,14 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 			//no default on nodes
 			if (strstr(bf->familly, "Node")) continue;
 			//extract default falue
-			
+
 			//
 			//		SF Fields
 			//
-			
+
 			//SFBool
 			if (!strcmp(bf->familly, "SFBool")) {
-				if (!strcmp(bf->def, "1") || !strcmp(bf->def, "TRUE")) 
+				if (!strcmp(bf->def, "1") || !strcmp(bf->def, "TRUE"))
 					fprintf(vrml_code, "\tp->%s = 1;\n", bf->name);
 			}
 			//SFFloat
@@ -562,10 +566,10 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 			}
 			//SFString
 			else if (!strcmp(bf->familly, "SFString")) {
-				fprintf(vrml_code, "\tp->%s.buffer = (char*) malloc(sizeof(char) * %d);\n", bf->name, strlen(bf->def)+1);
+				fprintf(vrml_code, "\tp->%s.buffer = (char*) gf_malloc(sizeof(char) * %u);\n", bf->name, (u32) strlen(bf->def)+1);
 				fprintf(vrml_code, "\tstrcpy(p->%s.buffer, \"%s\");\n", bf->name, bf->def);
 			}
-		
+
 			//
 			//		MF Fields
 			//
@@ -575,7 +579,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 				CurrentLine = bf->def;
 				while (GetNextToken(token, " ,")) j++;
 				j+=1;
-				fprintf(vrml_code, "\tp->%s.vals = (SFFloat *)malloc(sizeof(SFFloat)*%d);\n", bf->name, j);
+				fprintf(vrml_code, "\tp->%s.vals = (SFFloat *)gf_malloc(sizeof(SFFloat)*%d);\n", bf->name, j);
 				fprintf(vrml_code, "\tp->%s.count = %d;\n", bf->name, j);
 				j = 0;
 				go = 1;
@@ -593,7 +597,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 				CurrentLine = bf->def;
 				while (GetNextToken(token, " ,")) j++;
 				j+=1;
-				fprintf(vrml_code, "\tp->%s.vals = (SFFloat*)malloc(sizeof(SFFloat)*%d);\n", bf->name, j);
+				fprintf(vrml_code, "\tp->%s.vals = (SFFloat*)gf_malloc(sizeof(SFFloat)*%d);\n", bf->name, j);
 				fprintf(vrml_code, "\tp->%s.count = %d;\n", bf->name, j);
 				j = 0;
 				go = 1;
@@ -611,7 +615,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 				CurrentLine = bf->def;
 				while (GetNextToken(token, ",")) j++;
 				j+=1;
-				fprintf(vrml_code, "\tp->%s.vals = (SFVec2f*) malloc(sizeof(SFVec2f)*%d);\n", bf->name, j);
+				fprintf(vrml_code, "\tp->%s.vals = (SFVec2f*) gf_malloc(sizeof(SFVec2f)*%d);\n", bf->name, j);
 				fprintf(vrml_code, "\tp->%s.count = %d;\n", bf->name, j);
 				j = 0;
 				go = 1;
@@ -636,7 +640,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 				CurrentLine = bf->def;
 				while (GetNextToken(token, ",")) j++;
 				j+=1;
-				fprintf(vrml_code, "\tp->%s.vals = (SFVec2f*)malloc(sizeof(SFVec2f)*%d);\n", bf->name, j);
+				fprintf(vrml_code, "\tp->%s.vals = (SFVec2f*)gf_malloc(sizeof(SFVec2f)*%d);\n", bf->name, j);
 				fprintf(vrml_code, "\tp->%s.count = %d;\n", bf->name, j);
 				j = 0;
 				go = 1;
@@ -661,7 +665,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 				CurrentLine = bf->def;
 				while (GetNextToken(token, ",")) j++;
 				j+=1;
-				fprintf(vrml_code, "\tp->%s.vals = (SFVec3f*)malloc(sizeof(SFVec3f)*%d);\n", bf->name, j);
+				fprintf(vrml_code, "\tp->%s.vals = (SFVec3f*)gf_malloc(sizeof(SFVec3f)*%d);\n", bf->name, j);
 				fprintf(vrml_code, "\tp->%s.count = %d;\n", bf->name, j);
 				j = 0;
 				go = 1;
@@ -689,7 +693,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 				CurrentLine = bf->def;
 				while (GetNextToken(token, ",")) j++;
 				j+=1;
-				fprintf(vrml_code, "\tp->%s.vals = (SFVec2f*)malloc(sizeof(SFVec3f)*%d);\n", bf->name, j);
+				fprintf(vrml_code, "\tp->%s.vals = (SFVec2f*)gf_malloc(sizeof(SFVec3f)*%d);\n", bf->name, j);
 				fprintf(vrml_code, "\tp->%s.count = %d;\n", bf->name, j);
 				j = 0;
 				go = 1;
@@ -717,7 +721,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 				CurrentLine = bf->def;
 				while (GetNextToken(token, ",")) j++;
 				j+=1;
-				fprintf(vrml_code, "\tp->%s.vals = (GF_Vec4*)malloc(sizeof(GF_Vec4)*%d);\n", bf->name, j);
+				fprintf(vrml_code, "\tp->%s.vals = (GF_Vec4*)gf_malloc(sizeof(GF_Vec4)*%d);\n", bf->name, j);
 				fprintf(vrml_code, "\tp->%s.count = %d;\n", bf->name, j);
 				j = 0;
 				go = 1;
@@ -748,7 +752,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 				CurrentLine = bf->def;
 				while (GetNextToken(token, ",")) j++;
 				j+=1;
-				fprintf(vrml_code, "\tp->%s.vals = (SFInt32*)malloc(sizeof(SFInt32)*%d);\n", bf->name, j);
+				fprintf(vrml_code, "\tp->%s.vals = (SFInt32*)gf_malloc(sizeof(SFInt32)*%d);\n", bf->name, j);
 				fprintf(vrml_code, "\tp->%s.count = %d;\n", bf->name, j);
 				j = 0;
 				go = 1;
@@ -769,7 +773,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 				CurrentLine = bf->def;
 				while (GetNextToken(token, ",")) j++;
 				j+=1;
-				fprintf(vrml_code, "\tp->%s.vals = (SFColor*)malloc(sizeof(SFColor)*%d);\n", bf->name, j);
+				fprintf(vrml_code, "\tp->%s.vals = (SFColor*)gf_malloc(sizeof(SFColor)*%d);\n", bf->name, j);
 				fprintf(vrml_code, "\tp->%s.count = %d;\n", bf->name, j);
 				j = 0;
 				go = 1;
@@ -794,7 +798,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 				CurrentLine = bf->def;
 				while (GetNextToken(token, ",")) j++;
 				j+=1;
-				fprintf(vrml_code, "\tp->%s.vals = (char**)malloc(sizeof(SFString)*%d);\n", bf->name, j);
+				fprintf(vrml_code, "\tp->%s.vals = (char**)gf_malloc(sizeof(SFString)*%d);\n", bf->name, j);
 				fprintf(vrml_code, "\tp->%s.count = %d;\n", bf->name, j);
 				j = 0;
 				go = 1;
@@ -804,7 +808,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 					store = CurrentLine;
 					CurrentLine = token;
 					GetNextToken(tok, " \"");
-					fprintf(vrml_code, "\tp->%s.vals[%d] = (char*)malloc(sizeof(char) * %d);\n", bf->name, j, strlen(tok)+1);
+					fprintf(vrml_code, "\tp->%s.vals[%d] = (char*)gf_malloc(sizeof(char) * %u);\n", bf->name, j, (u32) strlen(tok)+1);
 					fprintf(vrml_code, "\tstrcpy(p->%s.vals[%d], \"%s\");\n", bf->name, j, tok);
 					j+=1;
 					CurrentLine = store;
@@ -816,7 +820,7 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 				CurrentLine = bf->def;
 				while (GetNextToken(token, ",")) j++;
 				j+=1;
-				fprintf(vrml_code, "\tp->%s.vals = (SFTime*)malloc(sizeof(SFTime)*%d);\n", bf->name, j);
+				fprintf(vrml_code, "\tp->%s.vals = (SFTime*)gf_malloc(sizeof(SFTime)*%d);\n", bf->name, j);
 				fprintf(vrml_code, "\tp->%s.count = %d;\n", bf->name, j);
 				j = 0;
 				go = 1;
@@ -907,10 +911,9 @@ void WriteNodeCode(GF_List *BNodes, FILE *vrml_code)
 static u32 IsNodeInTable(X3DNode *node, char *NDTName)
 {
 	u32 i;
-	char *ndt;
 
 	for (i=0; i<gf_list_count(node->NDT); i++) {
-		ndt = gf_list_get(node->NDT, i);
+		char *ndt = gf_list_get(node->NDT, i);
 		if (!strcmp(ndt, NDTName)) return 1;
 	}
 	return 0;
@@ -918,11 +921,9 @@ static u32 IsNodeInTable(X3DNode *node, char *NDTName)
 
 static u32 GetNDTCount(char *NDTName, GF_List *XNodes)
 {
-	u32 i, nodeCount;
-	X3DNode *n;
-	nodeCount = 0;
+	u32 i, nodeCount = 0;
 	for (i=0; i<gf_list_count(XNodes); i++) {
-		n = gf_list_get(XNodes, i);
+		X3DNode *n = gf_list_get(XNodes, i);
 		if (!IsNodeInTable(n, NDTName)) continue;
 		nodeCount++;
 	}
@@ -943,7 +944,7 @@ void WriteNDT(FILE *f, GF_List *XNodes, GF_List *NDTs)
 		NDTName = gf_list_get(NDTs, i);
 		count = GetNDTCount(NDTName, XNodes);
 		if (!count) continue;
-		
+
 		fprintf(f, "#define %s_X3D_Count\t%d\n", NDTName, count);
 		fprintf(f, "static const u32 %s_X3D_TypeToTag[%d] = {\n", NDTName, count);
 		first = 1;
@@ -951,7 +952,7 @@ void WriteNDT(FILE *f, GF_List *XNodes, GF_List *NDTs)
 		for (j=0; j<gf_list_count(XNodes); j++) {
 			n = gf_list_get(XNodes, j);
 			if (!IsNodeInTable(n, NDTName)) continue;
-			
+
 			if (first) {
 				fprintf(f, " TAG_X3D_%s", n->name);
 				first = 0;
@@ -996,7 +997,7 @@ void ParseTemplateFile(FILE *nodes, GF_List *BNodes, GF_List *NDTs)
 		if (sLine[0] == '\n') continue;
 
 		CurrentLine = sLine;
-		
+
 		//parse the line till end of line
 		while (GetNextToken(token, " \t")) {
 
@@ -1021,7 +1022,7 @@ void ParseTemplateFile(FILE *nodes, GF_List *BNodes, GF_List *NDTs)
 
 					//update the NDT list
 					CheckInTable(token, NDTs);
-					p = malloc(strlen(token)+1);
+					p = gf_malloc(strlen(token)+1);
 					strcpy(p, token);
 					gf_list_add(n->NDT, p);
 				}
@@ -1038,7 +1039,7 @@ void ParseTemplateFile(FILE *nodes, GF_List *BNodes, GF_List *NDTs)
 				}
 				f = BlankField();
 				gf_list_add(n->Fields, f);
-				
+
 				//get the field type
 				strcpy(f->type, token);
 				GetNextToken(f->familly, " \t");
@@ -1101,7 +1102,7 @@ void ParseTemplateFile(FILE *nodes, GF_List *BNodes, GF_List *NDTs)
 					case 'a':
 						printf("Corrupted X3D template file (quantization/animation not allowed)\n");
 						gf_list_del_item(n->Fields, f);
-						free(f);
+						gf_free(f);
 						return;
 					default:
 						break;
@@ -1110,7 +1111,7 @@ void ParseTemplateFile(FILE *nodes, GF_List *BNodes, GF_List *NDTs)
 				/*we ignore these*/
 				if (!stricmp(f->name, "bboxCenter") || !stricmp(f->name, "bboxSize")) {
 					gf_list_del_item(n->Fields, f);
-					free(f);
+					gf_free(f);
 				}
 			}
 		}
@@ -1135,12 +1136,11 @@ void ParseTemplateFile(FILE *nodes, GF_List *BNodes, GF_List *NDTs)
 
 void WriteNodeDump(FILE *f, X3DNode *n)
 {
-	X3DField *bf;
 	u32 i;
 
 	fprintf(f, "static const char *%s_FieldName[] = {\n", n->name);
 	for (i=0; i<gf_list_count(n->Fields); i++) {
-		bf = gf_list_get(n->Fields, i);
+		X3DField *bf = gf_list_get(n->Fields, i);
 		if (!i) {
 			fprintf(f, " \"%s\"", bf->name);
 		} else {
@@ -1163,7 +1163,7 @@ void parse_profile(GF_List *nodes, FILE *prof)
 		//skip comment and empty lines
 		if (sLine[0] == '#') continue;
 		if (sLine[0] == '\n') continue;
-		if (strstr(sLine, "Proximity")) 
+		if (strstr(sLine, "Proximity"))
 			found = 0;
 		found = 1;
 		while (found) {
@@ -1198,13 +1198,12 @@ void parse_profile(GF_List *nodes, FILE *prof)
 
 int main (int argc, char **argv)
 {
-	FILE *nodes, *pf;
+	FILE *nodes;
 	GF_List *XNodes, *NDTs;
-	X3DNode *n;
 	X3DField *bf;
 	u32 nb_nodes, nb_imp;
 
-	nodes = fopen("templates_X3D.txt", "rt");
+	nodes = gf_fopen("templates_X3D.txt", "rt");
 	if (!nodes) {
 		fprintf(stdout, "cannot open \"templates_X3D.txt\" - aborting\n");
 		return 0;
@@ -1214,33 +1213,35 @@ int main (int argc, char **argv)
 	NDTs = gf_list_new();
 	//all nodes are in the same list but we keep version info
 	ParseTemplateFile(nodes, XNodes, NDTs);
-	fclose(nodes);
+	gf_fclose(nodes);
 
 	if (argc>1) {
-		pf = fopen(argv[1], "rt");
+		FILE *pf;
+		pf = gf_fopen(argv[1], "rt");
 		if (!pf) fprintf(stdout, "Cannot open profile file %s\n", argv[1]);
 		else {
 			parse_profile(XNodes, pf);
-			fclose(pf);
+			gf_fclose(pf);
 		}
 	}
-	
+
 	//write the nodes def
 	WriteNodesFile(XNodes, NDTs);
 
 	nodes = BeginFile(1);
-	
+
 	//write all nodes init stuff
 	WriteNodeCode(XNodes, nodes);
 
 	WriteNDT(nodes, XNodes, NDTs);
+	fprintf(nodes, "#endif /*GPAC_DISABLE_X3D*/\n\n");
 
 	EndFile(nodes, 1);
 
 	//free NDTs
 	while (gf_list_count(NDTs)) {
 		char *tmp = gf_list_get(NDTs, 0);
-		free(tmp);
+		gf_free(tmp);
 		gf_list_rem(NDTs, 0);
 	}
 	gf_list_del(NDTs);
@@ -1249,22 +1250,22 @@ int main (int argc, char **argv)
 	nb_imp = 0;
 	//free nodes
 	while (gf_list_count(XNodes)) {
-		n = gf_list_get(XNodes, 0);
+		X3DNode *n = gf_list_get(XNodes, 0);
 		if (!n->skip_impl) nb_imp++;
 		gf_list_rem(XNodes, 0);
 		while (gf_list_count(n->NDT)) {
 			char *tmp = gf_list_get(n->NDT, 0);
-			free(tmp);
+			gf_free(tmp);
 			gf_list_rem(n->NDT, 0);
 		}
 		gf_list_del(n->NDT);
 		while (gf_list_count(n->Fields)) {
 			bf = gf_list_get(n->Fields, 0);
-			free(bf);
+			gf_free(bf);
 			gf_list_rem(n->Fields, 0);
 		}
 		gf_list_del(n->Fields);
-		free(n);
+		gf_free(n);
 	}
 	gf_list_del(XNodes);
 
