@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2019
  *					All rights reserved
  *
  *  This file is part of GPAC / alsa audio output module
@@ -10,16 +11,16 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
- *		
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  */
 
 
@@ -27,11 +28,11 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <poll.h>
-#include <alsa/asoundlib.h>	      
+#include <alsa/asoundlib.h>
 #include <gpac/modules/audio_out.h>
 
 
-typedef struct 
+typedef struct
 {
 	snd_pcm_t *playback_handle;
 	u32 nb_ch, buf_size, delay, num_buffers, total_duration, block_align;
@@ -44,17 +45,12 @@ typedef struct
 static GF_Err ALSA_Setup(GF_AudioOutput*dr, void *os_handle, u32 num_buffers, u32 total_duration)
 {
 	int err;
-	const char *opt;
 	ALSAContext *ctx = (ALSAContext*)dr->opaque;
-	
 
-	opt = gf_modules_get_option((GF_BaseInterface *)dr, "ALSA", "ForceSampleRate");
-	if (opt) ctx->force_sr = atoi(opt);
-	ctx->dev_name = gf_modules_get_option((GF_BaseInterface *)dr, "ALSA", "DeviceName");
-	if (!ctx->dev_name) {
-		ctx->dev_name = "hw:0,0";
-		gf_modules_set_option((GF_BaseInterface *)dr, "ALSA", "DeviceName", ctx->dev_name);
-	}
+
+	ctx->force_sr = gf_module_get_int((GF_BaseInterface *)dr, "force-rate");
+	ctx->dev_name = gf_module_get_key((GF_BaseInterface *)dr, "devname");
+	if (!ctx->dev_name) ctx->dev_name = "hw:0,0";
 
 	/*test device*/
 	err = snd_pcm_open(&ctx->playback_handle, ctx->dev_name, SND_PCM_STREAM_PLAYBACK, 0);
@@ -75,11 +71,11 @@ static void ALSA_Shutdown(GF_AudioOutput*dr)
 		snd_pcm_close(ctx->playback_handle);
 		ctx->playback_handle = NULL;
 	}
-	if (ctx->wav_buf) free(ctx->wav_buf);
+	if (ctx->wav_buf) gf_free(ctx->wav_buf);
 	ctx->wav_buf = NULL;
 }
 
-static GF_Err ALSA_ConfigureOutput(GF_AudioOutput*dr, u32 *SampleRate, u32 *NbChannels, u32 *nbBitsPerSample, u32 channel_cfg)
+static GF_Err ALSA_Configure(GF_AudioOutput*dr, u32 *SampleRate, u32 *NbChannels, u32 *audioFormat, u64 channel_cfg)
 {
 	snd_pcm_hw_params_t *hw_params = NULL;
 	int err;
@@ -94,7 +90,7 @@ static GF_Err ALSA_ConfigureOutput(GF_AudioOutput*dr, u32 *SampleRate, u32 *NbCh
 		snd_pcm_close(ctx->playback_handle);
 		ctx->playback_handle = NULL;
 	}
-	if (ctx->wav_buf) free(ctx->wav_buf);
+	if (ctx->wav_buf) gf_free(ctx->wav_buf);
 	ctx->wav_buf = NULL;
 
 	err = snd_pcm_open(&ctx->playback_handle, ctx->dev_name, SND_PCM_STREAM_PLAYBACK, 0/*SND_PCM_NONBLOCK*/);
@@ -121,12 +117,20 @@ static GF_Err ALSA_ConfigureOutput(GF_AudioOutput*dr, u32 *SampleRate, u32 *NbCh
 	/*set output format*/
 	ctx->nb_ch = (int) (*NbChannels);
 	ctx->block_align = ctx->nb_ch;
-	if ((*nbBitsPerSample) == 16) {
+
+	//only support for PCM 8/16/24/32 packet mode
+	switch (*audioFormat) {
+	case GF_AUDIO_FMT_U8:
+		err = snd_pcm_hw_params_set_format(ctx->playback_handle, hw_params, SND_PCM_FORMAT_U8);
+		break;
+	default:
+		*audioFormat = GF_AUDIO_FMT_S16;
+	case GF_AUDIO_FMT_S16:
 		ctx->block_align *= 2;
 		err = snd_pcm_hw_params_set_format(ctx->playback_handle, hw_params, SND_PCM_FORMAT_S16_LE);
-	} else {
-		err = snd_pcm_hw_params_set_format(ctx->playback_handle, hw_params, SND_PCM_FORMAT_U8);
+		break;
 	}
+
 	if (err < 0) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[ALSA] Cannot set sample format: %s\n", snd_strerror (err)) );
 		goto err_exit;
@@ -179,7 +183,7 @@ static GF_Err ALSA_ConfigureOutput(GF_AudioOutput*dr, u32 *SampleRate, u32 *NbCh
 	ctx->buf_size *= ctx->block_align;
 	/*get period time*/
 	snd_pcm_hw_params_get_period_time(hw_params, &period_time, 0);
-	
+
 	err = snd_pcm_hw_params(ctx->playback_handle, hw_params);
 	if (err < 0) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[ALSA] Cannot set parameters: %s\n", snd_strerror (err)) );
@@ -191,11 +195,11 @@ static GF_Err ALSA_ConfigureOutput(GF_AudioOutput*dr, u32 *SampleRate, u32 *NbCh
 	ctx->delay = (ctx->buf_size*1000) / (sr*ctx->block_align);
 
 	/*allocate a single buffer*/
-	ctx->wav_buf = malloc(ctx->buf_size*sizeof(char));
+	ctx->wav_buf = gf_malloc(ctx->buf_size*sizeof(char));
 	if(!ctx->wav_buf) return GF_OUT_OF_MEM;
 	memset(ctx->wav_buf, 0, ctx->buf_size*sizeof(char));
 	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[ALSA] Setup %d ch @ %d hz - %d periods of %d us - total buffer size %d - overall delay %d ms\n", ctx->nb_ch, sr, nb_bufs, period_time, ctx->buf_size, ctx->delay));
-	
+
 	return GF_OK;
 
 err_exit:
@@ -238,7 +242,7 @@ static void ALSA_WriteAudio(GF_AudioOutput*dr)
 		GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[ALSA] no frame to write\n" ));
 		return;
 	}
-	
+
 	//assert(nb_frames*ctx->block_align<=ctx->buf_size);
 	written = dr->FillBuffer(dr->audio_renderer, ctx->wav_buf, (u32) (ctx->block_align*nb_frames) );
 	if (!written) return;
@@ -255,18 +259,6 @@ static void ALSA_WriteAudio(GF_AudioOutput*dr)
 	}
 }
 
-static void ALSA_SetVolume(GF_AudioOutput*dr, u32 Volume)
-{
-}
-
-static void ALSA_SetPan(GF_AudioOutput*dr, u32 Pan) 
-{
-}
-
-static void ALSA_SetPriority(GF_AudioOutput*dr, u32 Priority) 
-{
-}
-
 static u32 ALSA_GetAudioDelay(GF_AudioOutput*dr)
 {
 	ALSAContext *ctx = (ALSAContext*)dr->opaque;
@@ -310,6 +302,12 @@ err_exit:
 	return GF_IO_ERR;
 }
 
+static GF_GPACArg ALSAArgs[] = {
+	GF_DEF_ARG("devname", NULL, "alsa device name", "hw:0,0", NULL, GF_ARG_STRING, 0),
+	GF_DEF_ARG("force-rate", NULL, "force alsa sample rate", "0", NULL, GF_ARG_INT, 0),
+	{0},
+};
+
 void *NewALSAOutput()
 {
 	ALSAContext *ctx;
@@ -319,21 +317,21 @@ void *NewALSAOutput()
 
 	GF_SAFEALLOC(driv, GF_AudioOutput);
 	if(!driv) {
-		free(ctx);
+		gf_free(ctx);
 		return NULL;
 	}
 	driv->opaque = ctx;
 	driv->SelfThreaded = 0;
 	driv->Setup = ALSA_Setup;
 	driv->Shutdown = ALSA_Shutdown;
-	driv->ConfigureOutput = ALSA_ConfigureOutput;
+	driv->Configure = ALSA_Configure;
 	driv->GetAudioDelay = ALSA_GetAudioDelay;
-	driv->SetVolume = ALSA_SetVolume;
-	driv->SetPan = ALSA_SetPan;
-	driv->SetPriority = ALSA_SetPriority;
 	driv->QueryOutputSampleRate = ALSA_QueryOutputSampleRate;
 	driv->WriteAudio = ALSA_WriteAudio;
-	GF_REGISTER_MODULE_INTERFACE(driv, GF_AUDIO_OUTPUT_INTERFACE, "ALSA Audio Output", "gpac distribution");
+	driv->args = ALSAArgs;
+	driv->description = "Audio output using ASLA";
+
+	GF_REGISTER_MODULE_INTERFACE(driv, GF_AUDIO_OUTPUT_INTERFACE, "alsa", "gpac distribution");
 	return driv;
 }
 
@@ -341,8 +339,8 @@ void DeleteALSAOutput(void *ifce)
 {
 	GF_AudioOutput*dr = (GF_AudioOutput*) ifce;
 	ALSAContext *ctx = (ALSAContext *)dr->opaque;
-	free(ctx);
-	free(dr);
+	gf_free(ctx);
+	gf_free(dr);
 }
 
 
@@ -350,22 +348,29 @@ void DeleteALSAOutput(void *ifce)
  * ********************************************************************
  * interface
  */
-Bool QueryInterface(u32 InterfaceType)
+GPAC_MODULE_EXPORT
+const u32 *QueryInterfaces()
 {
-	if (InterfaceType == GF_AUDIO_OUTPUT_INTERFACE) 
-		return 1;
-	return 0;
+	static u32 si [] = {
+		GF_AUDIO_OUTPUT_INTERFACE,
+		0
+	};
+	return si;
 }
 
+GPAC_MODULE_EXPORT
 GF_BaseInterface *LoadInterface(u32 InterfaceType)
 {
-	if (InterfaceType == GF_AUDIO_OUTPUT_INTERFACE) 
+	if (InterfaceType == GF_AUDIO_OUTPUT_INTERFACE)
 		return NewALSAOutput();
 	return NULL;
 }
 
+GPAC_MODULE_EXPORT
 void ShutdownInterface(GF_BaseInterface *ifce)
 {
 	if (ifce->InterfaceType==GF_AUDIO_OUTPUT_INTERFACE)
 		DeleteALSAOutput((GF_AudioOutput*)ifce);
 }
+
+GPAC_MODULE_STATIC_DECLARATION( alsa )
