@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / DirectX audio and video render module
@@ -10,16 +11,16 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
- *		
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  */
 
 
@@ -33,17 +34,19 @@
 #include <ks.h>
 #include <ksmedia.h>
 const static GUID  GPAC_KSDATAFORMAT_SUBTYPE_PCM = {0x00000001,0x0000,0x0010,
-{0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71}};
+	{0x80,0x00,0x00,0xaa,0x00,0x38,0x9b,0x71}
+};
 #endif
 
 
+typedef HRESULT (WINAPI *DIRECTSOUNDCREATEPROC)(LPCGUID pcGuidDevice, LPDIRECTSOUND *ppDS, LPUNKNOWN pUnkOuter);
 
 /*
 		DirectSound audio output
 */
 #define MAX_NUM_BUFFERS		20
 
-typedef struct 
+typedef struct
 {
 	Bool force_config;
 	u32 cfg_num_buffers, cfg_duration;
@@ -54,12 +57,15 @@ typedef struct
 	IDirectSoundBuffer *pOutput;
 
 	u32 buffer_size, num_audio_buffer, total_audio_buffer_ms;
-	
+
 	/*notifs*/
 	Bool use_notif;
-    u32 frame_state[MAX_NUM_BUFFERS];
-    DSBPOSITIONNOTIFY notif_events[MAX_NUM_BUFFERS];
-    HANDLE events[MAX_NUM_BUFFERS];
+	u32 frame_state[MAX_NUM_BUFFERS];
+	DSBPOSITIONNOTIFY notif_events[MAX_NUM_BUFFERS];
+	HANDLE events[MAX_NUM_BUFFERS];
+
+	HMODULE hDSoundLib;
+	DIRECTSOUNDCREATEPROC DirectSoundCreate;
 } DSContext;
 
 #define DSCONTEXT()		DSContext *ctx = (DSContext *)dr->opaque;
@@ -69,8 +75,7 @@ void DS_WriteAudio_Notifs(GF_AudioOutput *dr);
 
 static GF_Err DS_Setup(GF_AudioOutput *dr, void *os_handle, u32 num_buffers, u32 total_duration)
 {
-	DWORD flags;
-    HRESULT hr;
+	HRESULT hr;
 
 	DSCONTEXT();
 	ctx->hWnd = (HWND) os_handle;
@@ -79,15 +84,15 @@ static GF_Err DS_Setup(GF_AudioOutput *dr, void *os_handle, u32 num_buffers, u32
 	/*too bad, use desktop as window*/
 	if (!ctx->hWnd) ctx->hWnd = GetDesktopWindow();
 
-	ctx->force_config = (num_buffers && total_duration) ? 1 : 0;
+	ctx->force_config = (num_buffers && total_duration) ? GF_TRUE : GF_FALSE;
 	ctx->cfg_num_buffers = num_buffers;
 	ctx->cfg_duration = total_duration;
 	if (ctx->cfg_num_buffers <= 1) ctx->cfg_num_buffers = 2;
 
-	if ( FAILED( hr = DirectSoundCreate( NULL, &ctx->pDS, NULL ) ) ) return GF_IO_ERR;
-	flags = DSSCL_EXCLUSIVE;
-	if( FAILED( hr = IDirectSound_SetCooperativeLevel(ctx->pDS, ctx->hWnd, DSSCL_EXCLUSIVE) ) ) {
-		SAFE_DS_RELEASE( ctx->pDS ); 
+	if ( FAILED( hr = ctx->DirectSoundCreate( NULL, &ctx->pDS, NULL ) ) ) return GF_IO_ERR;
+
+	if( FAILED( hr = ctx->pDS->lpVtbl->SetCooperativeLevel(ctx->pDS, ctx->hWnd, DSSCL_EXCLUSIVE) ) ) {
+		SAFE_DS_RELEASE( ctx->pDS );
 		return GF_IO_ERR;
 	}
 	return GF_OK;
@@ -96,13 +101,13 @@ static GF_Err DS_Setup(GF_AudioOutput *dr, void *os_handle, u32 num_buffers, u32
 
 void DS_ResetBuffer(DSContext *ctx)
 {
-    VOID *pLock = NULL;
-    DWORD size;
+	VOID *pLock = NULL;
+	DWORD size;
 
-    if( FAILED(IDirectSoundBuffer_Lock(ctx->pOutput, 0, ctx->buffer_size*ctx->num_audio_buffer, &pLock, &size, NULL, NULL, 0 ) ) )
-        return;
+	if( FAILED(ctx->pOutput->lpVtbl->Lock(ctx->pOutput, 0, ctx->buffer_size*ctx->num_audio_buffer, &pLock, &size, NULL, NULL, 0 ) ) )
+		return;
 	memset(pLock, 0, (size_t) size);
-	IDirectSoundBuffer_Unlock(ctx->pOutput, pLock, size, NULL, 0L); 
+	ctx->pOutput->lpVtbl->Unlock(ctx->pOutput, pLock, size, NULL, 0L);
 }
 
 void DS_ReleaseBuffer(GF_AudioOutput *dr)
@@ -111,29 +116,28 @@ void DS_ReleaseBuffer(GF_AudioOutput *dr)
 	DSCONTEXT();
 
 	/*stop playing and notif proc*/
-	if (ctx->pOutput) IDirectSoundBuffer_Stop(ctx->pOutput);
+	if (ctx->pOutput) ctx->pOutput->lpVtbl->Stop(ctx->pOutput);
 	SAFE_DS_RELEASE(ctx->pOutput);
-	
+
 	/*use notif, shutdown notifier and event watcher*/
 	if (ctx->use_notif) {
 		for (i=0; i<ctx->num_audio_buffer; i++) CloseHandle(ctx->events[i]);
 	}
-	ctx->use_notif = 0;
+	ctx->use_notif = GF_FALSE;
 }
 
 static void DS_Shutdown(GF_AudioOutput *dr)
 {
 	DSCONTEXT();
 	DS_ReleaseBuffer(dr);
-	SAFE_DS_RELEASE(ctx->pDS ); 
+	SAFE_DS_RELEASE(ctx->pDS );
 }
 
 /*we assume what was asked is what we got*/
-static GF_Err DS_ConfigureOutput(GF_AudioOutput *dr, u32 *SampleRate, u32 *NbChannels, u32 *nbBitsPerSample, u32 channel_cfg)
+static GF_Err DS_Configure(GF_AudioOutput *dr, u32 *SampleRate, u32 *NbChannels, u32 *audioFormat, u64 channel_cfg)
 {
-    u32 i;
-	HRESULT hr; 
-	const char *sOpt;
+	u32 i;
+	HRESULT hr;
 	DSBUFFERDESC dsbBufferDesc;
 	IDirectSoundNotify *pNotify;
 #ifdef USE_WAVE_EX
@@ -142,15 +146,29 @@ static GF_Err DS_ConfigureOutput(GF_AudioOutput *dr, u32 *SampleRate, u32 *NbCha
 	DSCONTEXT();
 
 	DS_ReleaseBuffer(dr);
-
+	//only support for PCM 8/16/24/32 packet mode
+	switch (*audioFormat) {
+	case GF_AUDIO_FMT_U8:
+	case GF_AUDIO_FMT_S16:
+	case GF_AUDIO_FMT_S24:
+	case GF_AUDIO_FMT_S32:
+		break;
+	default:
+		//otherwise force PCM16
+		*audioFormat = GF_AUDIO_FMT_S16;
+		break;
+	}
 	ctx->format.nChannels = *NbChannels;
-	ctx->format.wBitsPerSample = *nbBitsPerSample;
+	ctx->format.wBitsPerSample = gf_audio_fmt_bit_depth( *audioFormat);
 	ctx->format.nSamplesPerSec = *SampleRate;
 	ctx->format.cbSize = sizeof (WAVEFORMATEX);
 	ctx->format.wFormatTag = WAVE_FORMAT_PCM;
 	ctx->format.nBlockAlign = ctx->format.nChannels * ctx->format.wBitsPerSample / 8;
 	ctx->format.nAvgBytesPerSec = ctx->format.nSamplesPerSec * ctx->format.nBlockAlign;
 
+	if (!ctx->format.nBlockAlign)
+		return GF_BAD_PARAM;
+
 	if (!ctx->force_config) {
 		ctx->buffer_size = ctx->format.nBlockAlign * 1024;
 		ctx->num_audio_buffer = 2;
@@ -162,9 +180,9 @@ static GF_Err DS_ConfigureOutput(GF_AudioOutput *dr, u32 *SampleRate, u32 *NbCha
 	/*make sure we're aligned*/
 	while (ctx->buffer_size % ctx->format.nBlockAlign) ctx->buffer_size++;
 
-	ctx->use_notif = 1;
-	sOpt = gf_modules_get_option((GF_BaseInterface *)dr, "Audio", "DisableNotification");
-	if (sOpt && !stricmp(sOpt, "yes")) ctx->use_notif = 0;
+	ctx->use_notif = GF_TRUE;
+	if (gf_module_get_bool((GF_BaseInterface*)dr, "disable-notif"))
+		ctx->use_notif = GF_FALSE;
 
 	memset(&dsbBufferDesc, 0, sizeof(DSBUFFERDESC));
 	dsbBufferDesc.dwSize = sizeof (DSBUFFERDESC);
@@ -179,36 +197,45 @@ static GF_Err DS_ConfigureOutput(GF_AudioOutput *dr, u32 *SampleRate, u32 *NbCha
 		format_ex.Format = ctx->format;
 		format_ex.Format.cbSize = sizeof(WAVEFORMATEXTENSIBLE);
 		format_ex.SubFormat = GPAC_KSDATAFORMAT_SUBTYPE_PCM;
-		format_ex.Samples.wValidBitsPerSample = *nbBitsPerSample;
+		format_ex.Samples.wValidBitsPerSample = gf_audio_fmt_bit_depth(*audioFormat);
+		
 		format_ex.dwChannelMask = 0;
 		if (channel_cfg & GF_AUDIO_CH_FRONT_LEFT) format_ex.dwChannelMask |= SPEAKER_FRONT_LEFT;
 		if (channel_cfg & GF_AUDIO_CH_FRONT_RIGHT) format_ex.dwChannelMask |= SPEAKER_FRONT_RIGHT;
 		if (channel_cfg & GF_AUDIO_CH_FRONT_CENTER) format_ex.dwChannelMask |= SPEAKER_FRONT_CENTER;
 		if (channel_cfg & GF_AUDIO_CH_LFE) format_ex.dwChannelMask |= SPEAKER_LOW_FREQUENCY;
-		if (channel_cfg & GF_AUDIO_CH_BACK_LEFT) format_ex.dwChannelMask |= SPEAKER_BACK_LEFT;
-		if (channel_cfg & GF_AUDIO_CH_BACK_RIGHT) format_ex.dwChannelMask |= SPEAKER_BACK_RIGHT;
-		if (channel_cfg & GF_AUDIO_CH_BACK_CENTER) format_ex.dwChannelMask |= SPEAKER_BACK_CENTER;
-		if (channel_cfg & GF_AUDIO_CH_SIDE_LEFT) format_ex.dwChannelMask |= SPEAKER_SIDE_LEFT;
-		if (channel_cfg & GF_AUDIO_CH_SIDE_RIGHT) format_ex.dwChannelMask |= SPEAKER_SIDE_RIGHT;
+		if (channel_cfg & GF_AUDIO_CH_SURROUND_LEFT) format_ex.dwChannelMask |= SPEAKER_BACK_LEFT;
+		if (channel_cfg & GF_AUDIO_CH_SURROUND_RIGHT) format_ex.dwChannelMask |= SPEAKER_BACK_RIGHT;
+		if (channel_cfg & GF_AUDIO_CH_REAR_CENTER) format_ex.dwChannelMask |= SPEAKER_BACK_CENTER;
+		if (channel_cfg & GF_AUDIO_CH_SIDE_SURROUND_LEFT) format_ex.dwChannelMask |= SPEAKER_SIDE_LEFT;
+		if (channel_cfg & GF_AUDIO_CH_SIDE_SURROUND_RIGHT) format_ex.dwChannelMask |= SPEAKER_SIDE_RIGHT;
+
 		dsbBufferDesc.lpwfxFormat = (WAVEFORMATEX *) &format_ex;
 	}
 #endif
 
 
-	hr = IDirectSound_CreateSoundBuffer(ctx->pDS, &dsbBufferDesc, &ctx->pOutput, NULL );
+	hr = ctx->pDS->lpVtbl->CreateSoundBuffer(ctx->pDS, &dsbBufferDesc, &ctx->pOutput, NULL );
 	if (FAILED(hr)) {
 retry:
-		if (ctx->use_notif) gf_modules_set_option((GF_BaseInterface *)dr, "Audio", "DisableNotification", "yes");
-		ctx->use_notif = 0;
+		if (ctx->use_notif) gf_opts_set_key("directsnd", "disable-notif", "yes");
+		ctx->use_notif = GF_FALSE;
 		dsbBufferDesc.dwFlags = DSBCAPS_GETCURRENTPOSITION2 | DSBCAPS_GLOBALFOCUS;
-		hr = IDirectSound_CreateSoundBuffer(ctx->pDS, &dsbBufferDesc, &ctx->pOutput, NULL );
-		if (FAILED(hr)) return GF_IO_ERR;
+		hr = ctx->pDS->lpVtbl->CreateSoundBuffer(ctx->pDS, &dsbBufferDesc, &ctx->pOutput, NULL );
+		if (FAILED(hr)) {
+			if (ctx->format.nChannels>2) {
+				GF_LOG(GF_LOG_ERROR, GF_LOG_AUDIO, ("[DirectSound] failed to configure output for %d channels (error %08x) - falling back to stereo\n", *NbChannels, hr));
+				*NbChannels = 2;
+				return DS_Configure(dr, SampleRate, NbChannels, audioFormat, 0);
+			}
+			return GF_IO_ERR;
+		}
 	}
 
 	for (i=0; i<ctx->num_audio_buffer; i++) ctx->frame_state[i] = 0;
 
 	if (ctx->use_notif) {
-		hr = IDirectSoundBuffer_QueryInterface(ctx->pOutput, &IID_IDirectSoundNotify , (void **)&pNotify);
+		hr = ctx->pOutput->lpVtbl->QueryInterface(ctx->pOutput, &IID_IDirectSoundNotify , (void **)&pNotify);
 		if (hr == S_OK) {
 			/*setup the notification positions*/
 			for (i=0; i<ctx->num_audio_buffer; i++) {
@@ -218,18 +245,18 @@ retry:
 			}
 
 			/*Tell DirectSound when to notify us*/
-			hr = IDirectSoundNotify_SetNotificationPositions(pNotify, ctx->num_audio_buffer, ctx->notif_events);
+			hr = pNotify->lpVtbl->SetNotificationPositions(pNotify, ctx->num_audio_buffer, ctx->notif_events);
 
 			if (hr != S_OK) {
-				IDirectSoundNotify_Release(pNotify);
+				pNotify->lpVtbl->Release(pNotify);
 				for (i=0; i<ctx->num_audio_buffer; i++) CloseHandle(ctx->events[i]);
 				SAFE_DS_RELEASE(ctx->pOutput);
 				goto retry;
 			}
 
-			IDirectSoundNotify_Release(pNotify);
+			pNotify->lpVtbl->Release(pNotify);
 		} else {
-			ctx->use_notif = 0;
+			ctx->use_notif = GF_FALSE;
 		}
 	}
 	if (ctx->use_notif) {
@@ -243,54 +270,54 @@ retry:
 	/*reset*/
 	DS_ResetBuffer(ctx);
 	/*play*/
-	IDirectSoundBuffer_Play(ctx->pOutput, 0, 0, DSBPLAY_LOOPING);	
+	ctx->pOutput->lpVtbl->Play(ctx->pOutput, 0, 0, DSBPLAY_LOOPING);
 	return GF_OK;
 }
 
 static Bool DS_RestoreBuffer(LPDIRECTSOUNDBUFFER pDSBuffer)
 {
 	DWORD dwStatus;
-    IDirectSoundBuffer_GetStatus(pDSBuffer, &dwStatus );
-    if( dwStatus & DSBSTATUS_BUFFERLOST ) {
+	pDSBuffer->lpVtbl->GetStatus(pDSBuffer, &dwStatus );
+	if( dwStatus & DSBSTATUS_BUFFERLOST ) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[DirectSound] buffer lost\n"));
-		IDirectSoundBuffer_Restore(pDSBuffer);
-	    IDirectSoundBuffer_GetStatus(pDSBuffer, &dwStatus);
-		if( dwStatus & DSBSTATUS_BUFFERLOST ) return 1;
-    }
-	return 0;
+		pDSBuffer->lpVtbl->Restore(pDSBuffer);
+		pDSBuffer->lpVtbl->GetStatus(pDSBuffer, &dwStatus);
+		if( dwStatus & DSBSTATUS_BUFFERLOST ) return GF_TRUE;
+	}
+	return GF_FALSE;
 }
 
 
 
-void DS_FillBuffer(GF_AudioOutput *dr, u32 buffer) 
+void DS_FillBuffer(GF_AudioOutput *dr, u32 buffer)
 {
 	HRESULT hr;
-    VOID *pLock;
+	VOID *pLock;
 	u32 pos;
-    DWORD size;
+	DWORD size;
 	DSCONTEXT();
 
 	/*restoring*/
-    if (DS_RestoreBuffer(ctx->pOutput)) {
+	if (DS_RestoreBuffer(ctx->pOutput)) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[DirectSound] restoring sound buffer\n"));
 		return;
 	}
-	
+
 	/*lock and fill from current pos*/
 	pos = buffer * ctx->buffer_size;
 	pLock = NULL;
-    if( FAILED( hr = IDirectSoundBuffer_Lock(ctx->pOutput, pos, ctx->buffer_size,  
-			&pLock,  &size, NULL, NULL, 0L ) ) ) {
+	if( FAILED( hr = ctx->pOutput->lpVtbl->Lock(ctx->pOutput, pos, ctx->buffer_size,
+	                 &pLock,  &size, NULL, NULL, 0L ) ) ) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[DirectSound] Error locking sound buffer\n"));
-        return;
+		return;
 	}
 
-	assert(size == ctx->buffer_size);
+	gf_assert(size == ctx->buffer_size);
 
 	dr->FillBuffer(dr->audio_renderer, pLock, size);
 
 	/*update current pos*/
-    if( FAILED( hr = IDirectSoundBuffer_Unlock(ctx->pOutput, pLock, size, NULL, 0)) ) {
+	if( FAILED( hr = ctx->pOutput->lpVtbl->Unlock(ctx->pOutput, pLock, size, NULL, 0)) ) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[DirectSound] Error unlocking sound buffer\n"));
 	}
 	ctx->frame_state[buffer] = 1;
@@ -319,19 +346,20 @@ void DS_WriteAudio_Notifs(GF_AudioOutput *dr)
 void DS_WriteAudio(GF_AudioOutput *dr)
 {
 	u32 retry;
-    DWORD in_play, cur_play;
+	DWORD in_play, cur_play;
 	DSCONTEXT();
+	if (!ctx->pOutput) return;
 
 	/*wait for end of current play buffer*/
-	if (IDirectSoundBuffer_GetCurrentPosition(ctx->pOutput, &in_play, NULL) != DS_OK ) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[DirectSound] error getting sound buffer poitions\n"));
+	if (ctx->pOutput->lpVtbl->GetCurrentPosition(ctx->pOutput, &in_play, NULL) != DS_OK ) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[DirectSound] error getting sound buffer positions\n"));
 		return;
 	}
 	in_play = (in_play / ctx->buffer_size);
 	retry = 6;
 	while (retry) {
-		if (IDirectSoundBuffer_GetCurrentPosition(ctx->pOutput, &cur_play, NULL) != DS_OK ) {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[DirectSound] error getting sound buffer poitions\n"));
+		if (ctx->pOutput->lpVtbl->GetCurrentPosition(ctx->pOutput, &cur_play, NULL) != DS_OK ) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[DirectSound] error getting sound buffer positions\n"));
 			return;
 		}
 		cur_play = (cur_play / ctx->buffer_size);
@@ -358,12 +386,12 @@ static void DS_Play(GF_AudioOutput *dr, u32 PlayType)
 	DSCONTEXT();
 	switch (PlayType) {
 	case 0:
-		IDirectSoundBuffer_Stop(ctx->pOutput);
+		ctx->pOutput->lpVtbl->Stop(ctx->pOutput);
 		break;
 	case 2:
 		DS_ResetBuffer(ctx);
 	case 1:
-		IDirectSoundBuffer_Play(ctx->pOutput, 0, 0, DSBPLAY_LOOPING);
+		ctx->pOutput->lpVtbl->Play(ctx->pOutput, 0, 0, DSBPLAY_LOOPING);
 		break;
 	}
 }
@@ -372,15 +400,18 @@ static void DS_SetVolume(GF_AudioOutput *dr, u32 Volume)
 {
 	LONG Vol;
 	DSCONTEXT();
-	if (Volume > 100) Volume = 100;
-	Vol = DSBVOLUME_MIN/2 + Volume * (DSBVOLUME_MAX-DSBVOLUME_MIN/2) / 100;
-	IDirectSoundBuffer_SetVolume(ctx->pOutput, Vol);
+	if (!ctx->pOutput) return;
+	if (Volume > 100) Vol = DSBVOLUME_MAX;
+	else if(Volume == 0) Vol = DSBVOLUME_MIN;
+	else Vol = DSBVOLUME_MIN/2 + Volume * (DSBVOLUME_MAX-DSBVOLUME_MIN/2) / 100;
+	ctx->pOutput->lpVtbl->SetVolume(ctx->pOutput, Vol);
 }
 
 static void DS_SetPan(GF_AudioOutput *dr, u32 Pan)
 {
 	LONG dspan;
 	DSCONTEXT();
+	if (!ctx->pOutput) return;
 
 	if (Pan > 100) Pan = 100;
 	if (Pan > 50) {
@@ -390,7 +421,7 @@ static void DS_SetPan(GF_AudioOutput *dr, u32 Pan)
 	} else {
 		dspan = 0;
 	}
-	IDirectSoundBuffer_SetPan(ctx->pOutput, dspan);
+	ctx->pOutput->lpVtbl->SetPan(ctx->pOutput, dspan);
 }
 
 
@@ -410,6 +441,11 @@ static u32 DS_GetTotalBufferTime(GF_AudioOutput *dr)
 	return ctx->total_audio_buffer_ms;
 }
 
+static GF_GPACArg DSAudioArgs[] = {
+	GF_DEF_ARG("disable-notif", NULL, "disable buffer position notifications", "false", NULL, GF_ARG_BOOL, 0),
+	{0},
+};
+
 void *NewAudioOutput()
 {
 	HRESULT hr;
@@ -418,19 +454,33 @@ void *NewAudioOutput()
 
 	if( FAILED( hr = CoInitialize(NULL) ) ) return NULL;
 
-	
-	ctx = malloc(sizeof(DSContext));
-	memset(ctx, 0, sizeof(DSContext));
 
-	driv = malloc(sizeof(GF_AudioOutput));
+	ctx = (DSContext*)gf_malloc(sizeof(DSContext));
+	memset(ctx, 0, sizeof(DSContext));
+#ifdef UNICODE
+	ctx->hDSoundLib = LoadLibrary(L"dsound.dll");
+#else
+	ctx->hDSoundLib = LoadLibrary("dsound.dll");
+#endif
+	if (ctx->hDSoundLib) {
+		ctx->DirectSoundCreate = (DIRECTSOUNDCREATEPROC) GetProcAddress(ctx->hDSoundLib, "DirectSoundCreate");
+	}
+	if (!ctx->DirectSoundCreate) {
+		if (ctx->hDSoundLib) FreeLibrary(ctx->hDSoundLib);
+		gf_free(ctx);
+		return NULL;
+	}
+
+
+	driv = (GF_AudioOutput*)gf_malloc(sizeof(GF_AudioOutput));
 	memset(driv, 0, sizeof(GF_AudioOutput));
-	GF_REGISTER_MODULE_INTERFACE(driv, GF_AUDIO_OUTPUT_INTERFACE, "DirectSound Audio Output", "gpac distribution");
+	GF_REGISTER_MODULE_INTERFACE(driv, GF_AUDIO_OUTPUT_INTERFACE, "directsnd", "gpac distribution");
 
 	driv->opaque = ctx;
 
 	driv->Setup = DS_Setup;
 	driv->Shutdown = DS_Shutdown;
-	driv->ConfigureOutput = DS_ConfigureOutput;
+	driv->Configure = DS_Configure;
 	driv->SetVolume = DS_SetVolume;
 	driv->SetPan = DS_SetPan;
 	driv->Play = DS_Play;
@@ -440,17 +490,21 @@ void *NewAudioOutput()
 	driv->WriteAudio = DS_WriteAudio;
 	driv->QueryOutputSampleRate = DS_QueryOutputSampleRate;
 	/*never threaded*/
-	driv->SelfThreaded = 0;
+	driv->SelfThreaded = GF_FALSE;
+	driv->args = DSAudioArgs;
+	driv->description = "DirectSound Audio output";
 	return driv;
 }
 
-void DeleteAudioOutput(void *ifce)
+void DeleteDxAudioOutput(void *ifce)
 {
 	GF_AudioOutput *dr = (GF_AudioOutput *)ifce;
 	DSCONTEXT();
 
-	free(ctx);
-	free(ifce);
+	if (ctx->hDSoundLib)
+		FreeLibrary(ctx->hDSoundLib);
+	gf_free(ctx);
+	gf_free(ifce);
 	CoUninitialize();
 }
 
