@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2012
  *					All rights reserved
  *
  *  This file is part of GPAC / wave audio render module
@@ -10,16 +11,16 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
- *		
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
  */
 
 
@@ -28,7 +29,7 @@
 
 #define MAX_AUDIO_BUFFER	30
 
-typedef struct 
+typedef struct
 {
 	HWAVEOUT hwo;
 	WAVEHDR wav_hdr[MAX_AUDIO_BUFFER];
@@ -90,15 +91,15 @@ typedef struct
 
 #ifndef _WAVEFORMATEXTENSIBLE_
 typedef struct {
-    WAVEFORMATEX    Format;
-    union {
-        WORD wValidBitsPerSample;       /* bits of precision  */
-        WORD wSamplesPerBlock;          /* valid if wBitsPerSample==0 */
-        WORD wReserved;                 /* If neither applies, set to zero. */
-    } Samples;
-    DWORD           dwChannelMask;      /* which channels are */
-                                        /* present in stream  */
-    GUID            SubFormat;
+	WAVEFORMATEX    Format;
+	union {
+		WORD wValidBitsPerSample;       /* bits of precision  */
+		WORD wSamplesPerBlock;          /* valid if wBitsPerSample==0 */
+		WORD wReserved;                 /* If neither applies, set to zero. */
+	} Samples;
+	DWORD           dwChannelMask;      /* which channels are */
+	/* present in stream  */
+	GUID            SubFormat;
 } WAVEFORMATEXTENSIBLE, *PWAVEFORMATEXTENSIBLE;
 #endif
 
@@ -116,7 +117,7 @@ static GF_Err WAV_Setup(GF_AudioOutput *dr, void *os_handle, u32 num_buffers, u3
 {
 	WAVCTX();
 
-	ctx->force_config = (num_buffers && total_duration) ? 1 : 0;
+	ctx->force_config = (num_buffers && total_duration) ? GF_TRUE : GF_FALSE;
 	ctx->cfg_num_buffers = num_buffers;
 	if (ctx->cfg_num_buffers <= 1) ctx->cfg_num_buffers = 2;
 	ctx->cfg_duration = total_duration;
@@ -125,7 +126,7 @@ static GF_Err WAV_Setup(GF_AudioOutput *dr, void *os_handle, u32 num_buffers, u3
 	return GF_OK;
 }
 
-static void CALLBACK WaveProc(HWAVEOUT hwo, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
+static void CALLBACK WaveProc(HWAVEOUT hwo, UINT uMsg, DWORD_PTR dwInstance, DWORD_PTR dwParam1, DWORD_PTR dwParam2)
 {
 	GF_AudioOutput *dr = (GF_AudioOutput *) dwInstance;
 	WAVCTX();
@@ -133,28 +134,28 @@ static void CALLBACK WaveProc(HWAVEOUT hwo, UINT uMsg, DWORD dwInstance, DWORD d
 	if (uMsg != WOM_DONE) return;
 	if (ctx->exit_request) return;
 	SetEvent(ctx->event);
-} 
+}
 
 
 static void close_waveform(GF_AudioOutput *dr)
 {
 	WAVCTX();
 
-    if (!ctx->event) return;
+	if (!ctx->event) return;
 
 	/*brute-force version, actually much safer on winCE*/
 #ifdef _WIN32_WCE
-	ctx->exit_request = 1;
+	ctx->exit_request = GF_TRUE;
 	SetEvent(ctx->event);
 	waveOutReset(ctx->hwo);
 	waveOutClose(ctx->hwo);
-	if (ctx->wav_buf) free(ctx->wav_buf);
+	if (ctx->wav_buf) gf_free(ctx->wav_buf);
 	ctx->wav_buf = NULL;
-    CloseHandle(ctx->event);
+	CloseHandle(ctx->event);
 	ctx->event = NULL;
-	ctx->exit_request = 0;
+	ctx->exit_request = GF_FALSE;
 #else
-	ctx->exit_request = 1;
+	ctx->exit_request = GF_TRUE;
 	SetEvent(ctx->event);
 	if (ctx->hwo) {
 		u32 i;
@@ -162,10 +163,10 @@ static void close_waveform(GF_AudioOutput *dr)
 		MMRESULT res;
 		/*wait for all buffers to complete, otherwise this locks waveOutReset*/
 		while (1) {
-			not_done = 0;
+			not_done = GF_FALSE;
 			for (i=0 ; i< ctx->num_buffers; i++) {
 				if (! (ctx->wav_hdr[i].dwFlags & WHDR_DONE)) {
-					not_done = 1;
+					not_done = GF_TRUE;
 					break;
 				}
 			}
@@ -179,11 +180,11 @@ static void close_waveform(GF_AudioOutput *dr)
 		}
 		ctx->hwo = NULL;
 	}
-	if (ctx->wav_buf) free(ctx->wav_buf);
+	if (ctx->wav_buf) gf_free(ctx->wav_buf);
 	ctx->wav_buf = NULL;
-    CloseHandle(ctx->event);
+	CloseHandle(ctx->event);
 	ctx->event = NULL;
-	ctx->exit_request = 0;
+	ctx->exit_request = GF_FALSE;
 #endif
 }
 
@@ -194,7 +195,7 @@ static void WAV_Shutdown(GF_AudioOutput *dr)
 
 
 /*we assume what was asked is what we got*/
-static GF_Err WAV_ConfigureOutput(GF_AudioOutput *dr, u32 *SampleRate, u32 *NbChannels, u32 *nbBitsPerSample, u32 channel_cfg)
+static GF_Err WAV_Configure(GF_AudioOutput *dr, u32 *SampleRate, u32 *NbChannels, u32 *audioFormat, u64 channel_cfg)
 {
 	u32 i, retry;
 	HRESULT	hr;
@@ -214,14 +215,27 @@ static GF_Err WAV_ConfigureOutput(GF_AudioOutput *dr, u32 *SampleRate, u32 *NbCh
 	if (*NbChannels>2) *NbChannels=2;
 #endif
 
+	//only support for PCM 8/16/24/32 packet mode
+	switch (*audioFormat) {
+	case GF_AUDIO_FMT_U8:
+	case GF_AUDIO_FMT_S16:
+	case GF_AUDIO_FMT_S24:
+	case GF_AUDIO_FMT_S32:
+		break;
+	default:
+		//otherwise force PCM16
+		*audioFormat = GF_AUDIO_FMT_S16;
+		break;
+	}
+
 	memset (&ctx->fmt, 0, sizeof(ctx->fmt));
 	ctx->fmt.cbSize = sizeof(WAVEFORMATEX);
 	ctx->fmt.wFormatTag = WAVE_FORMAT_PCM;
 	ctx->fmt.nChannels = *NbChannels;
-	ctx->fmt.wBitsPerSample = *nbBitsPerSample;
+	ctx->fmt.wBitsPerSample = gf_audio_fmt_bit_depth(*audioFormat);
 	ctx->fmt.nSamplesPerSec = *SampleRate;
 	ctx->fmt.nBlockAlign = ctx->fmt.wBitsPerSample * ctx->fmt.nChannels / 8;
-	ctx->fmt.nAvgBytesPerSec = *SampleRate * ctx->fmt.nBlockAlign; 
+	ctx->fmt.nAvgBytesPerSec = *SampleRate * ctx->fmt.nBlockAlign;
 	fmt = &ctx->fmt;
 
 #ifdef USE_WAVE_EXT
@@ -237,21 +251,21 @@ static GF_Err WAV_ConfigureOutput(GF_AudioOutput *dr, u32 *SampleRate, u32 *NbCh
 		if (channel_cfg & GF_AUDIO_CH_FRONT_RIGHT) format_ex.dwChannelMask |= SPEAKER_FRONT_RIGHT;
 		if (channel_cfg & GF_AUDIO_CH_FRONT_CENTER) format_ex.dwChannelMask |= SPEAKER_FRONT_CENTER;
 		if (channel_cfg & GF_AUDIO_CH_LFE) format_ex.dwChannelMask |= SPEAKER_LOW_FREQUENCY;
-		if (channel_cfg & GF_AUDIO_CH_BACK_LEFT) format_ex.dwChannelMask |= SPEAKER_BACK_LEFT;
-		if (channel_cfg & GF_AUDIO_CH_BACK_RIGHT) format_ex.dwChannelMask |= SPEAKER_BACK_RIGHT;
-		if (channel_cfg & GF_AUDIO_CH_BACK_CENTER) format_ex.dwChannelMask |= SPEAKER_BACK_CENTER;
-		if (channel_cfg & GF_AUDIO_CH_SIDE_LEFT) format_ex.dwChannelMask |= SPEAKER_SIDE_LEFT;
-		if (channel_cfg & GF_AUDIO_CH_SIDE_RIGHT) format_ex.dwChannelMask |= SPEAKER_SIDE_RIGHT;
+		if (channel_cfg & GF_AUDIO_CH_SURROUND_LEFT) format_ex.dwChannelMask |= SPEAKER_BACK_LEFT;
+		if (channel_cfg & GF_AUDIO_CH_SURROUND_RIGHT) format_ex.dwChannelMask |= SPEAKER_BACK_RIGHT;
+		if (channel_cfg & GF_AUDIO_CH_REAR_CENTER) format_ex.dwChannelMask |= SPEAKER_BACK_CENTER;
+		if (channel_cfg & GF_AUDIO_CH_SIDE_SURROUND_LEFT) format_ex.dwChannelMask |= SPEAKER_SIDE_LEFT;
+		if (channel_cfg & GF_AUDIO_CH_SIDE_SURROUND_RIGHT) format_ex.dwChannelMask |= SPEAKER_SIDE_RIGHT;
 		fmt = (WAVEFORMATEX *) &format_ex;
 	}
 #endif
 
-	/* Open a waveform device for output using window callback. */ 
+	/* Open a waveform device for output using window callback. */
 	retry = 10;
-	while (retry) {		
-		hr = waveOutOpen((LPHWAVEOUT)&ctx->hwo, WAVE_MAPPER, &ctx->fmt, (DWORD) WaveProc, (DWORD) dr, 
-			CALLBACK_FUNCTION | WAVE_ALLOWSYNC | WAVE_FORMAT_DIRECT
-			);
+	while (retry) {
+		hr = waveOutOpen((LPHWAVEOUT)&ctx->hwo, WAVE_MAPPER, &ctx->fmt, (DWORD_PTR) WaveProc, (DWORD_PTR) dr,
+		                 CALLBACK_FUNCTION | WAVE_ALLOWSYNC | WAVE_FORMAT_DIRECT
+		                );
 
 		if (hr == MMSYSERR_NOERROR) break;
 		/*couldn't open audio*/
@@ -259,6 +273,8 @@ static GF_Err WAV_ConfigureOutput(GF_AudioOutput *dr, u32 *SampleRate, u32 *NbCh
 		retry--;
 	}
 	if (hr != MMSYSERR_NOERROR) return GF_IO_ERR;
+	ctx->fmt.nBlockAlign = fmt->nBlockAlign;
+	ctx->fmt.nAvgBytesPerSec = fmt->nAvgBytesPerSec;
 
 	if (!ctx->force_config) {
 		/*one wave buffer size*/
@@ -269,18 +285,18 @@ static GF_Err WAV_ConfigureOutput(GF_AudioOutput *dr, u32 *SampleRate, u32 *NbCh
 		ctx->buffer_size = (ctx->fmt.nAvgBytesPerSec * ctx->cfg_duration) / (1000 * ctx->cfg_num_buffers);
 	}
 
-    ctx->event = CreateEvent( NULL, FALSE, FALSE, NULL);
+	ctx->event = CreateEvent( NULL, FALSE, FALSE, NULL);
 
 	/*make sure we're aligned*/
 	while (ctx->buffer_size % ctx->fmt.nBlockAlign) ctx->buffer_size++;
 
-	ctx->wav_buf = malloc(ctx->buffer_size*ctx->num_buffers*sizeof(char));
+	ctx->wav_buf = (char*)gf_malloc(ctx->buffer_size*ctx->num_buffers*sizeof(char));
 	memset(ctx->wav_buf, 0, ctx->buffer_size*ctx->num_buffers*sizeof(char));
 
 	/*setup wave headers*/
 	for (i=0 ; i < ctx->num_buffers; i++) {
 		memset(& ctx->wav_hdr[i], 0, sizeof(WAVEHDR));
-		ctx->wav_hdr[i].dwBufferLength = ctx->buffer_size; 
+		ctx->wav_hdr[i].dwBufferLength = ctx->buffer_size;
 		ctx->wav_hdr[i].lpData = & ctx->wav_buf[i*ctx->buffer_size];
 		ctx->wav_hdr[i].dwFlags = WHDR_DONE;
 		waveOutPrepareHeader(ctx->hwo, &ctx->wav_hdr[i], sizeof(WAVEHDR));
@@ -296,7 +312,6 @@ static GF_Err WAV_ConfigureOutput(GF_AudioOutput *dr, u32 *SampleRate, u32 *NbCh
 static void WAV_WriteAudio(GF_AudioOutput *dr)
 {
 	LPWAVEHDR hdr;
-	HRESULT hr;
 	u32 i;
 	WAVCTX();
 
@@ -319,7 +334,7 @@ static void WAV_WriteAudio(GF_AudioOutput *dr)
 			/*fill it*/
 			hdr->dwBufferLength = dr->FillBuffer(dr->audio_renderer, hdr->lpData, ctx->buffer_size);
 			hdr->dwFlags = 0;
-		    hr = waveOutPrepareHeader(ctx->hwo, hdr, sizeof(WAVEHDR));
+			waveOutPrepareHeader(ctx->hwo, hdr, sizeof(WAVEHDR));
 			/*write it*/
 			waveOutWrite(ctx->hwo, hdr, sizeof(WAVEHDR));
 		}
@@ -428,23 +443,22 @@ static u32 WAV_GetTotalBufferTime(GF_AudioOutput *dr)
 
 void *NewWAVRender()
 {
-	WAVContext *ctx;
 	GF_AudioOutput *driv;
-	ctx = malloc(sizeof(WAVContext));
+	WAVContext *ctx = (WAVContext*)gf_malloc(sizeof(WAVContext));
 	memset(ctx, 0, sizeof(WAVContext));
 	ctx->num_buffers = 10;
 	ctx->pan = 50;
 	ctx->vol = 100;
-	driv = malloc(sizeof(GF_AudioOutput));
+	driv = (GF_AudioOutput*)gf_malloc(sizeof(GF_AudioOutput));
 	memset(driv, 0, sizeof(GF_AudioOutput));
-	GF_REGISTER_MODULE_INTERFACE(driv, GF_AUDIO_OUTPUT_INTERFACE, "Windows MME Output", "gpac distribution")
+	GF_REGISTER_MODULE_INTERFACE(driv, GF_AUDIO_OUTPUT_INTERFACE, "mme", "gpac distribution")
 
 	driv->opaque = ctx;
 
-	driv->SelfThreaded = 0;
+	driv->SelfThreaded = GF_FALSE;
 	driv->Setup = WAV_Setup;
 	driv->Shutdown = WAV_Shutdown;
-	driv->ConfigureOutput = WAV_ConfigureOutput;
+	driv->Configure= WAV_Configure;
 	driv->GetAudioDelay = WAV_GetAudioDelay;
 	driv->GetTotalBufferTime = WAV_GetTotalBufferTime;
 	driv->SetVolume = WAV_SetVolume;
@@ -460,22 +474,29 @@ void DeleteWAVRender(void *ifce)
 {
 	GF_AudioOutput *dr = (GF_AudioOutput *) ifce;
 	WAVContext *ctx = (WAVContext *)dr->opaque;
-	free(ctx);
-	free(dr);
+	gf_free(ctx);
+	gf_free(dr);
 }
 
-Bool QueryInterface(u32 InterfaceType)
+GPAC_MODULE_EXPORT
+const u32 *QueryInterfaces()
 {
-	if (InterfaceType == GF_AUDIO_OUTPUT_INTERFACE) return 1;
-	return 0;
+	static u32 si [] = {
+		GF_AUDIO_OUTPUT_INTERFACE,
+		0
+	};
+	return si;
 }
 
+GPAC_MODULE_EXPORT
 GF_BaseInterface *LoadInterface(u32 InterfaceType)
 {
-	if (InterfaceType == GF_AUDIO_OUTPUT_INTERFACE) return NewWAVRender();
+	if (InterfaceType == GF_AUDIO_OUTPUT_INTERFACE)
+		return (GF_BaseInterface*)NewWAVRender();
 	return NULL;
 }
 
+GPAC_MODULE_EXPORT
 void ShutdownInterface(GF_BaseInterface *ifce)
 {
 	switch (ifce->InterfaceType) {
@@ -484,3 +505,5 @@ void ShutdownInterface(GF_BaseInterface *ifce)
 		break;
 	}
 }
+
+GPAC_MODULE_STATIC_DECLARATION( wave_out )
