@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / BIFS codec sub-project
@@ -10,24 +11,26 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
 
 
 #include <gpac/internal/bifs_dev.h>
-#include "quant.h" 
-#include "script.h" 
+#include <gpac/scene_manager.h>
+#include "quant.h"
+#include "script.h"
 
+#ifndef GPAC_DISABLE_BIFS
 
 void SFCommandBufferChanged(GF_BifsDecoder * codec, GF_Node *node)
 {
@@ -51,8 +54,8 @@ void BD_OffsetSFTime(GF_BifsDecoder * codec, Double *time)
 
 void BD_CheckSFTimeOffset(GF_BifsDecoder *codec, GF_Node *node, GF_FieldInfo *inf)
 {
-	if (gf_node_get_tag(node) != TAG_ProtoNode) {
-		if (!stricmp(inf->name, "startTime") || !stricmp(inf->name, "stopTime")) 
+	if (inf->name && (gf_node_get_tag(node) != TAG_ProtoNode)) {
+		if (!stricmp(inf->name, "startTime") || !stricmp(inf->name, "stopTime"))
 			BD_OffsetSFTime(codec,  (Double *)inf->far_ptr);
 	} else if (gf_sg_proto_field_is_sftime_offset(node, inf)) {
 		BD_OffsetSFTime(codec,  (Double *)inf->far_ptr);
@@ -62,14 +65,14 @@ void BD_CheckSFTimeOffset(GF_BifsDecoder *codec, GF_Node *node, GF_FieldInfo *in
 
 Fixed BD_ReadSFFloat(GF_BifsDecoder * codec, GF_BitStream *bs)
 {
-	if (codec->ActiveQP && codec->ActiveQP->useEfficientCoding) 
+	if (codec->ActiveQP && codec->ActiveQP->useEfficientCoding)
 		return gf_bifs_dec_mantissa_float(codec, bs);
 
 	return FLT2FIX(gf_bs_read_float(bs));
 }
 
 
-GF_Err gf_bifs_dec_sf_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field)
+GF_Err gf_bifs_dec_sf_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field, Bool is_mem_com)
 {
 	GF_Err e;
 	GF_Node *new_node;
@@ -87,7 +90,7 @@ GF_Err gf_bifs_dec_sf_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *n
 		* ((SFBool *) field->far_ptr) = (SFBool) gf_bs_read_int(bs, 1);
 		break;
 	case GF_SG_VRML_SFCOLOR:
-		((SFColor *)field->far_ptr)->red = BD_ReadSFFloat(codec, bs);;
+		((SFColor *)field->far_ptr)->red = BD_ReadSFFloat(codec, bs);
 		((SFColor *)field->far_ptr)->green = BD_ReadSFFloat(codec, bs);
 		((SFColor *)field->far_ptr)->blue = BD_ReadSFFloat(codec, bs);
 		break;
@@ -121,11 +124,25 @@ GF_Err gf_bifs_dec_sf_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *n
 		length = gf_bs_read_int(bs, size);
 		if (gf_bs_available(bs) < length) return GF_NON_COMPLIANT_BITSTREAM;
 
-		if ( ((SFString *)field->far_ptr)->buffer ) free( ((SFString *)field->far_ptr)->buffer);
-		((SFString *)field->far_ptr)->buffer = (char *)malloc(sizeof(char)*(length+1));
-		memset(((SFString *)field->far_ptr)->buffer , 0, length+1);
-		for (i=0; i<length; i++) {
-			 ((SFString *)field->far_ptr)->buffer[i] = gf_bs_read_int(bs, 8);
+		if (node && (node->sgprivate->tag==TAG_MPEG4_CacheTexture) && (field->fieldIndex<=2)) {
+			M_CacheTexture *ct = (M_CacheTexture *) node;
+			ct->data_len = length;
+			if (ct->data) gf_free(ct->data);
+			ct->data = (u8*)gf_malloc(sizeof(char)*length);
+			gf_bs_read_data(bs, (char*)ct->data, length);
+		} else if (node && (node->sgprivate->tag==TAG_MPEG4_BitWrapper) ) {
+			M_BitWrapper *bw = (M_BitWrapper*) node;
+			if (bw->buffer.buffer) gf_free(bw->buffer.buffer);
+			bw->buffer_len = length;
+			bw->buffer.buffer = (char*)gf_malloc(sizeof(char)*length);
+			gf_bs_read_data(bs, (char*)bw->buffer.buffer, length);
+		} else {
+			if ( ((SFString *)field->far_ptr)->buffer ) gf_free( ((SFString *)field->far_ptr)->buffer);
+			((SFString *)field->far_ptr)->buffer = (char *)gf_malloc(sizeof(char)*(length+1));
+			memset(((SFString *)field->far_ptr)->buffer , 0, length+1);
+			for (i=0; i<length; i++) {
+				((SFString *)field->far_ptr)->buffer[i] = gf_bs_read_int(bs, 8);
+			}
 		}
 		break;
 	case GF_SG_VRML_SFURL:
@@ -133,7 +150,7 @@ GF_Err gf_bifs_dec_sf_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *n
 		SFURL *url = (SFURL *) field->far_ptr;
 		size = gf_bs_read_int(bs, 1);
 		if (size) {
-			if (url->url) free(url->url );
+			if (url->url) gf_free(url->url );
 			url->url = NULL;
 			length = gf_bs_read_int(bs, 10);
 			url->OD_ID = length;
@@ -144,23 +161,26 @@ GF_Err gf_bifs_dec_sf_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *n
 			if (gf_bs_available(bs) < length) return GF_NON_COMPLIANT_BITSTREAM;
 			buffer = NULL;
 			if (length) {
-				buffer = (char *)malloc(sizeof(char)*(length+1));
+				buffer = (char *)gf_malloc(sizeof(char)*(length+1));
 				memset(buffer, 0, length+1);
 				for (i=0; i<length; i++) buffer[i] = gf_bs_read_int(bs, 8);
 			}
-			if (url->url) free( url->url);
+			if (url->url) gf_free( url->url);
 			/*if URL is empty set it to NULL*/
 			if (buffer && strlen(buffer)) {
 				url->url = buffer;
 			} else {
-				free(buffer);
+				gf_free(buffer);
 				url->url = NULL;
 			}
 		}
 	}
-		break;
+	break;
 	case GF_SG_VRML_SFIMAGE:
-		if (((SFImage *)field->far_ptr)->pixels) free(((SFImage *)field->far_ptr)->pixels);
+		if (((SFImage *)field->far_ptr)->pixels) {
+			gf_free(((SFImage *)field->far_ptr)->pixels);
+			((SFImage *)field->far_ptr)->pixels = NULL;
+		}
 		w = gf_bs_read_int(bs, 12);
 		h = gf_bs_read_int(bs, 12);
 		length = gf_bs_read_int(bs, 2);
@@ -172,7 +192,7 @@ GF_Err gf_bifs_dec_sf_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *n
 		((SFImage *)field->far_ptr)->width = w;
 		((SFImage *)field->far_ptr)->height = h;
 		((SFImage *)field->far_ptr)->numComponents = length;
-		((SFImage *)field->far_ptr)->pixels = (unsigned char *)malloc(sizeof(char)*size);
+		((SFImage *)field->far_ptr)->pixels = (unsigned char *)gf_malloc(sizeof(char)*size);
 		//WARNING: Buffers are NOT ALIGNED IN THE BITSTREAM
 		for (i=0; i<size; i++) {
 			((SFImage *)field->far_ptr)->pixels[i] = gf_bs_read_int(bs, 8);
@@ -181,9 +201,11 @@ GF_Err gf_bifs_dec_sf_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *n
 	case GF_SG_VRML_SFCOMMANDBUFFER:
 	{
 		SFCommandBuffer *sfcb = (SFCommandBuffer *)field->far_ptr;
+		if (!node) return GF_BAD_PARAM;
 		if (sfcb->buffer) {
-			free(sfcb->buffer);		
+			gf_free(sfcb->buffer);
 			sfcb->buffer = NULL;
+			sfcb->bufferSize = 0;
 		}
 		while (gf_list_count(sfcb->commandList)) {
 			GF_Command *com = (GF_Command*)gf_list_get(sfcb->commandList, 0);
@@ -197,13 +219,13 @@ GF_Err gf_bifs_dec_sf_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *n
 
 		sfcb->bufferSize = length;
 		if (length) {
-			sfcb->buffer = (unsigned char *)malloc(sizeof(char)*(length));
+			sfcb->buffer = (unsigned char *)gf_malloc(sizeof(char)*(length));
 			//WARNING Buffers are NOT ALIGNED IN THE BITSTREAM
 			for (i=0; i<length; i++) {
 				sfcb->buffer[i] = gf_bs_read_int(bs, 8);
 			}
 		}
-		//notify the node - this is needed in case an enhencement layer replaces the buffer, in which case 
+		//notify the node - this is needed in case an enhencement layer replaces the buffer, in which case
 		//the # ID Bits may change
 		SFCommandBufferChanged(codec, node);
 
@@ -212,63 +234,104 @@ GF_Err gf_bifs_dec_sf_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *n
 		 2 - InputSensor only works on decompressed commands
 		*/
 		if (codec->dec_memory_mode || (node->sgprivate->tag==TAG_MPEG4_InputSensor)) {
-			CommandBufferItem *cbi = (CommandBufferItem *)malloc(sizeof(CommandBufferItem));
+			CommandBufferItem *cbi = (CommandBufferItem *)gf_malloc(sizeof(CommandBufferItem));
 			cbi->node = node;
+			gf_node_register(cbi->node, NULL);
 			cbi->cb = sfcb;
 			gf_list_add(codec->command_buffers, cbi);
 		}
-	} 
-		break;
+	}
+	break;
 	case GF_SG_VRML_SFNODE:
+		//if not memory dec mode, unregister previous node
+		//otherwise the field points to the memory command internal field
+		if (!is_mem_com) {
+			GF_Node *old_node = *((GF_Node **) field->far_ptr);
+			if (old_node != NULL) {
+				u32 count = gf_list_count(codec->command_buffers);
+				for (i=0; i<count; i++) {
+					CommandBufferItem *cbi = (CommandBufferItem*) gf_list_get(codec->command_buffers, i);
+					if (cbi->node == old_node) {
+						gf_list_rem(codec->command_buffers, i);
+						i--;
+						count--;
+						gf_node_unregister(cbi->node, NULL);
+						gf_free(cbi);
+					}
+				}
+				gf_node_unregister(old_node, node);
+				 *((GF_Node **) field->far_ptr) = NULL;
+			}
+		}
 		//for nodes the field ptr is a ptr to the field, which is a node ptr ;)
 		new_node = gf_bifs_dec_node(codec, bs, field->NDTtype);
 		if (new_node) {
-			e = gf_node_register(new_node, node);
+			e = gf_node_register(new_node, is_mem_com ? NULL : node);
 			if (e) return e;
 		}
 		//it may happen that new_node is NULL (this is valid for a proto declaration)
 		*((GF_Node **) field->far_ptr) = new_node;
 		break;
 	case GF_SG_VRML_SFSCRIPT:
-		codec->LastError = SFScript_Parse(codec, (SFScript*)field->far_ptr, bs, node);
+#ifdef GPAC_HAS_QJS
+		codec->LastError = node ? SFScript_Parse(codec, (SFScript*)field->far_ptr, bs, node) : GF_NON_COMPLIANT_BITSTREAM;
+#else
+		return GF_NOT_SUPPORTED;
+#endif
 		break;
+	case GF_SG_VRML_SFATTRREF:
+	{
+		SFAttrRef *ar = (SFAttrRef *)field->far_ptr;
+		u32 nodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
+		ar->node = gf_sg_find_node(codec->current_graph, nodeID);
+		if (!ar->node) {
+
+		} else {
+			u32 nbBitsDEF = gf_get_bit_size(gf_node_get_num_fields_in_mode(ar->node, GF_SG_FIELD_CODING_DEF) - 1);
+			u32 field_ref = gf_bs_read_int(bs, nbBitsDEF);
+			codec->LastError = gf_bifs_get_field_index(ar->node, field_ref, GF_SG_FIELD_CODING_DEF, &ar->fieldIndex);
+		}
+	}
+	break;
 	default:
 		return GF_NON_COMPLIANT_BITSTREAM;
 	}
 	return codec->LastError;
 }
 
-GF_Err BD_DecMFFieldList(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field)
+GF_Err BD_DecMFFieldList(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field, Bool is_mem_com)
 {
 	GF_Node *new_node;
-	GF_Err e;
+	GF_Err e=GF_OK;
 	u8 endFlag, qp_local, qp_on, initial_qp;
 	GF_ChildNodeItem *last = NULL;
 	u32 nbF;
 
 	GF_FieldInfo sffield;
-	
+
 	memset(&sffield, 0, sizeof(GF_FieldInfo));
 	sffield.fieldIndex = field->fieldIndex;
 	sffield.fieldType = gf_sg_vrml_get_sf_type(field->fieldType);
 	sffield.NDTtype = field->NDTtype;
+	sffield.name = field->name;
 
 	nbF = 0;
 	qp_on = qp_local = 0;
 	initial_qp = codec->ActiveQP ? 1 : 0;
 
 	endFlag = gf_bs_read_int(bs, 1);
-	while (!endFlag) {
-		e = GF_OK;;
+	while (!endFlag  && (codec->LastError>=0)) {
 		if (field->fieldType != GF_SG_VRML_MFNODE) {
 			e = gf_sg_vrml_mf_append(field->far_ptr, field->fieldType, & sffield.far_ptr);
-			e = gf_bifs_dec_sf_field(codec, bs, node, &sffield);
+			if (e) goto exit;
+			e = gf_bifs_dec_sf_field(codec, bs, node, &sffield, GF_FALSE);
+			if (e) goto exit;
 		} else {
 			new_node = gf_bifs_dec_node(codec, bs, field->NDTtype);
 			//append
 			if (new_node) {
-				e = gf_node_register(new_node, node);
-				if (e) return e;
+				e = gf_node_register(new_node, is_mem_com ? NULL : node);
+				if (e) goto exit;
 
 				//regular coding
 				if (node) {
@@ -276,21 +339,21 @@ GF_Err BD_DecMFFieldList(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node
 					if (gf_node_get_tag(new_node) == TAG_MPEG4_QuantizationParameter) {
 						qp_local = ((M_QuantizationParameter *)new_node)->isLocal;
 						//we have a QP in the same scope, remove previous
-						if (qp_on) gf_bifs_dec_qp_remove(codec, 0);
+						if (qp_on) gf_bifs_dec_qp_remove(codec, GF_FALSE);
 						e = gf_bifs_dec_qp_set(codec, new_node);
-						if (e) return e;
+						if (e) goto exit;
 						qp_on = 1;
 						if (qp_local) qp_local = 2;
 						if (codec->force_keep_qp) {
-							e = gf_node_list_add_child_last( field->far_ptr, new_node, &last);
+							e = gf_node_list_add_child_last(field->far_ptr, new_node, &last);
 						} else {
 							gf_node_register(new_node, NULL);
 							gf_node_unregister(new_node, node);
 						}
-					} else 
+					} else
 						//this is generic MFNode container
 						e = gf_node_list_add_child_last(field->far_ptr, new_node, &last);
-					
+
 				}
 				//proto coding: directly add the child
 				else if (codec->pCurrentProto) {
@@ -298,14 +361,15 @@ GF_Err BD_DecMFFieldList(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node
 					e = gf_node_list_add_child_last( (GF_ChildNodeItem **)field->far_ptr, new_node, &last);
 				}
 			} else {
-				return codec->LastError;
+				e = codec->LastError;
+				goto exit;
 			}
 		}
-		if (e) return e;
+		if (e) goto exit;
 
 		endFlag = gf_bs_read_int(bs, 1);
 
-		//according to the spec, the QP applies to the current node itself, 
+		//according to the spec, the QP applies to the current node itself,
 		//not just children. If IsLocal is TRUE remove the node
 		if (qp_on && qp_local) {
 			if (qp_local == 2) {
@@ -319,32 +383,36 @@ GF_Err BD_DecMFFieldList(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node
 		}
 		nbF += 1;
 	}
+
+exit:
 	/*finally delete the QP if any (local or not) as we get out of this node
 	and reactivate previous one*/
 	if (qp_on) gf_bifs_dec_qp_remove(codec, initial_qp);
+	if (e) return e;
+
 	/*this is for QP 14*/
 	gf_bifs_dec_qp14_set_length(codec, nbF);
 	return GF_OK;
 }
 
-GF_Err BD_DecMFFieldVec(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field)
+GF_Err BD_DecMFFieldVec(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field, Bool is_mem_com)
 {
 	GF_Err e;
 	u32 NbBits, nbFields;
 	u32 i;
 	GF_ChildNodeItem *last;
 	u8 qp_local, qp_on, initial_qp;
-	GF_Node *new_node;
 	GF_FieldInfo sffield;
-	
+
 	memset(&sffield, 0, sizeof(GF_FieldInfo));
 	sffield.fieldIndex = field->fieldIndex;
 	sffield.fieldType = gf_sg_vrml_get_sf_type(field->fieldType);
 	sffield.NDTtype = field->NDTtype;
+	sffield.name = field->name;
 
 	initial_qp = qp_local = qp_on = 0;
 
-	//vector description - alloc the MF size before 
+	//vector description - alloc the MF size before
 	NbBits = gf_bs_read_int(bs, 5);
 	nbFields = gf_bs_read_int(bs, NbBits);
 
@@ -358,97 +426,108 @@ GF_Err BD_DecMFFieldVec(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node,
 		e = gf_sg_vrml_mf_alloc(field->far_ptr, field->fieldType, nbFields);
 		if (e) return e;
 
-		for (i=0;i<nbFields; i++) {
+		for (i=0; i<nbFields; i++) {
 			e = gf_sg_vrml_mf_get_item(field->far_ptr, field->fieldType, & sffield.far_ptr, i);
 			if (e) return e;
-			e = gf_bifs_dec_sf_field(codec, bs, node, &sffield);
+			e = gf_bifs_dec_sf_field(codec, bs, node, &sffield, GF_FALSE);
+			if (e) return e;
 		}
-	} else {
-		last = NULL;
-		for (i=0;i<nbFields; i++) {
-			new_node = gf_bifs_dec_node(codec, bs, field->NDTtype);
-			if (new_node) {
-				e = gf_node_register(new_node, node);
-				if (e) return e;
+		return GF_OK;
+	}
 
-				if (node) {
-					/*special case for QP, register as the current QP*/
-					if (gf_node_get_tag(new_node) == TAG_MPEG4_QuantizationParameter) {
-						qp_local = ((M_QuantizationParameter *)new_node)->isLocal;
-						/*we have a QP in the same scope, remove previous
-						NB: we assume this is the right behaviour, the spec doesn't say 
-						whether QP is cumulative or not*/
-						if (qp_on) gf_bifs_dec_qp_remove(codec, 0);
+	e = GF_OK;
+	last = NULL;
+	for (i=0; i<nbFields; i++) {
+		GF_Node *new_node = gf_bifs_dec_node(codec, bs, field->NDTtype);
+		if (new_node) {
+			e = gf_node_register(new_node, is_mem_com ? NULL : node);
+			if (e) goto exit;
 
-						e = gf_bifs_dec_qp_set(codec, new_node);
-						if (e) return e;
-						qp_on = 1;
-						if (qp_local) qp_local = 2;
-						if (codec->force_keep_qp) {
-							e = gf_node_list_add_child_last(field->far_ptr, new_node, &last);
-						} else {
-							gf_node_register(new_node, NULL);
-							gf_node_unregister(new_node, node);
-						}
-					} else {
+			if (node) {
+				/*special case for QP, register as the current QP*/
+				if (gf_node_get_tag(new_node) == TAG_MPEG4_QuantizationParameter) {
+					qp_local = ((M_QuantizationParameter *)new_node)->isLocal;
+					/*we have a QP in the same scope, remove previous
+					NB: we assume this is the right behavior, the spec doesn't say
+					whether QP is cumulative or not*/
+					if (qp_on) gf_bifs_dec_qp_remove(codec, GF_FALSE);
+
+					e = gf_bifs_dec_qp_set(codec, new_node);
+					if (e) goto exit;
+					qp_on = 1;
+					if (qp_local) qp_local = 2;
+					if (codec->force_keep_qp) {
 						e = gf_node_list_add_child_last(field->far_ptr, new_node, &last);
+						if (e) goto exit;
+					} else {
+						gf_node_register(new_node, NULL);
+						gf_node_unregister(new_node, node);
 					}
-				} 
-				/*proto coding*/
-				else if (codec->pCurrentProto) {
-					/*TO DO: what happens if this is a QP node on the interface ?*/
-					e = gf_node_list_add_child_last( (GF_ChildNodeItem **)field->far_ptr, new_node, &last);
+				} else {
+					e = gf_node_list_add_child_last(field->far_ptr, new_node, &last);
+					if (e) goto exit;
 				}
-			} else {
-				return codec->LastError ? codec->LastError : GF_NON_COMPLIANT_BITSTREAM;
 			}
-		}
-		/*according to the spec, the QP applies to the current node itself, not just children. 
-		If IsLocal is TRUE remove the node*/
-		if (qp_on && qp_local) {
-			if (qp_local == 2) {
-				qp_local = 1;
-			} else {
-				//ask to get rid of QP and reactivate if we had a QP when entering the node
-				gf_bifs_dec_qp_remove(codec, initial_qp);
-				qp_local = 0;
+			/*proto coding*/
+			else if (codec->pCurrentProto) {
+				/*TO DO: what happens if this is a QP node on the interface ?*/
+				e = gf_node_list_add_child_last( (GF_ChildNodeItem **)field->far_ptr, new_node, &last);
+				if (e)goto exit;
 			}
+		} else {
+			e = codec->LastError ? codec->LastError : GF_NON_COMPLIANT_BITSTREAM;
+			goto exit;
 		}
 	}
+
+exit:
+
+	/*according to the spec, the QP applies to the current node itself, not just children.
+	If IsLocal is TRUE remove the node*/
+	if (qp_on && qp_local) {
+		if (qp_local == 2) {
+//				qp_local = 1;
+		} else {
+			//ask to get rid of QP and reactivate if we had a QP when entering the node
+			gf_bifs_dec_qp_remove(codec, initial_qp);
+//				qp_local = 0;
+		}
+	}
+
 	/*finally delete the QP if any (local or not) as we get out of this node*/
-	if (qp_on) gf_bifs_dec_qp_remove(codec, 1);
-	return GF_OK;
+	if (qp_on) gf_bifs_dec_qp_remove(codec, GF_TRUE);
+	return e;
 }
 
 
 void gf_bifs_check_field_change(GF_Node *node, GF_FieldInfo *field)
 {
-	if ((field->fieldType==GF_SG_VRML_MFNODE) || (field->fieldType==GF_SG_VRML_MFNODE)) node->sgprivate->flags |= GF_SG_CHILD_DIRTY;
+	if (field->fieldType==GF_SG_VRML_MFNODE) node->sgprivate->flags |= GF_SG_CHILD_DIRTY;
 	/*signal node modif*/
 	gf_node_changed(node, field);
 	/*Notify eventOut in all cases to handle protos*/
 	gf_node_event_out(node, field->fieldIndex);
 	/*and propagate eventIn if any*/
 	if (field->on_event_in) {
-		field->on_event_in(node);
+		field->on_event_in(node, NULL);
 	} else if ((gf_node_get_tag(node) == TAG_MPEG4_Script) && (field->eventType==GF_SG_EVENT_IN)) {
 		gf_sg_script_event_in(node, field);
 	}
 
 }
 
-GF_Err gf_bifs_dec_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field)
+GF_Err gf_bifs_dec_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field, Bool is_mem_com)
 {
 	GF_Err e;
 	u8 flag;
 
 //	if (codec->LastError) return codec->LastError;
 
-	assert(node);
+	if (!node) return GF_BAD_PARAM;
 //	if (field->fieldType == GF_SG_VRML_UNKNOWN) return GF_NON_COMPLIANT_BITSTREAM;
-	
+
 	if (gf_sg_vrml_is_sf_field(field->fieldType)) {
-		e = gf_bifs_dec_sf_field(codec, bs, node, field);
+		e = gf_bifs_dec_sf_field(codec, bs, node, field, is_mem_com);
 		if (e) return e;
 	} else {
 		/*clean up the eventIn field if not done*/
@@ -458,14 +537,22 @@ GF_Err gf_bifs_dec_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node
 				* (GF_ChildNodeItem **)field->far_ptr = NULL;
 			} else {
 				//remove all items of the MFField
-				gf_sg_vrml_mf_reset(field->far_ptr, field->fieldType);
+				e = gf_sg_vrml_mf_reset(field->far_ptr, field->fieldType);
+				if (e) return e;
 			}
 		}
 
 		/*predictiveMFField*/
 		if (codec->info->config.UsePredictiveMFField) {
 			flag = gf_bs_read_int(bs, 1);
-			if (flag)  return gf_bifs_dec_pred_mf_field(codec, bs, node, field);
+			if (flag) {
+#ifdef GPAC_ENABLE_BIFS_PMF
+				return gf_bifs_dec_pred_mf_field(codec, bs, node, field);
+#else
+				GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[BIFS] Stream uses Predictive Field Coding, disabled in this build!\n"));
+				return GF_NOT_SUPPORTED;
+#endif
+			}
 		}
 
 		/*reserved*/
@@ -479,11 +566,11 @@ GF_Err gf_bifs_dec_field(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *node
 			/*List description - alloc is dynamic*/
 			flag = gf_bs_read_int(bs, 1);
 			if (flag) {
-				e = BD_DecMFFieldList(codec, bs, node, field);
+				e = BD_DecMFFieldList(codec, bs, node, field, is_mem_com);
 			} else {
-				e = BD_DecMFFieldVec(codec, bs, node, field);
+				e = BD_DecMFFieldVec(codec, bs, node, field, is_mem_com);
 			}
-			if (e) return e;
+			if (e) return codec->LastError = e;
 		}
 	}
 	return GF_OK;
@@ -509,8 +596,6 @@ GF_Err gf_bifs_dec_node_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *
 	u32 numBitsALL, numBitsDEF, field_all, field_ref, numProtoBits;
 	GF_FieldInfo field;
 
-	e = GF_OK;
-
 	numProtoBits = numBitsALL = 0;
 	if (codec->pCurrentProto) {
 		numProtoBits = gf_get_bit_size(gf_sg_proto_get_field_count(codec->pCurrentProto) - 1);
@@ -519,7 +604,7 @@ GF_Err gf_bifs_dec_node_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *
 	numBitsDEF = gf_get_bit_size(gf_node_get_num_fields_in_mode(node, GF_SG_FIELD_CODING_DEF)-1);
 
 	flag = gf_bs_read_int(bs, 1);
-	while (!flag) {
+	while (!flag && (codec->LastError>=0)) {
 		if (codec->pCurrentProto) {
 			//IS'ed flag
 			flag = gf_bs_read_int(bs, 1);
@@ -543,7 +628,7 @@ GF_Err gf_bifs_dec_node_list(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *
 		if (e) return e;
 		e = gf_node_get_field(node, field_all, &field);
 		if (e) return e;
-		e = gf_bifs_dec_field(codec, bs, node, &field);
+		e = gf_bifs_dec_field(codec, bs, node, &field, GF_FALSE);
 		if (e) return e;
 		flag = gf_bs_read_int(bs, 1);
 
@@ -576,12 +661,12 @@ GF_Err gf_bifs_dec_node_mask(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *
 				if (e) return e;
 				e = BD_SetProtoISed(codec, flag, node, i);
 			}
-			//regular field, parse it (nb: no contextual coding for protos in maskNode, 
+			//regular field, parse it (nb: no contextual coding for protos in maskNode,
 			//all node fields are coded
 			else {
 				e = gf_node_get_field(node, i, &field);
 				if (e) return e;
-				e = gf_bifs_dec_field(codec, bs, node, &field);
+				e = gf_bifs_dec_field(codec, bs, node, &field, GF_FALSE);
 			}
 			if (e) return e;
 		}
@@ -595,7 +680,7 @@ GF_Err gf_bifs_dec_node_mask(GF_BifsDecoder * codec, GF_BitStream *bs, GF_Node *
 			gf_bifs_get_field_index(node, i, GF_SG_FIELD_CODING_DEF, &index);
 			e = gf_node_get_field(node, index, &field);
 			if (e) return e;
-			e = gf_bifs_dec_field(codec, bs, node, &field);
+			e = gf_bifs_dec_field(codec, bs, node, &field, GF_FALSE);
 			if (e) return e;
 
 			if (is_proto) gf_sg_proto_mark_field_loaded(node, &field);
@@ -643,14 +728,13 @@ static void UpdateTimeNode(GF_BifsDecoder * codec, GF_Node *node)
 			BD_CheckSFTimeOffset(codec, node, &inf);
 		}
 	}
-		break;
+	break;
 	}
 }
 
 GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 {
 	u32 nodeID, NDTBits, node_type, node_tag, ProtoID, BVersion;
-	u8 node_flag;
 	Bool skip_init, reset_qp14;
 	GF_Node *new_node;
 	GF_Err e;
@@ -670,7 +754,6 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 
 
 	BVersion = GF_BIFS_V1;
-	node_flag = 0;
 
 	/*this is a USE statement*/
 	if (gf_bs_read_int(bs, 1)) {
@@ -678,30 +761,37 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 		/*NULL node is encoded as USE with ID = all bits to 1*/
 		if (nodeID == (u32) (1<<codec->info->config.NodeIDBits))
 			return NULL;
-		//find node and return it
+		//find node
 		new_node = gf_sg_find_node(codec->current_graph, nodeID);
 
+		//check node is allowed for the given NDT
+		if (new_node && !gf_node_in_table(new_node, NDT_Tag)) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[BIFS] Node %s not allowed as field/child of NDT type %d\n", gf_node_get_class_name(new_node), NDT_Tag));
+			codec->LastError = GF_SG_UNKNOWN_NODE;
+			return NULL;
+		}
+
 		if (!new_node) {
 			codec->LastError = GF_SG_UNKNOWN_NODE;
 		} else {
 			/*restore QP14 length*/
 			switch (gf_node_get_tag(new_node)) {
 			case TAG_MPEG4_Coordinate:
-				{
-					u32 nbCoord = ((M_Coordinate *)new_node)->point.count;
-					gf_bifs_dec_qp14_enter(codec, 1);
-					gf_bifs_dec_qp14_set_length(codec, nbCoord);
-					gf_bifs_dec_qp14_enter(codec, 0);
-				}
-				break;
+			{
+				u32 nbCoord = ((M_Coordinate *)new_node)->point.count;
+				gf_bifs_dec_qp14_enter(codec, GF_TRUE);
+				gf_bifs_dec_qp14_set_length(codec, nbCoord);
+				gf_bifs_dec_qp14_enter(codec, GF_FALSE);
+			}
+			break;
 			case TAG_MPEG4_Coordinate2D:
-				{
-					u32 nbCoord = ((M_Coordinate2D *)new_node)->point.count;
-					gf_bifs_dec_qp14_enter(codec, 1);
-					gf_bifs_dec_qp14_set_length(codec, nbCoord);
-					gf_bifs_dec_qp14_enter(codec, 0);
-				}
-				break;
+			{
+				u32 nbCoord = ((M_Coordinate2D *)new_node)->point.count;
+				gf_bifs_dec_qp14_enter(codec, GF_TRUE);
+				gf_bifs_dec_qp14_set_length(codec, nbCoord);
+				gf_bifs_dec_qp14_enter(codec, GF_FALSE);
+			}
+			break;
 			}
 		}
 		return new_node;
@@ -752,13 +842,13 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 	/*special handling of 3D mesh*/
 	if ((node_tag == TAG_MPEG4_IndexedFaceSet) && codec->info->config.Use3DMeshCoding) {
 		if (gf_bs_read_int(bs, 1)) {
-			nodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
-			if (codec->UseName) gf_bifs_dec_name(bs, name);
+			/*nodeID = 1 + */gf_bs_read_int(bs, codec->info->config.NodeIDBits);
+			if (codec->UseName) gf_bifs_dec_name(bs, name, 1000);
 		}
 		/*parse the 3DMesh node*/
 		return NULL;
 	}
-	/*unknow node*/
+	/*unknown node*/
 	if (!node_tag && !proto) {
 		codec->LastError = GF_SG_UNKNOWN_NODE;
 		return NULL;
@@ -772,11 +862,11 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 			return NULL;
 		}
 		nodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
-		if (codec->UseName) gf_bifs_dec_name(bs, name);
+		if (codec->UseName) gf_bifs_dec_name(bs, name, 1000);
 	}
 
 	new_node = NULL;
-	skip_init = 0;
+	skip_init = GF_FALSE;
 
 	/*don't check node IDs duplicate since VRML may use them...*/
 #if 0
@@ -802,13 +892,15 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 			}
 		}
 	}
+	if (!new_node)
 #endif
-	
-	if (!new_node) {
+
+	{
 		if (proto) {
-			skip_init = 1;
 			/*create proto interface*/
 			new_node = gf_sg_proto_create_instance(codec->current_graph, proto);
+            //don't init protos unless externProto (in which case we want init for hardcoded protos)
+            if (! proto->ExternProto.count) skip_init = GF_TRUE;
 		} else {
 			new_node = gf_node_new(codec->current_graph, node_tag);
 		}
@@ -818,42 +910,29 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 		return NULL;
 	}
 
-	/*VRML: "The transformation hierarchy shall be a directed acyclic graph; results are undefined if a node 
-	in the transformation hierarchy is its own ancestor"
-	that's good, because the scene graph can't handle cyclic graphs (destroy will never be called).
-	We therefore only register the node once parsed*/
-	if (nodeID) {
-		if (strlen(name)) {
-			gf_node_set_id(new_node, nodeID, name);
-		} else {
-			gf_node_set_id(new_node, nodeID, NULL);
-		}
-	}
-
-
 	/*update default time fields except in proto parsing*/
 	if (!codec->pCurrentProto) UpdateTimeNode(codec, new_node);
-	/*nodes are only init outside protos */
-	else skip_init = 1;
+	/*nodes are only init outside protos, nodes internal to protos are never intialized */
+	else skip_init = GF_TRUE;
 
 	/*if coords were not stored for QP14 before coding this node, reset QP14 it when leaving*/
 	reset_qp14 = !codec->coord_stored;
 
-	/*QP 14 is a special quant mode for IndexFace/Line(2D)Set to quantize the 
+	/*QP 14 is a special quant mode for IndexFace/Line(2D)Set to quantize the
 	coordonate(2D) child, based on the first field parsed
 	we must check the type of the node and notfy the QP*/
 	switch (node_tag) {
 	case TAG_MPEG4_Coordinate:
 	case TAG_MPEG4_Coordinate2D:
-		gf_bifs_dec_qp14_enter(codec, 1);
+		gf_bifs_dec_qp14_enter(codec, GF_TRUE);
 	}
 
 	if (gf_bs_read_int(bs, 1)) {
-		e = gf_bifs_dec_node_mask(codec, bs, new_node, proto ? 1 : 0);
+		e = gf_bifs_dec_node_mask(codec, bs, new_node, proto ? GF_TRUE : GF_FALSE);
 	} else {
-		e = gf_bifs_dec_node_list(codec, bs, new_node, proto ? 1 : 0);
+		e = gf_bifs_dec_node_list(codec, bs, new_node, proto ? GF_TRUE : GF_FALSE);
 	}
-	if (codec->coord_stored && reset_qp14) 
+	if (codec->coord_stored && reset_qp14)
 		gf_bifs_dec_qp14_reset(codec);
 
 	if (e) {
@@ -865,13 +944,25 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 		return NULL;
 	}
 
-	if (!skip_init) 
+	/*VRML: "The transformation hierarchy shall be a directed acyclic graph; results are undefined if a node
+	in the transformation hierarchy is its own ancestor"
+	that's good, because the scene graph can't handle cyclic graphs (destroy will never be called).
+	We therefore only register the node once parsed*/
+	if (nodeID) {
+		if (strlen(name)) {
+			gf_node_set_id(new_node, nodeID, name);
+		} else {
+			gf_node_set_id(new_node, nodeID, NULL);
+		}
+	}
+
+	if (!skip_init)
 		gf_node_init(new_node);
 
 	switch (node_tag) {
 	case TAG_MPEG4_Coordinate:
 	case TAG_MPEG4_Coordinate2D:
-		gf_bifs_dec_qp14_enter(codec, 0);
+		gf_bifs_dec_qp14_enter(codec, GF_FALSE);
 		break;
 	case TAG_MPEG4_Script:
 		/*load script if in main graph (useless to load in proto declaration)*/
@@ -885,10 +976,15 @@ GF_Node *gf_bifs_dec_node(GF_BifsDecoder * codec, GF_BitStream *bs, u32 NDT_Tag)
 		break;
 	}
 
+	/*proto is initialized upon the first traversal to have the same behavior as wth BT/XMT loading*/
+#if 0
 	/*if new node is a proto and we're in the top scene, load proto code*/
 	if (proto && (codec->scenegraph == codec->current_graph)) {
 		codec->LastError = gf_sg_proto_load_code(new_node);
 	}
+#endif
+
 	return new_node;
 }
 
+#endif /*GPAC_DISABLE_BIFS*/
