@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / BIFS codec sub-project
@@ -10,15 +11,15 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
@@ -26,6 +27,9 @@
 #include <gpac/internal/bifs_dev.h>
 #include "quant.h"
 
+
+#ifndef GPAC_DISABLE_BIFS
+
 GF_Err ParseMFFieldList(GF_BifsDecoder *codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field);
 GF_Err ParseMFFieldVec(GF_BifsDecoder *codec, GF_BitStream *bs, GF_Node *node, GF_FieldInfo *field);
 
@@ -44,7 +48,7 @@ static GF_Err BM_ParseMultipleIndexedReplace(GF_BifsDecoder *codec, GF_BitStream
 	GF_Command *com;
 	GF_CommandField *inf;
 	GF_FieldInfo field;
-	
+
 	ID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
 	node = gf_sg_find_node(codec->current_graph, ID);
 	if (!node) return GF_NON_COMPLIANT_BITSTREAM;
@@ -58,7 +62,7 @@ static GF_Err BM_ParseMultipleIndexedReplace(GF_BifsDecoder *codec, GF_BitStream
 	lenpos = gf_bs_read_int(bs, 5);
 	lennum = gf_bs_read_int(bs, 5);
 	count = gf_bs_read_int(bs, lennum);
-	
+
 	com = gf_sg_command_new(codec->current_graph, GF_SG_MULTIPLE_INDEXED_REPLACE);
 	BM_SetCommandNode(com, node);
 	field.fieldType = gf_sg_vrml_get_sf_type(field.fieldType);
@@ -68,17 +72,18 @@ static GF_Err BM_ParseMultipleIndexedReplace(GF_BifsDecoder *codec, GF_BitStream
 		inf->pos = gf_bs_read_int(bs, lenpos);
 		inf->fieldIndex = field.fieldIndex;
 		inf->fieldType = field.fieldType;
-		
+
 		if (field.fieldType==GF_SG_VRML_SFNODE) {
 			inf->new_node = gf_bifs_dec_node(codec, bs, field.NDTtype);
-			if (codec->LastError) goto err;
+			e = codec->LastError;
 			inf->field_ptr = &inf->new_node;
-			gf_node_register(inf->new_node, node);
+			if (inf->new_node)
+				gf_node_register(inf->new_node, NULL);
 		} else {
 			field.far_ptr = inf->field_ptr = gf_sg_vrml_field_pointer_new(inf->fieldType);
-			e = gf_bifs_dec_sf_field(codec, bs, node, &field);
-			if (e) goto err;
+			e = gf_bifs_dec_sf_field(codec, bs, node, &field, GF_TRUE);
 		}
+		if (e) goto err;
 		count--;
 	}
 err:
@@ -100,7 +105,7 @@ static GF_Err BM_ParseMultipleReplace(GF_BifsDecoder *codec, GF_BitStream *bs, G
 	NodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
 	node = gf_sg_find_node(codec->current_graph, NodeID);
 	if (!node) return GF_NON_COMPLIANT_BITSTREAM;
-	
+
 	e = GF_OK;
 	com = gf_sg_command_new(codec->current_graph, GF_SG_MULTIPLE_REPLACE);
 	BM_SetCommandNode(com, node);
@@ -123,13 +128,13 @@ static GF_Err BM_ParseMultipleReplace(GF_BifsDecoder *codec, GF_BitStream *bs, G
 			} else {
 				field.far_ptr = inf->field_ptr = gf_sg_vrml_field_pointer_new(inf->fieldType);
 			}
-			e = gf_bifs_dec_field(codec, bs, node, &field);
+			e = gf_bifs_dec_field(codec, bs, node, &field, GF_TRUE);
 			if (e) goto exit;
 		}
 	} else {
 		flag = gf_bs_read_int(bs, 1);
 		nbBits = gf_get_bit_size(gf_node_get_num_fields_in_mode(node, GF_SG_FIELD_CODING_DEF)-1);
-		while (!flag) {
+		while (!flag && (codec->LastError>=0)) {
 			field_ref = gf_bs_read_int(bs, nbBits);
 			e = gf_bifs_get_field_index(node, field_ref, GF_SG_FIELD_CODING_DEF, &fieldind);
 			if (e) goto exit;
@@ -145,13 +150,13 @@ static GF_Err BM_ParseMultipleReplace(GF_BifsDecoder *codec, GF_BitStream *bs, G
 			} else {
 				field.far_ptr = inf->field_ptr = gf_sg_vrml_field_pointer_new(inf->fieldType);
 			}
-			e = gf_bifs_dec_field(codec, bs, node, &field);
+			e = gf_bifs_dec_field(codec, bs, node, &field, GF_TRUE);
 			if (e) goto exit;
 			flag = gf_bs_read_int(bs, 1);
 		}
 	}
 
-	
+
 exit:
 	if (e) gf_sg_command_del(com);
 	else gf_list_add(com_list, com);
@@ -164,6 +169,7 @@ static GF_Err BM_ParseGlobalQuantizer(GF_BifsDecoder *codec, GF_BitStream *bs, G
 	GF_Command *com;
 	GF_CommandField *inf;
 	node = gf_bifs_dec_node(codec, bs, NDT_SFWorldNode);
+	if (!node) return GF_NON_COMPLIANT_BITSTREAM;
 
 	/*reset global QP*/
 	if (codec->scenegraph->global_qp) {
@@ -171,9 +177,14 @@ static GF_Err BM_ParseGlobalQuantizer(GF_BifsDecoder *codec, GF_BitStream *bs, G
 	}
 	codec->ActiveQP = NULL;
 	codec->scenegraph->global_qp = NULL;
-	
-	if (node && (gf_node_get_tag(node) != TAG_MPEG4_QuantizationParameter)) {
-		gf_node_unregister(node, NULL);
+
+	if (gf_node_get_tag(node) != TAG_MPEG4_QuantizationParameter) {
+		//if node was just created (num_instances == 0), unregister
+		//otherwise (USE node) don't do anything
+		if (!node->sgprivate->num_instances) {
+			node->sgprivate->num_instances = 1;
+			gf_node_unregister(node, NULL);
+		}
 		return GF_NON_COMPLIANT_BITSTREAM;
 	}
 
@@ -181,10 +192,11 @@ static GF_Err BM_ParseGlobalQuantizer(GF_BifsDecoder *codec, GF_BitStream *bs, G
 	codec->ActiveQP = (M_QuantizationParameter *) node;
 	codec->ActiveQP->isLocal = 0;
 	codec->scenegraph->global_qp = node;
-	if (node) {
-		/*register TWICE: once for the command, and for the scenegraph globalQP*/
-		node->sgprivate->num_instances = 2;
-	}
+
+	/*register TWICE: once for the command, and for the scenegraph globalQP*/
+	gf_node_register(node, NULL);
+	gf_node_register(node, NULL);
+
 	com = gf_sg_command_new(codec->current_graph, GF_SG_GLOBAL_QUANTIZER);
 	inf = gf_sg_command_field_new(com);
 	inf->new_node = node;
@@ -203,7 +215,7 @@ static GF_Err BM_ParseProtoDelete(GF_BifsDecoder *codec, GF_BitStream *bs, GF_Li
 		count = 0;
 		flag = gf_bs_read_int(bs, 1);
 		while (flag) {
-			com->del_proto_list = (u32*)realloc(com->del_proto_list, sizeof(u32) * (com->del_proto_list_size+1));
+			com->del_proto_list = (u32*)gf_realloc(com->del_proto_list, sizeof(u32) * (com->del_proto_list_size+1));
 			com->del_proto_list[count] = gf_bs_read_int(bs, codec->info->config.ProtoIDBits);
 			com->del_proto_list_size++;
 			flag = gf_bs_read_int(bs, 1);
@@ -211,7 +223,7 @@ static GF_Err BM_ParseProtoDelete(GF_BifsDecoder *codec, GF_BitStream *bs, GF_Li
 	} else {
 		flag = gf_bs_read_int(bs, 5);
 		com->del_proto_list_size = gf_bs_read_int(bs, flag);
-		com->del_proto_list = (u32*)realloc(com->del_proto_list, sizeof(u32) * (com->del_proto_list_size));
+		com->del_proto_list = (u32*)gf_realloc(com->del_proto_list, sizeof(u32) * (com->del_proto_list_size));
 		flag = 0;
 		while (flag<com->del_proto_list_size) {
 			com->del_proto_list[flag] = gf_bs_read_int(bs, codec->info->config.ProtoIDBits);
@@ -222,6 +234,129 @@ static GF_Err BM_ParseProtoDelete(GF_BifsDecoder *codec, GF_BitStream *bs, GF_Li
 	return GF_OK;
 }
 
+static GF_Err BM_XReplace(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com_list)
+{
+	GF_FieldInfo targetField, fromField, decfield;
+	GF_Node *target, *fromNode;
+	s32 pos = -2;
+	u32 id, nbBits, ind, aind;
+	GF_Err e;
+	GF_Command *com;
+	GF_CommandField *inf;
+
+	id = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
+	target = gf_sg_find_node(codec->current_graph, id);
+	if (!target) return GF_SG_UNKNOWN_NODE;
+
+	com = gf_sg_command_new(codec->current_graph, GF_SG_XREPLACE);
+	BM_SetCommandNode(com, target);
+	gf_list_add(com_list, com);
+
+	nbBits = gf_get_bit_size(gf_node_get_num_fields_in_mode(target, GF_SG_FIELD_CODING_IN)-1);
+	ind = gf_bs_read_int(bs, nbBits);
+	e = gf_bifs_get_field_index(target, ind, GF_SG_FIELD_CODING_IN, &aind);
+	if (e) return e;
+	e = gf_node_get_field(target, aind, &targetField);
+	if (e) return e;
+
+	inf = gf_sg_command_field_new(com);
+	inf->fieldIndex = aind;
+
+	if (!gf_sg_vrml_is_sf_field(targetField.fieldType)) {
+		/*this is indexed replacement*/
+		if (gf_bs_read_int(bs, 1)) {
+			/*index is dynamic*/
+			if (gf_bs_read_int(bs, 1)) {
+				GF_Node *n;
+				id = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
+				n = gf_sg_find_node(codec->current_graph, id);
+				if (!n) return GF_SG_UNKNOWN_NODE;
+				com->toNodeID = id;
+
+				nbBits = gf_get_bit_size(gf_node_get_num_fields_in_mode(n, GF_SG_FIELD_CODING_DEF)-1);
+				ind = gf_bs_read_int(bs, nbBits);
+				e = gf_bifs_get_field_index(n, ind, GF_SG_FIELD_CODING_DEF, &aind);
+				if (e) return e;
+				e = gf_node_get_field(n, aind, &fromField);
+				if (e) return e;
+				com->toFieldIndex = aind;
+			} else {
+				u32 type = gf_bs_read_int(bs, 2);
+				switch (type) {
+				case 0:
+					pos = gf_bs_read_int(bs, 16);
+					break;
+				case 2:
+					pos = 0;
+					break;
+				case 3:
+					pos = -1;
+					break;
+				}
+			}
+		}
+		if (targetField.fieldType==GF_SG_VRML_MFNODE) {
+			if (gf_bs_read_int(bs, 1)) {
+				target = gf_node_list_get_child(*(GF_ChildNodeItem **)targetField.far_ptr, pos);
+				if (!target) return GF_SG_UNKNOWN_NODE;
+
+				nbBits = gf_get_bit_size(gf_node_get_num_fields_in_mode(target, GF_SG_FIELD_CODING_IN)-1);
+				ind = gf_bs_read_int(bs, nbBits);
+				e = gf_bifs_get_field_index(target, ind, GF_SG_FIELD_CODING_IN, &aind);
+				if (e) return e;
+				e = gf_node_get_field(target, aind, &targetField);
+				if (e) return e;
+				pos = -2;
+				com->child_field = aind;
+				com->ChildNodeTag = gf_node_get_tag(target);
+				if (com->ChildNodeTag == TAG_ProtoNode) {
+					s32 p_id = gf_sg_proto_get_id(gf_node_get_proto(target));
+					com->ChildNodeTag = -p_id;
+				}
+			}
+		}
+		inf->pos = pos;
+	}
+
+	fromNode = NULL;
+	if (gf_bs_read_int(bs, 1)) {
+		id = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
+		fromNode = gf_sg_find_node(codec->current_graph, id);
+		if (!fromNode) return GF_SG_UNKNOWN_NODE;
+		com->fromNodeID = id;
+
+		nbBits = gf_get_bit_size(gf_node_get_num_fields_in_mode(fromNode, GF_SG_FIELD_CODING_DEF)-1);
+		ind = gf_bs_read_int(bs, nbBits);
+		e = gf_bifs_get_field_index(fromNode, ind, GF_SG_FIELD_CODING_DEF, &aind);
+		if (e) return e;
+		e = gf_node_get_field(fromNode, aind, &fromField);
+		if (e) return e;
+		com->fromFieldIndex = aind;
+
+		return GF_OK;
+	}
+
+
+	if (pos>= -1) {
+		inf->fieldType = gf_sg_vrml_get_sf_type(targetField.fieldType);
+	} else {
+		inf->fieldType = targetField.fieldType;
+	}
+	decfield.fieldIndex = inf->fieldIndex;
+	decfield.fieldType = inf->fieldType;
+	decfield.name = targetField.name;
+
+	if (inf->fieldType==GF_SG_VRML_SFNODE) {
+		decfield.far_ptr = inf->field_ptr = &inf->new_node;
+	} else if (inf->fieldType==GF_SG_VRML_MFNODE) {
+		decfield.far_ptr = inf->field_ptr = &inf->node_list;
+	} else {
+		decfield.far_ptr = inf->field_ptr = gf_sg_vrml_field_pointer_new(inf->fieldType);
+	}
+	e = gf_bifs_dec_sf_field(codec, bs, target, &decfield, GF_TRUE);
+	return e;
+}
+
 static GF_Err BM_ParseExtendedUpdates(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com_list)
 {
 	u32 type = gf_bs_read_int(bs, 8);
@@ -235,7 +370,7 @@ static GF_Err BM_ParseExtendedUpdates(GF_BifsDecoder *codec, GF_BitStream *bs, G
 		if (e) gf_sg_command_del(com);
 		else gf_list_add(com_list, com);
 	}
-		return e;
+	return e;
 	case 1:
 		return BM_ParseProtoDelete(codec, bs, com_list);
 	case 2:
@@ -259,7 +394,10 @@ static GF_Err BM_ParseExtendedUpdates(GF_BifsDecoder *codec, GF_BitStream *bs, G
 		BM_SetCommandNode(com, n);
 		gf_list_add(com_list, com);
 	}
-		return GF_OK;
+	return GF_OK;
+	case 7:
+		return BM_XReplace(codec, bs, com_list);
+
 	default:
 		return GF_BIFS_UNKNOWN_VERSION;
 	}
@@ -269,7 +407,6 @@ static GF_Err BM_ParseExtendedUpdates(GF_BifsDecoder *codec, GF_BitStream *bs, G
 GF_Err BM_ParseNodeInsert(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com_list)
 {
 	u32 NodeID, NDT;
-	GF_Command *com;
 	GF_CommandField *inf;
 	s32 type, pos;
 	GF_Node *node, *def;
@@ -297,7 +434,7 @@ GF_Err BM_ParseNodeInsert(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com_
 	}
 	node = gf_bifs_dec_node(codec, bs, NDT);
 	if (!codec->LastError) {
-		com = gf_sg_command_new(codec->current_graph, GF_SG_NODE_INSERT);
+		GF_Command *com = gf_sg_command_new(codec->current_graph, GF_SG_NODE_INSERT);
 		BM_SetCommandNode(com, def);
 		inf = gf_sg_command_field_new(com);
 		inf->pos = pos;
@@ -306,7 +443,7 @@ GF_Err BM_ParseNodeInsert(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com_
 		inf->fieldType = GF_SG_VRML_SFNODE;
 		gf_list_add(com_list, com);
 		/*register*/
-		gf_node_register(node, def);
+		gf_node_register(node, NULL);
 	}
 	return codec->LastError;
 }
@@ -370,7 +507,7 @@ GF_Err BM_ParseIndexInsert(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com
 			inf->field_ptr = &inf->new_node;
 			gf_list_add(com_list, com);
 			/*register*/
-			gf_node_register(node, def);
+			gf_node_register(node, NULL);
 		}
 	} else {
 		com = gf_sg_command_new(codec->current_graph, GF_SG_INDEXED_INSERT);
@@ -380,7 +517,7 @@ GF_Err BM_ParseIndexInsert(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com
 		inf->fieldIndex = field_ind;
 		inf->fieldType = sffield.fieldType;
 		sffield.far_ptr = inf->field_ptr = gf_sg_vrml_field_pointer_new(sffield.fieldType);
-		codec->LastError = gf_bifs_dec_sf_field(codec, bs, def, &sffield);
+		codec->LastError = gf_bifs_dec_sf_field(codec, bs, def, &sffield, GF_TRUE);
 		gf_list_add(com_list, com);
 	}
 	return codec->LastError;
@@ -402,7 +539,7 @@ GF_Err BM_ParseRouteInsert(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com
 	/*def'ed route*/
 	if (flag) {
 		RouteID = 1 + gf_bs_read_int(bs, codec->info->config.RouteIDBits);
-		if (codec->UseName) gf_bifs_dec_name(bs, name);
+		if (codec->UseName) gf_bifs_dec_name(bs, name, 1000);
 	}
 	/*origin*/
 	node_id = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
@@ -413,6 +550,7 @@ GF_Err BM_ParseRouteInsert(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com
 	numBits = gf_get_bit_size(numBits);
 	ind = gf_bs_read_int(bs, numBits);
 	e = gf_bifs_get_field_index(OutNode, ind, GF_SG_FIELD_CODING_OUT, &outField);
+	if (e) return e;
 
 	/*target*/
 	node_id = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
@@ -427,7 +565,7 @@ GF_Err BM_ParseRouteInsert(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com
 
 	com = gf_sg_command_new(codec->current_graph, GF_SG_ROUTE_INSERT);
 	com->RouteID = RouteID;
-	if (codec->UseName) com->def_name = strdup( name);
+	if (codec->UseName) com->def_name = gf_strdup( name);
 	com->fromNodeID = gf_node_get_id(OutNode);
 	com->fromFieldIndex = outField;
 	com->toNodeID = gf_node_get_id(InNode);
@@ -467,7 +605,7 @@ GF_Err BM_ParseIndexDelete(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com
 	GF_Err e;
 	GF_CommandField *inf;
 	GF_FieldInfo field;
-	
+
 	NodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
 	node = gf_sg_find_node(codec->current_graph, NodeID);
 	if (!node) return GF_NON_COMPLIANT_BITSTREAM;
@@ -543,7 +681,7 @@ GF_Err BM_ParseNodeReplace(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com
 	GF_Command *com;
 	GF_Node *node;
 	GF_CommandField *inf;
-	
+
 	NodeID = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
 	/*this is delete / new on a DEF node: replace ALL instances*/
 	node = gf_sg_find_node(codec->current_graph, NodeID);
@@ -578,6 +716,7 @@ GF_Err BM_ParseFieldReplace(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *co
 	if (e) return e;
 
 	e = gf_node_get_field(node, field_ind, &field);
+	if (e) return e;
 
 	com = gf_sg_command_new(codec->current_graph, GF_SG_FIELD_REPLACE);
 	BM_SetCommandNode(com, node);
@@ -592,20 +731,8 @@ GF_Err BM_ParseFieldReplace(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *co
 		field.far_ptr = inf->field_ptr = gf_sg_vrml_field_pointer_new(field.fieldType);
 	}
 	/*parse the field*/
-	codec->LastError = gf_bifs_dec_field(codec, bs, node, &field);
+	codec->LastError = gf_bifs_dec_field(codec, bs, node, &field, GF_TRUE);
 
-	/*register nodes*/
-#if 0
-	if (inf->fieldType == GF_SG_VRML_SFNODE) {
-		gf_node_register(inf->new_node, com->node);
-	} else if (inf->fieldType == GF_SG_VRML_MFNODE) {
-		GF_Node *p;
-		u32 i=0;
-		while ((p = gf_list_enum(inf->node_list, &i))) {
-			gf_node_register(p, com->node);
-		}
-	}
-#endif
 	gf_list_add(com_list, com);
 	return codec->LastError;
 }
@@ -631,6 +758,7 @@ GF_Err BM_ParseIndexValueReplace(GF_BifsDecoder *codec, GF_BitStream *bs, GF_Lis
 	if (e) return e;
 
 	e = gf_node_get_field(node, field_ind, &field);
+	if (e) return e;
 	if (gf_sg_vrml_is_sf_field(field.fieldType)) return GF_NON_COMPLIANT_BITSTREAM;
 
 	type = gf_bs_read_int(bs, 2);
@@ -642,12 +770,13 @@ GF_Err BM_ParseIndexValueReplace(GF_BifsDecoder *codec, GF_BitStream *bs, GF_Lis
 		pos = 0;
 		break;
 	case 3:
+		if (!field.far_ptr) return GF_NON_COMPLIANT_BITSTREAM;
 		pos = ((GenMFField *) field.far_ptr)->count - 1;
 		break;
 	default:
 		return GF_NON_COMPLIANT_BITSTREAM;
 	}
-	
+
 	com = gf_sg_command_new(codec->current_graph, GF_SG_INDEXED_REPLACE);
 	BM_SetCommandNode(com, node);
 	inf = gf_sg_command_field_new(com);
@@ -658,13 +787,13 @@ GF_Err BM_ParseIndexValueReplace(GF_BifsDecoder *codec, GF_BitStream *bs, GF_Lis
 		inf->fieldType = GF_SG_VRML_SFNODE;
 		inf->new_node = gf_bifs_dec_node(codec, bs, field.NDTtype);
 		inf->field_ptr = &inf->new_node;
-		if (inf->new_node) gf_node_register(inf->new_node, com->node);
+		if (inf->new_node) gf_node_register(inf->new_node, NULL);
 	} else {
 		memcpy(&sffield, &field, sizeof(GF_FieldInfo));
 		sffield.fieldType = gf_sg_vrml_get_sf_type(field.fieldType);
 		inf->fieldType = sffield.fieldType;
 		sffield.far_ptr = inf->field_ptr = gf_sg_vrml_field_pointer_new(sffield.fieldType);
-		codec->LastError = gf_bifs_dec_sf_field(codec, bs, node, &sffield);
+		codec->LastError = gf_bifs_dec_sf_field(codec, bs, node, &sffield, GF_TRUE);
 	}
 	gf_list_add(com_list, com);
 	return codec->LastError;
@@ -675,12 +804,9 @@ GF_Err BM_ParseRouteReplace(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *co
 	GF_Err e;
 	GF_Command *com;
 	u32 RouteID, numBits, ind, node_id, fromID, toID;
-	GF_Route *r;
 	GF_Node *OutNode, *InNode;
 
 	RouteID = 1+gf_bs_read_int(bs, codec->info->config.RouteIDBits);
-	
-	r = gf_sg_route_find(codec->current_graph, RouteID);
 
 	/*origin*/
 	node_id = 1 + gf_bs_read_int(bs, codec->info->config.NodeIDBits);
@@ -757,7 +883,7 @@ GF_Err BM_SceneReplace(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com_lis
 		ri->toFieldIndex = r->ToField.fieldIndex;
 		ri->toNodeID = gf_node_get_id(r->ToNode);
 		if (r->ID) ri->RouteID = r->ID;
-		ri->def_name = r->name ? strdup(r->name) : NULL;
+		ri->def_name = r->name ? gf_strdup(r->name) : NULL;
 		gf_list_add(com_list, ri);
 		gf_sg_route_del(r);
 	}
@@ -770,14 +896,15 @@ GF_Err BM_SceneReplace(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com_lis
 GF_Err BM_ParseCommand(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com_list)
 {
 	u8 go, type;
-	u32 count;
 	GF_Err e;
 	go = 1;
 	e = GF_OK;
+	GF_SceneGraph *cur_graph = codec->current_graph;
+	GF_Proto *cur_proto = codec->pCurrentProto;
+
+	u32 nbBufs = gf_list_count(codec->command_buffers);
 
 	codec->LastError = GF_OK;
-	count = 0;
-
 	while (go) {
 		type = gf_bs_read_int(bs, 2);
 		switch (type) {
@@ -794,21 +921,30 @@ GF_Err BM_ParseCommand(GF_BifsDecoder *codec, GF_BitStream *bs, GF_List *com_lis
 			e = BM_SceneReplace(codec, bs, com_list);
 			break;
 		}
-		if (e) return e;
-
+		if (e) break;
 		go = gf_bs_read_int(bs, 1);
-		count++;
-
 	}
 	while (gf_list_count(codec->QPs)) {
-		gf_bifs_dec_qp_remove(codec, 1);
+		gf_bifs_dec_qp_remove(codec, GF_TRUE);
 	}
-	return GF_OK;
+
+	if (e) {
+		while (gf_list_count(codec->command_buffers) > nbBufs) {
+			CommandBufferItem *cbi;
+			cbi = (CommandBufferItem *)gf_list_pop_back(codec->command_buffers);
+			if (cbi->node) gf_node_unregister(cbi->node, NULL);
+			gf_free(cbi);
+		}
+	}
+
+	codec->current_graph = cur_graph;
+	codec->pCurrentProto = cur_proto;
+	return e;
 }
 
 void BM_EndOfStream(void *co)
 {
-	((GF_BifsDecoder *) co)->LastError = GF_IO_ERR;
+	((GF_BifsDecoder *) co)->LastError = GF_NON_COMPLIANT_BITSTREAM;
 }
 
 void gf_bs_set_eos_callback(GF_BitStream *bs, void (*EndOfStream)(void *par), void *par);
@@ -819,12 +955,19 @@ GF_Err gf_bifs_flush_command_list(GF_BifsDecoder *codec)
 	GF_BitStream *bs;
 	GF_Err e;
 	CommandBufferItem *cbi;
+	GF_SceneGraph *prev_root = codec->current_graph;
+	M_QuantizationParameter *prev_qp = codec->ActiveQP;
+	u32 prev_qp_count = gf_list_count(codec->QPs);
 	u32 NbPass = gf_list_count(codec->command_buffers);
+
+
+	codec->ActiveQP = NULL;
 	GF_List *nextPass = gf_list_new();
 	while (NbPass) {
 		while (gf_list_count(codec->command_buffers)) {
 			cbi = (CommandBufferItem *)gf_list_get(codec->command_buffers, 0);
 			gf_list_rem(codec->command_buffers, 0);
+
 			codec->current_graph = gf_node_get_graph(cbi->node);
 			e = GF_OK;
 			if (cbi->cb->bufferSize) {
@@ -834,7 +977,8 @@ GF_Err gf_bifs_flush_command_list(GF_BifsDecoder *codec)
 				gf_bs_del(bs);
 			}
 			if (!e) {
-				free(cbi);
+				gf_node_unregister(cbi->node, NULL);
+				gf_free(cbi);
 				continue;
 			}
 			/*this may be an error or a dependency pb - reset coimmand list and move to next pass*/
@@ -848,7 +992,8 @@ GF_Err gf_bifs_flush_command_list(GF_BifsDecoder *codec)
 					for (i=0; i<gf_list_count(codec->command_buffers); i++) {
 						CommandBufferItem *cbi2 = (CommandBufferItem *)gf_list_get(codec->command_buffers, i);
 						if (cbi2->cb == cf->field_ptr) {
-							free(cbi2);
+							gf_node_unregister(cbi2->node, NULL);
+							gf_free(cbi2);
 							gf_list_rem(codec->command_buffers, i);
 							i--;
 						}
@@ -867,14 +1012,22 @@ GF_Err gf_bifs_flush_command_list(GF_BifsDecoder *codec)
 		}
 		NbPass --;
 		if (NbPass > gf_list_count(codec->command_buffers)) NbPass = gf_list_count(codec->command_buffers);
+
+		//restore QP state
+		while (gf_list_count(codec->QPs) > prev_qp_count) {
+			gf_list_rem(codec->QPs, 0); //QPs are inserted at head of list
+		}
+		codec->ActiveQP = NULL;
 		codec->LastError = GF_OK;
 	}
 	gf_list_del(nextPass);
+	codec->current_graph = prev_root;
+	codec->ActiveQP = prev_qp;
 	return GF_OK;
 }
 
 GF_EXPORT
-GF_Err gf_bifs_decode_command_list(GF_BifsDecoder *codec, u16 ESID, char *data, u32 data_length, GF_List *com_list)
+GF_Err gf_bifs_decode_command_list(GF_BifsDecoder *codec, u16 ESID, u8 *data, u32 data_length, GF_List *com_list)
 {
 	GF_BitStream *bs;
 	GF_Err e;
@@ -886,7 +1039,7 @@ GF_Err gf_bifs_decode_command_list(GF_BifsDecoder *codec, u16 ESID, char *data,
 	if (codec->info->config.elementaryMasks ) return GF_NOT_SUPPORTED;
 
 	/*root parse (not conditionals)*/
-	assert(codec->scenegraph);
+	if (!codec->scenegraph) return GF_BAD_PARAM;
 	/*setup current scene graph*/
 	codec->current_graph = codec->scenegraph;
 
@@ -905,7 +1058,8 @@ GF_Err gf_bifs_decode_command_list(GF_BifsDecoder *codec, u16 ESID, char *data,
 	/*if err or not reset conditionals*/
 	while (gf_list_count(codec->command_buffers)) {
 		CommandBufferItem *cbi = (CommandBufferItem *)gf_list_get(codec->command_buffers, 0);
-		free(cbi);
+		gf_node_unregister(cbi->node, NULL);
+		gf_free(cbi);
 		gf_list_rem(codec->command_buffers, 0);
 	}
 
@@ -918,3 +1072,6 @@ GF_Err gf_bifs_decode_command_list(GF_BifsDecoder *codec, u16 ESID, char *data,
 //	gf_mx_v(codec->mx);
 	return e;
 }
+
+#endif /*GPAC_DISABLE_BIFS*/
+
