@@ -1,9 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
- *			
- *			Authors: Jean Le Feuvre 
  *
- *			Copyright (c) ENST 2007-200X
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Rendering sub-project
@@ -12,26 +11,28 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
 #include <gpac/internal/compositor_dev.h>
+
+#ifndef GPAC_DISABLE_FONTS
+
 #include <gpac/modules/font.h>
-#include <gpac/options.h>
 #include "visual_manager.h"
 #include "nodes_stacks.h"
-
 #include "texturing.h"
 
+
 struct _gf_ft_mgr
 {
 	GF_FontReader *reader;
@@ -41,47 +42,30 @@ struct _gf_ft_mgr
 
 	u32 *id_buffer;
 	u32 id_buffer_size;
+
+	Bool wait_font_load;
 };
 
-
-GF_FontManager *gf_font_manager_new(GF_User *user)
+GF_EXPORT
+GF_FontManager *gf_font_manager_new()
 {
 	char *def_font = "SERIF";
-	u32 i, count;
 	GF_FontManager *font_mgr;
 	GF_FontReader *ifce;
-	const char *opt;
+	Bool wait_for_fonts = gf_opts_get_bool("core", "wait-fonts");
 
-	ifce = NULL;
-	opt = gf_cfg_get_key(user->config, "FontEngine", "FontReader");
-	if (opt) {
-		ifce = (GF_FontReader *) gf_modules_load_interface_by_name(user->modules, opt, GF_FONT_READER_INTERFACE);
-		if (ifce && ifce->init_font_engine(ifce) != GF_OK) {
-			gf_modules_close_interface((GF_BaseInterface *)ifce);
-			ifce = NULL;
-		}
-	}
+	ifce = (GF_FontReader *) gf_module_load(GF_FONT_READER_INTERFACE, NULL);
+	if (ifce) ifce->init_font_engine(ifce);
 
-	if (!ifce) {
-		count = gf_modules_get_count(user->modules);
-		for (i=0; i<count; i++) {
-			ifce = (GF_FontReader *) gf_modules_load_interface(user->modules, i, GF_FONT_READER_INTERFACE);
-			if (!ifce) continue;
 
-			if (ifce->init_font_engine(ifce) != GF_OK) {
-				gf_modules_close_interface((GF_BaseInterface *)ifce);
-				ifce = NULL;
-				continue;
-			}
-
-			gf_cfg_set_key(user->config, "FontEngine", "FontReader", ifce->module_name);
-			break;
-		}
-	}
 	GF_SAFEALLOC(font_mgr, GF_FontManager);
+	if (!font_mgr) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[Compositor] Failed to allocate font manager\n"));
+		return NULL;
+	}
 	font_mgr->reader = ifce;
 	font_mgr->id_buffer_size = 20;
-	font_mgr->id_buffer = malloc(sizeof(u32)*font_mgr->id_buffer_size);
+	font_mgr->id_buffer = gf_malloc(sizeof(u32)*font_mgr->id_buffer_size);
 	gf_font_manager_set_font(font_mgr, &def_font, 1, 0);
 	font_mgr->default_font = font_mgr->font;
 
@@ -91,6 +75,10 @@ GF_FontManager *gf_font_manager_new(GF_User *user)
 	gf_path_add_line_to(font_mgr->line_path, FIX_ONE/2, -FIX_ONE/2);
 	gf_path_add_line_to(font_mgr->line_path, -FIX_ONE/2, -FIX_ONE/2);
 	gf_path_close(font_mgr->line_path);
+
+
+	font_mgr->wait_font_load = wait_for_fonts;
+
 	return font_mgr;
 }
 
@@ -100,7 +88,9 @@ void gf_font_predestroy(GF_Font *font)
 		while (gf_list_count(font->spans)) {
 			GF_TextSpan *ts = gf_list_get(font->spans, 0);
 			gf_list_rem(font->spans, 0);
+#ifndef GPAC_DISABLE_COMPOSITOR
 			gf_node_dirty_set(ts->user, 0, 0);
+#endif
 			ts->user=NULL;
 		}
 		gf_list_del(font->spans);
@@ -117,12 +107,12 @@ void gf_font_del(GF_Font *font)
 		while (glyph) {
 			GF_Glyph *next = glyph->next;
 			gf_path_del(glyph->path);
-			free(glyph);
+			gf_free(glyph);
 			glyph = next;
 		}
 	}
-	free(font->name);
-	free(font);
+	gf_free(font->name);
+	gf_free(font);
 }
 
 
@@ -140,9 +130,9 @@ void gf_font_manager_del(GF_FontManager *fm)
 		gf_font_del(font);
 		font = next;
 	}
-	free(fm->id_buffer);
+	gf_free(fm->id_buffer);
 	gf_path_del(fm->line_path);
-	free(fm);
+	gf_free(fm);
 }
 
 GF_Err gf_font_manager_register_font(GF_FontManager *fm, GF_Font *font)
@@ -162,7 +152,7 @@ GF_Err gf_font_manager_register_font(GF_FontManager *fm, GF_Font *font)
 GF_Err gf_font_manager_unregister_font(GF_FontManager *fm, GF_Font *font)
 {
 	GF_Font *prev_font, *a_font;
-	
+
 	prev_font = NULL;
 	a_font = fm->font;
 	while (a_font) {
@@ -180,7 +170,7 @@ GF_Err gf_font_manager_unregister_font(GF_FontManager *fm, GF_Font *font)
 }
 
 
-
+GF_EXPORT
 GF_Font *gf_font_manager_set_font_ex(GF_FontManager *fm, char **alt_fonts, u32 nb_fonts, u32 styles, Bool check_only)
 {
 	u32 i;
@@ -190,11 +180,35 @@ GF_Font *gf_font_manager_set_font_ex(GF_FontManager *fm, char **alt_fonts, u32 n
 	GF_Font *the_font = NULL;
 
 	for (i=0; i<nb_fonts; i++) {
+		char *font_name;
+		const char *opt;
 		u32 weight_diff = 0xFFFFFFFF;
 		GF_Font *best_font = NULL;
 		GF_Font *font = fm->font;
+		font_name = alt_fonts[i];
+		if (!font_name) font_name = "SANS";
+
+		if (!stricmp(font_name, "SERIF")) {
+			opt = gf_opts_get_key("FontCache", "FontSerif");
+			if (opt) font_name = (char*)opt;
+		}
+		else if (!stricmp(font_name, "SANS") || !stricmp(font_name, "sans-serif")) {
+			opt = gf_opts_get_key("FontCache", "FontSans");
+			if (opt) font_name = (char*)opt;
+		}
+		else if (!stricmp(font_name, "TYPEWRITER") || !stricmp(font_name, "monospace")) {
+			opt = gf_opts_get_key("FontCache", "FontFixed");
+			if (opt) font_name = (char*)opt;
+		}
+
 		while (font) {
-			if ((check_only || !font->not_loaded) && font->name && !stricmp(font->name, alt_fonts[i])) {
+			if (fm->wait_font_load && font->not_loaded && !check_only && !stricmp(font->name, font_name)) {
+				GF_Font *a_font = NULL;
+				if (font->get_alias) a_font = font->get_alias(font->udta);
+				if (!a_font || a_font->not_loaded)
+					return font;
+			}
+			if ((check_only || !font->not_loaded) && font->name && !stricmp(font->name, font_name)) {
 				s32 fw;
 				s32 w;
 				u32 diff;
@@ -202,7 +216,7 @@ GF_Font *gf_font_manager_set_font_ex(GF_FontManager *fm, char **alt_fonts, u32 n
 				Bool ft_has_weight;
 
 				if (check_only) return font;
-				
+
 				ft_has_weight = (font->styles & GF_FONT_WEIGHT_MASK) ? 1 : 0;
 				if (font->styles == styles) {
 					the_font = font;
@@ -254,13 +268,16 @@ GF_Font *gf_font_manager_set_font_ex(GF_FontManager *fm, char **alt_fonts, u32 n
 		}
 		if (the_font) break;
 		if (fm->reader) {
-			e = fm->reader->set_font(fm->reader, alt_fonts[i], styles);
+			e = fm->reader->set_font(fm->reader, font_name, styles);
 			if (!e) {
 				GF_SAFEALLOC(the_font, GF_Font);
+				if (!the_font) {
+					return NULL;
+				}
 				fm->reader->get_font_info(fm->reader, &the_font->name, &the_font->em_size, &the_font->ascent, &the_font->descent, &the_font->underline, &the_font->line_spacing, &the_font->max_advance_h, &the_font->max_advance_v);
 				the_font->styles = styles;
-				if (!the_font->name) the_font->name = strdup(alt_fonts[i]);
-			
+				if (!the_font->name) the_font->name = gf_strdup(font_name);
+
 				if (fm->font) {
 					font = fm->font;
 					while (font->next) font = font->next;
@@ -289,7 +306,7 @@ GF_Font *gf_font_manager_set_font_ex(GF_FontManager *fm, char **alt_fonts, u32 n
 		the_font = fm->default_font;
 	}
 	/*embeded font*/
-	if (the_font && !the_font->get_glyphs) 
+	if (fm->reader && the_font && !the_font->get_glyphs)
 		fm->reader->set_font(fm->reader, the_font->name, the_font->styles);
 
 	return the_font;
@@ -306,14 +323,40 @@ static GF_Glyph *gf_font_get_glyph(GF_FontManager *fm, GF_Font *font, u32 name)
 		if (glyph->ID==name) return glyph;
 		glyph = glyph->next;
 	}
-	/*load glyph*/
 
-	if (font->load_glyph) {
-		glyph = font->load_glyph(font->udta, name);
+	if (name==GF_CARET_CHAR) {
+		GF_SAFEALLOC(glyph, GF_Glyph);
+		if (!glyph) return NULL;
+		glyph->height = font->ascent;
+		glyph->horiz_advance = 0;
+		glyph->width = 0;
+		glyph->ID = GF_CARET_CHAR;
+		glyph->path = gf_path_new();
+		gf_path_add_move_to(glyph->path, 0, INT2FIX(font->descent));
+		gf_path_add_line_to(glyph->path, 0, INT2FIX(font->ascent));
+		gf_path_add_line_to(glyph->path, 1, INT2FIX(font->ascent));
+		gf_path_add_line_to(glyph->path, 1, INT2FIX(font->descent));
+		gf_path_close(glyph->path);
+		glyph->utf_name=0;
+	} else if (name==(u32) '\n') {
+		GF_SAFEALLOC(glyph, GF_Glyph);
+		if (!glyph) return NULL;
+		glyph->height = font->ascent;
+		glyph->horiz_advance = 0;
+		glyph->width = 0;
+		glyph->ID = name;
+		glyph->utf_name=name;
+	} else if (name==(u32) '\t') {
+		return NULL;
 	} else {
-		if (!fm->reader) return NULL;
-//		fm->reader->set_font(fm->reader, font->name, font->styles);
-		glyph = fm->reader->load_glyph(fm->reader, name);
+		/*load glyph*/
+		if (font->load_glyph) {
+			glyph = font->load_glyph(font->udta, name);
+		} else {
+			if (!fm->reader) return NULL;
+			//		fm->reader->set_font(fm->reader, font->name, font->styles);
+			glyph = fm->reader->load_glyph(fm->reader, name);
+		}
 	}
 	if (!glyph) return NULL;
 
@@ -324,7 +367,7 @@ static GF_Glyph *gf_font_get_glyph(GF_FontManager *fm, GF_Font *font, u32 name)
 		a_glyph->next = glyph;
 	}
 	/*space character - this may need adjustment for other empty glyphs*/
-	if (!glyph->path->n_points) {
+	if (glyph->path && !glyph->path->n_points) {
 		glyph->path->bbox.x = 0;
 		glyph->path->bbox.width = INT2FIX(font->max_advance_h);
 		glyph->path->bbox.y = INT2FIX(font->ascent);
@@ -333,7 +376,7 @@ static GF_Glyph *gf_font_get_glyph(GF_FontManager *fm, GF_Font *font, u32 name)
 	return glyph;
 }
 
-
+GF_EXPORT
 GF_TextSpan *gf_font_manager_create_span(GF_FontManager *fm, GF_Font *font, char *text, Fixed font_size, Bool needs_x_offset, Bool needs_y_offset, Bool needs_rotate, const char *xml_lang, Bool fliped_text, u32 styles, GF_Node *user)
 {
 	GF_Err e;
@@ -341,7 +384,7 @@ GF_TextSpan *gf_font_manager_create_span(GF_FontManager *fm, GF_Font *font, char
 	u32 len, i;
 	GF_TextSpan *span;
 
-	if (!user || !strlen(text)) return NULL;
+	if (!strlen(text)) return NULL;
 
 	len = fm->id_buffer_size;
 	if (font->get_glyphs)
@@ -351,9 +394,9 @@ GF_TextSpan *gf_font_manager_create_span(GF_FontManager *fm, GF_Font *font, char
 
 	if (e==GF_BUFFER_TOO_SMALL) {
 		fm->id_buffer_size = len;
-		fm->id_buffer = realloc(fm->id_buffer, sizeof(u32) * len);
+		fm->id_buffer = gf_realloc(fm->id_buffer, sizeof(u32) * len);
 		if (!fm->id_buffer) return NULL;
-	
+
 		if (font->get_glyphs)
 			e = font->get_glyphs(font->udta, text, fm->id_buffer, &len, xml_lang, &is_rtl);
 		else
@@ -362,42 +405,45 @@ GF_TextSpan *gf_font_manager_create_span(GF_FontManager *fm, GF_Font *font, char
 	if (e) return NULL;
 
 	GF_SAFEALLOC(span, GF_TextSpan);
+	if (!span) return NULL;
 	span->font = font;
 	span->font_size = font_size;
-	if (font->em_size) 
+	if (font->em_size)
 		span->font_scale = font_size / font->em_size;
 	span->x_scale = span->y_scale = FIX_ONE;
 //	span->lang = xml_lang;
 	if (fliped_text) span->flags |= GF_TEXT_SPAN_FLIP;
 	if (styles & GF_FONT_UNDERLINED) span->flags |= GF_TEXT_SPAN_UNDERLINE;
+	if (styles & GF_FONT_STRIKEOUT) span->flags |= GF_TEXT_SPAN_STRIKEOUT;
 	span->nb_glyphs = len;
-	span->glyphs = malloc(sizeof(void *)*len);
+	span->glyphs = gf_malloc(sizeof(void *)*len);
 	if (needs_x_offset) {
-		span->dx = malloc(sizeof(Fixed)*len);
+		span->dx = gf_malloc(sizeof(Fixed)*len);
 		memset(span->dx, 0, sizeof(Fixed)*len);
 	}
 	if (needs_y_offset) {
-		span->dy = malloc(sizeof(Fixed)*len);
+		span->dy = gf_malloc(sizeof(Fixed)*len);
 		memset(span->dy, 0, sizeof(Fixed)*len);
 	}
 	if (needs_rotate) {
-		span->rot = malloc(sizeof(Fixed)*len);
+		span->rot = gf_malloc(sizeof(Fixed)*len);
 		memset(span->rot, 0, sizeof(Fixed)*len);
 	}
-	
+
 	for (i=0; i<len; i++) {
 		span->glyphs[i] = gf_font_get_glyph(fm, font, fm->id_buffer[i]);
 	}
 	span->user = user;
-	if (span->font->spans) 
+	if (span->font->spans)
 		gf_list_add(font->spans, span);
 	if (is_rtl) span->flags |= GF_TEXT_SPAN_RIGHT_TO_LEFT;
 	return span;
 }
 
 
+#ifndef GPAC_DISABLE_COMPOSITOR
 
-typedef struct _span_internal 
+typedef struct _span_internal
 {
 	/*zoom when texture was computed*/
 	Fixed last_zoom;
@@ -416,16 +462,19 @@ typedef struct _span_internal
 #endif
 } GF_SpanExtensions;
 
+#endif
+
 
 void gf_font_manager_delete_span(GF_FontManager *fm, GF_TextSpan *span)
 {
 	if (span->user && span->font->spans) gf_list_del_item(span->font->spans, span);
 
-	free(span->glyphs);
-	if (span->dx) free(span->dx);
-	if (span->dy) free(span->dy);
-	if (span->rot) free(span->rot);
+	gf_free(span->glyphs);
+	if (span->dx) gf_free(span->dx);
+	if (span->dy) gf_free(span->dy);
+	if (span->rot) gf_free(span->rot);
 
+#ifndef GPAC_DISABLE_COMPOSITOR
 	if (span->ext) {
 		if (span->ext->path) gf_path_del(span->ext->path);
 #ifndef GPAC_DISABLE_3D
@@ -435,19 +484,22 @@ void gf_font_manager_delete_span(GF_FontManager *fm, GF_TextSpan *span)
 #endif
 		if (span->ext->txh) {
 			gf_sc_texture_destroy(span->ext->txh);
-			if (span->ext->txh->data) free(span->ext->txh->data);
-			free(span->ext->txh);
+			if (span->ext->txh->data) gf_free(span->ext->txh->data);
+			gf_free(span->ext->txh);
 		}
-		free(span->ext);
+		gf_free(span->ext);
 	}
-	free(span);
+#endif
+
+	gf_free(span);
 }
 
+GF_EXPORT
 void gf_font_manager_refresh_span_bounds(GF_TextSpan *span)
 {
 	u32 i;
 	Fixed descent, ascent, bline;
-	Fixed min_x, min_y, width, max_y;
+	Fixed min_x, min_y, max_y;
 
 	if (!span->nb_glyphs) {
 		span->bounds.width = span->bounds.height = 0;
@@ -466,8 +518,6 @@ void gf_font_manager_refresh_span_bounds(GF_TextSpan *span)
 
 	bline = span->font->baseline * span->font_scale;
 
-	width = (span->flags & GF_TEXT_SPAN_HORIZONTAL) ? 0 : span->font->max_advance_h*span->font_scale;
-
 	min_x = span->dx ? FIX_MAX : span->off_x;
 	min_y = span->dy ? FIX_MAX : span->off_y - descent;
 	max_y = span->dy ? -FIX_MAX : span->off_y + ascent;
@@ -478,21 +528,21 @@ void gf_font_manager_refresh_span_bounds(GF_TextSpan *span)
 	}
 	span->bounds = gf_rect_center(0, 0);
 
-	for (i=0;i<span->nb_glyphs; i++) {
+	for (i=0; i<span->nb_glyphs; i++) {
 		Fixed g_width;
 		GF_Rect rc;
 
 		/*compute glyph size*/
 		if (!span->glyphs[i]) g_width = span->font->max_advance_h * span->font_scale;
 		/*if last glyph of the span, increase by width only*/
-		else if (i+1==span->nb_glyphs) g_width = span->glyphs[i]->width * span->font_scale;
+//		else if (i+1==span->nb_glyphs) g_width = span->glyphs[i]->width * span->font_scale;
 		/*otherwise increase by the horizontal advance*/
 		else g_width = span->glyphs[i]->horiz_advance * span->font_scale;
 
 		if (span->dy) {
 			if (span->dy[i] - descent < min_y) min_y = span->dy[i] - descent;
 			if (span->dy[i] + ascent > max_y) max_y = span->dy[i] + ascent;
-		} 
+		}
 		else if (span->glyphs[i]) {
 			Fixed size = span->glyphs[i]->height * span->font_scale;
 			if (size > max_y-min_y) max_y = size + min_y;
@@ -555,6 +605,9 @@ GF_Path *gf_font_span_create_path(GF_TextSpan *span)
 				if (span->rot) {
 					gf_mx2d_add_rotation(&mx, 0, 0, -span->rot[i]);
 				}
+				if (span->glyphs[i]->ID==GF_CARET_CHAR) {
+					gf_mx2d_add_scale(&mx, mat.m[0], FIX_ONE);
+				}
 				gf_mx2d_add_translation(&mx, dx, dy);
 				gf_path_add_subpath(path, span->glyphs[i]->path, &mx);
 			}
@@ -573,6 +626,8 @@ GF_Path *gf_font_span_create_path(GF_TextSpan *span)
 	return path;
 }
 
+#ifndef GPAC_DISABLE_COMPOSITOR
+
 static void span_alloc_extensions(GF_TextSpan *span)
 {
 	if (span->ext) return;
@@ -608,12 +663,10 @@ static Bool span_setup_texture(GF_Compositor *compositor, GF_TextSpan *span, Boo
 	Fixed cx, cy, sx, sy, max, min;
 	u32 tw, th;
 	GF_Matrix2D mx;
-	GF_STENCIL stencil, brush;
-	GF_SURFACE surface;
+	GF_EVGStencil *stencil, *brush;
+	GF_EVGSurface *surface;
 	u32 width, height;
 	Fixed scale;
-	GF_Raster2D *raster = compositor->rasterizer;
-
 	span_alloc_extensions(span);
 
 	/*something failed*/
@@ -641,7 +694,7 @@ static Bool span_setup_texture(GF_Compositor *compositor, GF_TextSpan *span, Boo
 	}
 	if ((gf_mulfix(scale, bounds.width)>max) || (gf_mulfix(scale, bounds.height)>max)) {
 		scale = MIN(gf_divfix(max, bounds.width), gf_divfix(max, bounds.height));
-	} 
+	}
 	else if ((gf_mulfix(scale, bounds.width)<min) || (gf_mulfix(scale, bounds.height)<min)) {
 		scale = MAX(gf_divfix(min, bounds.width), gf_divfix(min, bounds.height));
 	}
@@ -675,14 +728,15 @@ static Bool span_setup_texture(GF_Compositor *compositor, GF_TextSpan *span, Boo
 
 	if (span->ext->txh) {
 		gf_sc_texture_destroy(span->ext->txh);
-		if (span->ext->txh->data) free(span->ext->txh->data);
-		free(span->ext->txh);
+		if (span->ext->txh->data) gf_free(span->ext->txh->data);
+		gf_free(span->ext->txh);
 	}
 	GF_SAFEALLOC(span->ext->txh, GF_TextureHandler);
+	if (!span->ext->txh) return 0;
 	gf_sc_texture_setup(span->ext->txh, compositor, NULL);
 	gf_sc_texture_allocate(span->ext->txh);
 	stencil = gf_sc_texture_get_stencil(span->ext->txh);
-	if (!stencil) stencil = raster->stencil_new(raster, GF_STENCIL_TEXTURE);
+	if (!stencil) stencil = gf_evg_stencil_new(GF_STENCIL_TEXTURE);
 
 	/*FIXME - make it work with alphagrey...*/
 	span->ext->txh->width = width;
@@ -692,21 +746,21 @@ static Bool span_setup_texture(GF_Compositor *compositor, GF_TextSpan *span, Boo
 	span->ext->txh->transparent = 1;
 	span->ext->txh->flags |= GF_SR_TEXTURE_NO_GL_FLIP;
 
-	surface = raster->surface_new(raster, 1);
+	surface = gf_evg_surface_new(1);
 	if (!surface) {
 		gf_sc_texture_release(span->ext->txh);
 		return 0;
 	}
-	span->ext->txh->data = (char *) malloc(sizeof(char)*span->ext->txh->stride*span->ext->txh->height);
+	span->ext->txh->data = (char *) gf_malloc(sizeof(char)*span->ext->txh->stride*span->ext->txh->height);
 	memset(span->ext->txh->data, 0, sizeof(char)*span->ext->txh->stride*span->ext->txh->height);
 
-	raster->stencil_set_texture(stencil, span->ext->txh->data, span->ext->txh->width, span->ext->txh->height, span->ext->txh->stride, span->ext->txh->pixelformat, span->ext->txh->pixelformat, 1);
-	raster->surface_attach_to_texture(surface, stencil);
+	gf_evg_stencil_set_texture(stencil, span->ext->txh->data, span->ext->txh->width, span->ext->txh->height, span->ext->txh->stride, span->ext->txh->pixelformat);
+	gf_evg_surface_attach_to_texture(surface, stencil);
 
-	brush = raster->stencil_new(raster, GF_STENCIL_SOLID);
-	raster->stencil_set_brush_color(brush, 0xFF000000);
+	brush = gf_evg_stencil_new(GF_STENCIL_SOLID);
+	gf_evg_stencil_set_brush_color(brush, for_3d ? 0xFFFFFFFF : tr_state->ctx->aspect.fill_color);
 
-	cx = bounds.x + bounds.width/2; 
+	cx = bounds.x + bounds.width/2;
 	cy = bounds.y - bounds.height/2;
 
 	gf_mx2d_init(mx);
@@ -714,14 +768,14 @@ static Bool span_setup_texture(GF_Compositor *compositor, GF_TextSpan *span, Boo
 	gf_mx2d_add_scale(&mx, sx, sy);
 //	gf_mx2d_add_scale(&mx, 99*FIX_ONE/100, 99*FIX_ONE/100);
 
-	raster->surface_set_matrix(surface, &mx);
-	raster->surface_set_raster_level(surface, GF_RASTER_HIGH_QUALITY);
+	gf_evg_surface_set_matrix(surface, &mx);
+	gf_evg_surface_set_raster_level(surface, GF_RASTER_HIGH_QUALITY);
 	span_path = gf_font_span_create_path(span);
-	raster->surface_set_path(surface, span_path);
+	gf_evg_surface_set_path(surface, span_path);
 
-	raster->surface_fill(surface, brush);
-	raster->stencil_delete(brush);
-	raster->surface_delete(surface);
+	gf_evg_surface_fill(surface, brush);
+	gf_evg_stencil_delete(brush);
+	gf_evg_surface_delete(surface);
 	gf_path_del(span_path);
 
 	if (span->font->baseline) {
@@ -768,16 +822,15 @@ static void span_strike_3d(GF_TextSpan *span, GF_TraverseState *tr_state, DrawAs
 	if (!span->ext->outline) {
 		GF_Path *path = gf_font_span_create_path(span);
 		span->ext->outline = new_mesh();
-#ifndef GPAC_USE_OGL_ES
+#ifdef GPAC_HAS_GLU
 		if (vect_outline) {
 			GF_Path *outline = gf_path_get_outline(path, asp->pen_props);
-			TesselatePath(span->ext->outline, outline, asp->line_texture ? 2 : 1);
+			gf_mesh_tesselate_path(span->ext->outline, outline, asp->line_texture ? 2 : 1);
 			gf_path_del(outline);
 		} else {
 			mesh_get_outline(span->ext->outline, path);
 		}
 #else
-		/*VECTORIAL TEXT OUTLINE NOT SUPPORTED ON OGL-ES AT CURRENT TIME*/
 		vect_outline = 0;
 		mesh_get_outline(span->ext->outline, path);
 #endif
@@ -799,7 +852,7 @@ void gf_font_spans_draw_3d(GF_List *spans, GF_TraverseState *tr_state, DrawAspec
 	Bool fill_2d, vect_outline, can_texture_text;
 	GF_Compositor *compositor = (GF_Compositor*)tr_state->visual->compositor;
 
-	vect_outline = !compositor->raster_outlines;
+	vect_outline = !compositor->linegl;
 
 	visual_3d_set_state(tr_state->visual, V3D_STATE_BLEND, 0);
 
@@ -813,11 +866,12 @@ void gf_font_spans_draw_3d(GF_List *spans, GF_TraverseState *tr_state, DrawAspec
 
 	if (text_hl && (fill_2d || !asp) ) {
 		/*reverse video: highlighting uses the text color, and text color is inverted (except alpha channel)
-		the ideal impl would be to use the background color for the text, but since the text may be 
+		the ideal impl would be to use the background color for the text, but since the text may be
 		displayed over anything non uniform this would require clipping the highlight rect with the text
 		which is too onerous (and not supported anyway) */
 		if (text_hl == 0x00FFFFFF) {
 			if (!asp) {
+#ifndef GPAC_DISABLE_VRML
 				if (tr_state->appear) {
 					SFColor c, rc;
 					c = ((M_Material *) ((M_Appearance *)  tr_state->appear)->material)->diffuseColor;
@@ -830,7 +884,9 @@ void gf_font_spans_draw_3d(GF_List *spans, GF_TraverseState *tr_state, DrawAspec
 					((M_Material *) ((M_Appearance *)  tr_state->appear)->material)->diffuseColor = rc;
 					visual_3d_setup_appearance(tr_state);
 					((M_Material *) ((M_Appearance *)  tr_state->appear)->material)->diffuseColor = c;
-				} else {
+				} else
+#endif /*GPAC_DISABLE_VRML*/
+				{
 					hl_color.red = hl_color.green = hl_color.blue = 0;
 					hl_color.alpha = FIX_ONE;
 				}
@@ -860,7 +916,7 @@ void gf_font_spans_draw_3d(GF_List *spans, GF_TraverseState *tr_state, DrawAspec
 	can_texture_text = 0;
 	if (fill_2d || !asp) {
 		/*check if we can use text texturing*/
-		if (force_texturing || (compositor->texture_text_mode != GF_TEXTURE_TEXT_NEVER) ) {
+		if (force_texturing || (compositor->textxt != GF_TEXTURE_TEXT_NEVER) ) {
 			if (fill_2d && asp->pen_props.width) {
 				can_texture_text = 0;
 			} else {
@@ -869,36 +925,42 @@ void gf_font_spans_draw_3d(GF_List *spans, GF_TraverseState *tr_state, DrawAspec
 		}
 	}
 
-	visual_3d_enable_antialias(tr_state->visual, compositor->antiAlias);
+	visual_3d_enable_antialias(tr_state->visual, compositor->aa);
 	if (fill_2d || !asp || tr_state->mesh_num_textures) {
-		if (fill_2d) visual_3d_set_material_2d_argb(tr_state->visual, asp->fill_color);
+		if (fill_2d && asp) visual_3d_set_material_2d_argb(tr_state->visual, asp->fill_color);
 
 		i = tr_state->text_split_idx ? tr_state->text_split_idx-1 : 0;
 		while ((span = (GF_TextSpan *)gf_list_enum(spans, &i))) {
 			if (text_hl) {
 				visual_3d_fill_rect(tr_state->visual, span->bounds, hl_color);
-				if (fill_2d) 
-					visual_3d_set_material_2d_argb(tr_state->visual, asp->fill_color);
-				else
+
+				if (fill_2d) {
+					if (asp)
+						visual_3d_set_material_2d_argb(tr_state->visual, asp->fill_color);
+				} else
 					visual_3d_setup_appearance(tr_state);
 			}
 
 			if (can_texture_text && span_setup_texture(tr_state->visual->compositor, span, 1, tr_state)) {
-				tr_state->mesh_num_textures = 1;
-				gf_sc_texture_enable(span->ext->txh, NULL);
-				visual_3d_mesh_paint(tr_state, span->ext->tx_mesh);
-				gf_sc_texture_disable(span->ext->txh);
+				tr_state->mesh_num_textures = gf_sc_texture_enable(span->ext->txh, NULL);
+				if (tr_state->mesh_num_textures) {
+					Bool has_mat_2d = tr_state->visual->compositor->visual->has_material_2d;
+					visual_3d_mesh_paint(tr_state, span->ext->tx_mesh);
+					gf_sc_texture_disable(span->ext->txh);
+					tr_state->mesh_num_textures = 0;
+					tr_state->visual->has_material_2d = has_mat_2d;
+				}
 			} else {
 				span_fill_3d(span, tr_state);
 			}
 
 			if (tr_state->text_split_idx) break;
 		}
+		tr_state->visual->has_material_2d = GF_FALSE;
 
 		/*reset texturing in case of line texture*/
 		if (!asp) visual_3d_disable_texture(tr_state);
 	}
-
 	visual_3d_set_state(tr_state->visual, V3D_STATE_BLEND, 0);
 
 	if (asp && asp->pen_props.width) {
@@ -923,8 +985,6 @@ void gf_font_spans_draw_3d(GF_List *spans, GF_TraverseState *tr_state, DrawAspec
 
 #endif
 
-
-
 static void gf_font_span_draw_2d(GF_TraverseState *tr_state, GF_TextSpan *span, DrawableContext *ctx, GF_Rect *bounds)
 {
 	u32 flags, i;
@@ -956,7 +1016,15 @@ static void gf_font_span_draw_2d(GF_TraverseState *tr_state, GF_TextSpan *span,
 			}
 			continue;
 		}
-		ctx->transform.m[0] = sx;
+		if (span->glyphs[i]->ID==GF_CARET_CHAR) {
+			if (tr_state->visual->compositor->show_caret) {
+				ctx->transform.m[0] = FIX_ONE;
+			} else {
+				continue;
+			}
+		} else {
+			ctx->transform.m[0] = sx;
+		}
 		ctx->transform.m[4] = flip_text  ? -sy : sy;
 		ctx->transform.m[1] = ctx->transform.m[3] = 0;
 		ctx->transform.m[2] = span->dx ? span->dx[i] : dx;
@@ -981,7 +1049,7 @@ static void gf_font_span_draw_2d(GF_TraverseState *tr_state, GF_TextSpan *span,
 			gf_mx2d_inverse(&tx);
 
 			visual_2d_texture_path_extended(tr_state->visual, span->glyphs[i]->path, ctx->aspect.fill_texture, ctx, bounds ? bounds : &span->bounds, &tx, tr_state);
-			visual_2d_draw_path_extended(tr_state->visual, span->glyphs[i]->path, ctx, NULL, NULL, tr_state, bounds ? bounds : &span->bounds, &tx);
+			visual_2d_draw_path_extended(tr_state->visual, span->glyphs[i]->path, ctx, NULL, NULL, tr_state, bounds ? bounds : &span->bounds, &tx, GF_FALSE);
 		} else {
 			visual_2d_draw_path(tr_state->visual, span->glyphs[i]->path, ctx, NULL, NULL, tr_state);
 		}
@@ -997,7 +1065,7 @@ static void gf_font_span_draw_2d(GF_TraverseState *tr_state, GF_TextSpan *span,
 	ctx->aspect.line_scale = lscale;
 }
 
-void gf_font_underline_span(GF_TraverseState *tr_state, GF_TextSpan *span, DrawableContext *ctx)
+void gf_font_underline_span(GF_TraverseState *tr_state, GF_TextSpan *span, DrawableContext *ctx, Bool is_strikeout)
 {
 	GF_Matrix2D mx, m;
 	u32 col;
@@ -1006,12 +1074,15 @@ void gf_font_underline_span(GF_TraverseState *tr_state, GF_TextSpan *span, Drawa
 
 	gf_mx2d_copy(mx, ctx->transform);
 	sx = gf_mulfix(span->font_scale, span->x_scale);
-	diff = span->font_scale * span->font->underline;
-	if (span->flags & GF_TEXT_SPAN_FLIP) 
+
+	if (span->flags & GF_TEXT_SPAN_FLIP)
 		diff = sx * (span->font->descent - span->font->underline);
 	else
 		diff = sx * (- span->font->ascent + span->font->underline);
 
+	if (is_strikeout)
+		diff = 2*diff/3;
+
 	gf_mx2d_init(m);
 	gf_mx2d_add_scale(&m, span->bounds.width, FIX_ONE);
 	gf_mx2d_add_translation(&m, span->bounds.x + span->bounds.width / 2, span->bounds.y+diff);
@@ -1031,7 +1102,7 @@ void gf_font_underline_span(GF_TraverseState *tr_state, GF_TextSpan *span, Drawa
 }
 
 #if 0
-static u32 col_reverse_video(u32 col) 
+static u32 col_reverse_video(u32 col)
 {
 	u32 a, r, g, b;
 	a = GF_COL_A(col);
@@ -1047,7 +1118,7 @@ static GF_Rect font_get_sel_rect(GF_TraverseState *tr_state)
 	GF_Vec s, e;
 	GF_Rect rc;
 	GF_Compositor *compositor = tr_state->visual->compositor;
-	
+
 	e.x = compositor->end_sel.x;
 	e.y = compositor->end_sel.y;
 	e.z = 0;
@@ -1111,17 +1182,19 @@ static void gf_font_spans_select(GF_TextSpan *span, GF_TraverseState *tr_state,
 	}
 
 	color = compositor->text_sel_color;
-	
+
 	for (i=0; i<span->nb_glyphs; i++) {
 		GF_Rect g_rc;
 		Bool end_of_line = 0;
-		Fixed advance = sx * span->glyphs[i]->horiz_advance;
+		Fixed advance;
+		if (!span->glyphs[i]) continue;
+		advance = sx * span->glyphs[i]->horiz_advance;
 		if (span->dx) dx = span->dx[i];
 		if (span->dy) dy = span->dy[i];
 		if (dx + advance/2 < rc->x) {
 			dx += advance;
 			continue;
-		} 
+		}
 
 		g_rc.height = ascent-descent;
 
@@ -1164,14 +1237,15 @@ static void gf_font_spans_select(GF_TextSpan *span, GF_TraverseState *tr_state,
 
 		if (!end_of_line && !gf_rect_overlaps(g_rc, *rc))
 			continue;
-/*
-		if (dy < rc.y-rc.height) {
-			if (!span->dx) break;
-			continue;
-		} 
-*/				
+		/*
+				if (dy < rc.y-rc.height) {
+					if (!span->dx) break;
+					continue;
+				}
+		*/
 		has_selection = 1;
 		if (ctx) {
+			g_rc.width+=2;
 			if (span->rot) {
 				gf_mx2d_init(ctx->transform);
 				gf_mx2d_add_rotation(&ctx->transform, g_rc.x , g_rc.y, span->rot[i]);
@@ -1184,9 +1258,9 @@ static void gf_font_spans_select(GF_TextSpan *span, GF_TraverseState *tr_state,
 			if (!compositor->sel_buffer_alloc || compositor->sel_buffer_len == compositor->sel_buffer_alloc) {
 				if (!compositor->sel_buffer_alloc) compositor->sel_buffer_alloc ++;
 				compositor->sel_buffer_alloc = 2*compositor->sel_buffer_alloc;
-				compositor->sel_buffer = realloc(compositor->sel_buffer, sizeof(u16)*compositor->sel_buffer_alloc);
+				compositor->sel_buffer = gf_realloc(compositor->sel_buffer, sizeof(u16)*compositor->sel_buffer_alloc);
 			}
-			compositor->sel_buffer[compositor->sel_buffer_len] = span->glyphs[i]->utf_name;
+			compositor->sel_buffer[compositor->sel_buffer_len] = (u16) span->glyphs[i]->utf_name;
 			compositor->sel_buffer_len++;
 		}
 
@@ -1204,12 +1278,11 @@ static void gf_font_spans_select(GF_TextSpan *span, GF_TraverseState *tr_state,
 
 void gf_font_spans_get_selection(GF_Node *node, GF_List *spans, GF_TraverseState *tr_state)
 {
-	GF_TextSpan *span;
 	u32 i, count;
 	GF_Rect rc;
 	count = gf_list_count(spans);
 	for (i=0; i<count; i++) {
-		span = (GF_TextSpan *)gf_list_get(spans, i);
+		GF_TextSpan *span = (GF_TextSpan *)gf_list_get(spans, i);
 		gf_font_spans_select(span, tr_state, NULL, (i+1<count) ? 1 : 0, (i==0) ? 1 : 0, &rc);
 	}
 }
@@ -1223,22 +1296,22 @@ void gf_font_spans_draw_2d(GF_List *spans, GF_TraverseState *tr_state, u32 hl_co
 	GF_TextSpan *span;
 	DrawableContext *ctx = tr_state->ctx;
 	GF_Rect rc;
-	
+
 	use_texture_text = 0;
-	if (force_texture_text || (compositor->texture_text_mode==GF_TEXTURE_TEXT_ALWAYS) ) {
+	if (force_texture_text || (compositor->textxt==GF_TEXTURE_TEXT_ALWAYS) ) {
 		use_texture_text = !ctx->aspect.fill_texture && !ctx->aspect.pen_props.width;
 	}
 
 	is_rv = 0;
 	if (hl_color) {
 		/*reverse video: highlighting uses the text color, and text color is inverted (except alpha channel)
-		the ideal impl would be to use the background color for the text, but since the text may be 
+		the ideal impl would be to use the background color for the text, but since the text may be
 		displayed over anything non uniform this would require clipping the highlight rect with the text
 		which is too onerous (and not supported anyway) */
 		if (hl_color==0x00FFFFFF) {
 			u32 a, r, g, b;
 			hl_color = tr_state->ctx->aspect.fill_color;
-			
+
 			a = GF_COL_A(tr_state->ctx->aspect.fill_color);
 			if (a) {
 				r = GF_COL_R(tr_state->ctx->aspect.fill_color);
@@ -1251,13 +1324,14 @@ void gf_font_spans_draw_2d(GF_List *spans, GF_TraverseState *tr_state, u32 hl_co
 		if (GF_COL_A(hl_color) == 0) hl_color = 0;
 	}
 
+	memset(&rc, 0, sizeof(GF_Rect));
 	count = gf_list_count(spans);
 	i=ctx->sub_path_index ? ctx->sub_path_index-1 : 0;
-	for(;i<count; i++) {
+	for(; i<count; i++) {
 		span = (GF_TextSpan *)gf_list_get(spans, i);
 		if (compositor->text_selection)
-			gf_font_spans_select(span, tr_state, ctx, (i+1<count) ? 1 : 0, (i==0)?1:0, &rc);
-		else if (hl_color) 
+			gf_font_spans_select(span, tr_state, ctx, (i+1<count) ? 1 : 0, (i==0)? GF_TRUE : GF_FALSE, &rc);
+		else if (hl_color)
 			visual_2d_fill_rect(tr_state->visual, ctx, &span->bounds, hl_color, 0, tr_state);
 
 		if (use_texture_text && span_setup_texture(compositor, span, 0, tr_state)) {
@@ -1265,7 +1339,8 @@ void gf_font_spans_draw_2d(GF_List *spans, GF_TraverseState *tr_state, u32 hl_co
 		} else {
 			gf_font_span_draw_2d(tr_state, span, ctx, bounds);
 		}
-		if (span->anchor || (span->flags & GF_TEXT_SPAN_UNDERLINE) ) gf_font_underline_span(tr_state, span, ctx);
+		if (span->anchor || (span->flags & GF_TEXT_SPAN_UNDERLINE) ) gf_font_underline_span(tr_state, span, ctx, GF_FALSE);
+		if (span->flags & GF_TEXT_SPAN_STRIKEOUT) gf_font_underline_span(tr_state, span, ctx, GF_TRUE);
 		if (ctx->sub_path_index) break;
 	}
 	if (is_rv) tr_state->ctx->aspect.fill_color = hl_color;
@@ -1273,7 +1348,7 @@ void gf_font_spans_draw_2d(GF_List *spans, GF_TraverseState *tr_state, u32 hl_co
 
 void gf_font_spans_pick(GF_Node *node, GF_List *spans, GF_TraverseState *tr_state, GF_Rect *node_bounds, Bool use_dom_events, Drawable *drawable)
 {
-	u32 i, count, j;
+	u32 i, count, j, glyph_idx;
 	Fixed dx, dy;
 #ifndef GPAC_DISABLE_3D
 	GF_Matrix inv_mx;
@@ -1288,7 +1363,7 @@ void gf_font_spans_pick(GF_Node *node, GF_List *spans, GF_TraverseState *tr_stat
 		if (compositor->text_selection != tr_state->text_parent) return;
 		if (compositor->store_text_state==GF_SC_TSEL_FROZEN) return;
 	}
-	
+
 	/*TODO: pick the real glyph and not just the bounds of the text span*/
 	count = gf_list_count(spans);
 
@@ -1306,7 +1381,7 @@ void gf_font_spans_pick(GF_Node *node, GF_List *spans, GF_TraverseState *tr_stat
 
 		x = local_pt.x;
 		y = local_pt.y;
-	} else 
+	} else
 #endif
 	{
 		gf_mx2d_copy(inv_2d, tr_state->transform);
@@ -1321,24 +1396,37 @@ void gf_font_spans_pick(GF_Node *node, GF_List *spans, GF_TraverseState *tr_stat
 		drawable_get_aspect_2d_svg(node, &asp, tr_state);
 	}
 
-	dx = dy = 0;
 	span = NULL;
 	for (i=0; i<count; i++) {
 		Fixed loc_x, loc_y;
 		span = (GF_TextSpan*)gf_list_get(spans, i);
 
 		if ((x>=span->bounds.x)
-			&& (y<=span->bounds.y) 
-			&& (x<=span->bounds.x+span->bounds.width) 
-			&& (y>=span->bounds.y-span->bounds.height)) {
+		        && (y<=span->bounds.y)
+		        && (x<=span->bounds.x+span->bounds.width)
+		        && (y>=span->bounds.y-span->bounds.height)) {
 		} else {
 			continue;
 		}
 
+		glyph_idx = 0;
 		dx = span->off_x;
 		dy = span->off_y;
 		for (j=0; j<span->nb_glyphs; j++) {
 			GF_Rect rc;
+			if (!span->glyphs[j]) {
+				if (span->flags & GF_TEXT_SPAN_HORIZONTAL) {
+					dx += span->font_scale * span->font->max_advance_h;
+				} else {
+					dy -= span->font_scale * span->font->max_advance_v;
+				}
+				continue;
+			}
+			else if (span->glyphs[j]->ID==GF_CARET_CHAR) {
+				continue;
+			}
+			glyph_idx++;
+
 			loc_x = x - (span->dx ? span->dx[j] : dx);
 			loc_y = y - (span->dy ? span->dy[j] : dy);
 			loc_x = gf_divfix(loc_x, span->font_scale);
@@ -1348,14 +1436,7 @@ void gf_font_spans_pick(GF_Node *node, GF_List *spans, GF_TraverseState *tr_stat
 			gf_path_get_bounds(span->glyphs[j]->path, &rc);
 			rc.height = INT2FIX(span->font->ascent + span->font->descent);
 
-			if (0&&span->rot) {
-				GF_Matrix2D r;
-				gf_mx2d_init(r);
-				gf_mx2d_add_rotation(&r, 0, 0, span->rot[i]);
-				gf_mx2d_apply_coords(&r, &loc_x, &loc_y);
-			}
-
-			if (use_dom_events) {
+			if (use_dom_events && !compositor->sel_buffer) {
 				if (svg_drawable_is_over(drawable, loc_x, loc_y, &asp, tr_state, &rc))
 					goto picked;
 			} else {
@@ -1383,7 +1464,7 @@ picked:
 	if (tr_state->visual->type_3d) {
 		gf_mx_copy(compositor->hit_world_to_local, tr_state->model_matrix);
 		gf_mx_copy(compositor->hit_local_to_world, inv_mx);
-	} else 
+	} else
 #endif
 	{
 		gf_mx_from_mx2d(&compositor->hit_world_to_local, &tr_state->transform);
@@ -1396,21 +1477,28 @@ picked:
 	compositor->end_sel.x = compositor->hit_world_point.x;
 	compositor->end_sel.y = compositor->hit_world_point.y;
 	if (compositor->text_selection) {
-		compositor->draw_next_frame = 1;
+		gf_sc_next_frame_state(compositor, GF_SC_DRAW_FRAME);
+		if (tr_state->visual->offscreen) gf_node_dirty_set(tr_state->visual->offscreen, GF_SG_CHILD_DIRTY, 0);
 		span->flags |= GF_TEXT_SPAN_SELECTED;
 	} else {
 		compositor->start_sel = compositor->end_sel;
 	}
+	compositor->picked_span_idx = i;
+	compositor->picked_glyph_idx = glyph_idx;
 
 	compositor->hit_text = tr_state->text_parent;
 	compositor->hit_use_dom_events = use_dom_events;
-	compositor->hit_normal.x = compositor->hit_normal.y = 0; compositor->hit_normal.z = FIX_ONE;
+	compositor->hit_normal.x = compositor->hit_normal.y = 0;
+	compositor->hit_normal.z = FIX_ONE;
 	compositor->hit_texcoords.x = gf_divfix(x, node_bounds->width) + FIX_ONE/2;
 	compositor->hit_texcoords.y = gf_divfix(y, node_bounds->height) + FIX_ONE/2;
 
+#ifndef GPAC_DISABLE_VRML
 	if (compositor_is_composite_texture(tr_state->appear)) {
 		compositor->hit_appear = tr_state->appear;
-	} else {
+	} else
+#endif /*GPAC_DISABLE_VRML*/
+	{
 		compositor->hit_appear = NULL;
 	}
 
@@ -1420,3 +1508,55 @@ picked:
 		gf_list_add(tr_state->visual->compositor->sensors, gf_list_get(tr_state->vrml_sensors, i));
 	}
 }
+
+#endif // GPAC_DISABLE_COMPOSITOR
+
+#else
+GF_TextSpan *gf_font_manager_create_span(GF_FontManager *fm, GF_Font *font, char *text, Fixed font_size, Bool needs_x_offset, Bool needs_y_offset, Bool needs_rotate, const char *xml_lang, Bool fliped_text, u32 styles, GF_Node *user)
+{
+	return NULL;
+}
+void gf_font_manager_delete_span(GF_FontManager *fm, GF_TextSpan *span)
+{
+}
+void gf_font_manager_refresh_span_bounds(GF_TextSpan *span)
+{
+}
+GF_Err gf_font_manager_register_font(GF_FontManager *fm, GF_Font *font)
+{
+	return GF_NOT_SUPPORTED;
+}
+GF_Err gf_font_manager_unregister_font(GF_FontManager *fm, GF_Font *font)
+{
+	return GF_NOT_SUPPORTED;
+}
+GF_Font *gf_font_manager_set_font(GF_FontManager *fm, char **alt_fonts, u32 nb_fonts, u32 styles)
+{
+	return NULL;
+}
+GF_Font *gf_font_manager_set_font_ex(GF_FontManager *fm, char **alt_fonts, u32 nb_fonts, u32 styles, Bool check_only)
+{
+	return NULL;
+}
+GF_Path *gf_font_span_create_path(GF_TextSpan *span)
+{
+	return NULL;
+}
+#ifndef GPAC_DISABLE_COMPOSITOR
+#include "visual_manager.h"
+
+void gf_font_spans_draw_2d(GF_List *spans, GF_TraverseState *tr_state, u32 hl_color, Bool force_texture_text, GF_Rect *bounds)
+{
+}
+void gf_font_spans_draw_3d(GF_List *spans, GF_TraverseState *tr_state, DrawAspect2D *asp, u32 text_hl, Bool force_texturing)
+{
+}
+void gf_font_spans_get_selection(GF_Node *node, GF_List *spans, GF_TraverseState *tr_state)
+{
+}
+void gf_font_spans_pick(GF_Node *node, GF_List *spans, GF_TraverseState *tr_state, GF_Rect *node_bounds, Bool use_dom_events, Drawable *drawable)
+{
+}
+#endif
+
+#endif // GPAC_DISABLE_FONTS
