@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Compositor sub-project
@@ -10,38 +11,32 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
 
 #include "nodes_stacks.h"
 
-typedef struct
-{
-	GF_AudioInput input;
-	GF_TimeNode time_handle;
-	Double start_time;
-	Bool set_duration, failure;
-} AudioClipStack;
+#if !defined(GPAC_DISABLE_VRML) && !defined(GPAC_DISABLE_COMPOSITOR)
 
 
 static void audioclip_activate(AudioClipStack *st, M_AudioClip *ac)
 {
-	if (gf_sc_audio_open(&st->input, &ac->url, 0, -1) != GF_OK) {
-		st->failure = 1;
+	if (gf_sc_audio_open(&st->input, &ac->url, 0, -1, GF_FALSE) != GF_OK) {
+		st->failure = GF_TRUE;
 		return;
 	}
 	ac->isActive = 1;
-	gf_node_event_out_str((GF_Node *)ac, "isActive");
+	gf_node_event_out((GF_Node *)ac, 7/*"isActive"*/);
 
 	gf_mo_set_speed(st->input.stream, st->input.speed);
 	/*traverse all graph to get parent audio group*/
@@ -51,10 +46,11 @@ static void audioclip_activate(AudioClipStack *st, M_AudioClip *ac)
 static void audioclip_deactivate(AudioClipStack *st, M_AudioClip *ac)
 {
 	gf_sc_audio_stop(&st->input);
+	gf_sc_audio_unregister(&st->input);
 	ac->isActive = 0;
-	gf_node_event_out_str((GF_Node *)ac, "isActive");
+	gf_node_event_out((GF_Node *)ac, 7/*"isActive"*/);
 
-	st->time_handle.needs_unregister = 1;
+	st->time_handle.needs_unregister = GF_TRUE;
 }
 
 static void audioclip_traverse(GF_Node *node, void *rs, Bool is_destroy)
@@ -64,12 +60,11 @@ static void audioclip_traverse(GF_Node *node, void *rs, Bool is_destroy)
 	AudioClipStack *st = (AudioClipStack *)gf_node_get_private(node);
 
 	if (is_destroy) {
-		gf_sc_audio_stop(&st->input);
-		gf_sc_audio_unregister(&st->input);
+		gf_sc_audio_predestroy(&st->input);
 		if (st->time_handle.is_registered) {
 			gf_sc_unregister_time_node(st->input.compositor, &st->time_handle);
 		}
-		free(st);
+		gf_free(st);
 		return;
 	}
 	if (st->failure) return;
@@ -78,7 +73,7 @@ static void audioclip_traverse(GF_Node *node, void *rs, Bool is_destroy)
 	if (st->input.stream && st->input.stream_finished) {
 		if (gf_mo_get_loop(st->input.stream, ac->loop)) {
 			gf_sc_audio_restart(&st->input);
-		} else if (ac->isActive && gf_mo_should_deactivate(st->input.stream)) {
+		} else if (ac->isActive && (ac->startTime<ac->stopTime) && gf_mo_should_deactivate(st->input.stream)) {
 			/*deactivate*/
 			audioclip_deactivate(st, ac);
 		}
@@ -86,10 +81,10 @@ static void audioclip_traverse(GF_Node *node, void *rs, Bool is_destroy)
 	if (ac->isActive) {
 		gf_sc_audio_register(&st->input, (GF_TraverseState*)rs);
 	}
-	if (st->set_duration && st->input.stream) {
+	if (st->set_duration && st->input.stream && st->input.stream->odm) {
 		ac->duration_changed = gf_mo_get_duration(st->input.stream);
-		gf_node_event_out_str(node, "duration_changed");
-		st->set_duration = 0;
+		gf_node_event_out(node, 6/*"duration_changed"*/);
+		st->set_duration = GF_FALSE;
 	}
 
 	/*store mute flag*/
@@ -100,16 +95,19 @@ static void audioclip_update_time(GF_TimeNode *tn)
 {
 	Double time;
 	M_AudioClip *ac = (M_AudioClip *)tn->udta;
-	AudioClipStack *st = (AudioClipStack *)gf_node_get_private(tn->udta);
+	AudioClipStack *st = (AudioClipStack *)gf_node_get_private((GF_Node*)tn->udta);
 
-	if (st->failure) return;
+	if (st->failure) {
+		st->time_handle.needs_unregister = GF_TRUE;
+		return;
+	}
 	if (! ac->isActive) {
 		st->start_time = ac->startTime;
 		st->input.speed = ac->pitch;
 	}
-	time = gf_node_get_scene_time(tn->udta);
+	time = gf_node_get_scene_time((GF_Node*)tn->udta);
 	if ((time<st->start_time) || (st->start_time<0)) return;
-	
+
 	if (ac->isActive) {
 		if ( (ac->stopTime > st->start_time) && (time>=ac->stopTime)) {
 			audioclip_deactivate(st, ac);
@@ -124,11 +122,15 @@ void compositor_init_audioclip(GF_Compositor *compositor, GF_Node *node)
 {
 	AudioClipStack *st;
 	GF_SAFEALLOC(st, AudioClipStack);
+	if (!st) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[Compositor] Failed to allocate style group stack\n"));
+		return;
+	}
 	gf_sc_audio_setup(&st->input, compositor, node);
 
 	st->time_handle.UpdateTimeNode = audioclip_update_time;
 	st->time_handle.udta = node;
-	st->set_duration = 1;
+	st->set_duration = GF_TRUE;
 
 	gf_node_set_private(node, st);
 	gf_node_set_callback_function(node, audioclip_traverse);
@@ -142,13 +144,13 @@ void compositor_audioclip_modified(GF_Node *node)
 	AudioClipStack *st = (AudioClipStack *) gf_node_get_private(node);
 	if (!st) return;
 
-	st->failure = 0;
+	st->failure = GF_FALSE;
 
 	/*MPEG4 spec is not clear about that , so this is not forbidden*/
-	if (st->input.is_open && st->input.is_open) {
+	if (st->input.is_open) {
 		if (gf_sc_audio_check_url(&st->input, &ac->url)) {
 			gf_sc_audio_stop(&st->input);
-			gf_sc_audio_open(&st->input, &ac->url, 0, -1);
+			gf_sc_audio_open(&st->input, &ac->url, 0, -1, GF_FALSE);
 			/*force unregister to resetup audio cfg*/
 			gf_sc_audio_unregister(&st->input);
 			gf_sc_invalidate(st->input.compositor, NULL);
@@ -163,26 +165,18 @@ void compositor_audioclip_modified(GF_Node *node)
 	}
 
 	/*make sure we are still registered*/
-	if (!st->time_handle.is_registered && !st->time_handle.needs_unregister) 
+	if (!st->time_handle.is_registered && !st->time_handle.needs_unregister)
 		gf_sc_register_time_node(st->input.compositor, &st->time_handle);
 	else
-		st->time_handle.needs_unregister = 0;
+		st->time_handle.needs_unregister = GF_FALSE;
 }
 
 
-typedef struct
-{
-	GF_AudioInput input;
-	GF_TimeNode time_handle;
-	Bool is_active;
-	Double start_time;
-} AudioSourceStack;
-
 static void audiosource_activate(AudioSourceStack *st, M_AudioSource *as)
 {
-	if (gf_sc_audio_open(&st->input, &as->url, 0, -1) != GF_OK)
+	if (gf_sc_audio_open(&st->input, &as->url, 0, -1, GF_FALSE) != GF_OK)
 		return;
-	st->is_active = 1;
+	st->is_active = GF_TRUE;
 	gf_mo_set_speed(st->input.stream, st->input.speed);
 	/*traverse all graph to get parent audio group*/
 	gf_sc_invalidate(st->input.compositor, NULL);
@@ -191,8 +185,8 @@ static void audiosource_activate(AudioSourceStack *st, M_AudioSource *as)
 static void audiosource_deactivate(AudioSourceStack *st, M_AudioSource *as)
 {
 	gf_sc_audio_stop(&st->input);
-	st->is_active = 0;
-	st->time_handle.needs_unregister = 1;
+	st->is_active = GF_FALSE;
+	st->time_handle.needs_unregister = GF_TRUE;
 }
 
 static void audiosource_traverse(GF_Node *node, void *rs, Bool is_destroy)
@@ -203,19 +197,18 @@ static void audiosource_traverse(GF_Node *node, void *rs, Bool is_destroy)
 
 
 	if (is_destroy) {
-		gf_sc_audio_stop(&st->input);
-		gf_sc_audio_unregister(&st->input);
+		gf_sc_audio_predestroy(&st->input);
 		if (st->time_handle.is_registered) {
 			gf_sc_unregister_time_node(st->input.compositor, &st->time_handle);
 		}
-		free(st);
+		gf_free(st);
 		return;
 	}
-	
+
 
 	/*check end of stream*/
 	if (st->input.stream && st->input.stream_finished) {
-		if (gf_mo_get_loop(st->input.stream, 0)) {
+		if (gf_mo_get_loop(st->input.stream, GF_FALSE)) {
 			gf_sc_audio_restart(&st->input);
 		} else if (st->is_active && gf_mo_should_deactivate(st->input.stream)) {
 			/*deactivate*/
@@ -234,15 +227,15 @@ static void audiosource_update_time(GF_TimeNode *tn)
 {
 	Double time;
 	M_AudioSource *as = (M_AudioSource *)tn->udta;
-	AudioSourceStack *st = (AudioSourceStack *)gf_node_get_private(tn->udta);
+	AudioSourceStack *st = (AudioSourceStack *)gf_node_get_private((GF_Node*)tn->udta);
 
 	if (! st->is_active) {
 		st->start_time = as->startTime;
 		st->input.speed = as->speed;
 	}
-	time = gf_node_get_scene_time(tn->udta);
+	time = gf_node_get_scene_time((GF_Node*)tn->udta);
 	if ((time<st->start_time) || (st->start_time<0)) return;
-	
+
 	if (st->input.input_ifce.GetSpeed(st->input.input_ifce.callback) && st->is_active) {
 		if ( (as->stopTime > st->start_time) && (time>=as->stopTime)) {
 			audiosource_deactivate(st, as);
@@ -257,6 +250,10 @@ void compositor_init_audiosource(GF_Compositor *compositor, GF_Node *node)
 {
 	AudioSourceStack *st;
 	GF_SAFEALLOC(st, AudioSourceStack);
+	if (!st) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[Compositor] Failed to allocate style group stack\n"));
+		return;
+	}
 	gf_sc_audio_setup(&st->input, compositor, node);
 
 	st->time_handle.UpdateTimeNode = audiosource_update_time;
@@ -275,14 +272,13 @@ void compositor_audiosource_modified(GF_Node *node)
 	if (!st) return;
 
 	/*MPEG4 spec is not clear about that , so this is not forbidden*/
-	if (st->input.is_open&& st->input.is_open) {
-		if (gf_sc_audio_check_url(&st->input, &as->url)) {
-			gf_sc_audio_stop(&st->input);
-			gf_sc_audio_open(&st->input, &as->url, 0, -1);
-			/*force unregister to resetup audio cfg*/
-			gf_sc_audio_unregister(&st->input);
-			gf_sc_invalidate(st->input.compositor, NULL);
-		}
+	if (gf_sc_audio_check_url(&st->input, &as->url)) {
+		if (st->input.is_open) gf_sc_audio_stop(&st->input);
+		/*force unregister to resetup audio cfg*/
+		gf_sc_audio_unregister(&st->input);
+		gf_sc_invalidate(st->input.compositor, NULL);
+
+		if (st->is_active) gf_sc_audio_open(&st->input, &as->url, 0, -1, GF_FALSE);
 	}
 
 	//update state if we're active
@@ -292,10 +288,10 @@ void compositor_audiosource_modified(GF_Node *node)
 	}
 
 	/*make sure we are still registered*/
-	if (!st->time_handle.is_registered && !st->time_handle.needs_unregister) 
+	if (!st->time_handle.is_registered && !st->time_handle.needs_unregister)
 		gf_sc_register_time_node(st->input.compositor, &st->time_handle);
 	else
-		st->time_handle.needs_unregister = 0;
+		st->time_handle.needs_unregister = GF_FALSE;
 }
 
 
@@ -337,13 +333,13 @@ static void audiobuffer_traverse(GF_Node *node, void *rs, Bool is_destroy)
 
 	if (is_destroy) {
 		gf_sc_audio_unregister(&st->output);
-		if (st->time_handle.is_registered) 
+		if (st->time_handle.is_registered)
 			gf_sc_unregister_time_node(st->output.compositor, &st->time_handle);
 
 		gf_mixer_del(st->am);
-		if (st->buffer) free(st->buffer);
+		if (st->buffer) gf_free(st->buffer);
 		gf_list_del(st->new_inputs);
-		free(st);
+		gf_free(st);
 		return;
 	}
 	parent = tr_state->audio_parent;
@@ -354,18 +350,18 @@ static void audiobuffer_traverse(GF_Node *node, void *rs, Bool is_destroy)
 		l = l->next;
 	}
 
-	gf_mixer_lock(st->am, 1);
+	gf_mixer_lock(st->am, GF_TRUE);
 
 	/*if no new inputs don't change mixer config*/
-	update_mixer = gf_list_count(st->new_inputs) ? 1 : 0;
-	
+	update_mixer = gf_list_count(st->new_inputs) ? GF_TRUE : GF_FALSE;
+
 	if (gf_mixer_get_src_count(st->am) == gf_list_count(st->new_inputs)) {
 		u32 count = gf_list_count(st->new_inputs);
-		update_mixer = 0;
+		update_mixer = GF_FALSE;
 		for (j=0; j<count; j++) {
 			GF_AudioInput *cur = (GF_AudioInput *)gf_list_get(st->new_inputs, j);
 			if (!gf_mixer_is_src_present(st->am, &cur->input_ifce)) {
-				update_mixer = 1;
+				update_mixer = GF_TRUE;
 				break;
 			}
 		}
@@ -373,7 +369,7 @@ static void audiobuffer_traverse(GF_Node *node, void *rs, Bool is_destroy)
 
 	if (update_mixer) {
 		gf_mixer_remove_all(st->am);
-		gf_mixer_force_chanel_out(st->am, ab->numChan);
+		gf_mixer_force_channel_out(st->am, ab->numChan);
 	}
 
 	while (gf_list_count(st->new_inputs)) {
@@ -382,11 +378,11 @@ static void audiobuffer_traverse(GF_Node *node, void *rs, Bool is_destroy)
 		if (update_mixer) gf_mixer_add_input(st->am, &src->input_ifce);
 	}
 
-	gf_mixer_lock(st->am, 0);
+	gf_mixer_lock(st->am, GF_FALSE);
 	tr_state->audio_parent = parent;
 
-	/*Note the audio buffer is ALWAYS registered untill destroyed since buffer filling shall happen even when inactive*/
-	if (!st->output.register_with_parent || !st->output.register_with_renderer) 
+	/*Note the audio buffer is ALWAYS registered until destroyed since buffer filling shall happen even when inactive*/
+	if (!st->output.register_with_parent || !st->output.register_with_renderer)
 		gf_sc_audio_register(&st->output, tr_state);
 
 	/*store mute flag*/
@@ -398,32 +394,32 @@ static void audiobuffer_traverse(GF_Node *node, void *rs, Bool is_destroy)
 static void audiobuffer_activate(AudioBufferStack *st, M_AudioBuffer *ab)
 {
 	ab->isActive = 1;
-	gf_node_event_out_str((GF_Node *)ab, "isActive");
+	gf_node_event_out((GF_Node *)ab, 17/*"isActive"*/);
 	/*rerender all graph to get parent audio group*/
 	gf_sc_invalidate(st->output.compositor, NULL);
-	st->done = 0;
+	st->done = GF_FALSE;
 	st->read_pos = 0;
 }
 
 static void audiobuffer_deactivate(AudioBufferStack *st, M_AudioBuffer *ab)
 {
 	ab->isActive = 0;
-	gf_node_event_out_str((GF_Node *)ab, "isActive");
-	st->time_handle.needs_unregister = 1;
+	gf_node_event_out((GF_Node *)ab, 17/*"isActive"*/);
+	st->time_handle.needs_unregister = GF_TRUE;
 }
 
 static void audiobuffer_update_time(GF_TimeNode *tn)
 {
 	Double time;
 	M_AudioBuffer *ab = (M_AudioBuffer *)tn->udta;
-	AudioBufferStack *st = (AudioBufferStack *)gf_node_get_private(tn->udta);
+	AudioBufferStack *st = (AudioBufferStack *)gf_node_get_private((GF_Node*)tn->udta);
 
 	if (! ab->isActive) {
 		st->start_time = ab->startTime;
 	}
-	time = gf_node_get_scene_time(tn->udta);
+	time = gf_node_get_scene_time((GF_Node*)tn->udta);
 	if ((time<st->start_time) || (st->start_time<0)) return;
-	
+
 	if (ab->isActive) {
 		if ( (ab->stopTime > st->start_time) && (time>=ab->stopTime)) {
 			audiobuffer_deactivate(st, ab);
@@ -441,7 +437,7 @@ static void audiobuffer_update_time(GF_TimeNode *tn)
 
 
 
-static char *audiobuffer_fetch_frame(void *callback, u32 *size, u32 audio_delay_ms)
+static u8 *audiobuffer_fetch_frame(void *callback, u32 *size, u32 *planar_stride, u32 audio_delay_ms)
 {
 	u32 blockAlign;
 	AudioBufferStack *st = (AudioBufferStack *) gf_node_get_private( ((GF_AudioInput *) callback)->owner);
@@ -449,12 +445,12 @@ static char *audiobuffer_fetch_frame(void *callback, u32 *size, u32 audio_delay_
 
 	if (!st->is_init) return NULL;
 	if (!st->buffer) {
-		st->done = 0;
-		st->buffer_size = (u32) ceil(FIX2FLT(ab->length) * st->output.input_ifce.bps*st->output.input_ifce.samplerate*st->output.input_ifce.chan/8);
+		st->done = GF_FALSE;
+		st->buffer_size = (u32) ceil(FIX2FLT(ab->length) * gf_audio_fmt_bit_depth(st->output.input_ifce.afmt) * st->output.input_ifce.samplerate*st->output.input_ifce.chan/8);
 		blockAlign = gf_mixer_get_block_align(st->am);
 		/*BLOCK ALIGN*/
 		while (st->buffer_size%blockAlign) st->buffer_size++;
-		st->buffer = (char*)malloc(sizeof(char) * st->buffer_size);
+		st->buffer = (char*)gf_malloc(sizeof(char) * st->buffer_size);
 		memset(st->buffer, 0, sizeof(char) * st->buffer_size);
 		st->read_pos = st->write_pos = 0;
 	}
@@ -465,10 +461,10 @@ static char *audiobuffer_fetch_frame(void *callback, u32 *size, u32 audio_delay_
 		u32 written;
 		while (1) {
 			/*just try to completely fill it*/
-			written = gf_mixer_get_output(st->am, st->buffer + st->write_pos, st->buffer_size - st->write_pos);
+			written = gf_mixer_get_output(st->am, st->buffer + st->write_pos, st->buffer_size - st->write_pos, 0);
 			if (!written) break;
 			st->write_pos += written;
-			assert(st->write_pos<=st->buffer_size);
+			gf_fatal_assert(st->write_pos<=st->buffer_size);
 		}
 	}
 	/*not playing*/
@@ -481,7 +477,7 @@ static void audiobuffer_release_frame(void *callback, u32 nb_bytes)
 {
 	AudioBufferStack *st = (AudioBufferStack *) gf_node_get_private( ((GF_AudioInput *) callback)->owner);
 	st->read_pos += nb_bytes;
-	assert(st->read_pos<=st->write_pos);
+	gf_fatal_assert(st->read_pos<=st->write_pos);
 	if (st->read_pos==st->write_pos) {
 		if (st->write_pos<st->buffer_size) {
 			/*reading faster than buffering - let's still attempt to fill the buffer*/
@@ -492,7 +488,7 @@ static void audiobuffer_release_frame(void *callback, u32 nb_bytes)
 		} else if ( ((M_AudioBuffer*)st->output.owner)->loop) {
 			st->read_pos = 0;
 		} else {
-			st->done = 1;
+			st->done = GF_TRUE;
 		}
 	}
 }
@@ -510,8 +506,9 @@ static Bool audiobuffer_get_volume(void *callback, Fixed *vol)
 	if (ai->snd->GetChannelVolume) {
 		return ai->snd->GetChannelVolume(ai->snd->owner, vol);
 	} else {
-		vol[0] = vol[1] = vol[2] = vol[3] = vol[4] = vol[5] = FIX_ONE;
-		return 0;
+//		u32 i;
+//		for (i=0; i<GF_AUDIO_MIXER_MAX_CHANNELS; i++) vol[i] = FIX_ONE;
+		return GF_FALSE;
 	}
 }
 
@@ -526,18 +523,52 @@ static Bool audiobuffer_get_config(GF_AudioInterface *aifc, Bool for_reconf)
 	AudioBufferStack *st = (AudioBufferStack *) gf_node_get_private( ((GF_AudioInput *) aifc->callback)->owner);
 
 	if (gf_mixer_must_reconfig(st->am)) {
+		Bool force_config = GF_TRUE;
 		if (gf_mixer_reconfig(st->am)) {
-			if (st->buffer) free(st->buffer);
+			if (st->buffer) gf_free(st->buffer);
 			st->buffer = NULL;
 			st->buffer_size = 0;
 		}
 
-		gf_mixer_get_config(st->am, &aifc->samplerate, &aifc->chan, &aifc->bps, &aifc->ch_cfg);
-		st->is_init = (aifc->samplerate && aifc->chan && aifc->bps) ? 1 : 0;
-		assert(st->is_init);
-		if (!st->is_init) aifc->samplerate = aifc->chan = aifc->bps = aifc->ch_cfg = 0;
+		gf_mixer_get_config(st->am, &aifc->samplerate, &aifc->chan, &aifc->afmt, &aifc->ch_layout);
+		//we only work with packed formats
+		switch (aifc->afmt) {
+		case GF_AUDIO_FMT_U8P:
+			aifc->afmt = GF_AUDIO_FMT_U8;
+			break;
+		case GF_AUDIO_FMT_S16P:
+		case GF_AUDIO_FMT_S16_BE:
+			aifc->afmt = GF_AUDIO_FMT_S16;
+			break;
+		case GF_AUDIO_FMT_S24P:
+		case GF_AUDIO_FMT_S24_BE:
+			aifc->afmt = GF_AUDIO_FMT_S24;
+			break;
+		case GF_AUDIO_FMT_S32P:
+		case GF_AUDIO_FMT_S32_BE:
+			aifc->afmt = GF_AUDIO_FMT_S32;
+			break;
+		case GF_AUDIO_FMT_FLTP:
+			aifc->afmt = GF_AUDIO_FMT_FLT;
+			break;
+		case GF_AUDIO_FMT_DBLP:
+			aifc->afmt = GF_AUDIO_FMT_DBL;
+			break;
+		default:
+			force_config = GF_FALSE;
+			break;
+		}
+		if (force_config) {
+			gf_mixer_set_config(st->am, aifc->samplerate, aifc->chan, aifc->afmt, aifc->ch_layout);
+		}
+		st->is_init = (aifc->samplerate && aifc->chan && aifc->afmt) ? GF_TRUE : GF_FALSE;
+		gf_assert(st->is_init);
+		if (!st->is_init) {
+			aifc->samplerate = aifc->chan = aifc->afmt = 0;
+			aifc->ch_layout = 0;
+		}
 		/*this will force invalidation*/
-		return (for_reconf && st->is_init) ? 1 : 0;
+		return (for_reconf && st->is_init) ? GF_TRUE : GF_FALSE;
 	}
 	return st->is_init;
 }
@@ -551,7 +582,7 @@ void audiobuffer_add_source(GF_AudioGroup *_this, GF_AudioInput *src)
 }
 
 
-void setup_audiobufer(GF_AudioInput *ai, GF_Compositor *compositor, GF_Node *node)
+void setup_audiobuffer(GF_AudioInput *ai, GF_Compositor *compositor, GF_Node *node)
 {
 	memset(ai, 0, sizeof(GF_AudioInput));
 	ai->owner = node;
@@ -573,14 +604,18 @@ void compositor_init_audiobuffer(GF_Compositor *compositor, GF_Node *node)
 {
 	AudioBufferStack *st;
 	GF_SAFEALLOC(st, AudioBufferStack);
+	if (!st) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[Compositor] Failed to allocate audiobuffer stack\n"));
+		return;
+	}
 
 	/*use our private input*/
-	setup_audiobufer(&st->output, compositor, node);
+	setup_audiobuffer(&st->output, compositor, node);
 	st->add_source = audiobuffer_add_source;
 
 	st->time_handle.UpdateTimeNode = audiobuffer_update_time;
 	st->time_handle.udta = node;
-	st->set_duration = 1;
+	st->set_duration = GF_TRUE;
 
 	st->am = gf_mixer_new(NULL);
 	st->new_inputs = gf_list_new();
@@ -598,13 +633,15 @@ void compositor_audiobuffer_modified(GF_Node *node)
 	if (!st) return;
 
 	//update state if we're active
-	if (ab->isActive) 
+	if (ab->isActive)
 		audiobuffer_update_time(&st->time_handle);
 
 	/*make sure we are still registered*/
-	if (!st->time_handle.is_registered && !st->time_handle.needs_unregister) 
+	if (!st->time_handle.is_registered && !st->time_handle.needs_unregister)
 		gf_sc_register_time_node(st->output.compositor, &st->time_handle);
 	else
-		st->time_handle.needs_unregister = 0;
+		st->time_handle.needs_unregister = GF_FALSE;
 }
 
+#endif //!defined(GPAC_DISABLE_VRML) && !defined(GPAC_DISABLE_COMPOSITOR)
+
