@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Compositor sub-project
@@ -10,15 +11,15 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
@@ -27,6 +28,7 @@
 #include "nodes_stacks.h"
 #include "visual_manager.h"
 
+#if !defined(GPAC_DISABLE_COMPOSITOR)
 Bool compositor_get_2d_plane_intersection(GF_Ray *ray, SFVec3f *res)
 {
 	GF_Plane p;
@@ -34,9 +36,11 @@ Bool compositor_get_2d_plane_intersection(GF_Ray *ray, SFVec3f *res)
 	if (!ray->dir.x && !ray->dir.y) {
 		res->x = ray->orig.x;
 		res->y = ray->orig.y;
+		res->z = 0;
 		return 1;
 	}
-	p.normal.x = p.normal.y = 0; p.normal.z = FIX_ONE;
+	p.normal.x = p.normal.y = 0;
+	p.normal.z = FIX_ONE;
 	p.d = 0;
 	t2 = gf_vec_dot(p.normal, ray->dir);
 	if (t2 == 0) return 0;
@@ -47,8 +51,10 @@ Bool compositor_get_2d_plane_intersection(GF_Ray *ray, SFVec3f *res)
 	return 1;
 }
 
+#ifndef GPAC_DISABLE_VRML
+
 /*
-		Shape 
+		Shape
 */
 static void TraverseShape(GF_Node *node, void *rs, Bool is_destroy)
 {
@@ -67,18 +73,17 @@ static void TraverseShape(GF_Node *node, void *rs, Bool is_destroy)
 	/*reset this node dirty flag (because bitmap may trigger bounds invalidation on the fly)*/
 	gf_node_dirty_clear(node, 0);
 
-	
+
 	/*check traverse mode, and take care of switch-off flag*/
 	if (tr_state->traversing_mode==TRAVERSE_GET_BOUNDS) {
-		GF_Node *m;
 		tr_state->appear = (GF_Node *) shape->appearance;
-		
+
 		/*this is done regardless of switch flag*/
 		gf_node_traverse((GF_Node *) shape->geometry, tr_state);
 
 		if (tr_state->appear) {
 			/*apply line width*/
-			m = ((M_Appearance *)tr_state->appear)->material;
+			GF_Node *m = ((M_Appearance *)tr_state->appear)->material;
 			if (m && (gf_node_get_tag(m)==TAG_MPEG4_Material2D) ) {
 				DrawAspect2D asp;
 				Fixed width = 0;
@@ -125,11 +130,15 @@ static void TraverseShape(GF_Node *node, void *rs, Bool is_destroy)
 			break;
 #ifndef GPAC_DISABLE_3D
 		/*if we're here we passed culler already*/
-		case TRAVERSE_DRAW_3D: 
+		case TRAVERSE_DRAW_3D:
+			if (!tr_state->visual->type_3d && tr_state->visual->compositor->hybrid_opengl) {
+				tr_state->visual->compositor->root_visual_setup=0;
+				tr_state->visual->compositor->force_type_3d=1;
+			}
 			gf_node_traverse((GF_Node *) shape->geometry, tr_state);
 			break;
 		case TRAVERSE_COLLIDE:
-			visual_3d_drawable_collide(shape->geometry, tr_state);
+			visual_3d_vrml_drawable_collide(shape->geometry, tr_state);
 			break;
 #endif
 		}
@@ -172,7 +181,7 @@ static void TraverseCircle(GF_Node *node, void *rs, Bool is_destroy)
 		if (!stack->mesh) {
 			Fixed a = ((M_Circle *) node)->radius * 2;
 			stack->mesh = new_mesh();
-			mesh_new_ellipse(stack->mesh, a, a, tr_state->visual->compositor->high_speed);
+			mesh_new_ellipse(stack->mesh, a, a, tr_state->visual->compositor->fast);
 		}
 		visual_3d_draw_2d(stack, tr_state);
 		return;
@@ -181,7 +190,7 @@ static void TraverseCircle(GF_Node *node, void *rs, Bool is_destroy)
 		gf_path_get_bounds(stack->path, &tr_state->bounds);
 		return;
 	case TRAVERSE_PICK:
-		drawable_pick(stack, tr_state);
+		vrml_drawable_pick(stack, tr_state);
 		return;
 	case TRAVERSE_SORT:
 #ifndef GPAC_DISABLE_3D
@@ -227,13 +236,13 @@ static void TraverseEllipse(GF_Node *node, void *rs, Bool is_destroy)
 	case TRAVERSE_DRAW_3D:
 		if (!stack->mesh) {
 			stack->mesh = new_mesh();
-			mesh_new_ellipse(stack->mesh, ((M_Ellipse *) node)->radius.x * 2, ((M_Ellipse *) node)->radius.y * 2, tr_state->visual->compositor->high_speed);
+			mesh_new_ellipse(stack->mesh, ((M_Ellipse *) node)->radius.x * 2, ((M_Ellipse *) node)->radius.y * 2, tr_state->visual->compositor->fast);
 		}
 		visual_3d_draw_2d(stack, tr_state);
 		return;
 #endif
 	case TRAVERSE_PICK:
-		drawable_pick(stack, tr_state);
+		vrml_drawable_pick(stack, tr_state);
 		return;
 	case TRAVERSE_GET_BOUNDS:
 		gf_path_get_bounds(stack->path, &tr_state->bounds);
@@ -259,11 +268,15 @@ static void compositor_2d_draw_rectangle(GF_TraverseState *tr_state)
 {
 	DrawableContext *ctx = tr_state->ctx;
 
-	if (ctx->aspect.fill_texture) {
+	if (ctx->aspect.fill_texture && ctx->aspect.fill_texture->data
+#ifndef GPAC_DISABLE_3D
+	        && !tr_state->visual->compositor->hybrid_opengl
+#endif
+	   ) {
 		Bool res;
 
-		/*get image size WITHOUT line size*/
-		if (ctx->aspect.pen_props.width) {
+		/*get image size WITHOUT line size or antialias margin*/
+		if ( !(ctx->flags & CTX_NO_ANTIALIAS) ) {
 			GF_Rect orig_unclip;
 			GF_IRect orig_clip;
 			orig_unclip = ctx->bi->unclip;
@@ -274,7 +287,7 @@ static void compositor_2d_draw_rectangle(GF_TraverseState *tr_state)
 			ctx->bi->clip = gf_rect_pixelize(&ctx->bi->unclip);
 			gf_irect_intersect(&ctx->bi->clip, &orig_clip);
 
-			res = tr_state->visual->DrawBitmap(tr_state->visual, tr_state, ctx, NULL);
+			res = tr_state->visual->DrawBitmap(tr_state->visual, tr_state, ctx);
 
 			/*strike path*/
 			ctx->bi->unclip = orig_unclip;
@@ -284,11 +297,11 @@ static void compositor_2d_draw_rectangle(GF_TraverseState *tr_state)
 				visual_2d_draw_path(tr_state->visual, ctx->drawable->path, ctx, NULL, NULL, tr_state);
 			}
 		} else {
-			res = tr_state->visual->DrawBitmap(tr_state->visual, tr_state, ctx, NULL);
+			res = tr_state->visual->DrawBitmap(tr_state->visual, tr_state, ctx);
 		}
 		/*if failure retry with raster*/
 		if (res) return;
-	} 
+	}
 
 	visual_2d_texture_path(tr_state->visual, ctx->drawable->path, ctx, tr_state);
 	visual_2d_draw_path(tr_state->visual, ctx->drawable->path, ctx, NULL, NULL, tr_state);
@@ -305,6 +318,87 @@ static void rectangle_check_changes(GF_Node *node, Drawable *stack, GF_TraverseS
 		drawable_mark_modified(stack, tr_state);
 	}
 }
+
+Bool rectangle_check_adaptation(GF_Node *node, Drawable *stack, GF_TraverseState *tr_state)
+{
+	GF_TextureHandler *txh;
+	GF_MediaObjectVRInfo vrinfo;
+	s32 gaze_x, gaze_y;
+	Bool is_visible = GF_FALSE;
+	if (! tr_state->visual->compositor->gazer_enabled)
+		return GF_TRUE;
+
+	if (!tr_state->appear || ! ((M_Appearance *)tr_state->appear)->texture)
+		return GF_TRUE;
+	
+	txh = gf_sc_texture_get_handler( ((M_Appearance *) tr_state->appear)->texture );
+	if (!txh->stream) return GF_TRUE;
+
+	if (! gf_mo_get_srd_info(txh->stream, &vrinfo))
+		return GF_TRUE;
+
+	if (!vrinfo.srd_w && !vrinfo.srd_h && vrinfo.is_tiled_srd) {
+		if (txh->stream->srd_full_w && txh->stream->srd_full_h) {
+			gaze_x = tr_state->visual->compositor->gaze_x;
+			gaze_x *= txh->stream->srd_full_w;
+			gaze_x /= tr_state->visual->width;
+			gaze_x += txh->stream->srd_map_ox;
+
+			gaze_y = tr_state->visual->compositor->gaze_y;
+			gaze_y *= txh->stream->srd_full_h;
+			gaze_y /= tr_state->visual->height;
+			gaze_y += txh->stream->srd_map_oy;
+
+			gf_mo_hint_gaze(txh->stream, gaze_x, gaze_y);
+		}
+
+		return GF_TRUE;
+	}
+
+	gaze_x = tr_state->visual->compositor->gaze_x;
+	gaze_x *= vrinfo.srd_max_x;
+	gaze_x /= tr_state->visual->width;
+
+	gaze_y = tr_state->visual->compositor->gaze_y;
+	gaze_y *= vrinfo.srd_max_y;
+	gaze_y /= tr_state->visual->height;
+
+	//simple test condition: only keep the first row
+	if ((gaze_x>=vrinfo.srd_x) && (gaze_x<=vrinfo.srd_x+vrinfo.srd_w) && (gaze_y>=vrinfo.srd_y) && (gaze_y<=vrinfo.srd_y+vrinfo.srd_h)) {
+
+		GF_LOG(GF_LOG_INFO, GF_LOG_COMPOSE, ("[Compositor] Texture %d Partial plane is under gaze coord %d %d\n", txh->stream->OD_ID, tr_state->visual->compositor->gaze_x, tr_state->visual->compositor->gaze_y));
+		is_visible = GF_TRUE;
+	}
+
+	if (vrinfo.has_full_coverage) {
+		if (is_visible) {
+			if (!txh->is_open) {
+				GF_LOG(GF_LOG_INFO, GF_LOG_COMPOSE, ("[Compositor] Texture %d stopped on visible partial plane - starting it\n", txh->stream->OD_ID));
+				if (txh->stream->odm)
+					txh->stream->odm->disable_buffer_at_next_play = GF_TRUE;
+
+				gf_sc_texture_play(txh, NULL);
+			}
+			if (! txh->data)  return GF_FALSE;
+			return GF_TRUE;
+		} else {
+			if (txh->is_open) {
+				GF_LOG(GF_LOG_INFO, GF_LOG_COMPOSE, ("[Compositor] Texure %d playing on hidden partial plane - stopping it\n", txh->stream->OD_ID));
+				gf_sc_texture_stop_no_unregister(txh);
+			}
+			return GF_FALSE;
+		}
+	} else {
+		if (is_visible) {
+			gf_mo_hint_quality_degradation(txh->stream, 0);
+			if (! txh->data)  return GF_FALSE;
+		} else {
+			gf_mo_hint_quality_degradation(txh->stream, 100);
+		}
+	}
+	return GF_TRUE;
+}
+
 static void TraverseRectangle(GF_Node *node, void *rs, Bool is_destroy)
 {
 	DrawableContext *ctx;
@@ -318,6 +412,9 @@ static void TraverseRectangle(GF_Node *node, void *rs, Bool is_destroy)
 
 	rectangle_check_changes(node, stack, tr_state);
 
+	if (! rectangle_check_adaptation(node, stack, tr_state))
+		return;
+
 	switch (tr_state->traversing_mode) {
 	case TRAVERSE_DRAW_2D:
 		compositor_2d_draw_rectangle(tr_state);
@@ -326,13 +423,13 @@ static void TraverseRectangle(GF_Node *node, void *rs, Bool is_destroy)
 	case TRAVERSE_DRAW_3D:
 		if (!stack->mesh) {
 			stack->mesh = new_mesh();
-			mesh_new_rectangle(stack->mesh, ((M_Rectangle *) node)->size);
+			mesh_new_rectangle(stack->mesh, ((M_Rectangle *) node)->size, NULL, 0);
 		}
 		visual_3d_draw_2d(stack, tr_state);
 		return;
 #endif
 	case TRAVERSE_PICK:
-		drawable_pick(stack, tr_state);
+		vrml_drawable_pick(stack, tr_state);
 		return;
 	case TRAVERSE_GET_BOUNDS:
 		gf_path_get_bounds(stack->path, &tr_state->bounds);
@@ -349,21 +446,26 @@ static void TraverseRectangle(GF_Node *node, void *rs, Bool is_destroy)
 	ctx = drawable_init_context_mpeg4(stack, tr_state);
 	if (!ctx) return;
 
-	/*if alpha or not filled, transparent*/
-	if (GF_COL_A(ctx->aspect.fill_color) != 0xFF) {
-	} 
-	/*if texture transparent, transparent*/
-	else if (ctx->aspect.fill_texture && ctx->aspect.fill_texture->transparent) {
-	} 
-	/*if rotated, transparent (doesn't fill bounds)*/
-	else if (ctx->transform.m[1] || ctx->transform.m[3]) {
+	/*if rotated, object is transparent (doesn't fill bounds) and antialias must be used*/
+	if (tr_state->transform.m[1] || tr_state->transform.m[3]) {
 	}
-	/*TODO check matrix for alpha*/
-	else if (!tr_state->color_mat.identity) {
-	}
-	/*otherwsie, not transparent*/
 	else {
-		ctx->flags &= ~CTX_IS_TRANSPARENT;	
+
+		/*if alpha or not filled, transparent*/
+		if (ctx->aspect.fill_color && (GF_COL_A(ctx->aspect.fill_color) != 0xFF)) {
+		}
+		/*if texture transparent, transparent*/
+		else if (ctx->aspect.fill_texture && ctx->aspect.fill_texture->transparent) {
+		}
+		/*TODO check matrix for alpha*/
+		else if (!tr_state->color_mat.identity) {
+		}
+		/*otherwise, not transparent*/
+		else {
+			ctx->flags &= ~CTX_IS_TRANSPARENT;
+		}
+		/*if no line width, we skip antialiasing*/
+		if (!ctx->aspect.pen_props.width) ctx->flags |= CTX_NO_ANTIALIAS;
 	}
 	drawable_finalize_sort(ctx, tr_state, NULL);
 }
@@ -378,7 +480,7 @@ void compositor_init_rectangle(GF_Compositor  *compositor, GF_Node *node)
 
 #define CHECK_VALID_C2D(nbPts) if (!idx && cur_index+nbPts>=pt_count) { gf_path_reset(stack->path); return; }
 //#define CHECK_VALID_C2D(nbPts)
-#define GET_IDX(_i)	((idx && idx->count>_i) ? idx->vals[_i] : _i)
+#define GET_IDX(_i)	((idx && (idx->count>_i) && (idx->vals[_i]>=0) ) ? (u32) idx->vals[_i] : _i)
 
 void curve2d_check_changes(GF_Node *node, Drawable *stack, GF_TraverseState *tr_state, MFInt32 *idx)
 {
@@ -394,11 +496,11 @@ void curve2d_check_changes(GF_Node *node, Drawable *stack, GF_TraverseState *tr_
 	if (!coord) return;
 
 	stack->path->fineness = c2D->fineness;
-	if (tr_state->visual->compositor->high_speed)  stack->path->fineness /= 2;
+	if (tr_state->visual->compositor->fast)  stack->path->fineness /= 2;
 
 
 	pts = coord->point.vals;
-	if (!pts) 
+	if (!pts)
 		return;
 
 	cur_index = c2D->type.count ? 1 : 0;
@@ -454,7 +556,7 @@ void curve2d_check_changes(GF_Node *node, Drawable *stack, GF_TraverseState *tr_
 			ct_orig = ct_end;
 			orig = end;
 			break;
-		
+
 		/*all XCurve2D specific*/
 
 		/*CW and CCW ArcTo*/
@@ -496,7 +598,7 @@ void curve2d_check_changes(GF_Node *node, Drawable *stack, GF_TraverseState *tr_
 		if (remain>1)
 			gf_path_add_bezier(stack->path, &pts[cur_index], remain);
 	}
-	
+
 	gf_node_dirty_clear(node, 0);
 	drawable_mark_modified(stack, tr_state);
 }
@@ -526,7 +628,7 @@ static void TraverseCurve2D(GF_Node *node, void *rs, Bool is_destroy)
 		return;
 #endif
 	case TRAVERSE_PICK:
-		drawable_pick(stack, tr_state);
+		vrml_drawable_pick(stack, tr_state);
 		return;
 	case TRAVERSE_GET_BOUNDS:
 		gf_path_get_bounds(stack->path, &tr_state->bounds);
@@ -551,7 +653,7 @@ void compositor_init_curve2d(GF_Compositor  *compositor, GF_Node *node)
 
 
 /*
-	Note on point set 2D: this is a very bad node and should be avoided in DEF/USE, since the size 
+	Note on point set 2D: this is a very bad node and should be avoided in DEF/USE, since the size
 	of the rectangle representing the pixel shall always be 1 pixel w/h, therefore
 	the path object is likely not the same depending on transformation context...
 
@@ -578,11 +680,11 @@ static void pointset2d_check_changes(GF_Node *node, Drawable *stack, GF_Traverse
 
 	get_point_size(&tr_state->transform, &w, &h);
 	/*for PS2D don't add to avoid too  much antialiasing, just try to fill the given pixel*/
-	for (i=0; i < coord->point.count; i++) 
+	for (i=0; i < coord->point.count; i++)
 		gf_path_add_rect(stack->path, coord->point.vals[i].x, coord->point.vals[i].y, w, h);
 
 	stack->path->flags |= GF_PATH_FILL_ZERO_NONZERO;
-	
+
 	gf_node_dirty_clear(node, 0);
 	drawable_mark_modified(stack, tr_state);
 }
@@ -627,7 +729,7 @@ static void TraversePointSet2D(GF_Node *node, void *rs, Bool is_destroy)
 	M_PointSet2D *ps2D = (M_PointSet2D *)node;
 	Drawable *stack = (Drawable *)gf_node_get_private(node);
 	GF_TraverseState *tr_state = (GF_TraverseState *)rs;
-	
+
 	if (is_destroy) {
 		drawable_node_del(node);
 		return;
@@ -654,7 +756,7 @@ static void TraversePointSet2D(GF_Node *node, void *rs, Bool is_destroy)
 		visual_3d_set_material_2d_argb(tr_state->visual, asp.fill_color);
 		visual_3d_mesh_paint(tr_state, stack->mesh);
 		return;
-	} 
+	}
 #endif
 	case TRAVERSE_GET_BOUNDS:
 		gf_path_get_bounds(stack->path, &tr_state->bounds);
@@ -681,3 +783,41 @@ void compositor_init_pointset2d(GF_Compositor  *compositor, GF_Node *node)
 	stack->flags = DRAWABLE_USE_TRAVERSE_DRAW;
 	gf_node_set_callback_function(node, TraversePointSet2D);
 }
+
+static void TraverseBitWrapper(GF_Node *node, void *rs, Bool is_destroy)
+{
+	GF_TraverseState *tr_state;
+	M_BitWrapper *bitWrap;
+
+	if (is_destroy) {
+		gf_node_set_private(node, NULL);
+		return;
+	}
+#ifdef GPAC_ENABLE_COVERAGE
+	if (!rs) return;
+#endif
+
+	tr_state = (GF_TraverseState *)rs;
+	// Traverse the node here
+	bitWrap = (M_BitWrapper *)node;
+	gf_node_traverse(bitWrap->node, tr_state);
+}
+
+void compositor_init_bitwrapper(GF_Compositor *compositor, GF_Node *node)
+{
+	M_BitWrapper *bit;
+	bit = (M_BitWrapper *)node;
+	if (!bit->node) return;
+	gf_node_set_private(node, gf_node_get_private(bit->node));
+	gf_node_set_callback_function(node, TraverseBitWrapper);
+
+#ifdef GPAC_ENABLE_COVERAGE
+	if (gf_sys_is_cov_mode()) {
+		TraverseBitWrapper(node, NULL, GF_FALSE);
+	}
+#endif
+}
+
+
+#endif /*GPAC_DISABLE_VRML*/
+#endif //!defined(GPAC_DISABLE_COMPOSITOR)
