@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Compositor sub-project
@@ -10,32 +11,42 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
 #include "nodes_stacks.h"
 #include "texturing.h"
 
+#if !defined(GPAC_DISABLE_COMPOSITOR)
+
+
+#ifndef GPAC_DISABLE_VRML
+
 
 #define GRAD_TEXTURE_SIZE	128
 #define GRAD_TEXTURE_HSIZE	64
 
+enum
+{
+	GF_SR_TEXTURE_GRAD_REGISTERED = 1<<6,
+	GF_SR_TEXTURE_GRAD_NO_RGB = 1<<7,
+};
+
 /*linear/radial gradient*/
 typedef struct
 {
 	GF_TextureHandler txh;
 	char *tx_data;
-	Bool transparent;
-	Bool no_rgb_support;
+//	Bool no_rgb_support;
 } GradientStack;
 
 void GradientGetMatrix(GF_Node *transform, GF_Matrix2D *mat)
@@ -50,15 +61,19 @@ void GradientGetMatrix(GF_Node *transform, GF_Matrix2D *mat)
 			gf_mx2d_add_rotation(mat, tr->center.x, tr->center.y, tr->rotationAngle);
 			gf_mx2d_add_translation(mat, tr->translation.x, tr->translation.y);
 		}
-			break;
+		break;
 		case TAG_MPEG4_TransformMatrix2D:
 		{
 			M_TransformMatrix2D *tm = (M_TransformMatrix2D*)transform;
 			gf_mx2d_init(*mat);
-			mat->m[0] = tm->mxx; mat->m[1] = tm->mxy; mat->m[2] = tm->tx;
-			mat->m[3] = tm->myx; mat->m[4] = tm->myy; mat->m[5] = tm->ty;
+			mat->m[0] = tm->mxx;
+			mat->m[1] = tm->mxy;
+			mat->m[2] = tm->tx;
+			mat->m[3] = tm->myx;
+			mat->m[4] = tm->myy;
+			mat->m[5] = tm->ty;
 		}
-			break;
+		break;
 		default:
 			break;
 		}
@@ -70,8 +85,8 @@ static void DestroyGradient(GF_Node *node, void *rs, Bool is_destroy)
 	if (is_destroy) {
 		GradientStack *st = (GradientStack *) gf_node_get_private(node);
 		gf_sc_texture_destroy(&st->txh);
-		if (st->tx_data) free(st->tx_data);
-		free(st);
+		if (st->tx_data) gf_free(st->tx_data);
+		gf_free(st);
 	}
 }
 
@@ -80,7 +95,7 @@ static void UpdateLinearGradient(GF_TextureHandler *txh)
 	u32 i, *cols;
 	Fixed a;
 	Bool const_a;
-	GF_STENCIL stencil;
+	GF_EVGStencil *stencil;
 	M_LinearGradient *lg = (M_LinearGradient *) txh->owner;
 	GradientStack *st = (GradientStack *) gf_node_get_private(txh->owner);
 
@@ -90,10 +105,14 @@ static void UpdateLinearGradient(GF_TextureHandler *txh)
 	}
 	if (lg->key.count > lg->keyValue.count) return;
 
-	if (!txh->tx_io) gf_sc_texture_allocate(txh);
+	if (!txh->tx_io) {
+		gf_node_dirty_set(gf_node_get_parent(txh->owner, 0), 0, 1);
+		gf_node_dirty_set(txh->owner, 0, 1);
+		gf_sc_texture_allocate(txh);
+	}
 
 	stencil = gf_sc_texture_get_stencil(txh);
-	if (!stencil) stencil = txh->compositor->rasterizer->stencil_new(txh->compositor->rasterizer, GF_STENCIL_LINEAR_GRADIENT);
+	if (!stencil) stencil = gf_evg_stencil_new(GF_STENCIL_LINEAR_GRADIENT);
 	/*set stencil even if assigned, this invalidates the associated bitmap state in 3D*/
 	gf_sc_texture_set_stencil(txh, stencil);
 
@@ -102,21 +121,21 @@ static void UpdateLinearGradient(GF_TextureHandler *txh)
 
 	st->txh.transparent = 0;
 	const_a = (lg->opacity.count == 1) ? 1 : 0;
-	cols = (u32*)malloc(sizeof(u32) * lg->key.count);
+	cols = (u32*)gf_malloc(sizeof(u32) * lg->key.count);
 	for (i=0; i<lg->key.count; i++) {
 		a = (const_a ? lg->opacity.vals[0] : lg->opacity.vals[i]);
 		cols[i] = GF_COL_ARGB_FIXED(a, lg->keyValue.vals[i].red, lg->keyValue.vals[i].green, lg->keyValue.vals[i].blue);
 		if (a != FIX_ONE) txh->transparent = 1;
 	}
-	txh->compositor->rasterizer->stencil_set_gradient_interpolation(stencil, lg->key.vals, cols, lg->key.count);
-	free(cols);
-	txh->compositor->rasterizer->stencil_set_gradient_mode(stencil, (GF_GradientMode) lg->spreadMethod);
+	gf_evg_stencil_set_gradient_interpolation(stencil, lg->key.vals, cols, lg->key.count);
+	gf_free(cols);
+	gf_evg_stencil_set_gradient_mode(stencil, (GF_GradientMode) lg->spreadMethod);
 }
 
 
 static void LG_ComputeMatrix(GF_TextureHandler *txh, GF_Rect *bounds, GF_Matrix2D *mat, Bool for_3d)
 {
-	GF_STENCIL stencil;
+	GF_EVGStencil *stencil;
 	M_LinearGradient *lg = (M_LinearGradient *) txh->owner;
 
 	stencil = gf_sc_texture_get_stencil(txh);
@@ -133,12 +152,14 @@ static void LG_ComputeMatrix(GF_TextureHandler *txh, GF_Rect *bounds, GF_Matrix2
 	/*translate to the center of the bounds*/
 	gf_mx2d_add_translation(mat, gf_divfix(bounds->x, bounds->width), gf_divfix(bounds->y - bounds->height, bounds->height));
 	/*scale back to object coordinates - the gradient is still specified in texture coordinates
-	i order to avoid overflows in fixed point*/
+	in order to avoid overflows in fixed point*/
 	gf_mx2d_add_scale(mat, bounds->width, bounds->height);
 
-	txh->compositor->rasterizer->stencil_set_linear_gradient(stencil, lg->startPoint.x, lg->startPoint.y, lg->endPoint.x, lg->endPoint.y);
+	gf_evg_stencil_set_linear_gradient(stencil, lg->startPoint.x, lg->startPoint.y, lg->endPoint.x, lg->endPoint.y);
 }
 
+//TODO - replace this with shader-based code ...
+
 static void BuildLinearGradientTexture(GF_TextureHandler *txh)
 {
 	u32 i;
@@ -147,22 +168,22 @@ static void BuildLinearGradientTexture(GF_TextureHandler *txh)
 	Fixed a;
 	Bool const_a;
 	GF_Matrix2D mat;
-	GF_STENCIL stenc;
-	GF_SURFACE surface;
-	GF_STENCIL texture2D;
+	GF_EVGStencil * stenc;
+	GF_EVGSurface *surface;
+	GF_EVGStencil * texture2D;
 	GF_Path *path;
 	GF_Err e;
 	Bool transparent;
+	u32 pix_fmt = 0;
 	M_LinearGradient *lg = (M_LinearGradient *) txh->owner;
 	GradientStack *st = (GradientStack *) gf_node_get_private(txh->owner);
-	GF_Raster2D *raster = txh->compositor->rasterizer;
 
 	if (!txh->tx_io) return;
 
-
-	if (st->tx_data) {
-		free(st->tx_data);
-		st->tx_data = NULL;
+	if (!(txh->flags & GF_SR_TEXTURE_GRAD_REGISTERED)) {
+		txh->flags |= GF_SR_TEXTURE_GRAD_REGISTERED;
+		if (gf_list_find(txh->compositor->textures, txh)<0)
+			gf_list_insert(txh->compositor->textures, txh, 0);
 	}
 
 	if (lg->key.count<2) return;
@@ -172,64 +193,70 @@ static void BuildLinearGradientTexture(GF_TextureHandler *txh)
 	end = lg->endPoint;
 
 	transparent = (lg->opacity.count==1) ? (lg->opacity.vals[0]!=FIX_ONE) : 1;
-	
+
 	/*init our 2D graphics stuff*/
-	texture2D = raster->stencil_new(raster, GF_STENCIL_TEXTURE);
+	texture2D = gf_evg_stencil_new(GF_STENCIL_TEXTURE);
 	if (!texture2D) return;
-	surface = raster->surface_new(raster, 1);
+	surface = gf_evg_surface_new(1);
 	if (!surface) {
-		raster->stencil_delete(texture2D);
+		gf_evg_stencil_delete(texture2D);
 		return;
 	}
 
-	if (st->no_rgb_support) transparent = 1;
-	if (st->tx_data && (st->transparent != transparent)) {
-		free(st->tx_data);
+	if (st->txh.flags & GF_SR_TEXTURE_GRAD_NO_RGB) transparent = 1;
+
+	if (st->tx_data && (st->txh.transparent != transparent)) {
+		gf_free(st->tx_data);
 		st->tx_data = NULL;
 	}
-	
+
 	if (transparent) {
 		if (!st->tx_data) {
-			st->tx_data = (char *) malloc(sizeof(char)*GRAD_TEXTURE_SIZE*GRAD_TEXTURE_SIZE*4);
-		} else {
-			memset(st->tx_data, 0, sizeof(char)*txh->stride*txh->height);
+			st->tx_data = (char *) gf_malloc(sizeof(char)*GRAD_TEXTURE_SIZE*GRAD_TEXTURE_SIZE*4);
+		}
+		memset(st->tx_data, 0, sizeof(char)*txh->stride*txh->height);
+
+		pix_fmt = GF_PIXEL_RGBA;
+		e = gf_evg_stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 4*GRAD_TEXTURE_SIZE, pix_fmt);
+		if (e) {
+			pix_fmt = GF_PIXEL_ARGB;
+			e = gf_evg_stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 4*GRAD_TEXTURE_SIZE, pix_fmt);
 		}
-		e = raster->stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 4*GRAD_TEXTURE_SIZE, GF_PIXEL_ARGB, GF_PIXEL_ARGB, 1);
 	} else {
 		if (!st->tx_data) {
-			st->tx_data = (char *) malloc(sizeof(char)*GRAD_TEXTURE_SIZE*GRAD_TEXTURE_SIZE*3);
+			st->tx_data = (char *) gf_malloc(sizeof(char)*GRAD_TEXTURE_SIZE*GRAD_TEXTURE_SIZE*3);
 		}
-		e = raster->stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 3*GRAD_TEXTURE_SIZE, GF_PIXEL_RGB_24, GF_PIXEL_RGB_24, 1);
+		e = gf_evg_stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 3*GRAD_TEXTURE_SIZE, GF_PIXEL_RGB);
 		/*try with ARGB (it actually is needed for GDIplus module since GDIplus cannot handle native RGB texture (it works in BGR)*/
 		if (e) {
 			/*remember for later use*/
-			st->no_rgb_support = 1;
+			st->txh.flags |= GF_SR_TEXTURE_GRAD_NO_RGB;
 			transparent = 1;
-			free(st->tx_data);
-			st->tx_data = (char *) malloc(sizeof(char)*GRAD_TEXTURE_SIZE*GRAD_TEXTURE_SIZE*4);
-			e = raster->stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 4*GRAD_TEXTURE_SIZE, GF_PIXEL_ARGB, GF_PIXEL_ARGB, 1);
+			gf_free(st->tx_data);
+			st->tx_data = (char *) gf_malloc(sizeof(char)*GRAD_TEXTURE_SIZE*GRAD_TEXTURE_SIZE*4);
+			e = gf_evg_stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 4*GRAD_TEXTURE_SIZE, GF_PIXEL_ARGB);
 		}
 	}
-	st->transparent = transparent;
+	st->txh.transparent = transparent;
 
 	if (e) {
-		free(st->tx_data);
-		raster->stencil_delete(texture2D);
-		raster->surface_delete(surface);
+		gf_free(st->tx_data);
+		gf_evg_stencil_delete(texture2D);
+		gf_evg_surface_delete(surface);
 		return;
 	}
-	e = raster->surface_attach_to_texture(surface, texture2D);
+	e = gf_evg_surface_attach_to_texture(surface, texture2D);
 	if (e) {
-		raster->stencil_delete(texture2D);
-		raster->surface_delete(surface);
+		gf_evg_stencil_delete(texture2D);
+		gf_evg_surface_delete(surface);
 		return;
 	}
 
 	/*create & setup gradient*/
-	stenc = raster->stencil_new(raster, GF_STENCIL_LINEAR_GRADIENT);
+	stenc = gf_evg_stencil_new(GF_STENCIL_LINEAR_GRADIENT);
 	if (!stenc) {
-		raster->stencil_delete(texture2D);
-		raster->surface_delete(surface);
+		gf_evg_stencil_delete(texture2D);
+		gf_evg_surface_delete(surface);
 		return;
 	}
 	/*move line to object space*/
@@ -237,16 +264,16 @@ static void BuildLinearGradientTexture(GF_TextureHandler *txh)
 	end.x *= GRAD_TEXTURE_SIZE;
 	start.y *= GRAD_TEXTURE_SIZE;
 	end.y *= GRAD_TEXTURE_SIZE;
-	raster->stencil_set_linear_gradient(stenc, start.x, start.y, end.x, end.y);
+	gf_evg_stencil_set_linear_gradient(stenc, start.x, start.y, end.x, end.y);
 	const_a = (lg->opacity.count == 1) ? 1 : 0;
-	cols = (u32*)malloc(sizeof(u32) * lg->key.count);
+	cols = (u32*)gf_malloc(sizeof(u32) * lg->key.count);
 	for (i=0; i<lg->key.count; i++) {
 		a = (const_a ? lg->opacity.vals[0] : lg->opacity.vals[i]);
 		cols[i] = GF_COL_ARGB_FIXED(a, lg->keyValue.vals[i].red, lg->keyValue.vals[i].green, lg->keyValue.vals[i].blue);
 	}
-	raster->stencil_set_gradient_interpolation(stenc, lg->key.vals, cols, lg->key.count);
-	free(cols);
-	raster->stencil_set_gradient_mode(stenc, (GF_GradientMode)lg->spreadMethod);
+	gf_evg_stencil_set_gradient_interpolation(stenc, lg->key.vals, cols, lg->key.count);
+	gf_free(cols);
+	gf_evg_stencil_set_gradient_mode(stenc, (GF_GradientMode)lg->spreadMethod);
 
 	/*fill surface*/
 	path = gf_path_new();
@@ -266,14 +293,15 @@ static void BuildLinearGradientTexture(GF_TextureHandler *txh)
 	gf_mx2d_add_translation(&mat, -INT2FIX(GRAD_TEXTURE_HSIZE), -INT2FIX(GRAD_TEXTURE_HSIZE));
 	/*back to GL bottom->up order*/
 	gf_mx2d_add_scale(&mat, FIX_ONE, -FIX_ONE);
-	raster->stencil_set_matrix(stenc, &mat);
+	gf_evg_stencil_set_matrix(stenc, &mat);
+	gf_evg_stencil_set_auto_matrix(stenc, GF_FALSE);
 
-	raster->surface_set_raster_level(surface, GF_RASTER_HIGH_QUALITY);
-	raster->surface_set_path(surface, path);
-	raster->surface_fill(surface, stenc);
-	raster->stencil_delete(stenc);
-	raster->surface_delete(surface);
-	raster->stencil_delete(texture2D);
+	gf_evg_surface_set_raster_level(surface, GF_RASTER_HIGH_QUALITY);
+	gf_evg_surface_set_path(surface, path);
+	gf_evg_surface_fill(surface, stenc);
+	gf_evg_stencil_delete(stenc);
+	gf_evg_surface_delete(surface);
+	gf_evg_stencil_delete(texture2D);
 	gf_path_del(path);
 
 	txh->data = st->tx_data;
@@ -283,22 +311,24 @@ static void BuildLinearGradientTexture(GF_TextureHandler *txh)
 	if (transparent) {
 		u32 j;
 		txh->stride = GRAD_TEXTURE_SIZE*4;
+		txh->pixelformat = GF_PIXEL_RGBA;
 
 		/*back to RGBA texturing*/
-		txh->pixelformat = GF_PIXEL_RGBA;
-		for (i=0; i<txh->height; i++) {
-			char *data = txh->data + i*txh->stride;
-			for (j=0; j<txh->width; j++) {
-				u32 val = *(u32 *) &data[4*j];
-				data[4*j] = (val>>16) & 0xFF;
-				data[4*j+1] = (val>>8) & 0xFF;
-				data[4*j+2] = (val) & 0xFF;
-				data[4*j+3] = (val>>24) & 0xFF;
+		if (pix_fmt != GF_PIXEL_RGBA) {
+			for (i=0; i<txh->height; i++) {
+				char *data = txh->data + i*txh->stride;
+				for (j=0; j<txh->width; j++) {
+					u32 val = *(u32 *) &data[4*j];
+					data[4*j] = (val>>16) & 0xFF;
+					data[4*j+1] = (val>>8) & 0xFF;
+					data[4*j+2] = (val) & 0xFF;
+					data[4*j+3] = (val>>24) & 0xFF;
+				}
 			}
 		}
 	} else {
 		txh->stride = GRAD_TEXTURE_SIZE*3;
-		txh->pixelformat = GF_PIXEL_RGB_24;
+		txh->pixelformat = GF_PIXEL_RGB;
 	}
 	txh->flags |= GF_SR_TEXTURE_NO_GL_FLIP;
 	gf_sc_texture_set_data(txh);
@@ -308,11 +338,19 @@ void compositor_init_linear_gradient(GF_Compositor *compositor, GF_Node *node)
 {
 	GradientStack *st;
 	GF_SAFEALLOC(st, GradientStack);
+	if (!st) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[Compositor] Failed to allocate gradient stack\n"));
+		return;
+	}
 
-	gf_sc_texture_setup(&st->txh, compositor, node);
+	/*!!! Gradients are textures but are not registered as textures with the compositor in order to avoid updating
+	too many textures each frame - gradients are only registered with the compositor when they are used in OpenGL, in order
+	to release associated HW resource when no longer used*/
+	st->txh.owner = node;
+	st->txh.compositor = compositor;
 	st->txh.update_texture_fcnt = UpdateLinearGradient;
 	st->txh.compute_gradient_matrix = LG_ComputeMatrix;
-	
+
 	gf_node_set_private(node, st);
 	gf_node_set_callback_function(node, DestroyGradient);
 }
@@ -326,86 +364,94 @@ static void BuildRadialGradientTexture(GF_TextureHandler *txh)
 	Fixed a, radius;
 	Bool const_a;
 	GF_Matrix2D mat;
-	GF_STENCIL stenc;
-	GF_SURFACE surface;
-	GF_STENCIL texture2D;
+	GF_EVGStencil * stenc;
+	GF_EVGSurface *surface;
+	GF_EVGStencil * texture2D;
 	GF_Path *path;
 	GF_Err e;
+	u32 pix_fmt = 0;
 	Bool transparent;
 	M_RadialGradient *rg = (M_RadialGradient*) txh->owner;
 	GradientStack *st = (GradientStack *) gf_node_get_private(txh->owner);
-	GF_Raster2D *raster = txh->compositor->rasterizer;
-
 
 	if (!txh->tx_io) return;
 
-	if (st->tx_data) {
-		free(st->tx_data);
-		st->tx_data = NULL;
+	if (!(txh->flags & GF_SR_TEXTURE_GRAD_REGISTERED)) {
+		txh->flags |= GF_SR_TEXTURE_GRAD_REGISTERED;
+		if (gf_list_find(txh->compositor->textures, txh)<0)
+			gf_list_insert(txh->compositor->textures, txh, 0);
 	}
 
 	if (rg->key.count<2) return;
 	if (rg->key.count != rg->keyValue.count) return;
 
 	transparent = (rg->opacity.count==1) ? ((rg->opacity.vals[0]!=FIX_ONE) ? 1 : 0) : 1;
-	
+
 	/*init our 2D graphics stuff*/
-	texture2D = raster->stencil_new(raster, GF_STENCIL_TEXTURE);
+	texture2D = gf_evg_stencil_new(GF_STENCIL_TEXTURE);
 	if (!texture2D) return;
-	surface = raster->surface_new(raster, 1);
+	surface = gf_evg_surface_new(1);
 	if (!surface) {
-		raster->stencil_delete(texture2D);
+		gf_evg_stencil_delete(texture2D);
 		return;
 	}
 
-	if (st->no_rgb_support) transparent = 1;
-	if (st->tx_data && (st->transparent != transparent)) {
-		free(st->tx_data);
+	if (st->txh.flags & GF_SR_TEXTURE_GRAD_NO_RGB) transparent = 1;
+
+	if (st->tx_data && (st->txh.transparent != transparent)) {
+		gf_free(st->tx_data);
 		st->tx_data = NULL;
 	}
-	
+
 	if (transparent) {
 		if (!st->tx_data) {
-			st->tx_data = (char *) malloc(sizeof(char)*GRAD_TEXTURE_SIZE*GRAD_TEXTURE_SIZE*4);
-		} else {
-			memset(st->tx_data, 0, sizeof(char)*txh->stride*txh->height);
+			st->tx_data = (char *) gf_malloc(sizeof(char)*GRAD_TEXTURE_SIZE*GRAD_TEXTURE_SIZE*4);
+		}
+		memset(st->tx_data, 0, sizeof(char)*txh->stride*txh->height);
+
+		pix_fmt = GF_PIXEL_RGBA;
+
+		e = gf_evg_stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 4*GRAD_TEXTURE_SIZE, pix_fmt);
+
+		if (e) {
+			pix_fmt = GF_PIXEL_ARGB;
+			e = gf_evg_stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 4*GRAD_TEXTURE_SIZE, pix_fmt);
 		}
-		e = raster->stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 4*GRAD_TEXTURE_SIZE, GF_PIXEL_ARGB, GF_PIXEL_ARGB, 1);
 	} else {
 		if (!st->tx_data) {
-			st->tx_data = (char *) malloc(sizeof(char)*GRAD_TEXTURE_SIZE*GRAD_TEXTURE_SIZE*3);
+			st->tx_data = (char *) gf_malloc(sizeof(char)*GRAD_TEXTURE_SIZE*GRAD_TEXTURE_SIZE*3);
 		}
-		e = raster->stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 3*GRAD_TEXTURE_SIZE, GF_PIXEL_RGB_24, GF_PIXEL_RGB_24, 1);
+		e = gf_evg_stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 3*GRAD_TEXTURE_SIZE, GF_PIXEL_RGB);
 		/*try with ARGB (it actually is needed for GDIplus module since GDIplus cannot handle native RGB texture (it works in BGR)*/
 		if (e) {
 			/*remember for later use*/
-			st->no_rgb_support = 1;
+			st->txh.flags |= GF_SR_TEXTURE_GRAD_NO_RGB;
 			transparent = 1;
-			free(st->tx_data);
-			st->tx_data = (char *) malloc(sizeof(char)*GRAD_TEXTURE_SIZE*GRAD_TEXTURE_SIZE*4);
-			e = raster->stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 4*GRAD_TEXTURE_SIZE, GF_PIXEL_ARGB, GF_PIXEL_ARGB, 1);
+			gf_free(st->tx_data);
+			st->tx_data = (char *) gf_malloc(sizeof(char)*GRAD_TEXTURE_SIZE*GRAD_TEXTURE_SIZE*4);
+			e = gf_evg_stencil_set_texture(texture2D, st->tx_data, GRAD_TEXTURE_SIZE, GRAD_TEXTURE_SIZE, 4*GRAD_TEXTURE_SIZE, GF_PIXEL_ARGB);
 		}
 	}
-	st->transparent = transparent;
+	st->txh.transparent = transparent;
 
 	if (e) {
-		free(st->tx_data);
-		raster->stencil_delete(texture2D);
-		raster->surface_delete(surface);
+		gf_free(st->tx_data);
+		gf_evg_stencil_delete(texture2D);
+		gf_evg_surface_delete(surface);
 		return;
 	}
-	e = raster->surface_attach_to_texture(surface, texture2D);
+	e = gf_evg_surface_attach_to_texture(surface, texture2D);
 	if (e) {
-		raster->stencil_delete(texture2D);
-		raster->surface_delete(surface);
+		gf_evg_stencil_delete(texture2D);
+		gf_evg_surface_delete(surface);
 		return;
 	}
 
 	/*create & setup gradient*/
-	stenc = raster->stencil_new(raster, GF_STENCIL_RADIAL_GRADIENT);
+	stenc = gf_evg_stencil_new(GF_STENCIL_RADIAL_GRADIENT);
 	if (!stenc) {
-		raster->stencil_delete(texture2D);
-		raster->surface_delete(surface);
+		gf_evg_stencil_delete(texture2D);
+		gf_evg_surface_delete(surface);
 	}
 
 	center = rg->center;
@@ -419,17 +465,17 @@ static void BuildRadialGradientTexture(GF_TextureHandler *txh)
 	focal.y *= GRAD_TEXTURE_SIZE;
 	radius *= GRAD_TEXTURE_SIZE;
 
-	raster->stencil_set_radial_gradient(stenc, center.x, center.y, focal.x, focal.y, radius, radius);
+	gf_evg_stencil_set_radial_gradient(stenc, center.x, center.y, focal.x, focal.y, radius, radius);
 
 	const_a = (rg->opacity.count == 1) ? 1 : 0;
-	cols = (u32*) malloc(sizeof(u32) * rg->key.count);
+	cols = (u32*) gf_malloc(sizeof(u32) * rg->key.count);
 	for (i=0; i<rg->key.count; i++) {
 		a = (const_a ? rg->opacity.vals[0] : rg->opacity.vals[i]);
 		cols[i] = GF_COL_ARGB_FIXED(a, rg->keyValue.vals[i].red, rg->keyValue.vals[i].green, rg->keyValue.vals[i].blue);
 	}
-	raster->stencil_set_gradient_interpolation(stenc, rg->key.vals, cols, rg->key.count);
-	free(cols);
-	raster->stencil_set_gradient_mode(stenc, (GF_GradientMode)rg->spreadMethod);
+	gf_evg_stencil_set_gradient_interpolation(stenc, rg->key.vals, cols, rg->key.count);
+	gf_free(cols);
+	gf_evg_stencil_set_gradient_mode(stenc, (GF_GradientMode)rg->spreadMethod);
 
 	/*fill surface*/
 	path = gf_path_new();
@@ -448,14 +494,15 @@ static void BuildRadialGradientTexture(GF_TextureHandler *txh)
 	gf_mx2d_add_translation(&mat, -INT2FIX(GRAD_TEXTURE_HSIZE), -INT2FIX(GRAD_TEXTURE_HSIZE));
 	/*back to GL bottom->up order*/
 	gf_mx2d_add_scale(&mat, FIX_ONE, -FIX_ONE);
-	raster->stencil_set_matrix(stenc, &mat);
+	gf_evg_stencil_set_matrix(stenc, &mat);
+	gf_evg_stencil_set_auto_matrix(stenc, GF_FALSE);
 
-	raster->surface_set_raster_level(surface, GF_RASTER_HIGH_QUALITY);
-	raster->surface_set_path(surface, path);
-	raster->surface_fill(surface, stenc);
-	raster->stencil_delete(stenc);
-	raster->surface_delete(surface);
-	raster->stencil_delete(texture2D);
+	gf_evg_surface_set_raster_level(surface, GF_RASTER_HIGH_QUALITY);
+	gf_evg_surface_set_path(surface, path);
+	gf_evg_surface_fill(surface, stenc);
+	gf_evg_stencil_delete(stenc);
+	gf_evg_surface_delete(surface);
+	gf_evg_stencil_delete(texture2D);
 	gf_path_del(path);
 
 	txh->data = st->tx_data;
@@ -465,22 +512,28 @@ static void BuildRadialGradientTexture(GF_TextureHandler *txh)
 	if (transparent) {
 		u32 j;
 		txh->stride = GRAD_TEXTURE_SIZE*4;
+		txh->pixelformat = GF_PIXEL_RGBA;
 
 		/*back to RGBA texturing*/
-		txh->pixelformat = GF_PIXEL_RGBA;
-		for (i=0; i<txh->height; i++) {
-			char *data = txh->data + i*txh->stride;
-			for (j=0; j<txh->width; j++) {
-				u32 val = *(u32 *) &data[4*j];
-				data[4*j] = (val>>16) & 0xFF;
-				data[4*j+1] = (val>>8) & 0xFF;
-				data[4*j+2] = (val) & 0xFF;
-				data[4*j+3] = (val>>24) & 0xFF;
+		if (pix_fmt == GF_PIXEL_ARGB) {
+			for (i=0; i<txh->height; i++) {
+				char *data = txh->data + i*txh->stride;
+				for (j=0; j<txh->width; j++) {
+					u8 pa, pr, pg, pb;
+					pa = data[4*j];
+					pr = data[4*j+1];
+					pg = data[4*j+2];
+					pb = data[4*j+3];
+					data[4*j] = pr;
+					data[4*j+1] = pg;
+					data[4*j+2] = pb;
+					data[4*j+3] = pa;
+				}
 			}
 		}
 	} else {
 		txh->stride = GRAD_TEXTURE_SIZE*3;
-		txh->pixelformat = GF_PIXEL_RGB_24;
+		txh->pixelformat = GF_PIXEL_RGB;
 	}
 //	tx_set_blend_enable(txh, 1);
 	txh->flags |= GF_SR_TEXTURE_NO_GL_FLIP;
@@ -491,7 +544,7 @@ static void BuildRadialGradientTexture(GF_TextureHandler *txh)
 static void UpdateRadialGradient(GF_TextureHandler *txh)
 {
 	Bool const_a;
-	GF_STENCIL stencil;
+	GF_EVGStencil * stencil;
 	u32 i, *cols;
 	Fixed a;
 	M_RadialGradient *rg = (M_RadialGradient*) txh->owner;
@@ -506,7 +559,7 @@ static void UpdateRadialGradient(GF_TextureHandler *txh)
 	if (!txh->tx_io) gf_sc_texture_allocate(txh);
 
 	stencil = gf_sc_texture_get_stencil(txh);
-	if (!stencil)stencil = txh->compositor->rasterizer->stencil_new(txh->compositor->rasterizer, GF_STENCIL_RADIAL_GRADIENT);
+	if (!stencil) stencil = gf_evg_stencil_new(GF_STENCIL_RADIAL_GRADIENT);
 	/*set stencil even if assigned, this invalidates the associated bitmap state in 3D*/
 	gf_sc_texture_set_stencil(txh, stencil);
 
@@ -522,21 +575,21 @@ static void UpdateRadialGradient(GF_TextureHandler *txh)
 	}
 
 	const_a = (rg->opacity.count == 1) ? 1 : 0;
-	cols = (u32*)malloc(sizeof(u32) * rg->key.count);
+	cols = (u32*)gf_malloc(sizeof(u32) * rg->key.count);
 	for (i=0; i<rg->key.count; i++) {
 		a = (const_a ? rg->opacity.vals[0] : rg->opacity.vals[i]);
 		cols[i] = GF_COL_ARGB_FIXED(a, rg->keyValue.vals[i].red, rg->keyValue.vals[i].green, rg->keyValue.vals[i].blue);
 	}
-	txh->compositor->rasterizer->stencil_set_gradient_interpolation(stencil, rg->key.vals, cols, rg->key.count);
-	free(cols);
+	gf_evg_stencil_set_gradient_interpolation(stencil, rg->key.vals, cols, rg->key.count);
+	gf_free(cols);
 
-	txh->compositor->rasterizer->stencil_set_gradient_mode(stencil, (GF_GradientMode) rg->spreadMethod);
+	gf_evg_stencil_set_gradient_mode(stencil, (GF_GradientMode) rg->spreadMethod);
 
 }
 
 static void RG_ComputeMatrix(GF_TextureHandler *txh, GF_Rect *bounds, GF_Matrix2D *mat, Bool for_3d)
 {
-	GF_STENCIL stencil;
+	GF_EVGStencil * stencil;
 	M_RadialGradient *rg = (M_RadialGradient *) txh->owner;
 
 	if (rg->key.count<2) return;
@@ -549,11 +602,12 @@ static void RG_ComputeMatrix(GF_TextureHandler *txh, GF_Rect *bounds, GF_Matrix2
 
 	GradientGetMatrix((GF_Node *) rg->transform, mat);
 
-	txh->compositor->rasterizer->stencil_set_radial_gradient(stencil, rg->center.x, rg->center.y, rg->focalPoint.x, rg->focalPoint.y, rg->radius, rg->radius);
+	gf_evg_stencil_set_radial_gradient(stencil, rg->center.x, rg->center.y, rg->focalPoint.x, rg->focalPoint.y, rg->radius, rg->radius);
+
 	/*move to center of bounds*/
 	gf_mx2d_add_translation(mat, gf_divfix(bounds->x, bounds->width), gf_divfix(bounds->y - bounds->height, bounds->height));
 	/*scale back to object coordinates - the gradient is still specified in texture coordinates
-	i order to avoid overflows in fixed point*/
+	in order to avoid overflows in fixed point*/
 	gf_mx2d_add_scale(mat, bounds->width, bounds->height);
 }
 
@@ -562,8 +616,16 @@ void compositor_init_radial_gradient(GF_Compositor *compositor, GF_Node *node)
 {
 	GradientStack *st;
 	GF_SAFEALLOC(st, GradientStack);
+	if (!st) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[Compositor] Failed to allocate gradient stack\n"));
+		return;
+	}
 
-	gf_sc_texture_setup(&st->txh, compositor, node);
+	/*!!! Gradients are textures but are not registered as textures with the compositor in order to avoid updating
+	too many textures each frame - gradients are only registered with the compositor when they are used in OpenGL, in order
+	to release associated HW resource when no longer used*/
+	st->txh.owner = node;
+	st->txh.compositor = compositor;
 	st->txh.update_texture_fcnt = UpdateRadialGradient;
 	st->txh.compute_gradient_matrix = RG_ComputeMatrix;
 
@@ -574,18 +636,26 @@ void compositor_init_radial_gradient(GF_Compositor *compositor, GF_Node *node)
 GF_TextureHandler *compositor_mpeg4_get_gradient_texture(GF_Node *node)
 {
 	GradientStack *st = (GradientStack*) gf_node_get_private(node);
+	st->txh.update_texture_fcnt(&st->txh);
 	return &st->txh;
 }
 
+#endif /*GPAC_DISABLE_VRML*/
+
+#if !defined(GPAC_DISABLE_COMPOSITOR)
+
 void compositor_gradient_update(GF_TextureHandler *txh)
 {
 	switch (gf_node_get_tag(txh->owner) ) {
+#ifndef GPAC_DISABLE_VRML
 	case TAG_MPEG4_RadialGradient:
 		BuildRadialGradientTexture(txh);
 		break;
 	case TAG_MPEG4_LinearGradient:
 		BuildLinearGradientTexture(txh);
 		break;
+#endif
+
 #ifndef GPAC_DISABLE_SVG
 	case TAG_SVG_linearGradient:
 	case TAG_SVG_radialGradient:
@@ -595,3 +665,6 @@ void compositor_gradient_update(GF_TextureHandler *txh)
 	}
 }
 
+#endif
+
+#endif //!defined(GPAC_DISABLE_COMPOSITOR)
