@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Compositor sub-project
@@ -10,15 +11,15 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
@@ -26,10 +27,12 @@
 #include "mpeg4_grouping.h"
 #include "visual_manager.h"
 
+#if !defined(GPAC_DISABLE_VRML) && !defined(GPAC_DISABLE_COMPOSITOR)
+
 typedef struct
 {
-	PARENT_NODE_STACK_2D
-	
+	PARENT_MPEG4_STACK_2D
+
 	Bool is_scrolling;
 	u32 start_scroll_type;
 	Double start_time, pause_time;
@@ -40,12 +43,14 @@ typedef struct
 	/*for keyboard navigation*/
 	GF_SensorHandler hdl;
 	s32 key_scroll;
+	Bool keys_active;
 } LayoutStack;
 
 typedef struct
 {
 	Fixed width, height, ascent, descent;
 	u32 first_child, nb_children;
+	Bool line_break;
 } LineInfo;
 
 static void layout_reset_lines(LayoutStack *st)
@@ -53,7 +58,7 @@ static void layout_reset_lines(LayoutStack *st)
 	while (gf_list_count(st->lines)) {
 		LineInfo *li = (LineInfo *)gf_list_get(st->lines, 0);
 		gf_list_rem(st->lines, 0);
-		free(li);
+		gf_free(li);
 	}
 }
 
@@ -61,14 +66,10 @@ static LineInfo *new_line_info(LayoutStack *st)
 {
 	LineInfo *li;
 	GF_SAFEALLOC(li, LineInfo);
-
-	gf_list_add(st->lines, li);
+	if (li)
+		gf_list_add(st->lines, li);
 	return li;
 }
-static GFINLINE LineInfo *get_line_info(LayoutStack *st, u32 i)
-{
-	return (LineInfo *) gf_list_get(st->lines, i);
-}
 
 
 enum
@@ -101,7 +102,7 @@ static void get_lines_info(LayoutStack *st, M_Layout *l)
 	max_w = st->clip.width;
 	max_h = st->clip.height;
 	layout_reset_lines(st);
-	
+
 	count = gf_list_count(st->groups);
 	if (!count) return;
 
@@ -123,7 +124,8 @@ static void get_lines_info(LayoutStack *st, M_Layout *l)
 			li->height += cg->final.height;
 			li->nb_children ++;
 		} else {
-			if (i && (cg->final.width + li->width> max_w)) {
+			if ((cg->text_type==2) || (i && (cg->final.width + li->width> max_w))) {
+				if (cg->text_type==2) li->line_break = 1;
 				if (l->wrap) {
 					if (!li->ascent) {
 						li->ascent = li->height;
@@ -134,17 +136,17 @@ static void get_lines_info(LayoutStack *st, M_Layout *l)
 						li->width -= prev_discard_width;
 						li->nb_children--;
 					}
-					if (cg->discardable && (i+1==count)) break; 
+					if ((cg->text_type==1) && (i+1==count)) break;
 
 					li = new_line_info(st);
 					li->first_child = i;
-					if (cg->discardable) {
+					if (cg->text_type) {
 						li->first_child++;
 						continue;
 					}
-				} 
+				}
 			}
-		
+
 			/*get ascent/descent for text or height for non-text*/
 			if (cg->ascent) {
 				if (li->ascent < cg->ascent) li->ascent = cg->ascent;
@@ -156,7 +158,7 @@ static void get_lines_info(LayoutStack *st, M_Layout *l)
 			li->width += cg->final.width;
 			li->nb_children ++;
 
-			prev_discard_width = cg->discardable ? cg->final.width : 0;
+			prev_discard_width = (cg->text_type==1) ? cg->final.width : 0;
 
 		}
 	}
@@ -172,7 +174,7 @@ static void layout_justify(LayoutStack *st, M_Layout *l)
 	get_lines_info(st, l);
 	major = get_justify(l, 0);
 	minor = get_justify(l, 1);
-	
+
 	st->scroll_len = 0;
 	nbLines = gf_list_count(st->lines);
 	if (l->horizontal) {
@@ -192,9 +194,10 @@ static void layout_justify(LayoutStack *st, M_Layout *l)
 			if (!l->leftToRight) first += li->nb_children - 1;
 
 			if (!l->topToBottom && k) current_top += li->height;
-			
+
 			/*set major alignment (X) */
 			cg = (ChildGroup *)gf_list_get(st->groups, first);
+			if (!cg) continue;
 			switch (major) {
 			case L_END:
 				cg->final.x = st->clip.x + st->clip.width - li->width;
@@ -212,19 +215,22 @@ static void layout_justify(LayoutStack *st, M_Layout *l)
 					cg = (ChildGroup *)gf_list_get(st->groups, li->nb_children-1);
 					spacing = (st->clip.width - li->width) / (li->nb_children-1) ;
 					if (spacing<0) spacing = 0;
-					/*disable spacing for last text line*/
-					else if (cg->ascent && (k+1==nbLines) ) 
-						spacing = 0;
-					
+					else if (cg->ascent) {
+						/*disable spacing for last text line and line breaks*/
+						if ( (k+1==nbLines) || li->line_break) {
+							spacing = 0;
+						}
+					}
 				}
 				break;
-			}          
+			}
 
 
 			/*for each in the run */
 			i = first;
 			while (1) {
 				cg = (ChildGroup *)gf_list_get(st->groups, i);
+				if (!cg) break;
 				h = MAX(li->ascent, li->height);
 				switch (minor) {
 				case L_FIRST:
@@ -259,7 +265,7 @@ static void layout_justify(LayoutStack *st, M_Layout *l)
 				if (l->leftToRight && (i==li->first_child + li->nb_children))
 					break;
 				else if (!l->leftToRight && (i==li->first_child - 1))
-					break;		
+					break;
 			}
 			if (l->topToBottom) {
 				current_top -= gf_mulfix(l->spacing, li->height);
@@ -291,7 +297,7 @@ static void layout_justify(LayoutStack *st, M_Layout *l)
 
 		first = li->first_child;
 		if (!l->topToBottom) first += li->nb_children - 1;
-		
+
 		/*set major alignment (Y) */
 		cg = (ChildGroup *)gf_list_get(st->groups, first);
 		switch (major) {
@@ -312,7 +318,7 @@ static void layout_justify(LayoutStack *st, M_Layout *l)
 				if (spacing<0) spacing = 0;
 			}
 			break;
-		}          
+		}
 
 		/*for each in the run */
 		i = first;
@@ -344,7 +350,7 @@ static void layout_justify(LayoutStack *st, M_Layout *l)
 			if (l->topToBottom && (i==li->first_child + li->nb_children))
 				break;
 			else if (!l->topToBottom && (i==li->first_child - 1))
-				break;		
+				break;
 		}
 		if (l->leftToRight) {
 			current_left += gf_mulfix(l->spacing, li->width);
@@ -357,7 +363,7 @@ static void layout_justify(LayoutStack *st, M_Layout *l)
 		} else {
 			st->scroll_len += li->width;
 		}
-	}		
+	}
 }
 
 static void layout_setup_scroll_bounds(LayoutStack *st, M_Layout *l)
@@ -401,7 +407,7 @@ static void layout_setup_scroll_bounds(LayoutStack *st, M_Layout *l)
 			}
 			break;
 		}
-	} 
+	}
 	/*scroll-in only*/
 	else {
 		st->scroll_max = 0;
@@ -459,7 +465,7 @@ static void layout_setup_scroll_bounds(LayoutStack *st, M_Layout *l)
 static void layout_scroll(GF_TraverseState *tr_state, LayoutStack *st, M_Layout *l)
 {
 	u32 i, nb_lines;
-	Fixed scrolled, rate, ellapsed, scroll_diff;
+	Fixed scrolled, rate, elapsed, scroll_diff;
 	Bool smooth, do_scroll, stop_anim;
 	Double time = 0;
 	ChildGroup *cg;
@@ -478,12 +484,12 @@ static void layout_scroll(GF_TraverseState *tr_state, LayoutStack *st, M_Layout
 
 		time = gf_node_get_scene_time((GF_Node *)l);
 
-	//	if (st->scale_scroll && (st->prev_rate!=st->scale_scroll)) st->start_scroll_type = 1;
+		//	if (st->scale_scroll && (st->prev_rate!=st->scale_scroll)) st->start_scroll_type = 1;
 
-		/*if scroll rate changed to previous non-zero value, this is a 
+		/*if scroll rate changed to previous non-zero value, this is a
 		scroll restart, don't re-update bounds*/
 		if ((st->start_scroll_type==2) && (st->prev_rate==st->scale_scroll)) st->start_scroll_type = 0;
-		
+
 		if (st->start_scroll_type) {
 			st->start_time = time;
 			st->is_scrolling = 1;
@@ -518,8 +524,8 @@ static void layout_scroll(GF_TraverseState *tr_state, LayoutStack *st, M_Layout
 		}
 
 		/*compute advance in pixels for smooth scroll*/
-		ellapsed = FLT2FIX((Float) (time - st->start_time));
-		scrolled = gf_mulfix(ellapsed, rate);
+		elapsed = FLT2FIX((Float) (time - st->start_time));
+		scrolled = gf_mulfix(elapsed, rate);
 	}
 
 	smooth = l->smoothScroll;
@@ -531,7 +537,6 @@ static void layout_scroll(GF_TraverseState *tr_state, LayoutStack *st, M_Layout
 
 	stop_anim = 0;
 	/*compute scroll diff for non-smooth mode*/
-	do_scroll = 1;
 	if (smooth) {
 		do_scroll = 1;
 	} else {
@@ -554,8 +559,8 @@ static void layout_scroll(GF_TraverseState *tr_state, LayoutStack *st, M_Layout
 				}
 				break;
 			} else if (ABS(scroll_diff) >= dim) {
-				if (scroll_diff<0) scroll_diff = -dim;
-				else scroll_diff = dim;
+//				if (scroll_diff<0) scroll_diff = -dim;
+//				else scroll_diff = dim;
 				do_scroll = 1;
 				break;
 			}
@@ -572,7 +577,7 @@ static void layout_scroll(GF_TraverseState *tr_state, LayoutStack *st, M_Layout
 		scrolled = scroll_diff;
 	}
 
-	if (do_scroll) 
+	if (do_scroll)
 		st->last_scroll = scrolled;
 	else
 		scrolled = st->last_scroll;
@@ -594,7 +599,7 @@ static void layout_scroll(GF_TraverseState *tr_state, LayoutStack *st, M_Layout
 	}
 	/*draw next frame*/
 	if (!stop_anim) {
-		gf_sc_invalidate(tr_state->visual->compositor, NULL);
+		tr_state->visual->compositor->force_next_frame_redraw = GF_TRUE;
 		return;
 	}
 
@@ -613,9 +618,9 @@ static void TraverseLayout(GF_Node *node, void *rs, Bool is_destroy)
 	u32 i;
 	ChildGroup *cg;
 	GF_IRect prev_clip;
-	Bool mode_bckup, had_clip;
+	Bool mode_bckup, had_clip=GF_FALSE;
 	ParentNode2D *parent_bck;
-	GF_Rect clip, prev_clipper;
+	GF_Rect prev_clipper;
 	M_Layout *l = (M_Layout *)node;
 	LayoutStack *st = (LayoutStack *) gf_node_get_private(node);
 	GF_TraverseState *tr_state = (GF_TraverseState *)rs;
@@ -624,10 +629,10 @@ static void TraverseLayout(GF_Node *node, void *rs, Bool is_destroy)
 		layout_reset_lines(st);
 		parent_node_predestroy((ParentNode2D *)st);
 		gf_list_del(st->lines);
-		free(st);
+		gf_free(st);
 		return;
 	}
-	
+
 	/*note we don't clear dirty flag, this is done in traversing*/
 	if (gf_node_dirty_get(node) & GF_SG_NODE_DIRTY) {
 
@@ -643,15 +648,15 @@ static void TraverseLayout(GF_Node *node, void *rs, Bool is_destroy)
 		st->bounds = st->clip = gf_rect_center(st->clip.width, st->clip.height);
 
 		if (st->scale_scroll && !st->start_scroll_type) st->start_scroll_type = 1;
-	
+
 		drawable_reset_group_highlight(tr_state, node);
 	}
 
 	/*don't waste time traversing is pick ray not in clipper*/
-	if ((tr_state->traversing_mode==TRAVERSE_PICK) && !gf_sc_pick_in_clipper(tr_state, &st->clip)) 
+	if ((tr_state->traversing_mode==TRAVERSE_PICK) && !gf_sc_pick_in_clipper(tr_state, &st->clip))
 		goto layout_exit;
 
-	if (tr_state->traversing_mode==TRAVERSE_GET_BOUNDS) {
+	if ((tr_state->traversing_mode==TRAVERSE_GET_BOUNDS) && !tr_state->for_node) {
 		tr_state->bounds = st->clip;
 #ifndef GPAC_DISABLE_3D
 		gf_bbox_from_rect(&tr_state->bbox, &st->clip);
@@ -660,19 +665,20 @@ static void TraverseLayout(GF_Node *node, void *rs, Bool is_destroy)
 	}
 
 	recompute_layout = 0;
-	if (gf_node_dirty_get(node)) recompute_layout = 1;
+	if (gf_node_dirty_get(node))
+		recompute_layout = 1;
 
 	/*setup clipping*/
 	prev_clip = tr_state->visual->top_clipper;
 	if (tr_state->traversing_mode==TRAVERSE_SORT) {
-		clip = compositor_2d_update_clipper(tr_state, st->clip, &had_clip, &prev_clipper, 0);
+		compositor_2d_update_clipper(tr_state, st->clip, &had_clip, &prev_clipper, 0);
 		if (tr_state->has_clip) {
 			tr_state->visual->top_clipper = gf_rect_pixelize(&tr_state->clipper);
 			gf_irect_intersect(&tr_state->visual->top_clipper, &prev_clip);
 		}
 	}
 	if (recompute_layout) {
-		GF_LOG(GF_LOG_COMPOSE, GF_LOG_DEBUG, ("[Layout] recomputing positions\n"));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPOSE, ("[Layout] recomputing positions\n"));
 
 		parent_node_reset((ParentNode2D*)st);
 
@@ -699,10 +705,27 @@ static void TraverseLayout(GF_Node *node, void *rs, Bool is_destroy)
 		/*apply justification*/
 		layout_justify(st, l);
 
-		/*prepare initial scroll bounds*/
-//		layout_setup_scroll_bounds(st, l);
+		/*if scrolling, update bounds*/
+		if (l->scrollRate && st->is_scrolling) {
+			layout_setup_scroll_bounds(st, l);
+		}
 	}
 
+
+	if (tr_state->traversing_mode==TRAVERSE_GET_BOUNDS)  {
+		tr_state->bounds = st->bounds;
+		if (l->scrollVertical) {
+			tr_state->bounds.height = st->scroll_len;
+		} else {
+			tr_state->bounds.width = st->scroll_len;
+		}
+#ifndef GPAC_DISABLE_3D
+		gf_bbox_from_rect(&tr_state->bbox, &tr_state->bounds);
+#endif
+		goto layout_exit;
+	}
+
+
 	/*scroll*/
 	layout_scroll(tr_state, st, l);
 
@@ -722,50 +745,66 @@ layout_exit:
 	tr_state->text_split_mode = 0;
 }
 
-static void OnLayout(GF_SensorHandler *sh, Bool is_over, GF_Event *ev, GF_Compositor *compositor)
+static Bool OnLayout(GF_SensorHandler *sh, Bool is_over, Bool is_cancel, GF_Event *ev, GF_Compositor *compositor)
 {
-	Bool vertical = ((M_Layout *)sh->sensor)->scrollVertical;
-	LayoutStack *st = (LayoutStack *) gf_node_get_private(sh->sensor);
+	Bool vertical;
+	LayoutStack *st;
+	if (!sh || !ev) return GF_FALSE;
+
+	st = (LayoutStack *) gf_node_get_private(sh->sensor);
+	vertical = ((M_Layout *)sh->sensor)->scrollVertical;
 
 	if (!is_over) {
 		st->is_scrolling = 0;
 		st->key_scroll = 0;
-		return;
+		return 0;
 	}
 	if (ev->type!=GF_EVENT_KEYDOWN) {
 		st->is_scrolling = 0;
 		st->key_scroll = 0;
-		return;
+		return 0;
 	}
 
 	switch (ev->key.key_code) {
 	case GF_KEY_LEFT:
-		if (vertical) return;
+		if (!st->keys_active) return 0;
+
+		if (vertical) return 0;
 		st->key_scroll = -1;
 		break;
 	case GF_KEY_RIGHT:
-		if (vertical) return;
+		if (!st->keys_active) return 0;
+
+		if (vertical) return 0;
 		st->key_scroll = +1;
 		break;
 	case GF_KEY_UP:
-		if (!vertical) return;
+		if (!st->keys_active) return 0;
+
+		if (!vertical) return 0;
 		st->key_scroll = +1;
 		break;
 	case GF_KEY_DOWN:
-		if (!vertical) return;
+		if (!st->keys_active) return 0;
+
+		if (!vertical) return 0;
 		st->key_scroll = -1;
 		break;
+	case GF_KEY_ENTER:
+		st->keys_active = !st->keys_active;
+		break;
 	default:
 		st->key_scroll = 0;
-		return;
+		return 0;
 	}
 	gf_sc_invalidate(compositor, NULL);
+	return 1;
 }
 
 static Bool layout_is_enabled(GF_Node *node)
 {
 	M_Layout *l = (M_Layout *)node;
-	if (l->scrollRate != 0) return 0;
+	if (node && (l->scrollRate != 0)) return 0;
 	return 1;
 }
 
@@ -773,6 +812,10 @@ void compositor_init_layout(GF_Compositor *compositor, GF_Node *node)
 {
 	LayoutStack *stack;
 	GF_SAFEALLOC(stack, LayoutStack);
+	if (!stack) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[Compositor] Failed to allocate layout stack\n"));
+		return;
+	}
 
 	parent_node_setup((ParentNode2D*)stack);
 	stack->lines = gf_list_new();
@@ -781,6 +824,14 @@ void compositor_init_layout(GF_Compositor *compositor, GF_Node *node)
 	stack->hdl.sensor = node;
 	stack->hdl.IsEnabled = layout_is_enabled;
 	stack->hdl.OnUserEvent = OnLayout;
+#ifdef GPAC_ENABLE_COVERAGE
+	if (gf_sys_is_cov_mode()) {
+		OnLayout(NULL, GF_FALSE, GF_FALSE, NULL, compositor);
+		layout_is_enabled(node);
+		compositor_mpeg4_layout_get_sensor_handler(node);
+	}
+#endif
+
 }
 
 void compositor_layout_modified(GF_Compositor *compositor, GF_Node *node)
@@ -789,7 +840,7 @@ void compositor_layout_modified(GF_Compositor *compositor, GF_Node *node)
 	/*if modif other than scrollrate restart scroll*/
 	if (st->scroll_rate == ((M_Layout*)node)->scrollRate) {
 		st->start_scroll_type = 1;
-	} 
+	}
 	/*if modif on scroll rate only, indicate continous restart*/
 	else if (((M_Layout*)node)->scrollRate) {
 		st->start_scroll_type = 2;
@@ -800,7 +851,11 @@ void compositor_layout_modified(GF_Compositor *compositor, GF_Node *node)
 
 GF_SensorHandler *compositor_mpeg4_layout_get_sensor_handler(GF_Node *node)
 {
-	LayoutStack *st = (LayoutStack *) gf_node_get_private(node);
+	LayoutStack *st;
+	if (!node) return NULL;
+	st = (LayoutStack *) gf_node_get_private(node);
 	return &st->hdl;
 }
 
+
+#endif //!defined(GPAC_DISABLE_VRML) && !defined(GPAC_DISABLE_COMPOSITOR)
