@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Compositor sub-project
@@ -10,26 +11,25 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
-
-
 #include "nodes_stacks.h"
 #include "visual_manager.h"
 #include "mpeg4_grouping.h"
 #include "texturing.h"
 #include <gpac/utf.h>
-#include <gpac/options.h>
+
+#if !defined(GPAC_DISABLE_VRML) && !defined(GPAC_DISABLE_COMPOSITOR)
 
 /*default value when no fontStyle*/
 #define FSFAMILY	(fs && fs->family.count) ? (const char *)fs->family.vals[0]	: ""
@@ -49,32 +49,32 @@
 /*exported to access the strike list ...*/
 typedef struct
 {
-	Drawable *graph;
+	struct _drawable s_graph;
 	Fixed ascent, descent;
 	GF_List *spans;
 	GF_Rect bounds;
 	u32 texture_text_flag;
+	Bool is_dirty;
 	GF_Compositor *compositor;
 } TextStack;
 
 void text_clean_paths(GF_Compositor *compositor, TextStack *stack)
 {
-	GF_TextSpan *span;
 	/*delete all path objects*/
 	while (gf_list_count(stack->spans)) {
-		span = (GF_TextSpan*) gf_list_get(stack->spans, 0);
+		GF_TextSpan *span = (GF_TextSpan*) gf_list_get(stack->spans, 0);
 		gf_list_rem(stack->spans, 0);
 		gf_font_manager_delete_span(compositor->font_manager, span);
 	}
 	stack->bounds.width = stack->bounds.height = 0;
-	drawable_reset_path(stack->graph);
+	drawable_reset_path(&stack->s_graph);
 }
 
 
 static void build_text_split(TextStack *st, M_Text *txt, GF_TraverseState *tr_state)
 {
 	u32 i, j, k, len, styles, idx, first_char;
-	Bool split_words = 0;
+	Bool split_words = GF_FALSE;
 	GF_Font *font;
 	GF_TextSpan *tspan;
 	GF_FontManager *ft_mgr = tr_state->visual->compositor->font_manager;
@@ -85,13 +85,14 @@ static void build_text_split(TextStack *st, M_Text *txt, GF_TraverseState *tr_st
 	if (fontSize <= 0) {
 		fontSize = INT2FIX(12);
 		if (!tr_state->pixel_metrics) fontSize = gf_divfix(fontSize, tr_state->visual->compositor->output_width);
-    }
+	}
 
 	styles = 0;
 	if (fs && fs->style.buffer) {
 		if (strstr(fs->style.buffer, "BOLD") || strstr(fs->style.buffer, "bold")) styles |= GF_FONT_WEIGHT_BOLD;
 		if (strstr(fs->style.buffer, "ITALIC") || strstr(fs->style.buffer, "italic")) styles |= GF_FONT_ITALIC;
 		if (strstr(fs->style.buffer, "UNDERLINED") || strstr(fs->style.buffer, "underlined")) styles |= GF_FONT_UNDERLINED;
+		if (strstr(fs->style.buffer, "STRIKETHROUGH") || strstr(fs->style.buffer, "strikethrough")) styles |= GF_FONT_STRIKEOUT;
 	}
 
 	font = gf_font_manager_set_font(ft_mgr, fs ? fs->family.vals : NULL, fs ? fs->family.count : 0, styles);
@@ -112,17 +113,17 @@ static void build_text_split(TextStack *st, M_Text *txt, GF_TraverseState *tr_st
 	else {
 		start_y = st->ascent;
 	}
-	
+
 	st->bounds.width = st->bounds.x = st->bounds.height = 0;
 	idx = 0;
-	split_words = (tr_state->text_split_mode==1) ? 1 : 0;
+	split_words = (tr_state->text_split_mode==1) ? GF_TRUE : GF_FALSE;
 
 	for (i=0; i < txt->string.count; i++) {
 
 		char *str = txt->string.vals[i];
 		if (!str || !strlen(str)) continue;
 
-		tspan = gf_font_manager_create_span(ft_mgr, font, str, fontSize, 0, 0, 0, NULL, 0, styles, (GF_Node*)txt);
+		tspan = gf_font_manager_create_span(ft_mgr, font, str, fontSize, GF_FALSE, GF_FALSE, GF_FALSE, NULL, GF_FALSE, styles, (GF_Node*)txt);
 		if (!tspan) continue;
 
 		len = tspan->nb_glyphs;
@@ -137,20 +138,24 @@ static void build_text_split(TextStack *st, M_Text *txt, GF_TraverseState *tr_st
 
 			/*we currently only split sentences at spaces*/
 			if (tspan->glyphs[j]->utf_name == (unsigned short) ' ') is_space = 1;
-			if (split_words && (j+1!=len) && !is_space) 
+			else if (tspan->glyphs[j]->utf_name == (unsigned short) '\n')
+				is_space = 2;
+			if (split_words && (j+1!=len) && !is_space)
 				continue;
 
-			span = (GF_TextSpan*) malloc(sizeof(GF_TextSpan));
+			span = (GF_TextSpan*) gf_malloc(sizeof(GF_TextSpan));
 			memcpy(span, tspan, sizeof(GF_TextSpan));
 
 			span->nb_glyphs = split_words ? (j - first_char) : 1;
 			if (split_words && !is_space) span->nb_glyphs++;
-			span->glyphs = malloc(sizeof(void *)*span->nb_glyphs);
+			span->glyphs = (GF_Glyph**)gf_malloc(sizeof(void *)*span->nb_glyphs);
 
 			span->bounds.height = st->ascent + st->descent;
 			span->bounds.y = start_y;
 			span->bounds.x = 0;
 			span->bounds.width = 0;
+			span->bounds.y += 2;
+			span->bounds.height += 4;
 
 			if (split_words) {
 				for (k=0; k<span->nb_glyphs; k++) {
@@ -158,7 +163,7 @@ static void build_text_split(TextStack *st, M_Text *txt, GF_TraverseState *tr_st
 					span->bounds.width += tspan->font_scale * (span->glyphs[k] ? span->glyphs[k]->horiz_advance : tspan->font->max_advance_h);
 				}
 			} else {
-				span->glyphs[0] = tspan->glyphs[FSLTR ? j : (len - j - 1) ];
+				//span->glyphs[0] = tspan->glyphs[FSLTR ? j : (len - j - 1) ];
 				span->glyphs[0] = tspan->glyphs[j];
 				span->bounds.width = tspan->font_scale * (span->glyphs[0] ? span->glyphs[0]->horiz_advance : tspan->font->max_advance_h);
 			}
@@ -166,25 +171,27 @@ static void build_text_split(TextStack *st, M_Text *txt, GF_TraverseState *tr_st
 			gf_list_add(st->spans, span);
 
 			/*request a context (first one is always valid when entering sort phase)*/
-			if (idx) parent_node_start_group(tr_state->parent, NULL, 0);
+			if (idx) parent_node_start_group(tr_state->parent, NULL, GF_FALSE);
 
 			idx++;
 			parent_node_end_text_group(tr_state->parent, &span->bounds, st->ascent, st->descent, idx);
 
 			if (is_space && split_words) {
-				span = (GF_TextSpan*) malloc(sizeof(GF_TextSpan));
+				span = (GF_TextSpan*) gf_malloc(sizeof(GF_TextSpan));
 				memcpy(span, tspan, sizeof(GF_TextSpan));
 				span->nb_glyphs = 1;
-				span->glyphs = malloc(sizeof(void *));
+				span->glyphs = (GF_Glyph**)gf_malloc(sizeof(void *));
 
 				gf_list_add(st->spans, span);
 				span->bounds.height = st->ascent + st->descent;
 				span->bounds.y = start_y;
 				span->bounds.x = 0;
+				span->bounds.y += 2;
+				span->bounds.height += 4;
 				k = (j - first_char);
 				span->glyphs[0] = tspan->glyphs[FSLTR ? (first_char+k) : (len - first_char - k - 1)];
 				span->bounds.width = tspan->font_scale * (span->glyphs[0] ? span->glyphs[0]->horiz_advance : tspan->font->max_advance_h);
-				parent_node_start_group(tr_state->parent, NULL, 1);
+				parent_node_start_group(tr_state->parent, NULL, is_space);
 				idx++;
 				parent_node_end_text_group(tr_state->parent, &span->bounds, st->ascent, st->descent, idx);
 			}
@@ -198,9 +205,12 @@ static void build_text_split(TextStack *st, M_Text *txt, GF_TraverseState *tr_st
 static void build_text(TextStack *st, M_Text *txt, GF_TraverseState *tr_state)
 {
 	u32 i, j, int_major, k, styles, count;
-	Fixed fontSize, start_x, start_y, line_spacing, tot_width, tot_height, max_scale, space;
+	Fixed fontSize, start_x, start_y, line_spacing, tot_width, tot_height, max_scale, maxExtent;
+	u32 size, trim_size;
 	GF_Font *font;
-	Bool horizontal;
+	char szBuf[100];
+	Bool horizontal, use_pass = GF_FALSE;
+	GF_TextSpan *trim_tspan = NULL;
 	GF_FontManager *ft_mgr = tr_state->visual->compositor->font_manager;
 	M_FontStyle *fs = (M_FontStyle *)txt->fontStyle;
 
@@ -208,7 +218,7 @@ static void build_text(TextStack *st, M_Text *txt, GF_TraverseState *tr_state)
 	if (fontSize <= 0) {
 		fontSize = INT2FIX(12);
 		if (!tr_state->pixel_metrics) fontSize = gf_divfix(fontSize, tr_state->visual->compositor->output_width);
-    }
+	}
 	horizontal = FSHORIZ;
 	start_x = start_y = 0;
 
@@ -217,6 +227,8 @@ static void build_text(TextStack *st, M_Text *txt, GF_TraverseState *tr_state)
 		if (strstr(fs->style.buffer, "BOLD") || strstr(fs->style.buffer, "bold")) styles |= GF_FONT_WEIGHT_BOLD;
 		if (strstr(fs->style.buffer, "ITALIC") || strstr(fs->style.buffer, "italic")) styles |= GF_FONT_ITALIC;
 		if (strstr(fs->style.buffer, "UNDERLINED") || strstr(fs->style.buffer, "underlined")) styles |= GF_FONT_UNDERLINED;
+		if (strstr(fs->style.buffer, "STRIKETHROUGH") || strstr(fs->style.buffer, "strikethrough")) styles |= GF_FONT_STRIKEOUT;
+		if (strstr(fs->style.buffer, "PASSWD") || strstr(fs->style.buffer, "passwd")) use_pass = GF_TRUE;
 	}
 
 	font = gf_font_manager_set_font(ft_mgr, fs ? fs->family.vals : NULL, fs ? fs->family.count : 0, styles);
@@ -225,21 +237,79 @@ static void build_text(TextStack *st, M_Text *txt, GF_TraverseState *tr_state)
 	/*NOTA: we could use integer maths here but we have a risk of overflow with large fonts, so use fixed maths*/
 	st->ascent = gf_muldiv(fontSize, INT2FIX(font->ascent), INT2FIX(font->em_size));
 	st->descent = -gf_muldiv(fontSize, INT2FIX(font->descent), INT2FIX(font->em_size));
-	space = gf_muldiv(fontSize, INT2FIX(font->line_spacing), INT2FIX(font->em_size)) ;
 	line_spacing = gf_mulfix(FSSPACE, fontSize);
 
+	maxExtent = txt->maxExtent;
+	trim_size = 0;
+
+	if (maxExtent<0) {
+		trim_tspan = gf_font_manager_create_span(ft_mgr, font, "...", fontSize, GF_FALSE, GF_FALSE, GF_FALSE, NULL, GF_FALSE, styles, (GF_Node*)txt);
+		if (trim_tspan) {
+			for (i=0; i<trim_tspan->nb_glyphs; i++) {
+				if (horizontal) {
+					trim_size += trim_tspan->glyphs[i] ? trim_tspan->glyphs[i]->horiz_advance : trim_tspan->font->max_advance_h;
+				} else {
+					trim_size += trim_tspan->glyphs[i] ? trim_tspan->glyphs[i]->vert_advance : trim_tspan->font->max_advance_v;
+				}
+			}
+		}
+	}
+
 	tot_width = tot_height = 0;
 	for (i=0; i < txt->string.count; i++) {
 		GF_TextSpan *tspan;
-		u32 size;
+
 		char *str = txt->string.vals[i];
 		if (!str) continue;
 
-		tspan = gf_font_manager_create_span(ft_mgr, font, txt->string.vals[i], fontSize, 0, 0, 0, NULL, 0, styles, (GF_Node*)txt);
+		if (use_pass) {
+			u32 j=0, len = (u32) strlen(str);
+			if (len>=100) len=99;
+			while (j<len) {
+				if (str[j] == 0x1)
+					szBuf[j] = str[j];
+				else
+					szBuf[j] = '*';
+				j++;
+			}
+			szBuf[j] = 0;
+			str = szBuf;
+		}
+
+		tspan = gf_font_manager_create_span(ft_mgr, font, str, fontSize, GF_FALSE, GF_FALSE, GF_FALSE, NULL, GF_FALSE, styles, (GF_Node*)txt);
 		if (!tspan) continue;
-		
+
 		if (horizontal) tspan->flags |= GF_TEXT_SPAN_HORIZONTAL;
 
+		size = 0;
+		if (trim_size) {
+			for (j=0; j<tspan->nb_glyphs; j++) {
+				if (horizontal) {
+					size += tspan->glyphs[j] ? tspan->glyphs[j]->horiz_advance : tspan->font->max_advance_h;
+				} else {
+					size += tspan->glyphs[j] ? tspan->glyphs[j]->vert_advance : tspan->font->max_advance_v;
+				}
+				/*word is bigger than allowed extent, rewrite 3 previous chars*/
+				if ((s32)size*tspan->font_scale >= -maxExtent) {
+					u32 nb_chars = (j<2) ? j : 3;
+
+					for (k=0; k<nb_chars; k++) {
+						u32 idx = nb_chars-k-1;
+						if (horizontal) {
+							size -= tspan->glyphs[j-k] ? tspan->glyphs[j-k]->horiz_advance : tspan->font->max_advance_h;
+							size += trim_tspan->glyphs[idx] ? trim_tspan->glyphs[idx]->horiz_advance : tspan->font->max_advance_h;
+						} else {
+							size -= tspan->glyphs[j-k] ? tspan->glyphs[j-k]->vert_advance : tspan->font->max_advance_v;
+							size += trim_tspan->glyphs[idx] ? trim_tspan->glyphs[idx]->vert_advance : tspan->font->max_advance_v;
+						}
+						tspan->glyphs[j-k] = trim_tspan->glyphs[idx];
+					}
+					tspan->nb_glyphs = j+1;
+					break;
+				}
+			}
+		}
+
 		if ((horizontal && !FSLTR) || (!horizontal && !FSTTB)) {
 			for (k=0; k<tspan->nb_glyphs/2; k++) {
 				GF_Glyph *g = tspan->glyphs[k];
@@ -248,12 +318,13 @@ static void build_text(TextStack *st, M_Text *txt, GF_TraverseState *tr_state)
 			}
 		}
 
-		size = 0;
-		for (j=0; j<tspan->nb_glyphs; j++) {
-			if (horizontal) {
-				size += tspan->glyphs[j] ? tspan->glyphs[j]->horiz_advance : tspan->font->max_advance_h;
-			} else {
-				size += tspan->glyphs[j] ? tspan->glyphs[j]->vert_advance : tspan->font->max_advance_v;
+		if (!size) {
+			for (j=0; j<tspan->nb_glyphs; j++) {
+				if (horizontal) {
+					size += tspan->glyphs[j] ? tspan->glyphs[j]->horiz_advance : tspan->font->max_advance_h;
+				} else {
+					size += tspan->glyphs[j] ? tspan->glyphs[j]->vert_advance : tspan->font->max_advance_v;
+				}
 			}
 		}
 		gf_list_add(st->spans, tspan);
@@ -277,12 +348,13 @@ static void build_text(TextStack *st, M_Text *txt, GF_TraverseState *tr_state)
 			if (tot_height < tspan->bounds.height) tot_height = tspan->bounds.height;
 		}
 	}
-	
+	if (trim_tspan)	gf_font_manager_delete_span(ft_mgr, trim_tspan);
+
+
 	max_scale = FIX_ONE;
 	if (horizontal) {
-		if ((txt->maxExtent > 0) && (tot_width>txt->maxExtent)) {
-			max_scale = gf_divfix(txt->maxExtent, tot_width);
-			tot_width = txt->maxExtent;
+		if ((maxExtent > 0) && (tot_width>maxExtent)) {
+			max_scale = gf_divfix(maxExtent, tot_width);
 		}
 		tot_height = (txt->string.count-1) * line_spacing + (st->ascent + st->descent);
 		st->bounds.height = tot_height;
@@ -298,7 +370,6 @@ static void build_text(TextStack *st, M_Text *txt, GF_TraverseState *tr_state)
 		}
 		else if (!strcmp(FSMINOR, "BEGIN")) {
 			if (FSTTB) {
-				start_y = st->descent;
 				start_y = 0;
 				st->bounds.y = start_y;
 			} else {
@@ -320,9 +391,8 @@ static void build_text(TextStack *st, M_Text *txt, GF_TraverseState *tr_state)
 			st->bounds.y = FSTTB ? start_y : (tot_height - st->descent);
 		}
 	} else {
-		if ((txt->maxExtent > 0) && (tot_height>txt->maxExtent) ) {
-			max_scale = gf_divfix(txt->maxExtent, tot_height);
-			tot_height = txt->maxExtent;
+		if ((maxExtent > 0) && (tot_height>maxExtent) ) {
+			max_scale = gf_divfix(maxExtent, tot_height);
 		}
 		tot_width = txt->string.count * line_spacing;
 		st->bounds.width = tot_width;
@@ -355,7 +425,7 @@ static void build_text(TextStack *st, M_Text *txt, GF_TraverseState *tr_state)
 			}
 		}
 	}
-			
+
 
 	/*major-justification*/
 	if (!strcmp(FSMAJOR, "MIDDLE") ) {
@@ -370,7 +440,7 @@ static void build_text(TextStack *st, M_Text *txt, GF_TraverseState *tr_state)
 
 	count = gf_list_count(st->spans);
 	for (i=0; i < count; i++) {
-		GF_TextSpan *span = gf_list_get(st->spans, i);
+		GF_TextSpan *span = (GF_TextSpan*)gf_list_get(st->spans, i);
 		switch (int_major) {
 		/*major-justification MIDDLE*/
 		case 0:
@@ -425,7 +495,7 @@ static void build_text(TextStack *st, M_Text *txt, GF_TraverseState *tr_state)
 	}
 }
 
-static void text_get_draw_opt(GF_Node *node, TextStack *st, Bool *force_texture, u32 *hl_color)
+static void text_get_draw_opt(GF_Node *node, TextStack *st, Bool *force_texture, u32 *hl_color, DrawAspect2D *asp)
 {
 	const char *fs_style;
 	char *hlight;
@@ -434,7 +504,6 @@ static void text_get_draw_opt(GF_Node *node, TextStack *st, Bool *force_texture,
 	*hl_color = 0;
 
 	fs_style = FSSTYLE;
-	hlight = NULL;
 	hlight = strstr(fs_style, "HIGHLIGHT");
 	if (hlight) hlight = strchr(hlight, '#');
 	if (hlight) {
@@ -446,7 +515,15 @@ static void text_get_draw_opt(GF_Node *node, TextStack *st, Bool *force_texture,
 		}
 	}
 	*force_texture = st->texture_text_flag;
-	if (strstr(fs_style, "TEXTURED")) *force_texture  = 1;
+	if (strstr(fs_style, "TEXTURED")) *force_texture = GF_TRUE;
+	if (strstr(fs_style, "OUTLINED")) {
+		if (asp && !asp->pen_props.width) {
+			asp->pen_props.width = FIX_ONE/2;
+			asp->pen_props.align = GF_PATH_LINE_OUTSIDE;
+			asp->line_scale=FIX_ONE;
+			asp->line_color = 0xFF000000;
+		}
+	}
 }
 
 
@@ -456,11 +533,11 @@ static void text_get_draw_opt(GF_Node *node, TextStack *st, Bool *force_texture,
 static Bool text_is_3d_material(GF_TraverseState *tr_state)
 {
 	GF_Node *__mat;
-	if (!tr_state->appear) return 0;
+	if (!tr_state->appear) return GF_FALSE;
 	__mat = ((M_Appearance *)tr_state->appear)->material;
-	if (!__mat) return 0;
-	if (gf_node_get_tag(__mat)==TAG_MPEG4_Material2D) return 0;
-	return 1;
+	if (!__mat) return GF_FALSE;
+	if (gf_node_get_tag(__mat)==TAG_MPEG4_Material2D) return GF_FALSE;
+	return GF_TRUE;
 }
 
 static void text_draw_3d(GF_TraverseState *tr_state, GF_Node *node, TextStack *st)
@@ -476,7 +553,7 @@ static void text_draw_3d(GF_TraverseState *tr_state, GF_Node *node, TextStack *s
 		asp = &the_asp;
 		drawable_get_aspect_2d_mpeg4(node, asp, tr_state);
 	}
-	text_get_draw_opt(node, st, &force_texture, &hl_color);
+	text_get_draw_opt(node, st, &force_texture, &hl_color, asp);
 	gf_font_spans_draw_3d(st->spans, tr_state, asp, hl_color, force_texture);
 }
 
@@ -493,7 +570,7 @@ void text_draw_2d(GF_Node *node, GF_TraverseState *tr_state)
 
 	if (!GF_COL_A(tr_state->ctx->aspect.fill_color) && !tr_state->ctx->aspect.pen_props.width) return;
 
-	text_get_draw_opt(node, st, &force_texture, &hl_color);
+	text_get_draw_opt(node, st, &force_texture, &hl_color, &tr_state->ctx->aspect);
 
 	tr_state->text_parent = node;
 	gf_font_spans_draw_2d(st->spans, tr_state, hl_color, force_texture, &st->bounds);
@@ -508,7 +585,14 @@ static void text_check_changes(GF_Node *node, TextStack *stack, GF_TraverseState
 		text_clean_paths(tr_state->visual->compositor, stack);
 		build_text(stack, (M_Text*)node, tr_state);
 		gf_node_dirty_clear(node, 0);
-		drawable_mark_modified(stack->graph, tr_state);
+		drawable_mark_modified(&stack->s_graph, tr_state);
+	}
+
+	if (tr_state->visual->compositor->edited_text && (tr_state->visual->compositor->focus_node==node)) {
+		drawable_mark_modified(&stack->s_graph, tr_state);
+		tr_state->visual->has_text_edit = GF_TRUE;
+		if (!stack->bounds.width) stack->bounds.width = INT2FIX(1)/100;
+		if (!stack->bounds.height) stack->bounds.height = INT2FIX(1)/100;
 	}
 }
 
@@ -521,16 +605,18 @@ static void Text_Traverse(GF_Node *n, void *rs, Bool is_destroy)
 	GF_TraverseState *tr_state = (GF_TraverseState *)rs;
 
 	if (is_destroy) {
-		text_clean_paths(gf_sc_get_compositor(n), st);
-		drawable_del(st->graph);
+		GF_Compositor *compositor = gf_sc_get_compositor(n);
+		text_clean_paths(compositor, st);
+		drawable_del_ex(&st->s_graph, compositor, GF_TRUE);
 		gf_list_del(st->spans);
-		free(st);
+		gf_free(st);
 		return;
 	}
 
 	if (!txt->string.count) return;
 
 	if (tr_state->text_split_mode) {
+		st->is_dirty = gf_node_dirty_get(n) ? GF_TRUE : GF_FALSE;
 		gf_node_dirty_clear(n, 0);
 		text_clean_paths(tr_state->visual->compositor, st);
 		build_text_split(st, txt, tr_state);
@@ -550,7 +636,7 @@ static void Text_Traverse(GF_Node *n, void *rs, Bool is_destroy)
 #endif
 	case TRAVERSE_PICK:
 		tr_state->text_parent = n;
-		gf_font_spans_pick(n, st->spans, tr_state, &st->bounds, 0, NULL);
+		gf_font_spans_pick(n, st->spans, tr_state, &st->bounds, GF_FALSE, NULL);
 		tr_state->text_parent = NULL;
 		return;
 	case TRAVERSE_GET_BOUNDS:
@@ -571,7 +657,7 @@ static void Text_Traverse(GF_Node *n, void *rs, Bool is_destroy)
 	if (tr_state->visual->type_3d) return;
 #endif
 
-	ctx = drawable_init_context_mpeg4(st->graph, tr_state);
+	ctx = drawable_init_context_mpeg4(&st->s_graph, tr_state);
 	if (!ctx) return;
 	ctx->sub_path_index = tr_state->text_split_idx;
 
@@ -584,21 +670,23 @@ static void Text_Traverse(GF_Node *n, void *rs, Bool is_destroy)
 
 	/*if text selection mode, we must force redraw of the entire text span because we don't
 	if glyphs have been (un)selected*/
-	if (!tr_state->direct_draw && 
-		/*text selection on*/
-		(tr_state->visual->compositor->text_selection 
-		/*text sel release*/
-		|| (tr_state->visual->compositor->store_text_state==GF_SC_TSEL_RELEASED)) 
-	) {
+	if (!tr_state->immediate_draw &&
+	        /*text selection on*/
+	        (tr_state->visual->compositor->text_selection
+	         /*text sel release*/
+	         || (tr_state->visual->compositor->store_text_state==GF_SC_TSEL_RELEASED))
+	   ) {
 		GF_TextSpan *span;
 		u32 i = 0;
-		Bool unselect = (tr_state->visual->compositor->store_text_state==GF_SC_TSEL_RELEASED) ? 1 : 0;
-		while ((span = gf_list_enum(st->spans, &i))) {
+		Bool unselect = (tr_state->visual->compositor->store_text_state==GF_SC_TSEL_RELEASED) ? GF_TRUE : GF_FALSE;
+		while ((span = (GF_TextSpan*)gf_list_enum(st->spans, &i))) {
 			if (span->flags & GF_TEXT_SPAN_SELECTED) {
 				if (unselect) span->flags &= ~GF_TEXT_SPAN_SELECTED;
 				ctx->flags |= CTX_APP_DIRTY;
 			}
 		}
+	} else if (st->is_dirty) {
+		ctx->flags |= CTX_APP_DIRTY;
 	}
 
 	if (ctx->sub_path_index) {
@@ -612,10 +700,15 @@ static void Text_Traverse(GF_Node *n, void *rs, Bool is_destroy)
 
 void compositor_init_text(GF_Compositor *compositor, GF_Node *node)
 {
-	TextStack *stack = (TextStack *)malloc(sizeof(TextStack));
-	stack->graph = drawable_new();
-	stack->graph->node = node;
-	stack->graph->flags = DRAWABLE_USE_TRAVERSE_DRAW;
+	TextStack *stack;
+	GF_SAFEALLOC(stack, TextStack);
+	if (!stack) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[Compositor] Failed to allocate text stack\n"));
+		return;
+	}
+	drawable_init_ex(&stack->s_graph);
+	stack->s_graph.node = node;
+	stack->s_graph.flags = DRAWABLE_USE_TRAVERSE_DRAW;
 	stack->ascent = stack->descent = 0;
 	stack->spans = gf_list_new();
 	stack->texture_text_flag = 0;
@@ -625,32 +718,6 @@ void compositor_init_text(GF_Compositor *compositor, GF_Node *node)
 	gf_node_set_callback_function(node, Text_Traverse);
 }
 
-
-static void TraverseTextureText(GF_Node *node, void *rs, Bool is_destroy)
-{
-	TextStack *stack;
-	GF_Node *text;
-	GF_FieldInfo field;
-	if (is_destroy) return;
-	if (gf_node_get_field(node, 0, &field) != GF_OK) return;
-	if (field.fieldType != GF_SG_VRML_SFNODE) return;
-	text = *(GF_Node **)field.far_ptr;
-	if (!text) return;
-
-	if (gf_node_get_field(node, 1, &field) != GF_OK) return;
-	if (field.fieldType != GF_SG_VRML_SFBOOL) return;
-
-	if (gf_node_get_tag(text) != TAG_MPEG4_Text) return;
-	stack = (TextStack *) gf_node_get_private(text);
-	stack->texture_text_flag = *(SFBool*)field.far_ptr ? 1 : 0;
-}
-
-
-void compositor_init_texture_text(GF_Compositor *compositor, GF_Node *node)
-{
-	gf_node_set_callback_function(node, TraverseTextureText);
-}
-
 #ifndef GPAC_DISABLE_3D
 void compositor_extrude_text(GF_Node *node, GF_TraverseState *tr_state, GF_Mesh *mesh, MFVec3f *thespine, Fixed creaseAngle, Bool begin_cap, Bool end_cap, MFRotation *spine_ori, MFVec2f *spine_scale, Bool txAlongSpine)
 {
@@ -663,7 +730,7 @@ void compositor_extrude_text(GF_Node *node, GF_TraverseState *tr_state, GF_Mesh
 		ParentNode2D *parent = tr_state->parent;
 		tr_state->parent = NULL;
 		text_clean_paths(tr_state->visual->compositor, st);
-		drawable_reset_path(st->graph);
+		drawable_reset_path(&st->s_graph);
 		gf_node_dirty_clear(node, 0);
 		build_text(st, (M_Text *)node, tr_state);
 		tr_state->parent = parent;
@@ -686,7 +753,8 @@ void compositor_extrude_text(GF_Node *node, GF_TraverseState *tr_state, GF_Mesh
 	gf_mesh_build_aabbtree(mesh);
 }
 
-#endif
+#endif /*GPAC_DISABLE_3D*/
 
 
+#endif //!defined(GPAC_DISABLE_VRML) && !defined(GPAC_DISABLE_COMPOSITOR)
 
