@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Compositor sub-project
@@ -10,22 +11,22 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
 #include "nodes_stacks.h"
 #include "visual_manager.h"
-#include <gpac/options.h>
 
+#if !defined(GPAC_DISABLE_COMPOSITOR)
 
 #ifndef GPAC_DISABLE_3D
 
@@ -33,8 +34,22 @@ static void camera_changed(GF_Compositor *compositor, GF_Camera *cam)
 {
 	cam->flags |= CAM_IS_DIRTY;
 	gf_sc_invalidate(compositor, NULL);
+	if (compositor->active_layer) gf_node_dirty_set(compositor->active_layer, 0, 1);
 }
 
+#endif
+
+static void nav_set_zoom_trans_2d(GF_VisualManager *visual, Fixed zoom, Fixed dx, Fixed dy)
+{
+	compositor_2d_set_user_transform(visual->compositor, zoom, visual->compositor->trans_x + dx, visual->compositor->trans_y + dy, 0);
+#ifndef GPAC_DISABLE_3D
+	if (visual->type_3d) camera_changed(visual->compositor, &visual->camera);
+#endif
+}
+
+
+#ifndef GPAC_DISABLE_3D
+
 /*shortcut*/
 static void gf_mx_rotation_matrix(GF_Matrix *mx, SFVec3f axis_pt, SFVec3f axis, Fixed angle)
 {
@@ -43,7 +58,7 @@ static void gf_mx_rotation_matrix(GF_Matrix *mx, SFVec3f axis_pt, SFVec3f axis,
 	gf_mx_add_rotation(mx, angle, axis.x, axis.y, axis.z);
 	gf_mx_add_translation(mx, -axis_pt.x, -axis_pt.y, -axis_pt.z);
 }
-	
+
 static void view_orbit_x(GF_Compositor *compositor, GF_Camera *cam, Fixed dx)
 {
 	GF_Matrix mx;
@@ -104,26 +119,49 @@ static void view_roll(GF_Compositor *compositor, GF_Camera *cam, Fixed dd)
 	camera_changed(compositor, cam);
 }
 
+static void update_pan_up(GF_Compositor *compositor, GF_Camera *cam)
+{
+	SFVec3f axis, dir;
+	/*update up vector so that right is always horizontal (no y component)*/
+	dir = camera_get_pos_dir(cam);
+	axis = camera_get_right_dir(cam);
+	axis.y = 0;
+	gf_vec_norm(&axis);
+	cam->up = gf_vec_cross(dir, axis);
+	gf_vec_norm(&cam->up);
+
+	camera_changed(compositor, cam);
+}
+
 static void view_pan_x(GF_Compositor *compositor, GF_Camera *cam, Fixed dx)
 {
 	GF_Matrix mx;
 	if (!dx) return;
 	gf_mx_rotation_matrix(&mx, cam->position, cam->up, dx);
 	gf_mx_apply_vec(&mx, &cam->target);
-	camera_changed(compositor, cam);
+
+	update_pan_up(compositor, cam);
 }
 static void view_pan_y(GF_Compositor *compositor, GF_Camera *cam, Fixed dy)
 {
 	GF_Matrix mx;
-	SFVec3f axis;
+	GF_Vec prev_target = cam->target;
 	if (!dy) return;
-	axis = camera_get_right_dir(cam);
-	gf_mx_rotation_matrix(&mx, cam->position, axis, dy);
+	gf_mx_rotation_matrix(&mx, cam->position, camera_get_right_dir(cam), dy);
 	gf_mx_apply_vec(&mx, &cam->target);
-	/*update up vector*/
-	cam->up = gf_vec_cross(camera_get_pos_dir(cam), axis);
-	gf_vec_norm(&cam->up);
-	camera_changed(compositor, cam);
+	switch (cam->navigate_mode) {
+	case GF_NAVIGATE_WALK:
+	case GF_NAVIGATE_VR:
+	case GF_NAVIGATE_GAME:
+		if (cam->target.z*prev_target.z<0) {
+			cam->target = prev_target;
+			return;
+		}
+	default:
+		break;
+	}
+
+	update_pan_up(compositor, cam);
 }
 
 /*for translation moves when jumping*/
@@ -167,7 +205,8 @@ static void view_zoom(GF_Compositor *compositor, GF_Camera *cam, Fixed z)
 	Fixed oz;
 	if ((z>FIX_ONE) || (z<-FIX_ONE)) return;
 	oz = gf_divfix(cam->vp_fov, cam->fieldOfView);
-	if (oz<FIX_ONE) z/=4; oz += z;
+	if (oz<FIX_ONE) z/=4;
+	oz += z;
 	if (oz<=0) return;
 
 	cam->fieldOfView = gf_divfix(cam->vp_fov, oz);
@@ -175,7 +214,7 @@ static void view_zoom(GF_Compositor *compositor, GF_Camera *cam, Fixed z)
 	camera_changed(compositor, cam);
 }
 
-static void nav_fit_screen(GF_Compositor *compositor)
+Bool gf_sc_fit_world_to_screen(GF_Compositor *compositor)
 {
 	GF_TraverseState tr_state;
 	SFVec3f pos, diff;
@@ -183,26 +222,38 @@ static void nav_fit_screen(GF_Compositor *compositor)
 	GF_Camera *cam;
 	GF_Node *top;
 
-	if (gf_list_count(compositor->visual->back_stack)) return;
-	if (gf_list_count(compositor->visual->view_stack)) return;
+#ifndef GPAC_DISABLE_VRML
+//	if (gf_list_count(compositor->visual->back_stack)) return;
+	if (gf_list_count(compositor->visual->view_stack)) return 0;
+#endif
 
 	gf_mx_p(compositor->mx);
 	top = gf_sg_get_root_node(compositor->scene);
 	if (!top) {
 		gf_mx_v(compositor->mx);
-		return;
+		return 0;
 	}
 	memset(&tr_state, 0, sizeof(GF_TraverseState));
+	gf_mx_init(tr_state.model_matrix);
 	tr_state.traversing_mode = TRAVERSE_GET_BOUNDS;
 	tr_state.visual = compositor->visual;
 	gf_node_traverse(top, &tr_state);
+	if (gf_node_dirty_get(top)) {
+		tr_state.bbox.is_set = 0;
+	}
+
 	if (!tr_state.bbox.is_set) {
 		gf_mx_v(compositor->mx);
-		return;
+		/*empty world ...*/
+		if (tr_state.bbox.radius==-1) return 1;
+		/*2D world with 3D camera forced*/
+		if (tr_state.bounds.width&&tr_state.bounds.height) return 1;
+		return 0;
 	}
 
 	cam = &compositor->visual->camera;
 
+	cam->world_bbox = tr_state.bbox;
 	/*fit is based on bounding sphere*/
 	dist = gf_divfix(tr_state.bbox.radius, gf_sin(cam->fieldOfView/2) );
 	gf_vec_diff(diff, cam->center, tr_state.bbox.center);
@@ -212,20 +263,83 @@ static void nav_fit_screen(GF_Compositor *compositor)
 		d = gf_vec_len(diff);
 		if (d<dist) {
 			gf_mx_v(compositor->mx);
-			return;
+			return 1;
 		}
 	}
-		
+
 	diff = gf_vec_scale(camera_get_pos_dir(cam), dist);
 	gf_vec_add(pos, tr_state.bbox.center, diff);
 	diff = cam->position;
 	camera_set_vectors(cam, pos, cam->vp_orientation, cam->fieldOfView);
 	cam->position = diff;
 	camera_move_to(cam, pos, cam->target, cam->up);
+	if (!compositor->player) {
+		camera_stop_anim(cam);
+		camera_set_vectors(cam, cam->end_pos, cam->end_ori, cam->end_fov);
+	}
+
 	cam->examine_center = tr_state.bbox.center;
 	cam->flags |= CF_STORE_VP;
+	if (cam->z_far < dist) cam->z_far = 10*dist;
 	camera_changed(compositor, cam);
 	gf_mx_v(compositor->mx);
+	return 1;
+}
+
+static void handle_mouse_move_3d(GF_Compositor *compositor, GF_Camera *cam, u32 keys, Fixed dx, Fixed dy)
+{
+	Fixed trans_scale = cam->width/20;
+	//if default VP is quite far from center use larger dz/dy moves
+	if (cam->vp_dist>100) trans_scale *= 10;
+
+	switch (cam->navigate_mode) {
+	/*FIXME- we'll likely need a "step" value for walk at some point*/
+	case GF_NAVIGATE_WALK:
+	case GF_NAVIGATE_FLY:
+		view_pan_x(compositor, cam, -dx);
+		if (keys & GF_KEY_MOD_CTRL) view_pan_y(compositor, cam, dy);
+		else view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
+		break;
+	case GF_NAVIGATE_VR:
+		view_pan_x(compositor, cam, -dx);
+		if (keys & GF_KEY_MOD_CTRL) view_zoom(compositor, cam, dy);
+		else view_pan_y(compositor, cam, dy);
+		break;
+	case GF_NAVIGATE_PAN:
+		view_pan_x(compositor, cam, -dx);
+		if (keys & GF_KEY_MOD_CTRL) view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
+		else view_pan_y(compositor, cam, dy);
+		break;
+	case GF_NAVIGATE_SLIDE:
+		view_translate_x(compositor, cam, gf_mulfix(dx, trans_scale));
+		if (keys & GF_KEY_MOD_CTRL) view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
+		else view_translate_y(compositor, cam, gf_mulfix(dy, trans_scale));
+		break;
+	case GF_NAVIGATE_EXAMINE:
+		if (keys & GF_KEY_MOD_CTRL) {
+			view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
+			view_roll(compositor, cam, gf_mulfix(dx, trans_scale));
+		} else {
+			if (ABS(dx) > ABS(dy)) {
+				view_exam_x(compositor, cam, -gf_mulfix(GF_PI, dx));
+			} else {
+				view_exam_y(compositor, cam, gf_mulfix(GF_PI, dy));
+			}
+		}
+		break;
+	case GF_NAVIGATE_ORBIT:
+		if (keys & GF_KEY_MOD_CTRL) {
+			view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
+		} else {
+			view_orbit_x(compositor, cam, -gf_mulfix(GF_PI, dx));
+			view_orbit_y(compositor, cam, gf_mulfix(GF_PI, dy));
+		}
+		break;
+	case GF_NAVIGATE_GAME:
+		view_pan_x(compositor, cam, -dx);
+		view_pan_y(compositor, cam, dy);
+		break;
+	}
 }
 
 static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event *ev)
@@ -234,41 +348,44 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 	Fixed dx, dy, key_trans, key_pan, key_exam;
 	s32 key_inv;
 	u32 keys;
-	Bool is_pixel_metrics;
 	GF_Camera *cam;
+	Fixed zoom = compositor->zoom;
 
 	cam = NULL;
+#ifndef GPAC_DISABLE_VRML
 	if (compositor->active_layer) {
 		cam = compositor_layer3d_get_camera(compositor->active_layer);
-		is_pixel_metrics = gf_sg_use_pixel_metrics(gf_node_get_graph(compositor->active_layer));
-	} 
+	}
+#endif
+
 	if (!cam) {
 		cam = &compositor->visual->camera;
-		assert(compositor);
-		assert(compositor->scene);
-		is_pixel_metrics = compositor->traverse_state->pixel_metrics;
 	}
+	if (!cam || (cam->navigate_mode==GF_NAVIGATE_NONE)) return 0;
 
-	if (!cam->navigate_mode) return 0;
 	keys = compositor->key_states;
+	if (!cam->navigate_mode && !(keys & GF_KEY_MOD_ALT) ) return 0;
 	x = y = 0;
 	/*renorm between -1, 1*/
-	if (ev->type<=GF_EVENT_MOUSEWHEEL) {
-		x = gf_divfix( INT2FIX(ev->mouse.x + (s32) compositor->visual->width/2), INT2FIX(compositor->visual->width));
-		y = gf_divfix( INT2FIX(ev->mouse.y + (s32) compositor->visual->height/2), INT2FIX(compositor->visual->height));
+	if (ev->type <= GF_EVENT_LAST_MOUSE) {
+		x = gf_divfix( INT2FIX(ev->mouse.x - (s32) compositor->visual->width/2), INT2FIX(compositor->visual->width));
+		y = gf_divfix( INT2FIX(ev->mouse.y - (s32) compositor->visual->height/2), INT2FIX(compositor->visual->height));
 	}
 
-	dx = (x - compositor->grab_x); 
-	dy = (y - compositor->grab_y);
+	dx = (x - compositor->grab_x);
+	dy = (compositor->grab_y - y);
 
-/*	trans_scale = is_pixel_metrics ? cam->width/2 : INT2FIX(10);
-	key_trans = is_pixel_metrics ? INT2FIX(10) : cam->avatar_size.x;
-*/
-	trans_scale = cam->width/2;
-	key_trans = cam->avatar_size.x;
+	trans_scale = cam->width/20;
+	key_trans = cam->avatar_size.x/2;
+	//if default VP is quite far from center use larger dz/dy moves
+	if (cam->vp_dist>100) trans_scale *= 10;
 
-	key_pan = FIX_ONE/25;
-	key_exam = FIX_ONE/10;
+	if (cam->world_bbox.is_set && (key_trans*5 > cam->world_bbox.radius)) {
+		key_trans = cam->world_bbox.radius / 100;
+	}
+
+	key_pan = FIX_ONE/20;
+	key_exam = FIX_ONE/20;
 	key_inv = 1;
 
 	if (keys & GF_KEY_MOD_SHIFT) {
@@ -279,6 +396,14 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 		key_trans*=4;
 	}
 
+	if (! compositor->orientation_sensors_active) {
+		Fixed yaw, pitch, roll;
+		gf_mx_get_yaw_pitch_roll(&compositor->visual->camera.modelview, &yaw, &pitch, &roll);
+		compositor->audio_renderer->yaw = yaw;
+		compositor->audio_renderer->pitch = pitch;
+		compositor->audio_renderer->roll = roll;
+	}
+
 	switch (ev->type) {
 	case GF_EVENT_MOUSEDOWN:
 		/*left*/
@@ -286,6 +411,7 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 			compositor->grab_x = x;
 			compositor->grab_y = y;
 			compositor->navigation_state = 1;
+			compositor->auto_rotate=0;
 
 			/*change vp and examine center to current location*/
 			if ((keys & GF_KEY_MOD_CTRL) && compositor->hit_square_dist) {
@@ -296,7 +422,7 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 				camera_changed(compositor, cam);
 				return 1;
 			}
-		} 
+		}
 		/*right*/
 		else if (ev->mouse.button==GF_MOUSE_RIGHT) {
 			if (compositor->navigation_state && (cam->navigate_mode==GF_NAVIGATE_WALK)) {
@@ -304,12 +430,13 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 				gf_sc_invalidate(compositor, NULL);
 				return 1;
 			}
-			else if (keys & GF_KEY_MOD_CTRL) nav_fit_screen(compositor);
+			else if (keys & GF_KEY_MOD_CTRL) gf_sc_fit_world_to_screen(compositor);
 		}
 		break;
 
-	/* note: shortcuts are mostly the same as blaxxun contact, I don't feel like remembering 2 sets...*/
 	case GF_EVENT_MOUSEMOVE:
+		if (compositor->orientation_sensors_active) return 0;
+
 		if (!compositor->navigation_state) {
 			if (cam->navigate_mode==GF_NAVIGATE_GAME) {
 				/*init mode*/
@@ -317,55 +444,20 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 				compositor->grab_y = y;
 				compositor->navigation_state = 1;
 			}
+			compositor->auto_rotate=0;
 			return 0;
 		}
 		compositor->navigation_state++;
 
-		switch (cam->navigate_mode) {
-		/*FIXME- we'll likely need a "step" value for walk at some point*/
-		case GF_NAVIGATE_WALK:
-		case GF_NAVIGATE_FLY:
-			view_pan_x(compositor, cam, -dx);
-			if (keys & GF_KEY_MOD_CTRL) view_pan_y(compositor, cam, dy);
-			else view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
-			break;
-		case GF_NAVIGATE_VR:
-			view_pan_x(compositor, cam, -dx);
-			if (keys & GF_KEY_MOD_CTRL) view_zoom(compositor, cam, dy);
-			else view_pan_y(compositor, cam, dy);
-			break;
-		case GF_NAVIGATE_PAN:
-			view_pan_x(compositor, cam, -dx);
-			if (keys & GF_KEY_MOD_CTRL) view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
-			else view_pan_y(compositor, cam, dy);
-			break;
-		case GF_NAVIGATE_SLIDE:
-			view_translate_x(compositor, cam, gf_mulfix(dx, trans_scale));
-			if (keys & GF_KEY_MOD_CTRL) view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
-			else view_translate_y(compositor, cam, gf_mulfix(dy, trans_scale));
-			break;
-		case GF_NAVIGATE_EXAMINE:
-			if (keys & GF_KEY_MOD_CTRL) {
-				view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
-				view_roll(compositor, cam, gf_mulfix(dx, trans_scale));
-			} else {
-				view_exam_x(compositor, cam, -gf_mulfix(GF_PI, dx));
-				view_exam_y(compositor, cam, gf_mulfix(GF_PI, dy));
-			}
-			break;
-		case GF_NAVIGATE_ORBIT:
-			if (keys & GF_KEY_MOD_CTRL) {
-				view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
-			} else {
-				view_orbit_x(compositor, cam, -gf_mulfix(GF_PI, dx));
-				view_orbit_y(compositor, cam, gf_mulfix(GF_PI, dy));
-			}
-			break;
-		case GF_NAVIGATE_GAME:
-			view_pan_x(compositor, cam, -dx);
-			view_pan_y(compositor, cam, dy);
-			break;
-		}
+		if (x <= -0.49) compositor->auto_rotate = 1;
+		else if (x >= 0.49) compositor->auto_rotate = 2;
+		else if (y <= -0.49) compositor->auto_rotate = 3;
+		else if (y >= 0.49) compositor->auto_rotate = 4;
+		else
+			compositor->auto_rotate = 0;
+
+		handle_mouse_move_3d(compositor, cam, keys, dx, dy);
+
 		compositor->grab_x = x;
 		compositor->grab_y = y;
 		return 1;
@@ -384,17 +476,46 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 		case GF_NAVIGATE_EXAMINE:
 		case GF_NAVIGATE_ORBIT:
 		case GF_NAVIGATE_PAN:
-			if (is_pixel_metrics) {
+			if (cam->is_3D) {
 				view_translate_z(compositor, cam, gf_mulfix(trans_scale, ev->mouse.wheel_pos) * ((keys & GF_KEY_MOD_SHIFT) ? 4 : 1));
 			} else {
-				view_translate_z(compositor, cam, ev->mouse.wheel_pos * ((keys & GF_KEY_MOD_SHIFT) ? 4 : 1));
+				nav_set_zoom_trans_2d(compositor->visual, zoom + INT2FIX(ev->mouse.wheel_pos)/10, 0, 0);
 			}
-			break;
 		}
 		return 1;
 
+	case GF_EVENT_MULTITOUCH:
+		compositor->auto_rotate=0;
+		compositor->navigation_state = 0;
+		if (ev->mtouch.num_fingers==2) {
+			if( ABS(ev->mtouch.pinch) * 100 > 2 ) {
+				if (cam->is_3D) {
+					view_translate_z(compositor, cam, gf_mulfix(cam->width, compositor->visual->width* ev->mtouch.pinch));
+				} else {
+					nav_set_zoom_trans_2d(compositor->visual, zoom + gf_mulfix(trans_scale, ev->mtouch.pinch), 0, 0);
+				}
+				return 1;
+			}
+			if( ABS(ev->mtouch.rotation) > GF_PI/40 ) {
+				view_roll(compositor, cam, gf_mulfix(ev->mtouch.rotation, trans_scale));
+				return 1;
+			}
+		} else if (ev->mtouch.num_fingers==3) {
+			compositor->visual->camera.start_zoom = compositor->zoom;
+			compositor->zoom = FIX_ONE;
+			compositor->interoccular_offset = 0;
+			compositor->focdist = 0;
+			compositor->interoccular_offset = 0;
+			compositor->focdist = 0;
+			compositor_3d_reset_camera(compositor);
+			return 1;
+		}
+		return 0;
+
 	case GF_EVENT_MOUSEUP:
-		if (ev->mouse.button==GF_MOUSE_LEFT) compositor->navigation_state = 0;
+		if (ev->mouse.button==GF_MOUSE_LEFT) {
+			compositor->navigation_state = 0;
+		}
 		break;
 
 	case GF_EVENT_KEYDOWN:
@@ -413,7 +534,15 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 			}
 			break;
 		case GF_KEY_HOME:
-			if (!compositor->navigation_state) compositor_3d_reset_camera(compositor);
+			if (!compositor->navigation_state) {
+				compositor->visual->camera.start_zoom = compositor->zoom;
+				compositor->zoom = FIX_ONE;
+				compositor->interoccular_offset = 0;
+				compositor->focdist = 0;
+				compositor->interoccular_offset = 0;
+				compositor->focdist = 0;
+				compositor_3d_reset_camera(compositor);
+			}
 			break;
 		case GF_KEY_END:
 			if (cam->navigate_mode==GF_NAVIGATE_GAME) {
@@ -422,8 +551,22 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 				return 1;
 			}
 			break;
-		case GF_KEY_LEFT: key_inv = -1;
+		case GF_KEY_LEFT:
+			key_inv = -1;
 		case GF_KEY_RIGHT:
+			if (keys & GF_KEY_MOD_ALT) {
+				if ( (keys & GF_KEY_MOD_SHIFT) && (compositor->visual->nb_views > 1) ) {
+					/*+ or - 10 cm*/
+					compositor->focdist += INT2FIX(key_inv);
+					cam->flags |= CAM_IS_DIRTY;
+					GF_LOG(GF_LOG_INFO, GF_LOG_COMPOSE, ("AutoStereo view distance %f - focus %f\n", FIX2FLT(compositor->video_out->dispdist)/100, FIX2FLT(compositor->focdist)/100));
+					gf_sc_invalidate(compositor, NULL);
+					return 1;
+				}
+				return 0;
+			}
+
+
 			switch (cam->navigate_mode) {
 			case GF_NAVIGATE_SLIDE:
 				if (keys & GF_KEY_MOD_CTRL) view_pan_x(compositor, cam, key_inv * key_pan);
@@ -437,8 +580,12 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 				if (keys & GF_KEY_MOD_CTRL) view_translate_x(compositor, cam, key_inv * key_trans);
 				else view_orbit_x(compositor, cam, -key_inv * key_exam);
 				break;
-			case GF_NAVIGATE_GAME: view_translate_x(compositor, cam, key_inv * key_trans); break;
-			case GF_NAVIGATE_VR: view_pan_x(compositor, cam, -key_inv * key_pan); break;
+			case GF_NAVIGATE_GAME:
+				view_translate_x(compositor, cam, key_inv * key_trans);
+				break;
+			case GF_NAVIGATE_VR:
+				view_pan_x(compositor, cam, -key_inv * key_pan);
+				break;
 			/*walk/fly/pan*/
 			default:
 				if (keys & GF_KEY_MOD_CTRL) view_translate_x(compositor, cam, key_inv * key_trans);
@@ -446,9 +593,19 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 				break;
 			}
 			return 1;
-		case GF_KEY_DOWN: key_inv = -1;
+		case GF_KEY_DOWN:
+			key_inv = -1;
 		case GF_KEY_UP:
-			if (keys & GF_KEY_MOD_ALT) return 0;
+			if (keys & GF_KEY_MOD_ALT) {
+				if ( (keys & GF_KEY_MOD_SHIFT) && (compositor->visual->nb_views > 1) ) {
+					compositor->interoccular_offset += FLT2FIX(0.5) * key_inv;
+					GF_LOG(GF_LOG_INFO, GF_LOG_COMPOSE, ("AutoStereo interoccular distance %f\n", FIX2FLT(compositor->iod + compositor->interoccular_offset)));
+					cam->flags |= CAM_IS_DIRTY;
+					gf_sc_invalidate(compositor, NULL);
+					return 1;
+				}
+				return 0;
+			}
 			switch (cam->navigate_mode) {
 			case GF_NAVIGATE_SLIDE:
 				if (keys & GF_KEY_MOD_CTRL) view_translate_z(compositor, cam, key_inv * key_trans);
@@ -466,7 +623,9 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 				if (keys & GF_KEY_MOD_CTRL) view_translate_y(compositor, cam, key_inv * key_trans);
 				else view_pan_y(compositor, cam, key_inv * key_pan);
 				break;
-			case GF_NAVIGATE_GAME: view_translate_z(compositor, cam, key_inv * key_trans); break;
+			case GF_NAVIGATE_GAME:
+				view_translate_z(compositor, cam, key_inv * key_trans);
+				break;
 			case GF_NAVIGATE_VR:
 				if (keys & GF_KEY_MOD_CTRL) view_zoom(compositor, cam, key_inv * key_pan);
 				else view_pan_y(compositor, cam, key_inv * key_pan);
@@ -480,27 +639,86 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 			return 1;
 
 		case GF_KEY_PAGEDOWN:
-			if (keys & GF_KEY_MOD_CTRL) { view_zoom(compositor, cam, FIX_ONE/10); return 1; }
+			if (keys & GF_KEY_MOD_CTRL) {
+				view_zoom(compositor, cam, FIX_ONE/10);
+				return 1;
+			}
 			break;
 		case GF_KEY_PAGEUP:
-			if (keys & GF_KEY_MOD_CTRL) { view_zoom(compositor, cam, -FIX_ONE/10); return 1; }
+			if (keys & GF_KEY_MOD_CTRL) {
+				view_zoom(compositor, cam, -FIX_ONE/10);
+				return 1;
+			}
+			break;
+		case GF_KEY_D:
+			if (keys & GF_KEY_MOD_CTRL) {
+				if (compositor->tvtd==TILE_DEBUG_FULL) compositor->tvtd = 0;
+				else compositor->tvtd ++;
+				gf_sc_invalidate(compositor, NULL);
+				return 1;
+			}
+			break;
+		case GF_KEY_G:
+			if (keys & GF_KEY_MOD_CTRL) {
+				compositor->vrhud_mode++;
+				if (compositor->vrhud_mode==5) compositor->vrhud_mode=0;
+				gf_sc_invalidate(compositor, NULL);
+				return 1;
+			}
+			break;
+		case GF_KEY_A:
+			if (keys & GF_KEY_MOD_CTRL) {
+				compositor->tvtf = !compositor->tvtf;
+				gf_sc_invalidate(compositor, NULL);
+				return 1;
+			}
 			break;
 		}
 		break;
+	case GF_EVENT_SENSOR_ORIENTATION:
+	{
+		Fixed z, w, yaw, /*pitch, */roll;
+		GF_Vec target;
+		GF_Matrix mx;
+
+		/* In iOS we get x, y, z in quaternions (first measurement is the frame of reference) */
+		if (ev->sensor.w) {
+			x = ev->sensor.x;
+			y = ev->sensor.y;
+			z = ev->sensor.z;
+			w = ev->sensor.w;
+		
+			yaw = gf_atan2(2*gf_mulfix(z,w) - 2*gf_mulfix(y,x) , 1 - 2*gf_mulfix(z,z) - 2*gf_mulfix(x,x));
+			//pitch = asin(2*y*z + 2*x*w);
+			roll = gf_atan2(2*gf_mulfix(y,w) - 2*gf_mulfix(z,x) , 1 - 2*gf_mulfix(y,y) - 2*gf_mulfix(x,x));
+		} else {
+			/*
+			 * In Android we get yaw, pitch, roll values (in rad)
+			 * The frame of reference is absolute
+			 */
+			yaw = ev->sensor.x;
+			//pitch = ev->sensor.y;
+			roll = ev->sensor.z;
+		}
+		target.x = 0;
+		target.y = -FIX_ONE;
+		target.z = 0;
+		gf_mx_init(mx);
+		gf_mx_add_rotation(&mx, yaw, 0, FIX_ONE, 0);
+		gf_mx_add_rotation(&mx, -roll, FIX_ONE, 0, 0);
+		
+		gf_mx_apply_vec(&mx, &target);
+
+		cam->target = target;
+		update_pan_up(compositor, cam);
+	}
+		return 1;
 	}
 	return 0;
 }
 
 #endif
 
-static void nav_set_zoom_trans_2d(GF_VisualManager *visual, Fixed zoom, Fixed dx, Fixed dy) 
-{
-	compositor_2d_set_user_transform(visual->compositor, zoom, visual->compositor->trans_x + dx, visual->compositor->trans_y + dy, 0); 
-#ifndef GPAC_DISABLE_3D
-	if (visual->type_3d) camera_changed(visual->compositor, &visual->camera);
-#endif
-}
-
 
 static Bool compositor_handle_navigation_2d(GF_VisualManager *visual, GF_Event *ev)
 {
@@ -516,18 +734,26 @@ static Bool compositor_handle_navigation_2d(GF_VisualManager *visual, GF_Event *
 	if (visual->type_3d) navigation_mode = visual->camera.navigate_mode;
 #endif
 
-	if (!navigation_mode /*&& !(keys & GF_KEY_MOD_ALT) */) return 0;
+	if (navigation_mode==GF_NAVIGATE_NONE) return 0;
+	if (!navigation_mode && !(keys & GF_KEY_MOD_ALT) ) return 0;
 
 
 	x = y = 0;
 	/*renorm between -1, 1*/
-	if (ev->type<=GF_EVENT_MOUSEWHEEL) {
+	if (ev->type <= GF_EVENT_LAST_MOUSE) {
 		x = INT2FIX(ev->mouse.x);
 		y = INT2FIX(ev->mouse.y);
 	}
 	dx = x - visual->compositor->grab_x;
-	dy = y - visual->compositor->grab_y;
-	if (!is_pixel_metrics) { dx /= visual->width; dy /= visual->height; }
+	if (visual->center_coords) {
+		dy = visual->compositor->grab_y - y;
+	} else {
+		dy = y - visual->compositor->grab_y;
+	}
+	if (!is_pixel_metrics) {
+		dx /= visual->width;
+		dy /= visual->height;
+	}
 
 	key_inv = 1;
 	key_trans = INT2FIX(2);
@@ -540,7 +766,9 @@ static Bool compositor_handle_navigation_2d(GF_VisualManager *visual, GF_Event *
 		key_trans*=4;
 	}
 
-	if (!is_pixel_metrics) { key_trans /= visual->width;}
+	if (!is_pixel_metrics) {
+		key_trans /= visual->width;
+	}
 
 	switch (ev->type) {
 	case GF_EVENT_MOUSEDOWN:
@@ -551,13 +779,8 @@ static Bool compositor_handle_navigation_2d(GF_VisualManager *visual, GF_Event *
 			visual->compositor->navigation_state = 1;
 			/*update zoom center*/
 			if (keys & GF_KEY_MOD_CTRL) {
-				if (visual->center_coords) {
-					visual->compositor->trans_x -= visual->compositor->grab_x;
-					visual->compositor->trans_y -= visual->compositor->grab_y;
-				} else {
-					visual->compositor->trans_x -= visual->compositor->grab_x - INT2FIX(visual->width)/2;
-					visual->compositor->trans_y += INT2FIX(visual->height)/2 - visual->compositor->grab_y;
-				}
+				visual->compositor->trans_x -= visual->compositor->grab_x - INT2FIX(visual->width)/2;
+				visual->compositor->trans_y += INT2FIX(visual->height)/2 - visual->compositor->grab_y;
 				nav_set_zoom_trans_2d(visual, visual->compositor->zoom, 0, 0);
 			}
 			return 0;
@@ -570,12 +793,42 @@ static Bool compositor_handle_navigation_2d(GF_VisualManager *visual, GF_Event *
 			return 0;
 		}
 		break;
+	case GF_EVENT_MULTITOUCH:
+		if (ev->mtouch.num_fingers==2) {
+			if( ABS(ev->mtouch.pinch) * 100 > 2 ) {
+				new_zoom = zoom + ev->mtouch.pinch * MIN(visual->width, visual->height)/100;
+				nav_set_zoom_trans_2d(visual, new_zoom, 0, 0);
+				return 1;
+			}
+			if( ABS(ev->mtouch.rotation) > GF_PI/40 ) {
+				visual->compositor->rotation -= ev->mtouch.rotation;
+				nav_set_zoom_trans_2d(visual, zoom, 0, 0);
+				return 1;
+			}
+		} else if (ev->mtouch.num_fingers==3) {
+			visual->compositor->trans_x = visual->compositor->trans_y = 0;
+			visual->compositor->rotation = 0;
+			visual->compositor->zoom = FIX_ONE;
+			nav_set_zoom_trans_2d(visual, FIX_ONE, 0, 0);
+			return 1;
+		}
+		return 0;
 
 	case GF_EVENT_MOUSEWHEEL:
-		if (navigation_mode != GF_NAVIGATE_SLIDE) return 0;
-		new_zoom = zoom + INT2FIX(ev->mouse.wheel_pos)/10;
-		nav_set_zoom_trans_2d(visual, new_zoom, 0, 0);
-		return 1;
+		switch (navigation_mode) {
+		case GF_NAVIGATE_SLIDE:
+			new_zoom = zoom + INT2FIX(ev->mouse.wheel_pos)/10;
+			nav_set_zoom_trans_2d(visual, new_zoom, 0, 0);
+			return 1;
+		case GF_NAVIGATE_EXAMINE:
+			if (ev->mouse.wheel_pos>0)
+				visual->compositor->rotation += gf_asin( GF_PI / 10);
+			else
+				visual->compositor->rotation -= gf_asin( GF_PI / 10);
+			nav_set_zoom_trans_2d(visual, zoom, 0, 0);
+			return 1;
+		}
+		return 0;
 
 	case GF_EVENT_MOUSEMOVE:
 		if (!visual->compositor->navigation_state) return 0;
@@ -596,10 +849,15 @@ static Bool compositor_handle_navigation_2d(GF_VisualManager *visual, GF_Event *
 		case GF_NAVIGATE_EXAMINE:
 		{
 			Fixed sin = gf_mulfix(GF_PI, dy) / visual->height;
+			//truncate in [-1;1] for arcsin()
+			if (sin < -FIX_ONE)
+				sin = -FIX_ONE;
+			if (sin >  FIX_ONE)
+				sin =  FIX_ONE;
 			visual->compositor->rotation += gf_asin(sin);
 			nav_set_zoom_trans_2d(visual, zoom, 0, 0);
 		}
-			break;
+		break;
 		}
 		visual->compositor->grab_x = x;
 		visual->compositor->grab_y = y;
@@ -617,7 +875,8 @@ static Bool compositor_handle_navigation_2d(GF_VisualManager *visual, GF_Event *
 				nav_set_zoom_trans_2d(visual, FIX_ONE, 0, 0);
 			}
 			return 1;
-		case GF_KEY_LEFT: key_inv = -1;
+		case GF_KEY_LEFT:
+			key_inv = -1;
 		case GF_KEY_RIGHT:
 			if (navigation_mode == GF_NAVIGATE_SLIDE) {
 				nav_set_zoom_trans_2d(visual, zoom, key_inv*key_trans, 0);
@@ -627,11 +886,12 @@ static Bool compositor_handle_navigation_2d(GF_VisualManager *visual, GF_Event *
 				nav_set_zoom_trans_2d(visual, zoom, 0, 0);
 			}
 			return 1;
-		case GF_KEY_DOWN: key_inv = -1;
+		case GF_KEY_DOWN:
+			key_inv = -1;
 		case GF_KEY_UP:
 			if (navigation_mode == GF_NAVIGATE_SLIDE) {
 				if (keys & GF_KEY_MOD_CTRL) {
-					Fixed new_zoom = zoom;
+					new_zoom = zoom;
 					if (new_zoom > FIX_ONE) new_zoom += key_inv*FIX_ONE/10;
 					else new_zoom += key_inv*FIX_ONE/20;
 					nav_set_zoom_trans_2d(visual, new_zoom, 0, 0);
@@ -650,14 +910,52 @@ static Bool compositor_handle_navigation_2d(GF_VisualManager *visual, GF_Event *
 	return 0;
 }
 
+void compositor_handle_auto_navigation(GF_Compositor *compositor)
+{
+#ifndef GPAC_DISABLE_3D
+	GF_Camera *cam = NULL;
+	Fixed dx, dy;
+#ifndef GPAC_DISABLE_VRML
+	if (compositor->active_layer) {
+		cam = compositor_layer3d_get_camera(compositor->active_layer);
+	}
+	if (!cam)
+		cam = &compositor->visual->camera;
+#endif
+
+	dx = dy = 0;
+	switch (compositor->auto_rotate) {
+	case 1:
+		dx = -FIX_ONE/100;
+		break;
+	case 2:
+		dx = FIX_ONE/100;
+		break;
+	case 3:
+		dy = FIX_ONE/100;
+		if (cam->up.z>0.9999) compositor->auto_rotate=4;
+		break;
+	case 4:
+		dy = -FIX_ONE/100;
+		if (cam->up.z<-0.9999) compositor->auto_rotate=3;
+		break;
+	default:
+		return;
+	}
+	if (compositor->gazer_enabled)
+		compositor->gaze_changed = GF_FALSE;
+	handle_mouse_move_3d(compositor, cam, 0, dx, dy);
+#endif
+}
 
 Bool compositor_handle_navigation(GF_Compositor *compositor, GF_Event *ev)
 {
 	if (!compositor->scene) return 0;
 #ifndef GPAC_DISABLE_3D
-	if ( (compositor->visual->type_3d>1) || compositor->active_layer) {
+	if ( (compositor->visual->type_3d>0) || compositor->active_layer)
 		return compositor_handle_navigation_3d(compositor, ev);
-	}
 #endif
 	return compositor_handle_navigation_2d(compositor->visual, ev);
 }
+
+#endif //!defined(GPAC_DISABLE_COMPOSITOR)
