@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Compositor sub-project
@@ -10,15 +11,15 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
@@ -27,6 +28,8 @@
 #include "visual_manager.h"
 #include "drawable.h"
 
+#if !defined(GPAC_DISABLE_X3D) && !defined(GPAC_DISABLE_COMPOSITOR)
+
 static void disk2d_check_changes(GF_Node *node, Drawable *stack, GF_TraverseState *tr_state)
 {
 	if (gf_node_dirty_get(node)) {
@@ -68,7 +71,7 @@ static void TraverseDisk2D(GF_Node *node, void *rs, Bool is_destroy)
 		gf_path_get_bounds(stack->path, &tr_state->bounds);
 		return;
 	case TRAVERSE_PICK:
-		drawable_pick(stack, tr_state);
+		vrml_drawable_pick(stack, tr_state);
 		return;
 	case TRAVERSE_SORT:
 #ifndef GPAC_DISABLE_3D
@@ -136,7 +139,7 @@ static void TraverseArc2D(GF_Node *node, void *rs, Bool is_destroy)
 #endif
 		return;
 	case TRAVERSE_PICK:
-		drawable_pick(stack, tr_state);
+		vrml_drawable_pick(stack, tr_state);
 		return;
 	case TRAVERSE_SORT:
 #ifndef GPAC_DISABLE_3D
@@ -199,7 +202,7 @@ static void TraversePolyline2D(GF_Node *node, void *rs, Bool is_destroy)
 		gf_path_get_bounds(stack->path, &tr_state->bounds);
 		return;
 	case TRAVERSE_PICK:
-		drawable_pick(stack, tr_state);
+		vrml_drawable_pick(stack, tr_state);
 		return;
 	case TRAVERSE_SORT:
 #ifndef GPAC_DISABLE_3D
@@ -261,9 +264,10 @@ static void TraverseTriangleSet2D(GF_Node *node, void *rs, Bool is_destroy)
 
 			stack->mesh = new_mesh();
 			stack->mesh->mesh_type = MESH_TRIANGLES;
-			col.red = col.green = col.blue = 0; col.alpha = FIX_ONE;
+			col.red = col.green = col.blue = 0;
+			col.alpha = FIX_ONE;
 			v1.color = MESH_MAKE_COL(col);
-			v1.normal.x = v1.normal.y = 0; 
+			v1.normal.x = v1.normal.y = 0;
 			v1.normal.z = MESH_NORMAL_UNIT;
 			v1.pos.z = 0;
 			v3 = v2 = v1;
@@ -299,7 +303,7 @@ static void TraverseTriangleSet2D(GF_Node *node, void *rs, Bool is_destroy)
 		gf_path_get_bounds(stack->path, &tr_state->bounds);
 		return;
 	case TRAVERSE_PICK:
-		drawable_pick(stack, tr_state);
+		vrml_drawable_pick(stack, tr_state);
 		return;
 	case TRAVERSE_SORT:
 #ifndef GPAC_DISABLE_3D
@@ -327,7 +331,8 @@ static void build_polypoint2d(GF_Node *node, Drawable3D *stack, GF_TraverseState
 	X_Polypoint2D *p = (X_Polypoint2D *)node;
 
 	stack->mesh->mesh_type = MESH_POINTSET;
-	col.red = col.green = col.blue = 0; col.alpha = FIX_ONE;
+	col.red = col.green = col.blue = 0;
+	col.alpha = FIX_ONE;
 	for (i=0; i<p->point.count; i++) {
 		mesh_set_point(stack->mesh, p->point.vals[i].x, p->point.vals[i].y, 0, col);
 		mesh_set_index(stack->mesh, stack->mesh->v_count-1);
@@ -417,9 +422,9 @@ static void BuildTriangleSet(GF_Mesh *mesh, GF_Node *_coords, GF_Node *_color, G
 	Bool rgba_col;
 	SFColorRGBA rgba;
 	X_Coordinate *c = (X_Coordinate *) _coords;
-	
+
 	mesh_reset(mesh);
-	
+
 	cols = NULL;
 	rgba_col = 0;
 	if (_color) {
@@ -478,13 +483,22 @@ static void BuildTriangleSet(GF_Mesh *mesh, GF_Node *_coords, GF_Node *_color, G
 		}
 		if (txcoords) {
 			if (txcoords->count>idx) vx.texcoords = txcoords->vals[idx];
-		} 
+		}
 		/*X3D says nothing about default texture mapping here...*/
 		else if (!generate_tx) {
 			switch (i%3) {
-			case 2: vx.texcoords.x = FIX_ONE; vx.texcoords.y = 0; break;
-			case 1: vx.texcoords.x = FIX_ONE/2; vx.texcoords.y = FIX_ONE; break;
-			case 0: vx.texcoords.x = 0; vx.texcoords.y = 0; break;
+			case 2:
+				vx.texcoords.x = FIX_ONE;
+				vx.texcoords.y = 0;
+				break;
+			case 1:
+				vx.texcoords.x = FIX_ONE/2;
+				vx.texcoords.y = FIX_ONE;
+				break;
+			case 0:
+				vx.texcoords.x = 0;
+				vx.texcoords.y = 0;
+				break;
 			}
 		}
 		mesh_set_vertex_vx(mesh, &vx);
@@ -535,11 +549,13 @@ static void TraverseIndexedTriangleSet(GF_Node *node, void *rs, Bool is_destroy)
 	drawable_3d_base_traverse(node, rs, is_destroy, build_indexed_triangle_set);
 }
 
-static void ITS_SetIndex(GF_Node *node)
+static void ITS_SetIndex(GF_Node *node, GF_Route *route)
 {
-	X_IndexedTriangleSet *its = (X_IndexedTriangleSet*)node;
-	gf_sg_vrml_field_copy(&its->index, &its->set_index, GF_SG_VRML_MFINT32);
-	gf_sg_vrml_mf_reset(&its->set_index, GF_SG_VRML_MFINT32);
+	if (node) {
+		X_IndexedTriangleSet *its = (X_IndexedTriangleSet*)node;
+		gf_sg_vrml_field_copy(&its->index, &its->set_index, GF_SG_VRML_MFINT32);
+		gf_sg_vrml_mf_reset(&its->set_index, GF_SG_VRML_MFINT32);
+	}
 }
 
 void compositor_init_indexed_triangle_set(GF_Compositor *compositor, GF_Node *node)
@@ -548,6 +564,13 @@ void compositor_init_indexed_triangle_set(GF_Compositor *compositor, GF_Node *no
 	drawable_3d_new(node);
 	gf_node_set_callback_function(node, TraverseIndexedTriangleSet);
 	its->on_set_index = ITS_SetIndex;
+
+#ifdef GPAC_ENABLE_COVERAGE
+	if (gf_sys_is_cov_mode()) {
+		ITS_SetIndex(NULL, NULL);
+	}
+#endif
+
 }
 
 
@@ -561,9 +584,9 @@ static void BuildTriangleStripSet(GF_Mesh *mesh, GF_Node *_coords, GF_Node *_col
 	Bool rgba_col;
 	SFColorRGBA rgba;
 	X_Coordinate *c = (X_Coordinate *) _coords;
-	
+
 	mesh_reset(mesh);
-	
+
 	cols = NULL;
 	rgba_col = 0;
 	if (_color) {
@@ -627,13 +650,22 @@ static void BuildTriangleStripSet(GF_Mesh *mesh, GF_Node *_coords, GF_Node *_col
 			}
 			if (txcoords) {
 				if (txcoords->count>idx) vx.texcoords = txcoords->vals[idx];
-			} 
+			}
 			/*X3D says nothing about default texture mapping here...*/
 			else if (!generate_tx) {
 				switch (idx%3) {
-				case 2: vx.texcoords.x = FIX_ONE; vx.texcoords.y = 0; break;
-				case 1: vx.texcoords.x = FIX_ONE/2; vx.texcoords.y = FIX_ONE; break;
-				case 0: vx.texcoords.x = 0; vx.texcoords.y = 0; break;
+				case 2:
+					vx.texcoords.x = FIX_ONE;
+					vx.texcoords.y = 0;
+					break;
+				case 1:
+					vx.texcoords.x = FIX_ONE/2;
+					vx.texcoords.y = FIX_ONE;
+					break;
+				case 0:
+					vx.texcoords.x = 0;
+					vx.texcoords.y = 0;
+					break;
 				}
 			}
 			if (i>2) {
@@ -687,7 +719,10 @@ static void BuildTriangleStripSet(GF_Mesh *mesh, GF_Node *_coords, GF_Node *_col
 				SFVec3f n_0, n_1, n_2, n_avg;
 				u32 nb_face;
 				if (stripList->vals[strip] < 3) continue;
-				if (stripList->vals[strip] <= 3) { cur_face ++; continue; }
+				if (stripList->vals[strip] <= 3) {
+					cur_face ++;
+					continue;
+				}
 
 				/*first face normal*/
 				MESH_GET_NORMAL(n_0, mesh->vertices[mesh->indices[3*cur_face]]);
@@ -751,7 +786,8 @@ static void build_indexed_triangle_strip_set(GF_Node *node, Drawable3D *stack, G
 
 	if (!itss->coord) return;
 
-	stripList.count = 0; stripList.vals = NULL;
+	stripList.count = 0;
+	stripList.vals = NULL;
 	nb_strips = 0;
 	for (i=0; i<itss->index.count; i++) {
 		if (itss->index.vals[i]==-1) {
@@ -779,11 +815,13 @@ static void TraverseIndexedTriangleStripSet(GF_Node *node, void *rs, Bool is_des
 	drawable_3d_base_traverse(node, rs, is_destroy, build_indexed_triangle_strip_set);
 }
 
-static void ITSS_SetIndex(GF_Node *node)
+static void ITSS_SetIndex(GF_Node *node, GF_Route *route)
 {
-	X_IndexedTriangleStripSet *itss = (X_IndexedTriangleStripSet*)node;
-	gf_sg_vrml_field_copy(&itss->index, &itss->set_index, GF_SG_VRML_MFINT32);
-	gf_sg_vrml_mf_reset(&itss->set_index, GF_SG_VRML_MFINT32);
+	if (node) {
+		X_IndexedTriangleStripSet *itss = (X_IndexedTriangleStripSet*)node;
+		gf_sg_vrml_field_copy(&itss->index, &itss->set_index, GF_SG_VRML_MFINT32);
+		gf_sg_vrml_mf_reset(&itss->set_index, GF_SG_VRML_MFINT32);
+	}
 }
 
 void compositor_init_indexed_triangle_strip_set(GF_Compositor *compositor, GF_Node *node)
@@ -792,6 +830,12 @@ void compositor_init_indexed_triangle_strip_set(GF_Compositor *compositor, GF_No
 	drawable_3d_new(node);
 	gf_node_set_callback_function(node, TraverseIndexedTriangleStripSet);
 	itss->on_set_index = ITSS_SetIndex;
+
+#ifdef GPAC_ENABLE_COVERAGE
+	if (gf_sys_is_cov_mode()) {
+		ITSS_SetIndex(NULL, NULL);
+	}
+#endif
 }
 
 
@@ -806,7 +850,7 @@ static void BuildTriangleFanSet(GF_Mesh *mesh, GF_Node *_coords, GF_Node *_color
 	SFColorRGBA rgba;
 	X_Coordinate *c = (X_Coordinate *) _coords;
 	mesh_reset(mesh);
-		
+
 	cols = NULL;
 	rgba_col = 0;
 	if (_color) {
@@ -870,13 +914,22 @@ static void BuildTriangleFanSet(GF_Mesh *mesh, GF_Node *_coords, GF_Node *_color
 			}
 			if (txcoords) {
 				if (txcoords->count>idx) vx.texcoords = txcoords->vals[idx];
-			} 
+			}
 			/*X3D says nothing about default texture mapping here...*/
 			else if (!generate_tx) {
 				switch (idx%3) {
-				case 2: vx.texcoords.x = FIX_ONE; vx.texcoords.y = 0; break;
-				case 1: vx.texcoords.x = FIX_ONE/2; vx.texcoords.y = FIX_ONE; break;
-				case 0: vx.texcoords.x = 0; vx.texcoords.y = 0; break;
+				case 2:
+					vx.texcoords.x = FIX_ONE;
+					vx.texcoords.y = 0;
+					break;
+				case 1:
+					vx.texcoords.x = FIX_ONE/2;
+					vx.texcoords.y = FIX_ONE;
+					break;
+				case 0:
+					vx.texcoords.x = 0;
+					vx.texcoords.y = 0;
+					break;
 				}
 			}
 			mesh_set_vertex_vx(mesh, &vx);
@@ -885,7 +938,7 @@ static void BuildTriangleFanSet(GF_Mesh *mesh, GF_Node *_coords, GF_Node *_color
 			if (indices) {
 				if (cur_idx>=indices->count) break;
 			} else if (cur_idx==c->point.count) break;
-			
+
 			if (i>1) {
 				mesh_set_vertex_vx(mesh, &mesh->vertices[start_idx]);
 				mesh_set_vertex_vx(mesh, &vx);
@@ -910,8 +963,11 @@ static void BuildTriangleFanSet(GF_Mesh *mesh, GF_Node *_coords, GF_Node *_color
 				SFVec3f n_0, n_1, n_avg, n_tot;
 				u32 nb_face, start_face;
 				if (fanList->vals[fan] < 3) continue;
-				if (fanList->vals[fan] == 3) { cur_face++; continue; }
-	
+				if (fanList->vals[fan] == 3) {
+					cur_face++;
+					continue;
+				}
+
 				start_face = cur_face;
 
 				/*first face normal*/
@@ -967,7 +1023,8 @@ static void build_indexed_triangle_fan_set(GF_Node *node, Drawable3D *stack, GF_
 	gf_node_dirty_clear(node, 0);
 	if (!itfs->coord) return;
 
-	fanList.count = 0; fanList.vals = NULL;
+	fanList.count = 0;
+	fanList.vals = NULL;
 	nb_fans = 0;
 	for (i=0; i<itfs->index.count; i++) {
 		if (itfs->index.vals[i]==-1) {
@@ -995,11 +1052,13 @@ static void TraverseIndexedTriangleFanSet(GF_Node *node, void *rs, Bool is_destr
 	drawable_3d_base_traverse(node, rs, is_destroy, build_indexed_triangle_fan_set);
 }
 
-static void ITFS_SetIndex(GF_Node *node)
+static void ITFS_SetIndex(GF_Node *node, GF_Route *route)
 {
-	X_IndexedTriangleFanSet *itfs = (X_IndexedTriangleFanSet *)node;
-	gf_sg_vrml_field_copy(&itfs->index, &itfs->set_index, GF_SG_VRML_MFINT32);
-	gf_sg_vrml_mf_reset(&itfs->set_index, GF_SG_VRML_MFINT32);
+	if (node) {
+		X_IndexedTriangleFanSet *itfs = (X_IndexedTriangleFanSet *)node;
+		gf_sg_vrml_field_copy(&itfs->index, &itfs->set_index, GF_SG_VRML_MFINT32);
+		gf_sg_vrml_mf_reset(&itfs->set_index, GF_SG_VRML_MFINT32);
+	}
 }
 
 void compositor_init_indexed_triangle_fan_set(GF_Compositor *compositor, GF_Node *node)
@@ -1008,6 +1067,14 @@ void compositor_init_indexed_triangle_fan_set(GF_Compositor *compositor, GF_Node
 	drawable_3d_new(node);
 	gf_node_set_callback_function(node, TraverseIndexedTriangleFanSet);
 	itfs->on_set_index = ITFS_SetIndex;
+
+#ifdef GPAC_ENABLE_COVERAGE
+	if (gf_sys_is_cov_mode()) {
+		ITFS_SetIndex(NULL, NULL);
+	}
+#endif
 }
 
 #endif /*GPAC_DISABLE_3D*/
+
+#endif // !defined(GPAC_DISABLE_X3D) && !defined(GPAC_DISABLE_COMPOSITOR)
