@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2017-2022
+ *			Copyright (c) Telecom ParisTech 2017-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / ffmpeg decode filter
@@ -247,10 +247,14 @@ static GF_Err ffdec_process_video(GF_Filter *filter, struct _gf_ffdec_ctx *ctx)
         return GF_EOS;
     }
 
+restart:
+
 	frame = ctx->frame;
 
 	FF_INIT_PCK(ctx, pkt)
 
+	//reset as we may get re-called from goto
+	pck_src=NULL;
 	if (pck) {
 		data = gf_filter_pck_get_data(pck, &size);
 
@@ -311,7 +315,7 @@ static GF_Err ffdec_process_video(GF_Filter *filter, struct _gf_ffdec_ctx *ctx)
 	case AVERROR_EOF:
 		break;
 	default:
-		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[FFDec] PID %s failed to send frame PTS "LLU": %s\n", gf_filter_pid_get_name(ctx->in_pid), pkt->pts, av_err2str(res) ));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[FFDec] PID %s failed to queue packet PTS "LLU": %s\n", gf_filter_pid_get_name(ctx->in_pid), pkt->pts, av_err2str(res) ));
 		break;
 	}
 	gotpic = 0;
@@ -535,6 +539,9 @@ static GF_Err ffdec_process_video(GF_Filter *filter, struct _gf_ffdec_ctx *ctx)
 		gf_filter_pck_set_interlaced(dst_pck, frame->top_field_first ? 2 : 1);
 
 	gf_filter_pck_send(dst_pck);
+
+	if (!pck) goto restart;
+
 	return GF_OK;
 }
 
@@ -570,7 +577,7 @@ decode_next:
 
 	if (pck) pkt->data = (uint8_t *) gf_filter_pck_get_data(pck, &in_size);
 
-	if (pck) {
+	if (pck && in_size) {
 		src_pck = pck;
 		gf_filter_pck_ref_props(&src_pck);
 		if (src_pck) gf_list_add(ctx->src_packets, src_pck);
@@ -602,6 +609,7 @@ decode_next:
 
 	} else {
 		pkt->size = 0;
+		pkt->data = NULL;
 	}
 
 	prev_afmt = ctx->decoder->sample_fmt;
@@ -705,8 +713,13 @@ dispatch_next:
 	FF_RELEASE_PCK(pkt);
 
 
+#ifdef FFMPEG_OLD_CHLAYOUT
 	FF_CHECK_PROP(channels, channels, GF_PROP_PID_NUM_CHANNELS)
 	FF_CHECK_PROPL(channel_layout, channel_layout, GF_PROP_PID_CHANNEL_LAYOUT)
+#else
+	FF_CHECK_PROP(channels, ch_layout.nb_channels, GF_PROP_PID_NUM_CHANNELS)
+	FF_CHECK_PROPL(channel_layout, ch_layout.u.mask, GF_PROP_PID_CHANNEL_LAYOUT)
+#endif
 	FF_CHECK_PROP(sample_rate, sample_rate, GF_PROP_PID_SAMPLE_RATE)
 
 	if (prev_afmt != ctx->decoder->sample_fmt) {
@@ -814,10 +827,10 @@ dispatch_next:
 	if (pck) gf_filter_pid_drop_packet(ctx->in_pid);
 #endif
 
+	if (gf_filter_pid_would_block(ctx->out_pid))
+		return GF_OK;
 	//avoid recursion
 	goto decode_next;
-
-//	return ffdec_process_audio(filter, ctx);
 }
 
 #ifndef FFMPEG_NO_SUBS
@@ -1058,7 +1071,7 @@ static GF_Err ffdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 			prop = gf_filter_pid_get_property(pid, GF_PROP_PID_UNFRAMED);
 			if (!prop || !prop->value.boolean) {
 				gf_filter_override_caps(filter, FFDecodeAnnexBCaps, GF_ARRAY_LENGTH(FFDecodeAnnexBCaps));
-				gf_filter_pid_negociate_property(ctx->in_pid, GF_PROP_PID_UNFRAMED, &PROP_BOOL(GF_TRUE) );
+				gf_filter_pid_negotiate_property(ctx->in_pid, GF_PROP_PID_UNFRAMED, &PROP_BOOL(GF_TRUE) );
 				return GF_OK;
 			}
 		}
@@ -1194,7 +1207,11 @@ static GF_Err ffdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 		}
 		if (ctx->sample_rate && ctx->channels) {
 			ctx->decoder->sample_rate = ctx->sample_rate;
+#ifdef FFMPEG_OLD_CHLAYOUT
 			ctx->decoder->channels = ctx->channels;
+#else
+			ctx->decoder->ch_layout.nb_channels = ctx->channels;
+#endif
 		}
 	}
 
@@ -1241,6 +1258,8 @@ static GF_Err ffdec_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 
 	//clone options (in case we need to destroy/recreate the codec) and open codec
 	av_dict_copy(&options, ctx->options, 0);
+	ffmpeg_check_threads(filter, options, ctx->decoder);
+
 	res = avcodec_open2(ctx->decoder, codec, &options);
 	if (res < 0) {
 		if (options) av_dict_free(&options);
@@ -1342,13 +1361,23 @@ reuse_codec_context:
 			ctx->bytes_per_sample = gf_audio_fmt_bit_depth(ctx->sample_fmt) / 8;
 		}
 
+		u32 nb_ch=0;
+		u64 ff_ch_layout=0;
+#ifdef FFMPEG_OLD_CHLAYOUT
+		nb_ch = ctx->decoder->channels;
+		ff_ch_layout = ctx->decoder->channel_layout;
+#else
+		nb_ch = ctx->decoder->ch_layout.nb_channels;
+		ff_ch_layout = (ctx->decoder->ch_layout.order>=AV_CHANNEL_ORDER_CUSTOM) ? 0 : ctx->decoder->ch_layout.u.mask;
+#endif
+
 		//override PID props with what decoder gives us
-		if (ctx->decoder->channels) {
-			ctx->channels = 0;
-			FF_CHECK_PROP(channels, channels, GF_PROP_PID_NUM_CHANNELS)
+		if (nb_ch) {
+			gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_NUM_CHANNELS, &PROP_UINT(nb_ch ) );
+			ctx->channels = nb_ch;
 		}
-		if (ctx->decoder->channel_layout) {
-			u64 ch_lay = ffmpeg_channel_layout_to_gpac(ctx->decoder->channel_layout);
+		if (ff_ch_layout) {
+			u64 ch_lay = ffmpeg_channel_layout_to_gpac(ff_ch_layout);
 			if (ctx->channel_layout != ch_lay) {
 				gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_CHANNEL_LAYOUT, &PROP_LONGUINT(ch_lay ) );
 				ctx->channel_layout = ch_lay;
@@ -1443,6 +1472,7 @@ static const GF_FilterCapability FFDecodeCaps[] =
 	CAP_UINT(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_LHVC),
 	CAP_UINT(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_HEVC_TILES),
 	CAP_UINT(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_VVC_SUBPIC),
+	CAP_UINT(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_HEVC_MERGE),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_CODECID, GF_CODECID_RAW),
 	CAP_BOOL(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_TILE_BASE, GF_TRUE),
 	{ .code=GF_PROP_PID_SCALABLE, .val={.type=GF_PROP_BOOL, .value.boolean = GF_TRUE}, .flags=(GF_CAPS_INPUT_OPT), .priority=255 },
@@ -1498,7 +1528,7 @@ GF_FilterRegister FFDecodeRegister = {
 	.process = ffdec_process,
 	.update_arg = ffdec_update_arg,
 	.process_event = ffdec_process_event,
-	.flags = GF_FS_REG_META,
+	.flags = GF_FS_REG_META|GF_FS_REG_BLOCK_MAIN,
 	//use middle priorty, so that hardware decs/other native impl in gpac can take over if needed
 	//don't use lowest one since we use this for scalable codecs
 	.priority = 128
@@ -1515,9 +1545,11 @@ static const GF_FilterArgs FFDecodeArgs[] =
 	{0}
 };
 
+const int FFDEC_STATIC_ARGS = (sizeof (FFDecodeArgs) / sizeof (GF_FilterArgs)) - 1;
+
 const GF_FilterRegister *ffdec_register(GF_FilterSession *session)
 {
-	return ffmpeg_build_register(session, &FFDecodeRegister, FFDecodeArgs, 1, FF_REG_TYPE_DECODE);
+	return ffmpeg_build_register(session, &FFDecodeRegister, FFDecodeArgs, FFDEC_STATIC_ARGS, FF_REG_TYPE_DECODE);
 }
 
 #else
