@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2020-2022
+ *			Copyright (c) Telecom ParisTech 2020-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / MHAS reframer filter
@@ -27,7 +27,7 @@
 #include <gpac/constants.h>
 #include <gpac/filters.h>
 
-#if !defined(GPAC_DISABLE_AV_PARSERS)
+#if !defined(GPAC_DISABLE_AV_PARSERS) && !defined(GPAC_DISABLE_RFMHAS)
 
 
 static u32 USACSampleRates[] = {96000, 88200, 64000, 48000, 44100, 32000, 24000, 22050, 16000, 12000, 11025, 8000, 7350, 0, 0,
@@ -198,12 +198,11 @@ static void mhas_dmx_check_dur(GF_Filter *filter, GF_MHASDmxCtx *ctx)
 			if (!mhas_last_cfg) mhas_sap = 0;
 		//config
 		} else if (mhas_type==1) {
-			u32 sr = 0;
 			/*u32 pl = */gf_bs_read_u8(bs);
 			u32 idx = gf_bs_read_int(bs, 5);
 			if (idx==0x1f)
 				duration.den = gf_bs_read_int(bs, 24);
-			else if (sr < nb_usac_sr) {
+			else if (idx < nb_usac_sr) {
 				duration.den = USACSampleRates[idx];
 			}
 			idx = gf_bs_read_int(bs, 3);
@@ -543,7 +542,7 @@ GF_Err mhas_dmx_process(GF_Filter *filter)
 			remain=0;
 			break;
 		}
-		if ((hdr_start[1]==0x01) && (hdr_start[2]==0xA5)) {
+		if ( (u32)(hdr_start-start)+3 <= remain  && (hdr_start[1]==0x01) && (hdr_start[2]==0xA5)) {
 			GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[MHASDmx] Sync found !\n"));
 			ctx->nosync = GF_FALSE;
 			break;
@@ -621,7 +620,7 @@ GF_Err mhas_dmx_process(GF_Filter *filter)
 			u32 idx = gf_bs_read_int(ctx->bs, 5);
 			if (idx==0x1f)
 				sr = gf_bs_read_int(ctx->bs, 24);
-			else if (sr < nb_usac_sr) {
+			else if (idx < nb_usac_sr) {
 				sr = USACSampleRates[idx];
 			}
 			ctx->nb_unknown_pck = 0;
@@ -639,10 +638,11 @@ GF_Err mhas_dmx_process(GF_Filter *filter)
 				numSpeakers = (s32) gf_mpegh_escaped_value(ctx->bs, 5, 8, 16) + 1;
 				//TODO ...
 			}
+			if (sr) {
+				mhas_dmx_check_pid(filter, ctx, pl, sr, frame_len, CICPspeakerLayoutIdx, numSpeakers, start + pay_start, (u32) mhas_size);
 
-			mhas_dmx_check_pid(filter, ctx, pl, sr, frame_len, CICPspeakerLayoutIdx, numSpeakers, start + pay_start, (u32) mhas_size);
-
-			has_cfg = GF_TRUE;
+				has_cfg = GF_TRUE;
+			}
 		}
 		//audio truncation
 		else if (mhas_type==17) {
@@ -714,8 +714,7 @@ GF_Err mhas_dmx_process(GF_Filter *filter)
 			}
 
 			if (ctx->timescale) {
-				pck_dur *= ctx->timescale;
-				pck_dur /= ctx->sample_rate;
+				pck_dur = gf_timestamp_rescale(pck_dur, ctx->sample_rate, ctx->timescale);
 			}
 
 			dst = gf_filter_pck_new_alloc(ctx->opid, au_size, &output);
@@ -774,9 +773,12 @@ GF_Err mhas_dmx_process(GF_Filter *filter)
 		}
 	}
 	if (consumed) {
-		assert(remain>=consumed);
-		remain -= consumed;
-		start += consumed;
+		if (remain>=consumed) {
+			remain -= consumed;
+			start += consumed;
+		} else {
+			remain=0;
+		}
 	}
 
 skip:
@@ -837,6 +839,8 @@ static const char *mhas_dmx_probe_data(const u8 *data, u32 size, GF_FilterProbeS
 		u32 pos = (u32) (ptr - data);
 		const u8 *sync_start = memchr(ptr, 0xC0, size - pos);
 		if (!sync_start) return NULL;
+		u32 remain = size - (u32) (sync_start-data);
+		if (remain<2) return NULL;
 		if ((sync_start[1]== 0x01) && (sync_start[2]==0xA5)) {
 			sync_pos = pos;
 			break;
@@ -915,15 +919,13 @@ GF_FilterRegister MHASDmxRegister = {
 };
 
 
-const GF_FilterRegister *mhas_dmx_register(GF_FilterSession *session)
+const GF_FilterRegister *rfmhas_register(GF_FilterSession *session)
 {
 	return &MHASDmxRegister;
 }
-
 #else
-
-const GF_FilterRegister *mhas_dmx_register(GF_FilterSession *session)
+const GF_FilterRegister *rfmhas_register(GF_FilterSession *session)
 {
 	return NULL;
 }
-#endif //#if !defined(GPAC_DISABLE_AV_PARSERS)
+#endif //#if !defined(GPAC_DISABLE_AV_PARSERS) && !defined(GPAC_DISABLE_RFMHAS)
