@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2022
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / AMR&EVRC&SMV reframer filter
@@ -28,6 +28,8 @@
 #include <gpac/bitstream.h>
 #include <gpac/internal/media_dev.h>
 
+#ifndef GPAC_DISABLE_RFQCP
+
 typedef struct
 {
 	u64 pos;
@@ -432,7 +434,7 @@ GF_Err qcpdmx_process(GF_Filter *filter)
 		if (gf_filter_pid_is_eos(ctx->ipid)) {
 			if (ctx->opid)
 				gf_filter_pid_set_eos(ctx->opid);
-			assert(ctx->remaining == 0);
+			gf_assert(ctx->remaining == 0);
 			return GF_EOS;
 		}
 		return GF_OK;
@@ -656,7 +658,7 @@ GF_Err qcpdmx_process(GF_Filter *filter)
 		remain -= size;
 
 
-		//don't demux too much of input, abort when we would block. This avoid dispatching
+		//don't demux too much of input, abort when we would block. This avoids dispatching
 		//a huge number of frames in a single call
 		if (gf_filter_pid_would_block(ctx->opid)) {
 			ctx->resume_from = (u32) ((char *)start -  (char *)data);
@@ -736,8 +738,14 @@ GF_FilterRegister QCPDmxRegister = {
 };
 
 
-const GF_FilterRegister *qcpdmx_register(GF_FilterSession *session)
+const GF_FilterRegister *rfqcp_register(GF_FilterSession *session)
 {
 	return &QCPDmxRegister;
 }
+#else
+const GF_FilterRegister *rfqcp_register(GF_FilterSession *session)
+{
+	return NULL;
+}
+#endif //#ifndef GPAC_DISABLE_RFQCP
 
