@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005 
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / IETF RTP/RTSP/SDP sub-project
@@ -10,24 +11,27 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
 #include <gpac/internal/ietf_dev.h>
+
+#ifndef GPAC_DISABLE_STREAMING
+
 #include <gpac/constants.h>
 
 static void rtp_amr_flush(GP_RTPPacketizer *builder)
 {
-	char *hdr;
+	u8 *hdr;
 	u32 hdr_size;
 	if (!builder->bytesInPacket) return;
 	gf_bs_get_content(builder->pck_hdr, &hdr, &hdr_size);
@@ -35,14 +39,14 @@ static void rtp_amr_flush(GP_RTPPacketizer *builder)
 	builder->pck_hdr = NULL;
 	/*overwrite last frame F bit*/
 	hdr[builder->last_au_sn] &= 0x7F;
-	builder->OnData(builder->cbk_obj, hdr, hdr_size, 1);
-	free(hdr);
+	builder->OnData(builder->cbk_obj, hdr, hdr_size, GF_TRUE);
+	gf_free(hdr);
 	builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
 	builder->bytesInPacket = 0;
 	builder->last_au_sn = 0;
 }
 
-GF_Err gp_rtp_builder_do_amr(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+GF_Err gp_rtp_builder_do_amr(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
 {
 	u32 offset, rtp_ts, block_size;
 
@@ -59,10 +63,10 @@ GF_Err gp_rtp_builder_do_amr(GP_RTPPacketizer *builder, char *data, u32 data_siz
 		u8 size;
 
 		if (builder->rtp_payt == GF_RTP_PAYT_AMR_WB) {
-			size = GF_AMR_WB_FRAME_SIZE[ft];
+			size = (u32)GF_AMR_WB_FRAME_SIZE[ft];
 			block_size = 320;
 		} else {
-			size = GF_AMR_FRAME_SIZE[ft];
+			size = (u32)GF_AMR_FRAME_SIZE[ft];
 			block_size = 160;
 		}
 
@@ -76,7 +80,7 @@ GF_Err gp_rtp_builder_do_amr(GP_RTPPacketizer *builder, char *data, u32 data_siz
 			builder->rtp_header.Marker = 0;	/*never set*/
 			builder->rtp_header.SequenceNumber += 1;
 			builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
-			assert(builder->pck_hdr==NULL);
+			gf_assert(builder->pck_hdr==NULL);
 
 			/*always have header and TOC*/
 			builder->pck_hdr = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
@@ -102,20 +106,20 @@ GF_Err gp_rtp_builder_do_amr(GP_RTPPacketizer *builder, char *data, u32 data_siz
 		if (builder->OnDataReference) {
 			builder->OnDataReference(builder->cbk_obj, size, offset);
 		} else {
-			builder->OnData(builder->cbk_obj, data+offset, size, 0);
+			builder->OnData(builder->cbk_obj, data+offset, size, GF_FALSE);
 		}
 		builder->last_au_sn++;
 		builder->bytesInPacket += size;
 		offset += size;
 		rtp_ts += block_size;
-		assert(builder->bytesInPacket<=builder->Path_MTU);
+		gf_assert(builder->bytesInPacket<=builder->Path_MTU);
 		/*take care of aggregation, flush if needed*/
 		if (builder->last_au_sn==builder->auh_size) rtp_amr_flush(builder);
 	}
 	return GF_OK;
 }
 
-static GFINLINE u8 qes_get_rate_size(u32 idx, const u32 *rates, u32 nb_rates)
+static GFINLINE u8 qes_get_rate_size(u32 idx, const unsigned int *rates, const unsigned int nb_rates)
 {
 	u32 i;
 	for (i=0; i<nb_rates; i++) {
@@ -124,7 +128,7 @@ static GFINLINE u8 qes_get_rate_size(u32 idx, const u32 *rates, u32 nb_rates)
 	return 0;
 }
 
-GF_Err gp_rtp_builder_do_qcelp(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+GF_Err gp_rtp_builder_do_qcelp(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
 {
 	u32 offset, rtp_ts;
 	u8 hdr;
@@ -162,18 +166,18 @@ GF_Err gp_rtp_builder_do_qcelp(GP_RTPPacketizer *builder, char *data, u32 data_s
 			builder->rtp_header.SequenceNumber += 1;
 			builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
 			hdr = 0;/*no interleaving*/
-			builder->OnData(builder->cbk_obj, (char*)&hdr, 1, 0);
+			builder->OnData(builder->cbk_obj, (char*)&hdr, 1, GF_FALSE);
 			builder->bytesInPacket = 1;
 		}
 		if (builder->OnDataReference) {
 			builder->OnDataReference(builder->cbk_obj, size, offset);
 		} else {
-			builder->OnData(builder->cbk_obj, data+offset, size, 0);
+			builder->OnData(builder->cbk_obj, data+offset, size, GF_FALSE);
 		}
 		builder->bytesInPacket += size;
 		offset += size;
 		rtp_ts += 160;
-		assert(builder->bytesInPacket<=builder->Path_MTU);
+		gf_assert(builder->bytesInPacket<=builder->Path_MTU);
 
 		/*take care of aggregation, flush if needed*/
 		builder->last_au_sn++;
@@ -186,12 +190,11 @@ GF_Err gp_rtp_builder_do_qcelp(GP_RTPPacketizer *builder, char *data, u32 data_s
 	return GF_OK;
 }
 
-
 static void rtp_evrc_smv_flush(GP_RTPPacketizer *builder)
 {
 	if (!builder->bytesInPacket) return;
 	if (builder->auh_size>1) {
-		char *hdr;
+		u8 *hdr;
 		u32 hdr_size;
 		/*padding*/
 		if (builder->last_au_sn % 2) gf_bs_write_int(builder->pck_hdr, 0, 4);
@@ -201,15 +204,15 @@ static void rtp_evrc_smv_flush(GP_RTPPacketizer *builder)
 		/*overwrite count*/
 		hdr[0] = 0;
 		hdr[1] = builder->last_au_sn-1;/*MMM + frameCount-1*/
-		builder->OnData(builder->cbk_obj, hdr, hdr_size, 1);
-		free(hdr);
+		builder->OnData(builder->cbk_obj, hdr, hdr_size, GF_TRUE);
+		gf_free(hdr);
 	}
 	builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
 	builder->bytesInPacket = 0;
 	builder->last_au_sn = 0;
 }
 
-GF_Err gp_rtp_builder_do_smv(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+GF_Err gp_rtp_builder_do_smv(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
 {
 	u32 offset, rtp_ts;
 
@@ -223,7 +226,7 @@ GF_Err gp_rtp_builder_do_smv(GP_RTPPacketizer *builder, char *data, u32 data_siz
 	offset = 0;
 	while (data_size>offset) {
 		u8 frame_type = data[offset];
-		u8 size = qes_get_rate_size(frame_type, (u32 *)GF_SMV_EVRC_RATE_TO_SIZE, GF_SMV_EVRC_RATE_TO_SIZE_NB);
+		u8 size = qes_get_rate_size(frame_type, GF_SMV_EVRC_RATE_TO_SIZE, GF_SMV_EVRC_RATE_TO_SIZE_NB);
 
 		/*reserved, not sent)*/
 		if (frame_type>=5) {
@@ -231,7 +234,7 @@ GF_Err gp_rtp_builder_do_smv(GP_RTPPacketizer *builder, char *data, u32 data_siz
 			continue;
 		}
 		/*packet full or too long*/
-		if (builder->bytesInPacket + size > builder->Path_MTU) 
+		if (builder->bytesInPacket + size > builder->Path_MTU)
 			rtp_evrc_smv_flush(builder);
 
 		/*need new*/
@@ -240,13 +243,13 @@ GF_Err gp_rtp_builder_do_smv(GP_RTPPacketizer *builder, char *data, u32 data_siz
 			builder->rtp_header.Marker = 0;	/*never set*/
 			builder->rtp_header.SequenceNumber += 1;
 			builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
-			assert(builder->pck_hdr==NULL);
+			gf_assert(builder->pck_hdr==NULL);
 
 			if (builder->auh_size>1) {
 				builder->pck_hdr = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
 				/*RRLLLNNN (all 0, no interleaving)*/
 				gf_bs_write_u8(builder->pck_hdr, 0);
-				/*MMM + count-1 : overriden when flushing*/
+				/*MMM + count-1 : overridden when flushing*/
 				gf_bs_write_u8(builder->pck_hdr, 0);
 				builder->bytesInPacket = 2;
 			}
@@ -265,32 +268,31 @@ GF_Err gp_rtp_builder_do_smv(GP_RTPPacketizer *builder, char *data, u32 data_siz
 		if (builder->OnDataReference) {
 			builder->OnDataReference(builder->cbk_obj, size, offset);
 		} else {
-			builder->OnData(builder->cbk_obj, data+offset, size, 0);
+			builder->OnData(builder->cbk_obj, data+offset, size, GF_FALSE);
 		}
 		builder->last_au_sn++;
 		builder->bytesInPacket += size;
 		offset += size;
 		rtp_ts += 160;
-		assert(builder->bytesInPacket<=builder->Path_MTU);
+		gf_assert(builder->bytesInPacket<=builder->Path_MTU);
 		/*take care of aggregation, flush if needed*/
 		if (builder->last_au_sn==builder->auh_size) rtp_evrc_smv_flush(builder);
 	}
 	return GF_OK;
 }
 
-GF_Err gp_rtp_builder_do_h263(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+GF_Err gp_rtp_builder_do_h263(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
 {
-	GF_BitStream *bs;
-	char hdr[2];
+	u8 hdr[2];
 	Bool Pbit;
 	u32 offset, size, max_size;
 
 	builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
-	
+
 	/*the H263 hinter doesn't perform inter-sample concatenation*/
 	if (!data) return GF_OK;
 
-	Pbit = 1;
+	Pbit = GF_TRUE;
 
 	/*skip 16 0'ed bits of start code*/
 	offset = 2;
@@ -298,10 +300,11 @@ GF_Err gp_rtp_builder_do_h263(GP_RTPPacketizer *builder, char *data, u32 data_si
 	max_size = builder->Path_MTU - 2;
 
 	while(data_size > 0) {
-		if(data_size > max_size){
+		GF_BitStream *bs;
+		if(data_size > max_size) {
 			size = max_size;
 			builder->rtp_header.Marker = 0;
-		}else{
+		} else {
 			size = data_size;
 			builder->rtp_header.Marker = 1;
 		}
@@ -311,7 +314,7 @@ GF_Err gp_rtp_builder_do_h263(GP_RTPPacketizer *builder, char *data, u32 data_si
 		/*create new RTP Packet */
 		builder->rtp_header.SequenceNumber += 1;
 		builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
-		
+
 		bs = gf_bs_new(hdr, 2, GF_BITSTREAM_WRITE);
 		gf_bs_write_int(bs, 0, 5);
 		gf_bs_write_int(bs, Pbit, 1);
@@ -319,28 +322,64 @@ GF_Err gp_rtp_builder_do_h263(GP_RTPPacketizer *builder, char *data, u32 data_si
 		gf_bs_del(bs);
 
 		/*add header*/
-		builder->OnData(builder->cbk_obj, (char*) hdr, 2, 1);
+		builder->OnData(builder->cbk_obj, (char*) hdr, 2, GF_TRUE);
 		/*add payload*/
-		if (builder->OnDataReference) 
+		if (builder->OnDataReference)
 			builder->OnDataReference(builder->cbk_obj, size, offset);
 		else
-			builder->OnData(builder->cbk_obj, data + offset, size, 0);
+			builder->OnData(builder->cbk_obj, data + offset, size, GF_FALSE);
 
 		builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
 
 		offset += size;
-		Pbit = 0;
+		Pbit = GF_FALSE;
 	}
 	return GF_OK;
 }
 
-GF_Err gp_rtp_builder_do_tx3g(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize, u32 duration, u8 descIndex)
+GF_Err gp_rtp_builder_do_mp2t(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+{
+	u32 offset, size, max_size;
+
+	builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
+
+	if (!data) return GF_OK;
+
+	max_size = builder->Path_MTU;
+	offset = 0;
+	while (data_size > 0) {
+		if (data_size > max_size) {
+			size = max_size / 188;
+			size *= 188;
+		} else {
+			size = data_size;
+		}
+
+		data_size -= size;
+
+		/*create new RTP Packet */
+		builder->rtp_header.SequenceNumber += 1;
+		builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
+
+		/*add payload*/
+		if (builder->OnDataReference)
+			builder->OnDataReference(builder->cbk_obj, size, offset);
+		else
+			builder->OnData(builder->cbk_obj, data + offset, size, GF_TRUE);
+
+		builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+
+		offset += size;
+	}
+	return GF_OK;
+}
+GF_Err gp_rtp_builder_do_tx3g(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize, u32 duration, u8 descIndex)
 {
 	GF_BitStream *bs;
-	char *hdr;
+	u8 *hdr;
 	u32 samp_size, txt_size, pay_start, hdr_size, txt_done, cur_frag, nb_frag;
-	Bool is_utf_16 = 0;
-	
+	Bool is_utf_16 = GF_FALSE;
+
 	if (!data) {
 		/*flush packet*/
 		if (builder->bytesInPacket) {
@@ -349,14 +388,19 @@ GF_Err gp_rtp_builder_do_tx3g(GP_RTPPacketizer *builder, char *data, u32 data_si
 		}
 		return GF_OK;
 	}
+	if (data_size<2) return GF_NON_COMPLIANT_BITSTREAM;
+
 	/*cfg packet*/
-	txt_size = data[0]; txt_size <<= 8; txt_size |= (unsigned char) data[1];
+	txt_size = data[0];
+	txt_size <<= 8;
+	txt_size |= (unsigned char) data[1];
 	/*remove BOM*/
 	pay_start = 2;
 	if (txt_size>2) {
+		if (data_size<4) return GF_NON_COMPLIANT_BITSTREAM;
 		/*seems 3GP only accepts BE UTF-16 (no LE, no UTF32)*/
 		if (((u8) data[2]==(u8) 0xFE) && ((u8) data[3]==(u8) 0xFF)) {
-			is_utf_16 = 1;
+			is_utf_16 = GF_TRUE;
 			pay_start = 4;
 			txt_size -= 2;
 		}
@@ -368,6 +412,8 @@ GF_Err gp_rtp_builder_do_tx3g(GP_RTPPacketizer *builder, char *data, u32 data_si
 		builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
 		builder->bytesInPacket = 0;
 	}
+	//we only deal with static SIDX
+	descIndex += GF_RTP_TX3G_SIDX_OFFSET;
 
 	/*first TTU in packet*/
 	if (!builder->bytesInPacket) {
@@ -388,15 +434,15 @@ GF_Err gp_rtp_builder_do_tx3g(GP_RTPPacketizer *builder, char *data, u32 data_si
 		gf_bs_write_u16(bs, txt_size);
 		gf_bs_get_content(bs, &hdr, &hdr_size);
 		gf_bs_del(bs);
-		builder->OnData(builder->cbk_obj, (char *) hdr, hdr_size, 0);
+		builder->OnData(builder->cbk_obj, (char *) hdr, hdr_size, GF_FALSE);
 		builder->bytesInPacket += hdr_size;
-		free(hdr);
-		
+		gf_free(hdr);
+
 		if (txt_size) {
 			if (builder->OnDataReference) {
 				builder->OnDataReference(builder->cbk_obj, samp_size, pay_start);
 			} else {
-				builder->OnData(builder->cbk_obj, data + pay_start, samp_size, 0);
+				builder->OnData(builder->cbk_obj, data + pay_start, samp_size, GF_FALSE);
 			}
 			builder->bytesInPacket += samp_size;
 		}
@@ -456,18 +502,18 @@ GF_Err gp_rtp_builder_do_tx3g(GP_RTPPacketizer *builder, char *data, u32 data_si
 		gf_bs_write_u16(bs, samp_size);
 		gf_bs_get_content(bs, &hdr, &hdr_size);
 		gf_bs_del(bs);
-		builder->OnData(builder->cbk_obj, (char *) hdr, hdr_size, 0);
+		builder->OnData(builder->cbk_obj, (char *) hdr, hdr_size, GF_FALSE);
 		builder->bytesInPacket += hdr_size;
-		free(hdr);
-	
+		gf_free(hdr);
+
 		if (builder->OnDataReference) {
 			builder->OnDataReference(builder->cbk_obj, size, pay_start + txt_done);
 		} else {
-			builder->OnData(builder->cbk_obj, data + pay_start + txt_done, size, 0);
+			builder->OnData(builder->cbk_obj, data + pay_start + txt_done, size, GF_FALSE);
 		}
 		builder->bytesInPacket += size;
 		cur_frag++;
-		
+
 		/*flush packet*/
 		if (cur_frag == nb_frag) {
 			txt_done = txt_size;
@@ -510,14 +556,14 @@ GF_Err gp_rtp_builder_do_tx3g(GP_RTPPacketizer *builder, char *data, u32 data_si
 
 		gf_bs_get_content(bs, &hdr, &hdr_size);
 		gf_bs_del(bs);
-		builder->OnData(builder->cbk_obj, (char *) hdr, hdr_size, 0);
+		builder->OnData(builder->cbk_obj, (char *) hdr, hdr_size, GF_FALSE);
 		builder->bytesInPacket += hdr_size;
-		free(hdr);
+		gf_free(hdr);
 
 		if (builder->OnDataReference) {
 			builder->OnDataReference(builder->cbk_obj, size, pay_start + txt_done);
 		} else {
-			builder->OnData(builder->cbk_obj, data + pay_start + txt_done, size, 0);
+			builder->OnData(builder->cbk_obj, data + pay_start + txt_done, size, GF_FALSE);
 		}
 		builder->bytesInPacket += size;
 		cur_frag++;
@@ -538,6 +584,7 @@ GF_Err gp_rtp_builder_do_tx3g(GP_RTPPacketizer *builder, char *data, u32 data_si
 }
 
 
+#if GPAC_ENABLE_3GPP_DIMS_RTP
 GF_Err gp_rtp_builder_do_dims(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize, u32 duration)
 {
 	u32 frag_state;
@@ -555,7 +602,7 @@ GF_Err gp_rtp_builder_do_dims(GP_RTPPacketizer *builder, char *data, u32 data_si
 		u32 du_offset = 0;
 		u32 hdr_offset = 0;
 		u32 orig_size, du_size;
-		
+
 		orig_size = du_size = 2+gf_bs_read_u16(bs);
 		/*if dims size is >0xFFFF, use our internal hack for large units*/
 		if (du_size==2) {
@@ -565,12 +612,13 @@ GF_Err gp_rtp_builder_do_dims(GP_RTPPacketizer *builder, char *data, u32 data_si
 		gf_bs_skip_bytes(bs, du_size-2);
 
 		/*prepare M-bit*/
-		is_last_du = (offset+du_size==data_size) ? 1 : 0;
+		is_last_du = (offset+du_size==data_size) ? GF_TRUE : GF_FALSE;
 
 		frag_state = 0;
 		while (du_size) {
 			u32 size_offset = 0;
-			u32 size = du_size;
+			u32 size;
+			//size = du_size;
 
 			/*does not fit, flush required*/
 			if (builder->bytesInPacket && (du_size + 1 + builder->bytesInPacket > builder->Path_MTU)) {
@@ -595,7 +643,7 @@ GF_Err gp_rtp_builder_do_dims(GP_RTPPacketizer *builder, char *data, u32 data_si
 				else frag_state = 2;
 
 				builder->rtp_header.Marker = 0;
-			} 
+			}
 			/*last fragment*/
 			else if (frag_state) {
 				size = du_size;
@@ -614,7 +662,7 @@ GF_Err gp_rtp_builder_do_dims(GP_RTPPacketizer *builder, char *data, u32 data_si
 			/*need a new packet*/
 			if (!builder->bytesInPacket) {
 				char dims_rtp_hdr[1];
-				
+
 				/*the unit is critical, increase counter (coded on 3 bits)*/
 				if (! (data[2+hdr_offset] & GF_DIMS_UNIT_P) && (frag_state <= 1) ) {
 					builder->last_au_sn++;
@@ -629,17 +677,17 @@ GF_Err gp_rtp_builder_do_dims(GP_RTPPacketizer *builder, char *data, u32 data_si
 
 				builder->rtp_header.SequenceNumber += 1;
 				builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
-				builder->OnData(builder->cbk_obj, (char *) dims_rtp_hdr, 1, 1);
+				builder->OnData(builder->cbk_obj, (char *) dims_rtp_hdr, 1, GF_TRUE);
 				builder->bytesInPacket = 1;
 			}
 
 			/*add payload*/
-			if (builder->OnDataReference) 
+			if (builder->OnDataReference)
 				builder->OnDataReference(builder->cbk_obj, size, offset+du_offset+size_offset);
 			else
-				builder->OnData(builder->cbk_obj, data+offset+du_offset+size_offset, size, 0);
+				builder->OnData(builder->cbk_obj, data+offset+du_offset+size_offset, size, GF_FALSE);
 
-			/*if fragmentation, force packet flush even on last packet since aggregation unit do not 
+			/*if fragmentation, force packet flush even on last packet since aggregation unit do not
 			use the same packet format*/
 			if (frag_state) {
 				builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
@@ -648,7 +696,7 @@ GF_Err gp_rtp_builder_do_dims(GP_RTPPacketizer *builder, char *data, u32 data_si
 				builder->bytesInPacket += size;
 			}
 			du_offset += size+size_offset;
-			assert(du_size>= size+size_offset);
+			gf_assert(du_size>= size+size_offset);
 			du_size -= size+size_offset;
 		}
 		offset += orig_size;
@@ -660,6 +708,7 @@ GF_Err gp_rtp_builder_do_dims(GP_RTPPacketizer *builder, char *data, u32 data_si
 	gf_bs_del(bs);
 	return GF_OK;
 }
+#endif
 
 
 
@@ -670,7 +719,7 @@ static void gf_rtp_ac3_flush(GP_RTPPacketizer *builder)
 
 	hdr[0] = builder->ac3_ft;
 	hdr[1] = builder->last_au_sn;
-	builder->OnData(builder->cbk_obj, hdr, 2, 1);
+	builder->OnData(builder->cbk_obj, hdr, 2, GF_TRUE);
 
 	builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
 	builder->bytesInPacket = 0;
@@ -678,7 +727,7 @@ static void gf_rtp_ac3_flush(GP_RTPPacketizer *builder)
 	builder->ac3_ft = 0;
 }
 
-GF_Err gp_rtp_builder_do_ac3(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+GF_Err gp_rtp_builder_do_ac3(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
 {
 	char hdr[2];
 	u32 offset, nb_pck;
@@ -688,21 +737,21 @@ GF_Err gp_rtp_builder_do_ac3(GP_RTPPacketizer *builder, char *data, u32 data_siz
 		gf_rtp_ac3_flush(builder);
 		return GF_OK;
 	}
-	
+
 	if (
-		/*AU does not fit*/
-		(builder->bytesInPacket + data_size > builder->Path_MTU)
-	||
-		/*aggregation is not enabled*/
-		!(builder->flags & GP_RTP_PCK_USE_MULTI)
-	||
-		/*max ptime is exceeded*/
-		(builder->max_ptime && ( (u32) builder->sl_header.compositionTimeStamp >= builder->rtp_header.TimeStamp + builder->max_ptime) )
+	    /*AU does not fit*/
+	    (builder->bytesInPacket + data_size > builder->Path_MTU)
+	    ||
+	    /*aggregation is not enabled*/
+	    !(builder->flags & GP_RTP_PCK_USE_MULTI)
+	    ||
+	    /*max ptime is exceeded*/
+	    (builder->max_ptime && ( (u32) builder->sl_header.compositionTimeStamp >= builder->rtp_header.TimeStamp + builder->max_ptime) )
 
 	) {
 		gf_rtp_ac3_flush(builder);
 	}
-	
+
 	/*fits*/
 	if (builder->bytesInPacket + data_size < builder->Path_MTU) {
 		/*need a new packet*/
@@ -717,10 +766,10 @@ GF_Err gp_rtp_builder_do_ac3(GP_RTPPacketizer *builder, char *data, u32 data_siz
 		}
 
 		/*add payload*/
-		if (builder->OnDataReference) 
+		if (builder->OnDataReference)
 			builder->OnDataReference(builder->cbk_obj, data_size, 0);
 		else
-			builder->OnData(builder->cbk_obj, data, data_size, 0);
+			builder->OnData(builder->cbk_obj, data, data_size, GF_FALSE);
 
 		builder->bytesInPacket += data_size;
 		builder->last_au_sn++;
@@ -728,7 +777,7 @@ GF_Err gp_rtp_builder_do_ac3(GP_RTPPacketizer *builder, char *data, u32 data_siz
 	}
 
 	/*need fragmentation*/
-	assert(!builder->bytesInPacket);
+	gf_assert(!builder->bytesInPacket);
 	offset = 0;
 	nb_pck = data_size / (builder->Path_MTU-2);
 	if (data_size % (builder->Path_MTU-2)) nb_pck++;
@@ -752,13 +801,13 @@ GF_Err gp_rtp_builder_do_ac3(GP_RTPPacketizer *builder, char *data, u32 data_siz
 
 		hdr[0] = builder->ac3_ft;
 		hdr[1] = builder->last_au_sn;
-		builder->OnData(builder->cbk_obj, hdr, 2, 1);
+		builder->OnData(builder->cbk_obj, hdr, 2, GF_TRUE);
 
 		/*add payload*/
-		if (builder->OnDataReference) 
+		if (builder->OnDataReference)
 			builder->OnDataReference(builder->cbk_obj, pck_size, offset);
 		else
-			builder->OnData(builder->cbk_obj, data+offset, pck_size, 0);
+			builder->OnData(builder->cbk_obj, data+offset, pck_size, GF_FALSE);
 
 		builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
 		offset += pck_size;
@@ -768,3 +817,32 @@ GF_Err gp_rtp_builder_do_ac3(GP_RTPPacketizer *builder, char *data, u32 data_siz
 	return GF_OK;
 }
 
+GF_Err gp_rtp_builder_do_opus(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+{
+	/*flush*/
+	if (!data) return GF_OK;
+
+	/*fits*/
+	if (data_size > builder->Path_MTU) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_RTP, ("[RTPOpus] Packet size %u larger MTU %u but Opus fragmentation is not yet supported, pacth welcome\n", data_size, builder->Path_MTU ));
+		return GF_NOT_SUPPORTED;
+	}
+	/*send new packet*/
+	builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
+	builder->rtp_header.Marker = 0;
+	builder->rtp_header.SequenceNumber += 1;
+	builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
+
+	/*add payload*/
+	if (builder->OnDataReference)
+		builder->OnDataReference(builder->cbk_obj, data_size, 0);
+	else
+		builder->OnData(builder->cbk_obj, data, data_size, GF_FALSE);
+
+	builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+	builder->last_au_sn++;
+	return GF_OK;
+}
+
+#endif /*GPAC_DISABLE_STREAMING*/
+
