@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / IETF RTP/RTSP/SDP sub-project
@@ -10,19 +11,22 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
 #include <gpac/internal/ietf_dev.h>
+
+#ifndef GPAC_DISABLE_STREAMING
+
 #include <gpac/constants.h>
 
 //get the size of the RSLH section given the GF_SLHeader and the SLMap
@@ -103,7 +107,7 @@ u32 gf_rtp_build_au_hdr_write(GP_RTPPacketizer *builder, u32 PayloadSize, u32 RT
 {
 	u32 nbBits = 0;
 	s32 delta;
-	
+
 	/*selective encryption*/
 	if (builder->flags & GP_RTP_PCK_SELECTIVE_ENCRYPTION) {
 		gf_bs_write_int(builder->pck_hdr, builder->is_encrypted, 1);
@@ -188,19 +192,25 @@ u32 gf_rtp_build_au_hdr_write(GP_RTPPacketizer *builder, u32 PayloadSize, u32 RT
 		gf_bs_write_int(builder->pck_hdr, builder->sl_header.randomAccessPointFlag, 1);
 		nbBits ++;
 	}
+	/*stream state - write AUSeqNum*/
+	if (builder->slMap.StreamStateIndication) {
+		gf_bs_write_int(builder->pck_hdr, builder->sl_header.AU_sequenceNumber, builder->slMap.StreamStateIndication);
+		nbBits += builder->slMap.StreamStateIndication;
+	}
+
 	return nbBits;
 }
 
 
-GF_Err gp_rtp_builder_do_mpeg4(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+GF_Err gp_rtp_builder_do_mpeg4(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
 {
-	char *sl_buffer, *payl_buffer;
-	u32 sl_buffer_size, payl_buffer_size; 
-	u32 auh_size_tmp, rslh_tmp, bytesLeftInPacket, infoSize, pckSize, pos;
+	u8 *sl_buffer, *payl_buffer;
+	u32 sl_buffer_size, payl_buffer_size;
+	u32 auh_size_tmp, bytesLeftInPacket, infoSize, pckSize;
+	u64 pos;
 	u8 flush_pck, no_split;
 
 	flush_pck = 0;
-	rslh_tmp = 0;
 
 	bytesLeftInPacket = data_size;
 	/*flush everything*/
@@ -226,11 +236,11 @@ GF_Err gp_rtp_builder_do_mpeg4(GP_RTPPacketizer *builder, char *data, u32 data_s
 			//we decide if we go with the same RTP TS serie or not
 			if (builder->payload) {
 				//don't store more than what we can (that is 2^slMap->CTSDelta - 1)
-				if ( (builder->flags & GP_RTP_PCK_SIGNAL_TS) 
-					&& (builder->sl_header.compositionTimeStamp - builder->rtp_header.TimeStamp >= (u32) ( 1 << builder->slMap.CTSDeltaLength) ) ) {
+				if ( (builder->flags & GP_RTP_PCK_SIGNAL_TS)
+				        && (builder->sl_header.compositionTimeStamp - builder->rtp_header.TimeStamp >= (u32) ( 1 << builder->slMap.CTSDeltaLength) ) ) {
 					goto flush_packet;
 				}
-				//don't split AU if # TS , start a new RTP pck 
+				//don't split AU if # TS , start a new RTP pck
 				if (builder->sl_header.compositionTimeStamp != builder->rtp_header.TimeStamp)
 					no_split = 1;
 			}
@@ -239,9 +249,9 @@ GF_Err gp_rtp_builder_do_mpeg4(GP_RTPPacketizer *builder, char *data, u32 data_s
 		/*new RTP Packet*/
 		if (!builder->payload) {
 			/*first SL in RTP*/
-			builder->first_sl_in_rtp = 1;
+			builder->first_sl_in_rtp = GF_TRUE;
 
-			/*if this is the end of an AU we will set it to 0 as soon as an AU is splited*/
+			/*if this is the end of an AU we will set it to 0 as soon as an AU is splitted*/
 			builder->rtp_header.Marker = 1;
 			builder->rtp_header.PayloadType = builder->PayloadType;
 			builder->rtp_header.SequenceNumber += 1;
@@ -250,7 +260,6 @@ GF_Err gp_rtp_builder_do_mpeg4(GP_RTPPacketizer *builder, char *data, u32 data_s
 			/*prepare the mapped headers*/
 			builder->pck_hdr = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
 			builder->payload = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
-			pckSize = infoSize = 0;
 			builder->bytesInPacket = 0;
 
 			/*in multiSL there is a MSLHSize structure on 2 bytes*/
@@ -258,19 +267,19 @@ GF_Err gp_rtp_builder_do_mpeg4(GP_RTPPacketizer *builder, char *data, u32 data_s
 			if (builder->has_AU_header) {
 				builder->auh_size = 16;
 				gf_bs_write_int(builder->pck_hdr, 0, 16);
-			}			
+			}
 			flush_pck = 0;
 			/*and create packet*/
 			builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
 		}
 
 		//make sure we are not interleaving too much - this should never happen actually
-		if (builder->slMap.IndexDeltaLength 
-			&& !builder->first_sl_in_rtp 
-			&& (builder->sl_header.AU_sequenceNumber - builder->last_au_sn >= (u32) 1<<builder->slMap.IndexDeltaLength)) {
+		if (builder->slMap.IndexDeltaLength
+		        && !builder->first_sl_in_rtp
+		        && (builder->sl_header.AU_sequenceNumber - builder->last_au_sn >= (u32) 1<<builder->slMap.IndexDeltaLength)) {
 			//we cannot write this packet here
 			goto flush_packet;
-		} 
+		}
 		/*check max ptime*/
 		if (builder->max_ptime && ( (u32) builder->sl_header.compositionTimeStamp >= builder->rtp_header.TimeStamp + builder->max_ptime) )
 			goto flush_packet;
@@ -289,7 +298,7 @@ GF_Err gp_rtp_builder_do_mpeg4(GP_RTPPacketizer *builder, char *data, u32 data_s
 			builder->sl_header.accessUnitEndFlag = IsAUEnd;
 
 			builder->auh_size += auh_size_tmp;
-			
+
 			builder->sl_header.paddingFlag = builder->sl_header.paddingBits ? 1 : 0;
 		} else {
 
@@ -306,9 +315,9 @@ GF_Err gp_rtp_builder_do_mpeg4(GP_RTPPacketizer *builder, char *data, u32 data_s
 		}
 
 		gf_rtp_build_au_hdr_write(builder, pckSize, builder->rtp_header.TimeStamp);
-		
+
 		//notify the user of our data structure
-		if (builder->OnDataReference) 
+		if (builder->OnDataReference)
 			builder->OnDataReference(builder->cbk_obj, pckSize, data_size - bytesLeftInPacket);
 		else
 			gf_bs_write_data(builder->payload, data + (data_size - bytesLeftInPacket), pckSize);
@@ -330,7 +339,7 @@ GF_Err gp_rtp_builder_do_mpeg4(GP_RTPPacketizer *builder, char *data, u32 data_s
 		}
 
 		//first SL in RTP is done
-		builder->first_sl_in_rtp = 0;
+		builder->first_sl_in_rtp = GF_FALSE;
 
 		//store current sl
 		builder->last_au_sn = builder->sl_header.AU_sequenceNumber;
@@ -345,11 +354,11 @@ flush_packet:
 		/*no aux data yet*/
 		if (builder->slMap.AuxiliaryDataSizeLength)	{
 			//write RSLH after the MSLH
-			gf_bs_write_int(builder->pck_hdr, 0, builder->slMap.AuxiliaryDataSizeLength);			
-		}				
+			gf_bs_write_int(builder->pck_hdr, 0, builder->slMap.AuxiliaryDataSizeLength);
+		}
 		/*rewrite the size header*/
 		if (builder->has_AU_header) {
-			pos = (u32) gf_bs_get_position(builder->pck_hdr);		
+			pos = gf_bs_get_position(builder->pck_hdr);
 			gf_bs_seek(builder->pck_hdr, 0);
 			builder->auh_size -= 16;
 			gf_bs_write_int(builder->pck_hdr, builder->auh_size, 16);
@@ -364,22 +373,22 @@ flush_packet:
 
 		payl_buffer = NULL;
 		payl_buffer_size = 0;
-		if (!builder->OnDataReference) 
+		if (!builder->OnDataReference)
 			gf_bs_get_content(builder->payload, &payl_buffer, &payl_buffer_size);
 
 		gf_bs_del(builder->payload);
 		builder->payload = NULL;
 
 		/*notify header*/
-		builder->OnData(builder->cbk_obj, sl_buffer, sl_buffer_size, 1);
+		builder->OnData(builder->cbk_obj, sl_buffer, sl_buffer_size, GF_TRUE);
 		/*notify payload*/
 		if (payl_buffer) {
-			builder->OnData(builder->cbk_obj, payl_buffer, payl_buffer_size, 0);
-			free(payl_buffer);
+			builder->OnData(builder->cbk_obj, payl_buffer, payl_buffer_size, GF_FALSE);
+			gf_free(payl_buffer);
 		}
 		/*flush packet*/
 		builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
-		free(sl_buffer);
+		gf_free(sl_buffer);
 	}
 	//packet is done, update AU markers
 	if (IsAUEnd) {
@@ -390,7 +399,7 @@ flush_packet:
 }
 
 
-GF_Err gp_rtp_builder_do_avc(GP_RTPPacketizer *builder, char *nalu, u32 nalu_size, u8 IsAUEnd, u32 FullAUSize)
+GF_Err gp_rtp_builder_do_avc(GP_RTPPacketizer *builder, u8 *nalu, u32 nalu_size, u8 IsAUEnd, u32 FullAUSize)
 {
 	u32 do_flush, bytesLeft, size, nal_type;
 	char shdr[2];
@@ -412,14 +421,16 @@ GF_Err gp_rtp_builder_do_avc(GP_RTPPacketizer *builder, char *nalu, u32 nalu_siz
 	}
 
 	if (!nalu) return GF_OK;
+	if (nalu_size<1) return GF_NON_COMPLIANT_BITSTREAM;
 
 	/*need a new RTP packet*/
 	if (!builder->bytesInPacket) {
 		builder->rtp_header.PayloadType = builder->PayloadType;
+		builder->rtp_header.Marker = 0;
 		builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
 		builder->rtp_header.SequenceNumber += 1;
 		builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
-		builder->avc_non_idr = 1;
+		builder->avc_non_idr = GF_TRUE;
 	}
 
 	/*check NAL type to see if disposable or not*/
@@ -432,7 +443,7 @@ GF_Err gp_rtp_builder_do_avc(GP_RTPPacketizer *builder, char *nalu, u32 nalu_siz
 	case GF_AVC_NALU_FILLER_DATA:
 		break;
 	default:
-		builder->avc_non_idr = 0;
+		builder->avc_non_idr = GF_FALSE;
 		break;
 	}
 
@@ -441,29 +452,29 @@ GF_Err gp_rtp_builder_do_avc(GP_RTPPacketizer *builder, char *nalu, u32 nalu_siz
 	/*pb: we don't know if next NALU from this AU will be small enough to fit in the packet, so we always
 	go for stap...*/
 	if (builder->bytesInPacket+nalu_size<builder->Path_MTU) {
-		Bool use_stap = 0;
+		Bool use_stap = GF_TRUE;
 		/*if this is the AU end and no NALU in packet, go for single mode*/
-		if (IsAUEnd && !builder->bytesInPacket) use_stap = 0;
-		
+		if (IsAUEnd && !builder->bytesInPacket) use_stap = GF_FALSE;
+
 		if (use_stap) {
 			/*declare STAP-A NAL*/
 			if (!builder->bytesInPacket) {
 				/*copy over F and NRI from first nal in packet and assign type*/
 				stap_hdr = (nalu[0] & 0xE0) | 24;
-				builder->OnData(builder->cbk_obj, (char *) &stap_hdr, 1, 0);
+				builder->OnData(builder->cbk_obj, (char *) &stap_hdr, 1, GF_FALSE);
 				builder->bytesInPacket = 1;
 			}
 			/*add NALU size*/
 			shdr[0] = nalu_size>>8;
 			shdr[1] = nalu_size&0x00ff;
-			builder->OnData(builder->cbk_obj, (char *)shdr, 2, 0);
+			builder->OnData(builder->cbk_obj, (char *)shdr, 2, GF_FALSE);
 			builder->bytesInPacket += 2;
 		}
 		/*add data*/
-		if (builder->OnDataReference) 
+		if (builder->OnDataReference)
 			builder->OnDataReference(builder->cbk_obj, nalu_size, 0);
 		else
-			builder->OnData(builder->cbk_obj, nalu, nalu_size, 0);
+			builder->OnData(builder->cbk_obj, nalu, nalu_size, GF_FALSE);
 
 		builder->bytesInPacket += nalu_size;
 
@@ -476,8 +487,8 @@ GF_Err gp_rtp_builder_do_avc(GP_RTPPacketizer *builder, char *nalu, u32 nalu_siz
 	/*fragmentation unit*/
 	else {
 		u32 offset;
-		assert(nalu_size>=builder->Path_MTU);
-		assert(!builder->bytesInPacket);
+		gf_assert(nalu_size>=builder->Path_MTU);
+		gf_assert(!builder->bytesInPacket);
 		/*FU payload doesn't have the NAL hdr*/
 		bytesLeft = nalu_size - 1;
 		offset = 1;
@@ -497,13 +508,13 @@ GF_Err gp_rtp_builder_do_avc(GP_RTPPacketizer *builder, char *nalu, u32 nalu_siz
 			/*end bit*/
 			else if (size == bytesLeft) shdr[1] |= 0x40;
 
-			builder->OnData(builder->cbk_obj, (char *)shdr, 2, 0);
+			builder->OnData(builder->cbk_obj, (char *)shdr, 2, GF_FALSE);
 
 			/*add data*/
-			if (builder->OnDataReference) 
+			if (builder->OnDataReference)
 				builder->OnDataReference(builder->cbk_obj, size, offset);
 			else
-				builder->OnData(builder->cbk_obj, nalu+offset, size, 0);
+				builder->OnData(builder->cbk_obj, nalu+offset, size, GF_FALSE);
 
 			offset += size;
 			bytesLeft -= size;
@@ -525,80 +536,396 @@ GF_Err gp_rtp_builder_do_avc(GP_RTPPacketizer *builder, char *nalu, u32 nalu_siz
 	return GF_OK;
 }
 
+GF_Err gp_rtp_builder_do_hevc(GP_RTPPacketizer *builder, u8 *nalu, u32 nalu_size, u8 IsAUEnd, u32 FullAUSize)
+{
+	u32 do_flush, bytesLeft, size;
+
+	do_flush = 0;
+	if (!nalu) do_flush = 1;
+	else if (builder->sl_header.accessUnitStartFlag) do_flush = 1;
+	/*we must NOT fragment a NALU*/
+	else if (builder->bytesInPacket + nalu_size + 4 >= builder->Path_MTU) do_flush = 2; //2 bytes PayloadHdr for AP + 2 bytes NAL size
+	/*aggregation is disabled*/
+	else if (! (builder->flags & GP_RTP_PCK_USE_MULTI) ) do_flush = 2;
+
+	if (builder->bytesInPacket && do_flush) {
+		builder->rtp_header.Marker = (do_flush==1) ? 1 : 0;
+		/*insert payload_hdr in case of AP*/
+		if (strlen(builder->hevc_payload_hdr)) {
+			builder->OnData(builder->cbk_obj, (char *)builder->hevc_payload_hdr, 2, GF_TRUE);
+			memset(builder->hevc_payload_hdr, 0, 2);
+		}
+		builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+		builder->bytesInPacket = 0;
+	}
+
+	if (!nalu) return GF_OK;
+	if (nalu_size<2) return GF_NON_COMPLIANT_BITSTREAM;
+
+	/*need a new RTP packet*/
+	if (!builder->bytesInPacket) {
+		builder->rtp_header.PayloadType = builder->PayloadType;
+		builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
+		builder->rtp_header.SequenceNumber += 1;
+		builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
+	}
+
+	/*at this point we're sure the NALU fits in current packet OR must be splitted*/
+	/*check that we should use single NALU packet mode or aggreation packets mode*/
+	if (builder->bytesInPacket+nalu_size+4 < builder->Path_MTU) {
+		Bool use_AP = (builder->flags & GP_RTP_PCK_USE_MULTI) ? GF_TRUE : GF_FALSE;
+		/*if this is the AU end and no NALU in packet, go for single NALU packet mode*/
+		if (IsAUEnd && !builder->bytesInPacket) use_AP = GF_FALSE;
+
+		if (use_AP) {
+			char nal_s[2];
+			/*declare PayloadHdr for AP*/
+			if (!builder->bytesInPacket) {
+				/*copy F bit and assign type*/
+				builder->hevc_payload_hdr[0] = (nalu[0] & 0x81) | (48 << 1);
+				/*copy LayerId and TID*/
+				builder->hevc_payload_hdr[1] = nalu[1];
+			}
+			else {
+				/*F bit of AP is 0 if the F nit of each aggreated NALU is 0; otherwise its must be 1*/
+				/*LayerId and TID must ne the lowest value of LayerId and TID of all aggreated NALU*/
+				u8 cur_LayerId, cur_TID, new_LayerId, new_TID;
+
+				builder->hevc_payload_hdr[0] |= (nalu[0] & 0x80);
+				cur_LayerId = ((builder->hevc_payload_hdr[0] & 0x01) << 5) + ((builder->hevc_payload_hdr[1] & 0xF8) >> 3);
+				new_LayerId = ((nalu[0] & 0x01) << 5) + ((nalu[1] & 0xF8) >> 3);
+				if (cur_LayerId > new_LayerId) {
+					builder->hevc_payload_hdr[0] = (builder->hevc_payload_hdr[0] & 0xFE) | (nalu[0] & 0x01);
+					builder->hevc_payload_hdr[1] = (builder->hevc_payload_hdr[1] & 0x07) | (nalu[1] & 0xF8);
+				}
+				cur_TID = builder->hevc_payload_hdr[1] & 0x07;
+				new_TID = nalu[1] & 0x07;
+				if (cur_TID > new_TID) {
+					builder->hevc_payload_hdr[1] = (builder->hevc_payload_hdr[1] & 0xF8) | (nalu[1] & 0x07);
+				}
+			}
+
+			/*add NALU size*/
+			nal_s[0] = nalu_size>>8;
+			nal_s[1] = nalu_size&0x00ff;
+			builder->OnData(builder->cbk_obj, (char *)nal_s, 2, GF_FALSE);
+			builder->bytesInPacket += 2;
+		}
+		/*add data*/
+		if (builder->OnDataReference)
+			builder->OnDataReference(builder->cbk_obj, nalu_size, 0);
+		else
+			builder->OnData(builder->cbk_obj, nalu, nalu_size, GF_FALSE);
+
+		builder->bytesInPacket += nalu_size;
+
+		if (IsAUEnd) {
+			builder->rtp_header.Marker = 1;
+			if (strlen(builder->hevc_payload_hdr)) {
+				builder->OnData(builder->cbk_obj, (char *)builder->hevc_payload_hdr, 2, GF_TRUE);
+				memset(builder->hevc_payload_hdr, 0, 2);
+			}
+			builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+			builder->bytesInPacket = 0;
+		}
+	}
+	/*fragmentation unit*/
+	else {
+		u32 offset;
+		char payload_hdr[2];
+		char shdr;
+
+		gf_assert(nalu_size + 4 >=builder->Path_MTU);
+		gf_assert(!builder->bytesInPacket);
+
+		/*FU payload doesn't have the NAL hdr (2 bytes*/
+		bytesLeft = nalu_size - 2;
+		offset = 2;
+		while (bytesLeft) {
+			if (3 + bytesLeft > builder->Path_MTU) {
+				size = builder->Path_MTU - 3;
+			} else {
+				size = bytesLeft;
+			}
+
+			/*declare PayloadHdr for FU*/
+			memset(payload_hdr, 0, 2);
+			/*copy F bit and assign type*/
+			payload_hdr[0] = (nalu[0] & 0x81) | (49 << 1);
+			/*copy LayerId and TID*/
+			payload_hdr[1] = nalu[1];
+			builder->OnData(builder->cbk_obj, (char *)payload_hdr, 2, GF_FALSE);
+
+			/*declare FU header*/
+			shdr = 0;
+			/*assign type*/
+			shdr |= (nalu[0] & 0x7E) >> 1;
+			/*start bit*/
+			if (offset==2) shdr |= 0x80;
+			/*end bit*/
+			else if (size == bytesLeft) shdr |= 0x40;
+
+			builder->OnData(builder->cbk_obj, &shdr, 1, GF_FALSE);
+
+			/*add data*/
+			if (builder->OnDataReference)
+				builder->OnDataReference(builder->cbk_obj, size, offset);
+			else
+				builder->OnData(builder->cbk_obj, nalu+offset, size, GF_FALSE);
+
+			offset += size;
+			bytesLeft -= size;
+
+			/*flush no matter what (FUs cannot be agreggated)*/
+			builder->rtp_header.Marker = (IsAUEnd && !bytesLeft) ? 1 : 0;
+			builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+			builder->bytesInPacket = 0;
+
+			if (bytesLeft) {
+				builder->rtp_header.PayloadType = builder->PayloadType;
+				builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
+				builder->rtp_header.SequenceNumber += 1;
+				builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
+			}
+		}
+	}
+	return GF_OK;
+}
+
+GF_Err gp_rtp_builder_do_vvc(GP_RTPPacketizer *builder, u8 *nalu, u32 nalu_size, u8 IsAUEnd, u32 FullAUSize)
+{
+	u32 do_flush, bytesLeft, size;
+
+	do_flush = 0;
+	if (!nalu) do_flush = 1;
+	else if (builder->sl_header.accessUnitStartFlag) do_flush = 1;
+	/*we must NOT fragment a NALU*/
+	else if (builder->bytesInPacket + nalu_size + 4 >= builder->Path_MTU) do_flush = 2; //2 bytes PayloadHdr for AP + 2 bytes NAL size
+	/*aggregation is disabled*/
+	else if (! (builder->flags & GP_RTP_PCK_USE_MULTI) ) do_flush = 2;
+
+	if (builder->bytesInPacket && do_flush) {
+		builder->rtp_header.Marker = (do_flush==1) ? 1 : 0;
+		/*insert payload_hdr in case of AP*/
+		if (strlen(builder->hevc_payload_hdr)) {
+			builder->OnData(builder->cbk_obj, (char *)builder->hevc_payload_hdr, 2, GF_TRUE);
+			memset(builder->hevc_payload_hdr, 0, 2);
+		}
+		builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+		builder->bytesInPacket = 0;
+	}
+
+	if (!nalu) return GF_OK;
+	if (nalu_size<2) return GF_NON_COMPLIANT_BITSTREAM;
+
+	/*need a new RTP packet*/
+	if (!builder->bytesInPacket) {
+		builder->rtp_header.PayloadType = builder->PayloadType;
+		builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
+		builder->rtp_header.SequenceNumber += 1;
+		builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
+	}
+
+	/*at this point we're sure the NALU fits in current packet OR must be splitted*/
+	/*check that we should use single NALU packet mode or aggreation packets mode*/
+	if (builder->bytesInPacket+nalu_size+4 < builder->Path_MTU) {
+		Bool use_AP = (builder->flags & GP_RTP_PCK_USE_MULTI) ? GF_TRUE : GF_FALSE;
+		/*if this is the AU end and no NALU in packet, go for single NALU packet mode*/
+		if (IsAUEnd && !builder->bytesInPacket) use_AP = GF_FALSE;
+
+		if (use_AP) {
+			char nal_s[2];
+			/*declare PayloadHdr for AP*/
+			if (!builder->bytesInPacket) {
+				//copy everything, reassign type (byte 1 bits 8-3)
+				builder->hevc_payload_hdr[0] = nalu[0];
+				builder->hevc_payload_hdr[1] = (RTP_VVC_AGG_NAL << 3) | (nalu[1] & 0x7);
+			}
+			else {
+				/*F bit of AP is 0 if the F bit of each aggreated NALU is 0; otherwise its must be 1*/
+				/*LayerId and TID must be the lowest value of LayerId and TID of all aggreated NALU*/
+				u8 cur_LayerId, cur_TID, new_LayerId, new_TID;
+
+				builder->hevc_payload_hdr[0] |= (nalu[0] & 0x80);
+				cur_LayerId = (builder->hevc_payload_hdr[0] & 0x3F);
+				new_LayerId = (nalu[0] & 0x3F);
+				if (cur_LayerId > new_LayerId) {
+					builder->hevc_payload_hdr[0] = (builder->hevc_payload_hdr[0] & 0xC0) | new_LayerId;
+				}
+				cur_TID = builder->hevc_payload_hdr[1] & 0x3;
+				new_TID = nalu[1] & 0x3;
+				if (cur_TID > new_TID) {
+					builder->hevc_payload_hdr[1] = (builder->hevc_payload_hdr[1] & 0xF8) | new_TID;
+				}
+			}
+
+			/*add NALU size*/
+			nal_s[0] = nalu_size>>8;
+			nal_s[1] = nalu_size&0x00ff;
+			builder->OnData(builder->cbk_obj, (char *)nal_s, 2, GF_FALSE);
+			builder->bytesInPacket += 2;
+		}
+		/*add data*/
+		if (builder->OnDataReference)
+			builder->OnDataReference(builder->cbk_obj, nalu_size, 0);
+		else
+			builder->OnData(builder->cbk_obj, nalu, nalu_size, GF_FALSE);
+
+		builder->bytesInPacket += nalu_size;
+
+		if (IsAUEnd) {
+			builder->rtp_header.Marker = 1;
+			if (strlen(builder->hevc_payload_hdr)) {
+				builder->OnData(builder->cbk_obj, (char *)builder->hevc_payload_hdr, 2, GF_TRUE);
+				memset(builder->hevc_payload_hdr, 0, 2);
+			}
+			builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+			builder->bytesInPacket = 0;
+		}
+	}
+	/*fragmentation unit*/
+	else {
+		u32 offset;
+		u8 payload_hdr[2];
+		u8 shdr;
+
+		gf_assert(nalu_size + 4 >=builder->Path_MTU);
+		gf_assert(!builder->bytesInPacket);
+
+		/*FU payload doesn't have the NAL hdr (2 bytes*/
+		bytesLeft = nalu_size - 2;
+		offset = 2;
+		while (bytesLeft) {
+			u8 nut;
+
+			if (3 + bytesLeft > builder->Path_MTU) {
+				size = builder->Path_MTU - 3;
+			} else {
+				size = bytesLeft;
+			}
+
+			/*declare PayloadHdr for FU*/
+			memset(payload_hdr, 0, 2);
+			/*copy everything and assign type*/
+			payload_hdr[0] = nalu[0];
+			payload_hdr[1] = (((u32) RTP_VVC_FRAG_NAL) << 3) | (nalu[1] & 0x7);
+			builder->OnData(builder->cbk_obj, (char *)payload_hdr, 2, GF_FALSE);
+
+			nut = nalu[1] >> 3;
+			/*declare FU header*/
+			shdr = 0;
+			/*assign type*/
+			shdr |= nut;
+			/*start bit*/
+			if (offset==2) shdr |= 0x80;
+			/*end bit*/
+			else if (size == bytesLeft) {
+				shdr |= 0x40;
+				//VCL, last packet of picture, set P bit
+				if (nut <= GF_VVC_NALU_SLICE_GDR) {
+					shdr |= 0x20;
+				}
+			}
+
+			builder->OnData(builder->cbk_obj, &shdr, 1, GF_FALSE);
+
+			/*add data*/
+			if (builder->OnDataReference)
+				builder->OnDataReference(builder->cbk_obj, size, offset);
+			else
+				builder->OnData(builder->cbk_obj, nalu+offset, size, GF_FALSE);
+
+			offset += size;
+			bytesLeft -= size;
+
+			/*flush no matter what (FUs cannot be agreggated)*/
+			builder->rtp_header.Marker = (IsAUEnd && !bytesLeft) ? 1 : 0;
+			builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+			builder->bytesInPacket = 0;
+
+			if (bytesLeft) {
+				builder->rtp_header.PayloadType = builder->PayloadType;
+				builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
+				builder->rtp_header.SequenceNumber += 1;
+				builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
+			}
+		}
+	}
+	return GF_OK;
+}
 void latm_flush(GP_RTPPacketizer *builder)
 {
 	if (builder->bytesInPacket) {
-		builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header); 
+		builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
 		builder->bytesInPacket = 0;
 	}
-	builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp; 
+	builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
 }
 
-GF_Err gp_rtp_builder_do_latm(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize, u32 duration) 
+GF_Err gp_rtp_builder_do_latm(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize, u32 duration)
 {
-	u32 size, latm_hdr_size, i, data_offset; 
-	Bool fragmented; 
-	unsigned char *latm_hdr; 
+	u32 size, latm_hdr_size, i, data_offset;
+	Bool fragmented;
 
 	if (!data) {
 		latm_flush(builder);
-		return GF_OK; 
+		return GF_OK;
 	}
-	
+
 	if ((builder->flags & GP_RTP_PCK_USE_MULTI) && builder->max_ptime) {
-		if ((u32) builder->sl_header.compositionTimeStamp + duration >= builder->rtp_header.TimeStamp + builder->max_ptime) 
+		if ((u32) builder->sl_header.compositionTimeStamp + duration >= builder->rtp_header.TimeStamp + builder->max_ptime)
 			latm_flush(builder);
 	}
 	/*compute max size for frame, flush current if this doesn't fit*/
-	latm_hdr_size = (data_size / 255) + 1; 
+	latm_hdr_size = (data_size / 255) + 1;
 	if (latm_hdr_size+data_size > builder->Path_MTU - builder->bytesInPacket) {
 		latm_flush(builder);
 	}
 
 	data_offset = 0;
-	fragmented = 0;
-	while (data_size > 0) { 
-
-		latm_hdr_size = (data_size / 255) + 1; 
+	fragmented = GF_FALSE;
+	while (data_size > 0) {
+		u8 *latm_hdr;
+		latm_hdr_size = (data_size / 255) + 1;
 		/*fragmenting*/
 		if (latm_hdr_size + data_size > builder->Path_MTU) {
-			assert(!builder->bytesInPacket);
-			fragmented = 1;
-			latm_hdr_size = (builder->Path_MTU / 255) + 1; 
-			size = builder->Path_MTU - latm_hdr_size; 
-			builder->rtp_header.Marker = 0; 
+			gf_assert(!builder->bytesInPacket);
+			fragmented = GF_TRUE;
+			latm_hdr_size = (builder->Path_MTU / 255) + 1;
+			size = builder->Path_MTU - latm_hdr_size;
+			builder->rtp_header.Marker = 0;
 		}
 		/*last fragment or full AU*/
-		else { 
-			fragmented = 0;
-			size = data_size; 
-			builder->rtp_header.Marker = 1; 
-		} 
-		data_size -= size; 
-		
-		/*create new RTP Packet if needed*/ 
-		if (!builder->bytesInPacket) {
-			builder->rtp_header.SequenceNumber += 1; 
-			builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp; 
-			builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header); 
+		else {
+			fragmented = GF_FALSE;
+			size = data_size;
+			builder->rtp_header.Marker = 1;
 		}
-	
-		/* compute AudioMuxUnit header */ 
-		latm_hdr_size = (size / 255) + 1; 
-		latm_hdr = (unsigned char *)malloc( sizeof(char) * latm_hdr_size); 
-		for (i=0; i<latm_hdr_size-1; i++)  latm_hdr[i] = 255; 
-		latm_hdr[latm_hdr_size-1] = size % 255; 
-		
-		/*add LATM header IN ORDER in case we aggregate audioMuxElements in RTP*/ 
-		builder->OnData(builder->cbk_obj, (char*) latm_hdr, latm_hdr_size, 0);
+		data_size -= size;
+
+		/*create new RTP Packet if needed*/
+		if (!builder->bytesInPacket) {
+			builder->rtp_header.SequenceNumber += 1;
+			builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
+			builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
+		}
+
+		/* compute AudioMuxUnit header */
+		latm_hdr_size = (size / 255) + 1;
+		latm_hdr = (unsigned char *)gf_malloc( sizeof(char) * latm_hdr_size);
+		for (i=0; i<latm_hdr_size-1; i++)  latm_hdr[i] = 255;
+		latm_hdr[latm_hdr_size-1] = size % 255;
+
+		/*add LATM header IN ORDER in case we aggregate audioMuxElements in RTP*/
+		builder->OnData(builder->cbk_obj, (char*) latm_hdr, latm_hdr_size, GF_FALSE);
 		builder->bytesInPacket += latm_hdr_size;
-		free(latm_hdr);
-		
-		/*add payload*/ 
+		gf_free(latm_hdr);
+
+		/*add payload*/
 		if (builder->OnDataReference) {
-			builder->OnDataReference(builder->cbk_obj, size, data_offset); 
-		} else 
-			builder->OnData(builder->cbk_obj, data, size, 0); 
+			builder->OnDataReference(builder->cbk_obj, size, data_offset);
+		} else
+			builder->OnData(builder->cbk_obj, data, size, GF_FALSE);
 
 		builder->bytesInPacket += size;
 
@@ -606,10 +933,13 @@ GF_Err gp_rtp_builder_do_latm(GP_RTPPacketizer *builder, char *data, u32 data_si
 
 		/*fragmented AU, always flush packet*/
 		if (!builder->rtp_header.Marker) latm_flush(builder);
-	} 
+	}
 	/*if the AU has been fragmented or we don't use RTP aggregation, flush*/
-	if (! (builder->flags & GP_RTP_PCK_USE_MULTI) ) fragmented = 1;
+	if (! (builder->flags & GP_RTP_PCK_USE_MULTI) ) fragmented = GF_TRUE;
 	if (fragmented) latm_flush(builder);
-	
-	return GF_OK; 
+
+	return GF_OK;
 }
+
+
+#endif /*GPAC_DISABLE_STREAMING*/
