@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / IETF RTP/RTSP/SDP sub-project
@@ -10,30 +11,34 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
 
 
 #include <gpac/ietf.h>
+
+#ifndef GPAC_DISABLE_STREAMING
+
 #include <gpac/token.h>
 
+
 #define SDP_WRITE_STEPALLOC		2048
 
 
 GF_EXPORT
 GF_SDP_FMTP *gf_sdp_fmtp_new()
 {
-	GF_SDP_FMTP *tmp = (GF_SDP_FMTP*)malloc(sizeof(GF_SDP_FMTP));
+	GF_SDP_FMTP *tmp = (GF_SDP_FMTP*)gf_malloc(sizeof(GF_SDP_FMTP));
 	tmp->PayloadType = 0;
 	tmp->Attributes = gf_list_new();
 	return tmp;
@@ -42,17 +47,16 @@ GF_SDP_FMTP *gf_sdp_fmtp_new()
 GF_EXPORT
 void gf_sdp_fmtp_del(GF_SDP_FMTP *fmtp)
 {
-	GF_X_Attribute *att;
 	if (!fmtp) return;
 	while (gf_list_count(fmtp->Attributes)) {
-		att = (GF_X_Attribute*)gf_list_get(fmtp->Attributes, 0);
+		GF_X_Attribute *att = (GF_X_Attribute*)gf_list_get(fmtp->Attributes, 0);
 		gf_list_rem(fmtp->Attributes, 0);
-		if (att->Name) free(att->Name);
-		if (att->Value) free(att->Value);
-		free(att);
+		if (att->Name) gf_free(att->Name);
+		if (att->Value) gf_free(att->Value);
+		gf_free(att);
 	}
 	gf_list_del(fmtp->Attributes);
-	free(fmtp);
+	gf_free(fmtp);
 }
 
 GF_SDP_FMTP *SDP_GetFMTPForPayload(GF_SDPMedia *media, u32 PayloadType)
@@ -72,34 +76,35 @@ void SDP_ParseAttribute(GF_SDPInfo *sdp, char *buffer, GF_SDPMedia *media)
 	s32 pos;
 	u32 PayT;
 	char comp[3000];
-	GF_RTPMap *map;
-	GF_SDP_FMTP *fmtp;
 	GF_X_Attribute *att;
 
 	pos = gf_token_get(buffer, 0, " :\t\r\n", comp, 3000);
 
 	if (!strcmp(comp, "cat")) {
 		if (media) return;
-		pos = gf_token_get(buffer, pos, ":\t\r\n", comp, 3000);
-		sdp->a_cat = strdup(comp);
+		/*pos = */gf_token_get(buffer, pos, ":\t\r\n", comp, 3000);
+		if (sdp->a_cat) return;
+		sdp->a_cat = gf_strdup(comp);
 		return;
 	}
 	if (!strcmp(comp, "keywds")) {
 		if (media) return;
-		pos = gf_token_get(buffer, pos, ":\t\r\n", comp, 3000);
-		sdp->a_keywds = strdup(comp);
+		/*pos = */gf_token_get(buffer, pos, ":\t\r\n", comp, 3000);
+		if (sdp->a_keywds) return;
+		sdp->a_keywds = gf_strdup(comp);
 		return;
 	}
 	if (!strcmp(comp, "tool")) {
 		if (media) return;
-		pos = gf_token_get(buffer, pos, ":\r\n", comp, 3000);
-		sdp->a_tool = strdup(comp);
+		/*pos = */gf_token_get(buffer, pos, ":\r\n", comp, 3000);
+		if (sdp->a_tool) return;
+		sdp->a_tool = gf_strdup(comp);
 		return;
 	}
 
 	if (!strcmp(comp, "ptime")) {
 		if (!media) return;
-		pos = gf_token_get(buffer, pos, ":\r\n", comp, 3000);
+		/*pos = */gf_token_get(buffer, pos, ":\r\n", comp, 3000);
 		media->PacketTime = atoi(comp);
 		return;
 	}
@@ -129,60 +134,68 @@ void SDP_ParseAttribute(GF_SDPInfo *sdp, char *buffer, GF_SDPMedia *media)
 	}
 	if (!strcmp(comp, "orient")) {
 		if (!media || media->Type) return;
-		pos = gf_token_get(buffer, pos, ":\r\n", comp, 3000);
-		media->orientation = strdup(comp);
+		/*pos = */gf_token_get(buffer, pos, ":\r\n", comp, 3000);
+		if (media->orientation) return;
+		media->orientation = gf_strdup(comp);
 		return;
 	}
 	if (!strcmp(comp, "type")) {
 		if (media) return;
-		pos = gf_token_get(buffer, pos, ":\r\n", comp, 3000);
-		sdp->a_type = strdup(comp);
+		/*pos = */gf_token_get(buffer, pos, ":\r\n", comp, 3000);
+		if (sdp->a_type) return;
+		sdp->a_type = gf_strdup(comp);
 		return;
 	}
 	if (!strcmp(comp, "charset")) {
 		if (media) return;
-		pos = gf_token_get(buffer, pos, ":\r\n", comp, 3000);
-		sdp->a_charset = strdup(comp);
+		/*pos = */gf_token_get(buffer, pos, ":\r\n", comp, 3000);
+		if (sdp->a_charset)
+		sdp->a_charset = gf_strdup(comp);
 		return;
 	}
 	if (!strcmp(comp, "sdplang")) {
-		pos = gf_token_get(buffer, pos, ":\r\n", comp, 3000);
+		/*pos = */gf_token_get(buffer, pos, ":\r\n", comp, 3000);
 		if (media) {
-			media->sdplang = strdup(comp);
+			if (media->sdplang) return;
+			media->sdplang = gf_strdup(comp);
 		} else {
-			sdp->a_sdplang = strdup(comp);
+			if (sdp->a_sdplang) return;
+			sdp->a_sdplang = gf_strdup(comp);
 		}
 		return;
 	}
 	if (!strcmp(comp, "lang")) {
-		pos = gf_token_get(buffer, pos, ":\r\n", comp, 3000);
+		/*pos = */gf_token_get(buffer, pos, ":\r\n", comp, 3000);
 		if (media) {
-			media->lang = strdup(comp);
+			if (media->lang) return;
+			media->lang = gf_strdup(comp);
 		} else {
-			sdp->a_lang = strdup(comp);
+			if (sdp->a_lang) return;
+			sdp->a_lang = gf_strdup(comp);
 		}
 		return;
 	}
 	if (!strcmp(comp, "framerate")) {
 		//only for video
 		if (!media || (media->Type != 1)) return;
-		pos = gf_token_get(buffer, pos, ":\r\n", comp, 3000);
+		/*pos = */gf_token_get(buffer, pos, ":\r\n", comp, 3000);
 		media->FrameRate = atof(comp);
 		return;
 	}
 	if (!strcmp(comp, "quality")) {
 		if (!media) return;
-		pos = gf_token_get(buffer, pos, ":\r\n", comp, 3000);
+		/*pos = */gf_token_get(buffer, pos, ":\r\n", comp, 3000);
 		media->Quality = atoi(comp);
 		return;
 	}
 	if (!strcmp(comp, "rtpmap")) {
+		GF_RTPMap *map;
 		if (!media) return;
-		map = (GF_RTPMap*)malloc(sizeof(GF_RTPMap));
+		map = (GF_RTPMap*)gf_malloc(sizeof(GF_RTPMap));
 		pos = gf_token_get(buffer, pos, ": \r\n", comp, 3000);
 		map->PayloadType = atoi(comp);
 		pos = gf_token_get(buffer, pos, " /\r\n", comp, 3000);
-		map->payload_name = strdup(comp);
+		map->payload_name = gf_strdup(comp);
 		pos = gf_token_get(buffer, pos, " /\r\n", comp, 3000);
 		map->ClockRate = atoi(comp);
 		pos = gf_token_get(buffer, pos, " /\r\n", comp, 3000);
@@ -192,6 +205,7 @@ void SDP_ParseAttribute(GF_SDPInfo *sdp, char *buffer, GF_SDPMedia *media)
 	}
 	//FMTP
 	if (!strcmp(comp, "fmtp")) {
+		GF_SDP_FMTP *fmtp;
 		if (!media) return;
 		pos = gf_token_get(buffer, pos, ": \r\n", comp, 3000);
 		PayT = atoi(comp);
@@ -204,12 +218,12 @@ void SDP_ParseAttribute(GF_SDPInfo *sdp, char *buffer, GF_SDPMedia *media)
 		while (1) {
 			pos = gf_token_get(buffer, pos, "; =\r\n", comp, 3000);
 			if (pos <= 0) break;
-			att = (GF_X_Attribute*)malloc(sizeof(GF_X_Attribute));
-			att->Name = strdup(comp);
+			att = (GF_X_Attribute*)gf_malloc(sizeof(GF_X_Attribute));
+			att->Name = gf_strdup(comp);
 			att->Value = NULL;
 			pos ++;
 			pos = gf_token_get(buffer, pos, ";\r\n", comp, 3000);
-			if (pos > 0) att->Value = strdup(comp);
+			if (pos > 0) att->Value = gf_strdup(comp);
 			gf_list_add(fmtp->Attributes, att);
 		}
 		return;
@@ -219,13 +233,17 @@ void SDP_ParseAttribute(GF_SDPInfo *sdp, char *buffer, GF_SDPMedia *media)
 	//a= <attribute> || <attribute>:<value>
 	//we add <attribute> <value> in case ...
 	pos = gf_token_get(buffer, 0, " :\r\n", comp, 3000);
-	att = (GF_X_Attribute*)malloc(sizeof(GF_X_Attribute));
-	att->Name = strdup(comp);
+	att = (GF_X_Attribute*)gf_malloc(sizeof(GF_X_Attribute));
+	att->Name = gf_strdup(comp);
 	att->Value = NULL;
-	pos += 1;
-	if (buffer[pos] == ' ') pos += 1;
+
+	if (pos+1 < (s32)strlen(buffer)) {
+		pos += 1;
+		if (buffer[pos] == ' ') pos += 1;
+	}
+
 	pos = gf_token_get(buffer, pos, "\r\n", comp, 3000);
-	if (pos > 0) att->Value = strdup(comp);
+	if (pos > 0) att->Value = gf_strdup(comp);
 
 	if (media) {
 		gf_list_add(media->Attributes, att);
@@ -236,53 +254,48 @@ void SDP_ParseAttribute(GF_SDPInfo *sdp, char *buffer, GF_SDPMedia *media)
 
 
 
-#define SDPM_DESTROY(p) if (media->p) free(media->p)
+#define SDPM_DESTROY(p) if (media->p) gf_free(media->p)
 GF_EXPORT
 void gf_sdp_media_del(GF_SDPMedia *media)
 {
-	GF_SDPBandwidth *bw;
-	GF_RTPMap *map;
-	GF_SDPConnection *conn;
-	GF_SDP_FMTP *fmtp;
-	GF_X_Attribute *att;
 	if (!media) return;
 
 	while (gf_list_count(media->FMTP)) {
-		fmtp = (GF_SDP_FMTP*)gf_list_get(media->FMTP, 0);
+		GF_SDP_FMTP *fmtp = (GF_SDP_FMTP*)gf_list_get(media->FMTP, 0);
 		gf_list_rem(media->FMTP, 0);
 		gf_sdp_fmtp_del(fmtp);
 	}
 	gf_list_del(media->FMTP);
 
 	while (gf_list_count(media->Attributes)) {
-		att = (GF_X_Attribute*)gf_list_get(media->Attributes, 0);
+		GF_X_Attribute *att = (GF_X_Attribute*)gf_list_get(media->Attributes, 0);
 		gf_list_rem(media->Attributes, 0);
-		if (att->Name) free(att->Name);
-		if (att->Value) free(att->Value);
-		free(att);
+		if (att->Name) gf_free(att->Name);
+		if (att->Value) gf_free(att->Value);
+		gf_free(att);
 	}
 	gf_list_del(media->Attributes);
 
 	while (gf_list_count(media->RTPMaps)) {
-		map = (GF_RTPMap*)gf_list_get(media->RTPMaps, 0);
-		free(map->payload_name);
-		free(map);
+		GF_RTPMap *map = (GF_RTPMap*)gf_list_get(media->RTPMaps, 0);
+		gf_free(map->payload_name);
+		gf_free(map);
 		gf_list_rem(media->RTPMaps, 0);
 	}
 	gf_list_del(media->RTPMaps);
-	
+
 	while (gf_list_count(media->Connections)) {
-		conn = (GF_SDPConnection*)gf_list_get(media->Connections, 0);
+		GF_SDPConnection *conn = (GF_SDPConnection*)gf_list_get(media->Connections, 0);
 		gf_list_rem(media->Connections, 0);
 		gf_sdp_conn_del(conn);
 	}
 	gf_list_del(media->Connections);
 
 	while (gf_list_count(media->Bandwidths)) {
-		bw = (GF_SDPBandwidth*)gf_list_get(media->Bandwidths, 0);
+		GF_SDPBandwidth *bw = (GF_SDPBandwidth*)gf_list_get(media->Bandwidths, 0);
 		gf_list_rem(media->Bandwidths, 0);
-		if (bw->name) free(bw->name);
-		free(bw);
+		if (bw->name) gf_free(bw->name);
+		gf_free(bw);
 	}
 	gf_list_del(media->Bandwidths);
 
@@ -293,7 +306,7 @@ void gf_sdp_media_del(GF_SDPMedia *media)
 	SDPM_DESTROY(fmt_list);
 	SDPM_DESTROY(k_method);
 	SDPM_DESTROY(k_key);
-	free(media);
+	gf_free(media);
 }
 
 
@@ -302,6 +315,7 @@ GF_SDPConnection *gf_sdp_conn_new()
 {
 	GF_SDPConnection *conn;
 	GF_SAFEALLOC(conn, GF_SDPConnection);
+	if (!conn) return NULL;
 	conn->TTL = -1;
 	return conn;
 }
@@ -309,10 +323,10 @@ GF_SDPConnection *gf_sdp_conn_new()
 GF_EXPORT
 void gf_sdp_conn_del(GF_SDPConnection *conn)
 {
-	if (conn->add_type) free(conn->add_type);
-	if (conn->host) free(conn->host);
-	if (conn->net_type) free(conn->net_type);
-	free(conn);
+	if (conn->add_type) gf_free(conn->add_type);
+	if (conn->host) gf_free(conn->host);
+	if (conn->net_type) gf_free(conn->net_type);
+	gf_free(conn);
 }
 
 GF_EXPORT
@@ -320,6 +334,7 @@ GF_SDPMedia *gf_sdp_media_new()
 {
 	GF_SDPMedia *tmp;
 	GF_SAFEALLOC(tmp, GF_SDPMedia);
+	if (!tmp) return NULL;
 	tmp->FMTP = gf_list_new();
 	tmp->RTPMaps = gf_list_new();
 	tmp->Attributes = gf_list_new();
@@ -334,6 +349,7 @@ GF_SDPInfo *gf_sdp_info_new()
 {
 	GF_SDPInfo *sdp;
 	GF_SAFEALLOC(sdp, GF_SDPInfo);
+	if (!sdp) return NULL;
 	sdp->b_bandwidth = gf_list_new();
 	sdp->media_desc = gf_list_new();
 	sdp->Attributes = gf_list_new();
@@ -341,43 +357,36 @@ GF_SDPInfo *gf_sdp_info_new()
 	return sdp;
 }
 
-#define SDP_DESTROY(p) if (sdp->p)	\
-					free(sdp->p);	\
-					sdp->p = NULL;
+#define SDP_DESTROY(p) if (sdp->p) { gf_free(sdp->p); sdp->p = NULL; }
 
 
 GF_EXPORT
 void gf_sdp_info_reset(GF_SDPInfo *sdp)
 {
-	GF_SDPBandwidth *bw;
-	GF_SDPMedia *media;
-	GF_SDPTiming *timing;
-	GF_X_Attribute *att;
-
 	if (!sdp) return;
-	
+
 	while (gf_list_count(sdp->media_desc)) {
-		media = (GF_SDPMedia*)gf_list_get(sdp->media_desc, 0);
+		GF_SDPMedia *media = (GF_SDPMedia*)gf_list_get(sdp->media_desc, 0);
 		gf_list_rem(sdp->media_desc, 0);
 		gf_sdp_media_del(media);
 	}
 	while (gf_list_count(sdp->Attributes)) {
-		att = (GF_X_Attribute*)gf_list_get(sdp->Attributes, 0);
+		GF_X_Attribute *att = (GF_X_Attribute*)gf_list_get(sdp->Attributes, 0);
 		gf_list_rem(sdp->Attributes, 0);
-		if (att->Name) free(att->Name);
-		if (att->Value) free(att->Value);
-		free(att);
+		if (att->Name) gf_free(att->Name);
+		if (att->Value) gf_free(att->Value);
+		gf_free(att);
 	}
 	while (gf_list_count(sdp->b_bandwidth)) {
-		bw = (GF_SDPBandwidth*)gf_list_get(sdp->b_bandwidth, 0);
+		GF_SDPBandwidth *bw = (GF_SDPBandwidth*)gf_list_get(sdp->b_bandwidth, 0);
 		gf_list_rem(sdp->b_bandwidth, 0);
-		if (bw->name) free(bw->name);
-		free(bw);
+		if (bw->name) gf_free(bw->name);
+		gf_free(bw);
 	}
 	while (gf_list_count(sdp->Timing)) {
-		timing = (GF_SDPTiming*)gf_list_get(sdp->Timing, 0);
+		GF_SDPTiming *timing = (GF_SDPTiming*)gf_list_get(sdp->Timing, 0);
 		gf_list_rem(sdp->Timing, 0);
-		free(timing);
+		gf_free(timing);
 	}
 
 	//then delete all info ...
@@ -411,14 +420,14 @@ void gf_sdp_info_reset(GF_SDPInfo *sdp)
 
 GF_EXPORT
 void gf_sdp_info_del(GF_SDPInfo *sdp)
-{	
+{
 	if (!sdp) return;
 	gf_sdp_info_reset(sdp);
 	gf_list_del(sdp->media_desc);
 	gf_list_del(sdp->Attributes);
 	gf_list_del(sdp->b_bandwidth);
 	gf_list_del(sdp->Timing);
-	free(sdp);
+	gf_free(sdp);
 }
 
 
@@ -430,13 +439,13 @@ Bool SDP_IsDynamicPayload(GF_SDPMedia *media, char *payt)
 	i=0;
 	while ((map = (GF_RTPMap*)gf_list_enum(media->RTPMaps, &i))) {
 		sprintf(buf, "%d", map->PayloadType);
-		if (!strcmp(payt, buf)) return 1;
+		if (!strcmp(payt, buf)) return GF_TRUE;
 	}
-	return 0;
+	return GF_FALSE;
 }
 
-//translate h || m || d in sec. Fractions are not allowed with this writing 
-s32 SDP_MakeSeconds(char *buf)
+//translate h || m || d in sec. Fractions are not allowed with this writing
+static s32 SDP_MakeSeconds(char *buf)
 {
 	s32 sign;
 	char num[30], *test;
@@ -448,17 +457,26 @@ s32 SDP_MakeSeconds(char *buf)
 	memset(num, 0, 30);
 	test = strstr(buf, "d");
 	if (test) {
-		strncpy(num, buf, strlen(buf)-strlen(test));
+		if (strlen(buf)-strlen(test) < sizeof(num))
+			memcpy(num, buf, MIN(sizeof(num)-1, strlen(buf)-strlen(test)));
+		else
+			gf_assert(0);
 		return (atoi(num)*sign*86400);
 	}
 	test = strstr(buf, "h");
 	if (test) {
-		strncpy(num, buf, strlen(buf)-strlen(test));
+		if (strlen(buf)-strlen(test) < sizeof(num))
+			memcpy(num, buf, MIN(sizeof(num)-1, strlen(buf)-strlen(test)));
+		else
+			gf_assert(0);
 		return (atoi(num)*sign*3600);
 	}
 	test = strstr(buf, "m");
 	if (test) {
-		strncpy(num, buf, strlen(buf)-strlen(test));
+		if (strlen(buf)-strlen(test) < sizeof(num))
+			memcpy(num, buf, MIN(sizeof(num)-1, strlen(buf)-strlen(test)));
+		else
+			gf_assert(0);
 		return (atoi(num)*sign*60);
 	}
 	return (atoi(buf) * sign);
@@ -481,6 +499,12 @@ GF_Err gf_sdp_info_parse(GF_SDPInfo *sdp, char *sdp_text, u32 text_size)
 
 	if (!sdp) return GF_BAD_PARAM;
 
+#ifdef GPAC_ENABLE_COVERAGE
+	if (gf_sys_is_cov_mode()) {
+		SDP_MakeSeconds("30m");
+	}
+#endif
+
 	//Clean SDP info
 	gf_sdp_info_reset(sdp);
 
@@ -490,73 +514,78 @@ GF_Err gf_sdp_info_parse(GF_SDPInfo *sdp, char *sdp_text, u32 text_size)
 		if (LinePos <= 0) break;
 		if (!strcmp(LineBuf, "\r\n") || !strcmp(LineBuf, "\n") || !strcmp(LineBuf, "\r")) continue;
 
-
+		pos=0;
 		switch (LineBuf[0]) {
 		case 'v':
-			pos = gf_token_get(LineBuf, 2, "\t\r\n", comp, 3000);
+			/*pos = */gf_token_get(LineBuf, 2, "\t\r\n", comp, 3000);
 			sdp->Version = atoi(comp);
 			break;
 		case 'o':
+			//only use first one
+			if (sdp->o_username) break;
 			pos = gf_token_get(LineBuf, 2, " \t\r\n", comp, 3000);
-			sdp->o_username = strdup(comp);
+			sdp->o_username = gf_strdup(comp);
 			pos = gf_token_get(LineBuf, pos, " \t\r\n", comp, 3000);
-			sdp->o_session_id = strdup(comp);
+			sdp->o_session_id = gf_strdup(comp);
 			pos = gf_token_get(LineBuf, pos, " \t\r\n", comp, 3000);
-			sdp->o_version = strdup(comp);
-			
-			pos = gf_token_get(LineBuf, pos, " \t\r\n", comp, 3000);
-			sdp->o_net_type = strdup(comp);
+			sdp->o_version = gf_strdup(comp);
 
 			pos = gf_token_get(LineBuf, pos, " \t\r\n", comp, 3000);
-			sdp->o_add_type = strdup(comp);
+			sdp->o_net_type = gf_strdup(comp);
 
 			pos = gf_token_get(LineBuf, pos, " \t\r\n", comp, 3000);
-			sdp->o_address = strdup(comp);
+			sdp->o_add_type = gf_strdup(comp);
+
+			/*pos = */gf_token_get(LineBuf, pos, " \t\r\n", comp, 3000);
+			sdp->o_address = gf_strdup(comp);
 			break;
 		case 's':
-			pos = gf_token_get(LineBuf, 2, "\t\r\n", comp, 3000);
-			sdp->s_session_name = strdup(comp);
+			if (sdp->s_session_name) break;
+			/*pos = */gf_token_get(LineBuf, 2, "\t\r\n", comp, 3000);
+			sdp->s_session_name = gf_strdup(comp);
 			break;
 		case 'i':
-			pos = gf_token_get(LineBuf, 2, "\t\r\n", comp, 3000);
-			sdp->i_description = strdup(comp);
+			if (sdp->i_description) break;
+			/*pos = */gf_token_get(LineBuf, 2, "\t\r\n", comp, 3000);
+			sdp->i_description = gf_strdup(comp);
 			break;
 		case 'u':
-			pos = gf_token_get(LineBuf, 2, "\t\r\n", comp, 3000);
-			sdp->u_uri = strdup(comp);
+			if (sdp->u_uri) break;
+			/*pos = */gf_token_get(LineBuf, 2, "\t\r\n", comp, 3000);
+			sdp->u_uri = gf_strdup(comp);
 			break;
 		case 'e':
-			pos = gf_token_get(LineBuf, 2, "\t\r\n", comp, 3000);
-			sdp->e_email = strdup(comp);
+			if (sdp->e_email) break;
+			/*pos = */gf_token_get(LineBuf, 2, "\t\r\n", comp, 3000);
+			sdp->e_email = gf_strdup(comp);
 			break;
 		case 'p':
-			pos = gf_token_get(LineBuf, 2, "\t\r\n", comp, 3000);
-			sdp->p_phone = strdup(comp);
+			if (sdp->p_phone) break;
+			/*pos = */gf_token_get(LineBuf, 2, "\t\r\n", comp, 3000);
+			sdp->p_phone = gf_strdup(comp);
 			break;
 		case 'c':
 			//if at session level, only 1 is allowed for all SDP
 			if (sdp->c_connection) break;
-			
+
 			conn = gf_sdp_conn_new();
 
 			pos = gf_token_get(LineBuf, 2, " \t\r\n", comp, 3000);
-			conn->net_type = strdup(comp);
-			
+			conn->net_type = gf_strdup(comp);
+
 			pos = gf_token_get(LineBuf, pos, " \t\r\n", comp, 3000);
-			conn->add_type = strdup(comp);
+			conn->add_type = gf_strdup(comp);
 
 			pos = gf_token_get(LineBuf, pos, " /\r\n", comp, 3000);
-			conn->host = strdup(comp);
+			conn->host = gf_strdup(comp);
 			if (gf_sk_is_multicast_address(conn->host)) {
 				//a valid SDP will have TTL if address is multicast
 				pos = gf_token_get(LineBuf, pos, "/\r\n", comp, 3000);
-				if (pos <= 0) {
-					gf_sdp_conn_del(conn);
-					break;
+				if (pos > 0) {
+					conn->TTL = atoi(comp);
+					//multiple address indication is only valid for media
+					pos = gf_token_get(LineBuf, pos, "/\r\n", comp, 3000);
 				}
-				conn->TTL = atoi(comp);
-				//multiple address indication is only valid for media
-				pos = gf_token_get(LineBuf, pos, "/\r\n", comp, 3000);
 				if (pos > 0) {
 					if (!media) {
 						gf_sdp_conn_del(conn);
@@ -565,9 +594,9 @@ GF_Err gf_sdp_info_parse(GF_SDPInfo *sdp, char *sdp_text, u32 text_size)
 					conn->add_count = atoi(comp);
 				}
 			}
-			if (!media) 
+			if (!media)
 				sdp->c_connection = conn;
-			else 
+			else
 				gf_list_add(media->Connections, conn);
 
 			break;
@@ -575,9 +604,10 @@ GF_Err gf_sdp_info_parse(GF_SDPInfo *sdp, char *sdp_text, u32 text_size)
 			pos = gf_token_get(LineBuf, 2, ":\r\n", comp, 3000);
 			if (strcmp(comp, "CT") && strcmp(comp, "AS") && (comp[0] != 'X')) break;
 
-			bw = (GF_SDPBandwidth*)malloc(sizeof(GF_SDPBandwidth));
-			bw->name = strdup(comp);
-			pos = gf_token_get(LineBuf, pos, ":\r\n", comp, 3000);
+			GF_SAFEALLOC(bw, GF_SDPBandwidth);
+			if (!bw) return GF_OUT_OF_MEM;
+			bw->name = gf_strdup(comp);
+			/*pos = */gf_token_get(LineBuf, pos, ":\r\n", comp, 3000);
 			bw->value = atoi(comp);
 			if (media) {
 				gf_list_add(media->Bandwidths, bw);
@@ -590,19 +620,22 @@ GF_Err gf_sdp_info_parse(GF_SDPInfo *sdp, char *sdp_text, u32 text_size)
 			if (media) break;
 			//create a new time structure for each entry
 			GF_SAFEALLOC(timing, GF_SDPTiming);
+			if (!timing) return GF_OUT_OF_MEM;
 			pos = gf_token_get(LineBuf, 2, " \t\r\n", comp, 3000);
 			timing->StartTime = atoi(comp);
-			pos = gf_token_get(LineBuf, pos, "\r\n", comp, 3000);
+			/*pos = */gf_token_get(LineBuf, pos, "\r\n", comp, 3000);
 			timing->StopTime = atoi(comp);
 			gf_list_add(sdp->Timing, timing);
 			break;
 		case 'r':
 			if (media) break;
 			pos = gf_token_get(LineBuf, 2, " \t\r\n", comp, 3000);
+			if (!timing) return GF_NON_COMPLIANT_BITSTREAM;
 			timing->RepeatInterval = SDP_MakeSeconds(comp);
 			pos = gf_token_get(LineBuf, pos, " \t\r\n", comp, 3000);
 			timing->ActiveDuration = SDP_MakeSeconds(comp);
-			while (1) {
+			while (pos>=0) {
+				if (timing->NbRepeatOffsets == GF_SDP_MAX_TIMEOFFSET) break;
 				pos = gf_token_get(LineBuf, pos, " \t\r\n", comp, 3000);
 				if (pos <= 0) break;
 				timing->OffsetFromStart[timing->NbRepeatOffsets] = SDP_MakeSeconds(comp);
@@ -612,9 +645,11 @@ GF_Err gf_sdp_info_parse(GF_SDPInfo *sdp, char *sdp_text, u32 text_size)
 		case 'z':
 			if (media) break;
 			pos = 2;
+			if (!timing) return GF_NON_COMPLIANT_BITSTREAM;
 			while (1) {
 				pos = gf_token_get(LineBuf, pos, " \t\r\n", comp, 3000);
 				if (pos <= 0) break;
+				if (timing->NbZoneOffsets >= GF_SDP_MAX_TIMEOFFSET) break;
 				timing->AdjustmentTime[timing->NbZoneOffsets] = atoi(comp);
 				pos = gf_token_get(LineBuf, pos, " \t\r\n", comp, 3000);
 				timing->AdjustmentOffset[timing->NbZoneOffsets] = SDP_MakeSeconds(comp);
@@ -622,18 +657,19 @@ GF_Err gf_sdp_info_parse(GF_SDPInfo *sdp, char *sdp_text, u32 text_size)
 			}
 			break;
 		case 'k':
+			if (sdp->k_method) break;
 			pos = gf_token_get(LineBuf, 2, ":\t\r\n", comp, 3000);
 			if (media) {
-				media->k_method = strdup(comp);
+				media->k_method = gf_strdup(comp);
 			} else {
-				sdp->k_method = strdup(comp);
+				sdp->k_method = gf_strdup(comp);
 			}
 			pos = gf_token_get(LineBuf, pos, ":\r\n", comp, 3000);
 			if (pos > 0) {
 				if (media) {
-					media->k_key = strdup(comp);
+					media->k_key = gf_strdup(comp);
 				} else {
-					sdp->k_key = strdup(comp);
+					sdp->k_key = gf_strdup(comp);
 				}
 			}
 			break;
@@ -642,12 +678,12 @@ GF_Err gf_sdp_info_parse(GF_SDPInfo *sdp, char *sdp_text, u32 text_size)
 			break;
 		case 'm':
 			pos = gf_token_get(LineBuf, 2, " \t\r\n", comp, 3000);
-			if (strcmp(comp, "audio") 
-				&& strcmp(comp, "data") 
-				&& strcmp(comp, "control") 
-				&& strcmp(comp, "video") 
-				&& strcmp(comp, "text") 
-				&& strcmp(comp, "application")) {
+			if (strcmp(comp, "audio")
+			        && strcmp(comp, "data")
+			        && strcmp(comp, "control")
+			        && strcmp(comp, "video")
+			        && strcmp(comp, "text")
+			        && strcmp(comp, "application")) {
 				return GF_SERVICE_ERROR;
 			}
 			media = gf_sdp_media_new();
@@ -658,7 +694,7 @@ GF_Err gf_sdp_info_parse(GF_SDPInfo *sdp, char *sdp_text, u32 text_size)
 			else if (!strcmp(comp, "data")) media->Type = 4;
 			else if (!strcmp(comp, "control")) media->Type = 5;
 			else media->Type = 0;
-			//port numbers			
+			//port numbers
 			gf_token_get(LineBuf, pos, " ", comp, 3000);
 			if (!strstr(comp, "/")) {
 				pos = gf_token_get(LineBuf, pos, " \r\n", comp, 3000);
@@ -672,15 +708,18 @@ GF_Err gf_sdp_info_parse(GF_SDPInfo *sdp, char *sdp_text, u32 text_size)
 			}
 			//transport Profile
 			pos = gf_token_get(LineBuf, pos, " \r\n", comp, 3000);
-			media->Profile = strdup(comp);
-			pos = gf_token_get(LineBuf, pos, " \r\n", comp, 3000);
-			media->fmt_list = strdup(comp);
+			media->Profile = gf_strdup(comp);
+			/*pos = */gf_token_get(LineBuf, pos, " \r\n", comp, 3000);
+			media->fmt_list = gf_strdup(comp);
 
 			gf_list_add(sdp->media_desc, media);
 			break;
 		}
+
+		if (pos<0)
+			return GF_NON_COMPLIANT_BITSTREAM;
 	}
-	//finally rewrite the fmt_list for all media, and remove dynamic payloads 
+	//finally rewrite the fmt_list for all media, and remove dynamic payloads
 	//from the list
 	i=0;
 	while ((media = (GF_SDPMedia*)gf_list_enum(sdp->media_desc, &i))) {
@@ -699,10 +738,10 @@ GF_Err gf_sdp_info_parse(GF_SDPInfo *sdp, char *sdp_text, u32 text_size)
 				}
 				strcat(LineBuf, comp);
 			}
-			free(media->fmt_list);
+			gf_free(media->fmt_list);
 			media->fmt_list = NULL;
 			if (strlen(LineBuf)) {
-				media->fmt_list = strdup(LineBuf);
+				media->fmt_list = gf_strdup(LineBuf);
 			}
 		}
 	}
@@ -710,8 +749,9 @@ GF_Err gf_sdp_info_parse(GF_SDPInfo *sdp, char *sdp_text, u32 text_size)
 }
 
 
+#if 0 //unused
 
-GF_Err SDP_CheckConnection(GF_SDPConnection *conn)
+static GF_Err SDP_CheckConnection(GF_SDPConnection *conn)
 {
 	if (!conn) return GF_BAD_PARAM;
 	if (!conn->host || !conn->add_type || !conn->net_type) return GF_REMOTE_SERVICE_ERROR;
@@ -726,7 +766,6 @@ GF_Err SDP_CheckConnection(GF_SDPConnection *conn)
 
 //return GF_BAD_PARAM if invalid structure, GF_REMOTE_SERVICE_ERROR if bad formatting
 //or GF_OK
-GF_EXPORT
 GF_Err gf_sdp_info_check(GF_SDPInfo *sdp)
 {
 	GF_Err e;
@@ -742,38 +781,39 @@ GF_Err gf_sdp_info_check(GF_SDPInfo *sdp)
 
 	//normative fields
 	//o=
-	if (!sdp->o_add_type || !sdp->o_address || !sdp->o_username || !sdp->o_session_id || !sdp->o_version) 
+	if (!sdp->o_add_type || !sdp->o_address || !sdp->o_username || !sdp->o_session_id || !sdp->o_version)
 		return GF_REMOTE_SERVICE_ERROR;
 	//s=
-	if (!sdp->s_session_name) return GF_REMOTE_SERVICE_ERROR;
+	//commented for intermedia demos
+//	if (!sdp->s_session_name) return GF_REMOTE_SERVICE_ERROR;
 	//t=
 //	if () return GF_REMOTE_SERVICE_ERROR;
 	//c=
-	if (sdp->c_connection) { 
+	if (sdp->c_connection) {
 		e = SDP_CheckConnection(sdp->c_connection);
 		if (e) return e;
 		//multiple addresses are only for media desc
 		if (sdp->c_connection->add_count >= 2) return GF_REMOTE_SERVICE_ERROR;
-		HasGlobalConnection = 1;
+		HasGlobalConnection = GF_TRUE;
 	} else {
-		HasGlobalConnection = 0;
+		HasGlobalConnection = GF_FALSE;
 	}
 
 	//then check all media
 	i=0;
 	while ((media = (GF_SDPMedia*)gf_list_enum(sdp->media_desc, &i))) {
-		HasSeveralPorts = 0;
+		HasSeveralPorts = GF_FALSE;
 
 		//m= : force non-null port, profile and fmt_list
 		if (/*!media->PortNumber || */ !media->Profile) return GF_REMOTE_SERVICE_ERROR;
-		if (media->NumPorts) HasSeveralPorts = 1;
+		if (media->NumPorts) HasSeveralPorts = GF_TRUE;
 
 		//no connections specified - THIS IS AN ERROR IN SDP BUT NOT IN ALL RTSP SESSIONS...
 //		if (!HasGlobalConnection && !gf_list_count(media->Connections)) return GF_REMOTE_SERVICE_ERROR;
 		//too many connections specified
 		if (HasGlobalConnection && gf_list_count(media->Connections)) return GF_REMOTE_SERVICE_ERROR;
 
-		//check all connections, and make sure we don't have multiple addresses 
+		//check all connections, and make sure we don't have multiple addresses
 		//and multiple ports at the same time
 		count = gf_list_count(media->Connections);
 		if (count>1 && HasSeveralPorts) return GF_REMOTE_SERVICE_ERROR;
@@ -782,7 +822,7 @@ GF_Err gf_sdp_info_check(GF_SDPInfo *sdp)
 			conn = (GF_SDPConnection*)gf_list_get(media->Connections, j);
 			e = SDP_CheckConnection(conn);
 			if (e) return e;
-			if ((conn->add_count >= 2) && HasSeveralPorts) return GF_REMOTE_SERVICE_ERROR; 
+			if ((conn->add_count >= 2) && HasSeveralPorts) return GF_REMOTE_SERVICE_ERROR;
 		}
 		//RTPMaps. 0 is tolerated, but if some are specified check them
 		j=0;
@@ -793,7 +833,7 @@ GF_Err gf_sdp_info_check(GF_SDPInfo *sdp)
 			if (!map->payload_name || !map->ClockRate) return GF_REMOTE_SERVICE_ERROR;
 		}
 	}
-	//Encryption: nothing tells wether the scope of the global key is eclusive or not.
+	//Encryption: nothing tells whether the scope of the global key is eclusive or not.
 	//we accept a global key + keys per media entry, assuming that the media key primes
 	//on the global key
 
@@ -801,33 +841,34 @@ GF_Err gf_sdp_info_check(GF_SDPInfo *sdp)
 }
 
 
-
+#define SDP_WRITE_ALLOC_STR_WITHOUT_CHECK(str, space)		\
+	if (strlen(str)+pos + (space ? 1 : 0) >= buf_size) {	\
+		buf_size += SDP_WRITE_STEPALLOC;	\
+		buf = (char*)gf_realloc(buf, sizeof(char)*buf_size);		\
+	}	\
+	strcpy(buf+pos, str);		\
+	pos += (u32) strlen(str);		\
+	if (space) {			\
+		strcat(buf+pos, " ");	\
+		pos += 1;		\
+	}
 
 #define SDP_WRITE_ALLOC_STR(str, space)		\
 	if (str) { \
-		if (strlen(str)+pos + (space ? 1 : 0) >= buf_size) {	\
-			buf_size += SDP_WRITE_STEPALLOC;	\
-			buf = (char*)realloc(buf, sizeof(char)*buf_size);		\
-		}	\
-		strcpy(buf+pos, str);		\
-		pos += strlen(str);		\
-		if (space) {			\
-			strcat(buf+pos, " ");	\
-			pos += 1;		\
-		}		\
+		SDP_WRITE_ALLOC_STR_WITHOUT_CHECK(str, space); \
 	}		\
 
 #define SDP_WRITE_ALLOC_INT(d, spa, sig)		\
-	if (sig) { \
+	if (sig < 0) { \
 		sprintf(temp, "%d", d);		\
 	} else { \
 		sprintf(temp, "%u", d);		\
 	}	\
-	SDP_WRITE_ALLOC_STR(temp, spa);
+	SDP_WRITE_ALLOC_STR_WITHOUT_CHECK(temp, spa);
 
 #define SDP_WRITE_ALLOC_FLOAT(d, spa)		\
 	sprintf(temp, "%.2f", d);		\
-	SDP_WRITE_ALLOC_STR(temp, spa);
+	SDP_WRITE_ALLOC_STR_WITHOUT_CHECK(temp, spa);
 
 #define TEST_SDP_WRITE_SINGLE(type, str, sep)		\
 	if (str) {		\
@@ -855,7 +896,6 @@ GF_Err gf_sdp_info_check(GF_SDPInfo *sdp)
 		SDP_WRITE_ALLOC_STR("\r\n", 0);		\
 	}
 
-GF_EXPORT
 GF_Err gf_sdp_info_write(GF_SDPInfo *sdp, char **out_str_buf)
 {
 	char *buf;
@@ -873,7 +913,7 @@ GF_Err gf_sdp_info_write(GF_SDPInfo *sdp, char **out_str_buf)
 	e = gf_sdp_info_check(sdp);
 	if (e) return e;
 
-	buf = (char *)malloc(SDP_WRITE_STEPALLOC);
+	buf = (char *)gf_malloc(SDP_WRITE_STEPALLOC);
 	buf_size = SDP_WRITE_STEPALLOC;
 	pos = 0;
 
@@ -1129,7 +1169,7 @@ GF_Err gf_sdp_info_write(GF_SDPInfo *sdp, char **out_str_buf)
 			SDP_WRITE_ALLOC_STR("a=quality:", 0);
 			SDP_WRITE_ALLOC_INT(media->Quality, 0, 0);
 			SDP_WRITE_ALLOC_STR("\r\n", 0);
-		}		
+		}
 		//the rest
 		j=0;
 		while ((att = (GF_X_Attribute*)gf_list_enum(media->Attributes, &j))) {
@@ -1143,11 +1183,14 @@ GF_Err gf_sdp_info_write(GF_SDPInfo *sdp, char **out_str_buf)
 		}
 	}
 
-	//finally realloc
+	//finally gf_realloc
 	//finall NULL char
 	pos += 1;
-	buf = (char *)realloc(buf, pos);
+	buf = (char *)gf_realloc(buf, pos);
 	*out_str_buf = buf;
 	return GF_OK;
-}	
+}
+#endif
 
+
+#endif /*GPAC_DISABLE_STREAMING*/
