@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2022
+ *			Copyright (c) Telecom ParisTech 2000-2024
  *					All rights reserved
  *
  *  This file is part of GPAC / ISO Media File Format sub-project
@@ -99,12 +99,13 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 	char uuid[16];
 	GF_Err e;
 	GF_BitStream *uncomp_bs = NULL;
+	GF_BitStream *orig_bs = NULL;
 	u8 *uncomp_data = NULL;
 	u32 compressed_size=0;
 	GF_Box *newBox;
 	Bool skip_logs = (gf_bs_get_cookie(bs) & GF_ISOM_BS_COOKIE_NO_LOGS ) ? GF_TRUE : GF_FALSE;
 	Bool is_special = GF_TRUE;
-	
+
 	if ((bs == NULL) || (outBox == NULL) ) return GF_BAD_PARAM;
 	*outBox = NULL;
 	if (gf_bs_available(bs) < 8) {
@@ -137,38 +138,69 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 					GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[iso file] Read Box type %s (0x%08X) at position "LLU" has size 0 but is not at root/file level. Forbidden, skipping end of parent box !\n", gf_4cc_to_str(type), type, start));
 					return GF_SKIP_BOX;
 				}
-				return GF_OK;
+				return GF_ISOM_INVALID_FILE;
 			}
 		}
-		if (is_root_box && (size>=8)) {
-			Bool do_uncompress = GF_FALSE;
+		if ((is_root_box && (size>=8))
+			|| (type==GF_QT_BOX_TYPE_CMOV)
+		) {
+			u32 do_uncompress = 0;
 			u8 *compb = NULL;
+			u32 extra_bytes = 0;
 			u32 osize = 0;
 			u32 otype = type;
-			if (type==GF_4CC('!', 'm', 'o', 'f')) {
-				do_uncompress = GF_TRUE;
-				type = GF_ISOM_BOX_TYPE_MOOF;
-			}
-			else if (type==GF_4CC('!', 'm', 'o', 'v')) {
-				do_uncompress = GF_TRUE;
+
+			if (type==GF_4CC('!', 'm', 'o', 'v')) {
+				do_uncompress = 1;
 				type = GF_ISOM_BOX_TYPE_MOOV;
 			}
+			else if (type==GF_QT_BOX_TYPE_CMOV) {
+				do_uncompress = 2;
+				u32 cbtype, cbsize, ctype;
+				//parse child boxes directly
+				cbsize = gf_bs_read_u32(bs);
+				if (cbsize != 12) return GF_ISOM_INVALID_FILE;
+				cbtype = gf_bs_read_u32(bs);
+				if (cbtype != GF_QT_BOX_TYPE_DCOM) return GF_ISOM_INVALID_FILE;
+				ctype = gf_bs_read_u32(bs);
+				if (ctype != GF_4CC('z', 'l', 'i', 'b')) return GF_NOT_SUPPORTED;
+				cbsize = gf_bs_read_u32(bs);
+				if (cbsize <= 12) return GF_ISOM_INVALID_FILE;
+				cbtype = gf_bs_read_u32(bs);
+				if (cbtype != GF_QT_BOX_TYPE_CMVD) return GF_ISOM_INVALID_FILE;
+
+				//uncompressed size
+				gf_bs_read_u32(bs);
+				extra_bytes = 6*4;
+				parent_size=0;
+				parent_type=type;
+				type = cbtype;
+			}
+#ifndef GPAC_DISABLE_ISOM_FRAGMENTS
+			else if (type==GF_4CC('!', 'm', 'o', 'f')) {
+				do_uncompress = 1;
+				type = GF_ISOM_BOX_TYPE_MOOF;
+			}
 			else if (type==GF_4CC('!', 's', 'i', 'x')) {
-				do_uncompress = GF_TRUE;
+				do_uncompress = 1;
 				type = GF_ISOM_BOX_TYPE_SIDX;
 			}
 			else if (type==GF_4CC('!', 's', 's', 'x')) {
-				do_uncompress = GF_TRUE;
+				do_uncompress = 1;
 				type = GF_ISOM_BOX_TYPE_SSIX;
 			}
-
+#endif
 			if (do_uncompress) {
+				if (size>100000000) {
+					GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[iso file] Compressed payload too large (%u)\n", size));
+					return GF_NOT_SUPPORTED;
+				}
 				compb = gf_malloc((u32) (size-8));
 				if (!compb) return GF_OUT_OF_MEM;
 
-				compressed_size = (u32) (size - 8);
+				compressed_size = (u32) (size - 8 - extra_bytes);
 				gf_bs_read_data(bs, compb, compressed_size);
-				e = gf_gz_decompress_payload(compb, compressed_size, &uncomp_data, &osize);
+				e = gf_gz_decompress_payload_ex(compb, compressed_size, &uncomp_data, &osize, GF_FALSE);
 				if (e) {
 					gf_free(compb);
 					GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[iso file] Failed to uncompress payload for box type %s (0x%08X)\n", gf_4cc_to_str(otype), otype));
@@ -178,6 +210,7 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 				//keep size as complete box size for tests below
 				size = osize + 8;
 				uncomp_bs = gf_bs_new(uncomp_data, osize, GF_BITSTREAM_READ);
+				orig_bs = bs;
 				bs = uncomp_bs;
 				start = 0;
 				gf_free(compb);
@@ -255,7 +288,7 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 	} else {
 		//OK, create the box based on the type
 		is_special = GF_FALSE;
-		newBox = gf_isom_box_new_ex(uuid_type ? uuid_type : type, parent_type, skip_logs, is_root_box);
+		newBox = gf_isom_box_new_ex(uuid_type ? uuid_type : type, parent_type, skip_logs, is_root_box, uuid_type ? GF_TRUE : GF_FALSE);
 		if (!newBox) ERR_EXIT(GF_OUT_OF_MEM);
 	}
 
@@ -297,15 +330,20 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 		}
 		//move size to real bitstream offsets for tests below
 		size -= 8;
-		//remember compressed vs real size info for moof in order to properly recompute data_offset/base_data_offset
-		if (type==GF_ISOM_BOX_TYPE_MOOF) {
-			((GF_MovieFragmentBox *)newBox)->compressed_diff = (s32)size - (s32)compressed_size;
-		}
 		//remember compressed vs real size info for moov in order to properly recompute chunk offset
-		else if (type==GF_ISOM_BOX_TYPE_MOOV) {
+		if (type==GF_ISOM_BOX_TYPE_MOOV) {
 			((GF_MovieBox *)newBox)->compressed_diff = (s32)size - (s32)compressed_size;
 			((GF_MovieBox *)newBox)->file_offset = comp_start;
 		}
+		else if (type==GF_QT_BOX_TYPE_CMVD) {
+			//do not store compressed diff or file offset, cmov uses offsets in the compressed file
+			newBox->size = compressed_size + 8 + 6*4;
+		}
+#ifndef GPAC_DISABLE_ISOM_FRAGMENTS
+		//remember compressed vs real size info for moof in order to properly recompute data_offset/base_data_offset
+		else if (type==GF_ISOM_BOX_TYPE_MOOF) {
+			((GF_MovieFragmentBox *)newBox)->compressed_diff = (s32)size - (s32)compressed_size;
+		}
 		//remember compressed vs real size info for dump
 		else if (type==GF_ISOM_BOX_TYPE_SIDX) {
 			((GF_SegmentIndexBox *)newBox)->compressed_diff = (s32)size - (s32)compressed_size;
@@ -314,7 +352,9 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 		else if (type==GF_ISOM_BOX_TYPE_SSIX) {
 			((GF_SubsegmentIndexBox *)newBox)->compressed_diff = (s32)size - (s32)compressed_size;
 		}
+#endif
 		newBox->internal_flags = GF_ISOM_BOX_COMPRESSED;
+		bs = orig_bs;
 	}
 
 
@@ -526,24 +566,38 @@ GF_Err unkn_box_dump(GF_Box *a, FILE * trace);
 //definition of boxes new/del/read/write/size. For now still exported since some files other than box_funcs.c call them
 //this should be fixed by only using gf_isom_box_new
 
+#ifndef GPAC_DISABLE_ISOM_DUMP
+#define ISOM_BOX_IMPL_DECL_DUMP(a_name) \
+		GF_Err a_name##_box_dump(GF_Box *a, FILE * trace);
+#else
+#define ISOM_BOX_IMPL_DECL_DUMP(a_name)
+
+#endif
+
+#ifndef GPAC_DISABLE_ISOM_WRITE
+#define ISOM_BOX_IMPL_DECL_WRITE(a_name) \
+		GF_Err a_name##_box_write(GF_Box *s, GF_BitStream *bs); \
+		GF_Err a_name##_box_size(GF_Box *s);
+#else
+#define ISOM_BOX_IMPL_DECL_WRITE(a_name)
+
+#endif
+
+
 #define ISOM_BOX_IMPL_DECL(a_name) \
 		GF_Box * a_name##_box_new(); \
 		void a_name##_box_del(GF_Box *); \
 		GF_Err a_name##_box_read(GF_Box *s, GF_BitStream *bs); \
-		GF_Err a_name##_box_write(GF_Box *s, GF_BitStream *bs); \
-		GF_Err a_name##_box_size(GF_Box *s);\
-		GF_Err a_name##_box_dump(GF_Box *a, FILE * trace);
+		ISOM_BOX_IMPL_DECL_WRITE(a_name) \
+		ISOM_BOX_IMPL_DECL_DUMP(a_name) \
 
 #define ISOM_BOX_IMPL_DECL_CHILD(a_name) \
-		GF_Box * a_name##_box_new(); \
-		void a_name##_box_del(GF_Box *); \
-		GF_Err a_name##_box_read(GF_Box *s, GF_BitStream *bs); \
-		GF_Err a_name##_box_write(GF_Box *s, GF_BitStream *bs); \
-		GF_Err a_name##_box_size(GF_Box *s);\
-		GF_Err a_name##_box_dump(GF_Box *a, FILE * trace);\
+		ISOM_BOX_IMPL_DECL(a_name)\
+		ISOM_BOX_IMPL_DECL_DUMP(a_name) \
 		GF_Err a_name##_on_child_box(GF_Box *s, GF_Box *a, Bool is_rem);\
 
 
+ISOM_BOX_IMPL_DECL(empty)
 ISOM_BOX_IMPL_DECL(reftype)
 ISOM_BOX_IMPL_DECL(ireftype)
 ISOM_BOX_IMPL_DECL(free)
@@ -884,6 +938,10 @@ ISOM_BOX_IMPL_DECL(dvvC)
 ISOM_BOX_IMPL_DECL(dvhe)
 ISOM_BOX_IMPL_DECL(dfla)
 
+/* DTS audio */
+ISOM_BOX_IMPL_DECL(ddts)
+ISOM_BOX_IMPL_DECL(udts)
+
 ISOM_BOX_IMPL_DECL(pcmC)
 ISOM_BOX_IMPL_DECL(chnl)
 
@@ -897,31 +955,67 @@ ISOM_BOX_IMPL_DECL(proj_type)
 
 ISOM_BOX_IMPL_DECL(keys)
 
-#define BOX_DEFINE(__type, b_rad, __par) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 0, 0, __par, "p12", GF_FALSE}
+#if defined(GPAC_DISABLE_ISOM_DUMP) && defined(GPAC_DISABLE_ISOM_WRITE)
 
-#define BOX_DEFINE_CHILD(__type, b_rad, __par) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 0, 0, __par, "p12", GF_FALSE, b_rad##_on_child_box}
+#define BOX_DECLARATION(_a, _b, _c, _d, _write, _size, _dump, _g, _h, _i, _j, _k, _l, _m) \
+	{ _a, _b, _c, _d, _g, _h, _i, _j, _k, _l, _m }
 
-#define BOX_DEFINE_S(__type, b_rad, __par, __spec) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 0, 0, __par, __spec, GF_FALSE }
+#elif defined(GPAC_DISABLE_ISOM_DUMP)
 
-#define BOX_DEFINE_S_CHILD(__type, b_rad, __par, __spec) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 0, 0, __par, __spec, GF_FALSE, b_rad##_on_child_box}
+#define BOX_DECLARATION(_a, _b, _c, _d, _write, _size, _dump, _g, _h, _i, _j, _k, _l, _m) \
+	{ _a, _b, _c, _d, _write, _size, _g, _h, _i, _j, _k, _l, _m }
 
-#define FBOX_DEFINE(__type, b_rad, __par, __max_v) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 1+__max_v, 0, __par, "p12", GF_FALSE }
+#elif defined(GPAC_DISABLE_ISOM_WRITE)
 
-#define FBOX_DEFINE_CHILD(__type, b_rad, __par, __max_v) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 1+__max_v, 0, __par, "p12", GF_FALSE, b_rad##_on_child_box}
+#define BOX_DECLARATION(_a, _b, _c, _d, _write, _size, _dump, _g, _h, _i, _j, _k, _l, _m) \
+	{ _a, _b, _c, _d, _dump, _g, _h, _i, _j, _k, _l, _m }
 
-#define FBOX_DEFINE_FLAGS(__type, b_rad, __par, __max_v, flags) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 1+__max_v, flags, __par, "p12", GF_FALSE }
+#else
 
-#define FBOX_DEFINE_FLAGS_S(__type, b_rad, __par, __max_v, flags, __spec) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 1+__max_v, flags, __par, __spec, GF_FALSE }
+#define BOX_DECLARATION(_a, _b, _c, _d, _write, _size, _dump, _g, _h, _i, _j, _k, _l, _m) \
+	{ _a, _b, _c, _d, _write, _size, _dump, _g, _h, _i, _j, _k, _l, _m }
 
-#define FBOX_DEFINE_S(__type, b_rad, __par, __max_v, __spec) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 1+__max_v, 0, __par, __spec, GF_FALSE }
+#endif
 
-#define FBOX_DEFINE_S_CHILD(__type, b_rad, __par, __max_v, __spec) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 1+__max_v, 0, __par, __spec, GF_FALSE, b_rad##_on_child_box}
 
-#define TREF_DEFINE(__type, b_rad, __par, __4cc, __spec) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, __4cc, 0, 0, __par, __spec, GF_FALSE }
+#define BOX_DEFINE(__type, b_rad, __par) \
+	BOX_DECLARATION(__type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 0, 0, __par, "p12", GF_FALSE, NULL)
 
-#define TRGT_DEFINE(__type, b_rad, __par, __4cc, max_version, __spec) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, __4cc, 1+max_version, 0, __par, __spec, GF_FALSE }
+#define BOX_DEFINE_CHILD(__type, b_rad, __par) \
+	BOX_DECLARATION(__type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 0, 0, __par, "p12", GF_FALSE, b_rad##_on_child_box)
 
-#define SGPD_DEFINE(__type, b_rad, __par, __4cc, __spec) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, __4cc, 1, 0, __par, __spec, GF_FALSE }
+#define BOX_DEFINE_S(__type, b_rad, __par, __spec) \
+	BOX_DECLARATION(__type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 0, 0, __par, __spec, GF_FALSE, NULL)
+
+#define BOX_DEFINE_S_CHILD(__type, b_rad, __par, __spec) \
+	BOX_DECLARATION(__type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 0, 0, __par, __spec, GF_FALSE, b_rad##_on_child_box)
+
+#define FBOX_DEFINE(__type, b_rad, __par, __max_v) \
+	BOX_DECLARATION(__type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 1+__max_v, 0, __par, "p12", GF_FALSE, NULL)
+
+#define FBOX_DEFINE_CHILD(__type, b_rad, __par, __max_v) \
+	BOX_DECLARATION(__type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 1+__max_v, 0, __par, "p12", GF_FALSE, b_rad##_on_child_box)
+
+#define FBOX_DEFINE_FLAGS(__type, b_rad, __par, __max_v, flags) \
+	BOX_DECLARATION(__type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 1+__max_v, flags, __par, "p12", GF_FALSE, NULL)
+
+#define FBOX_DEFINE_FLAGS_S(__type, b_rad, __par, __max_v, flags, __spec) \
+	BOX_DECLARATION(__type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 1+__max_v, flags, __par, __spec, GF_FALSE, NULL)
+
+#define FBOX_DEFINE_S(__type, b_rad, __par, __max_v, __spec) \
+	BOX_DECLARATION(__type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 1+__max_v, 0, __par, __spec, GF_FALSE, NULL)
+
+#define FBOX_DEFINE_S_CHILD(__type, b_rad, __par, __max_v, __spec) \
+	BOX_DECLARATION(__type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 1+__max_v, 0, __par, __spec, GF_FALSE, b_rad##_on_child_box)
+
+#define TREF_DEFINE(__type, b_rad, __par, __4cc, __spec) \
+	BOX_DECLARATION(__type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, __4cc, 0, 0, __par, __spec, GF_FALSE, NULL)
+
+#define TRGT_DEFINE(__type, b_rad, __par, __4cc, max_version, __spec) \
+	BOX_DECLARATION(__type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, __4cc, 1+max_version, 0, __par, __spec, GF_FALSE, NULL)
+
+#define SGPD_DEFINE(__type, b_rad, __par, __4cc, __spec) \
+	BOX_DECLARATION(__type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, __4cc, 1, 0, __par, __spec, GF_FALSE, NULL)
 
 #define ITUNES_TAG(_val) \
 	BOX_DEFINE_S( _val, ilst_item, "ilst data", "apple")
@@ -931,9 +1025,13 @@ static struct box_registry_entry {
 	GF_Box * (*new_fn)();
 	void (*del_fn)(GF_Box *a);
 	GF_Err (*read_fn)(GF_Box *s, GF_BitStream *bs);
+#ifndef GPAC_DISABLE_ISOM_WRITE
 	GF_Err (*write_fn)(GF_Box *s, GF_BitStream *bs);
 	GF_Err (*size_fn)(GF_Box *a);
+#endif
+#ifndef GPAC_DISABLE_ISOM_DUMP
 	GF_Err (*dump_fn)(GF_Box *a, FILE *trace);
+#endif
 	u32 alt_4cc;//used for sample grouping type and track / item reference types
 	u8 max_version_plus_one;
 	u32 flags;
@@ -1034,7 +1132,8 @@ static struct box_registry_entry {
 	BOX_DEFINE( GF_ISOM_BOX_TYPE_MDAT, mdat, "file"),
 	BOX_DEFINE( GF_ISOM_BOX_TYPE_IDAT, mdat, "meta"),
 	BOX_DEFINE( GF_ISOM_BOX_TYPE_IMDA, mdat, "file"),
-	BOX_DEFINE_CHILD( GF_ISOM_BOX_TYPE_MOOV, moov, "file"),
+	BOX_DEFINE_CHILD( GF_ISOM_BOX_TYPE_MOOV, moov, "file cmvd"),
+	BOX_DEFINE_CHILD( GF_QT_BOX_TYPE_CMVD, moov, "cmov moov"),
 	FBOX_DEFINE( GF_ISOM_BOX_TYPE_MVHD, mvhd, "moov", 1),
 	FBOX_DEFINE( GF_ISOM_BOX_TYPE_MDHD, mdhd, "mdia", 1),
 	FBOX_DEFINE( GF_ISOM_BOX_TYPE_VMHD, vmhd, "minf", 0),
@@ -1046,6 +1145,7 @@ static struct box_registry_entry {
 	BOX_DEFINE_CHILD( GF_ISOM_BOX_TYPE_DINF, dinf, "minf meta"),
 	FBOX_DEFINE_FLAGS( GF_ISOM_BOX_TYPE_URL, url, "dref", 0, 1),
 	FBOX_DEFINE_FLAGS( GF_ISOM_BOX_TYPE_URN, urn, "dref", 0, 1),
+	FBOX_DEFINE_FLAGS( GF_ISOM_BOX_TYPE_IMDT, url, "dref", 0, 0),
 	FBOX_DEFINE( GF_ISOM_BOX_TYPE_CPRT, cprt, "udta", 0),
 	FBOX_DEFINE( GF_ISOM_BOX_TYPE_KIND, kind, "udta", 0),
 	FBOX_DEFINE( GF_ISOM_BOX_TYPE_HDLR, hdlr, "mdia meta minf", 0),	//minf container is OK in QT ...
@@ -1101,7 +1201,7 @@ static struct box_registry_entry {
 	FBOX_DEFINE_CHILD( GF_ISOM_BOX_TYPE_IINF, iinf, "meta", 1),
 	FBOX_DEFINE_CHILD( GF_ISOM_BOX_TYPE_IREF, iref, "meta", 1),
 	BOX_DEFINE_CHILD( GF_ISOM_BOX_TYPE_SINF, sinf, "ipro sample_entry"),
-	BOX_DEFINE( GF_ISOM_BOX_TYPE_RINF, sinf, "sample_entry"),
+	BOX_DEFINE_CHILD( GF_ISOM_BOX_TYPE_RINF, sinf, "sample_entry"),
 	BOX_DEFINE( GF_ISOM_BOX_TYPE_FRMA, frma, "sinf rinf"),
 	FBOX_DEFINE_FLAGS(GF_ISOM_BOX_TYPE_SCHM, schm, "sinf rinf", 0, 1),
 	BOX_DEFINE_CHILD( GF_ISOM_BOX_TYPE_SCHI, schi, "sinf rinf"),
@@ -1318,6 +1418,7 @@ static struct box_registry_entry {
 	FBOX_DEFINE_S( GF_ISOM_BOX_TYPE_RLOC, rloc, "ipco", 0, "iff"),
 	BOX_DEFINE_S( GF_ISOM_BOX_TYPE_IROT, irot, "ipco", "iff"),
 	BOX_DEFINE_S( GF_ISOM_BOX_TYPE_IMIR, imir, "ipco", "iff"),
+	BOX_DEFINE_S( GF_ISOM_BOX_TYPE_ILCE, imir, "ipco", "iff"),
 	FBOX_DEFINE_FLAGS_S( GF_ISOM_BOX_TYPE_IPMA, ipma, "iprp", 1, 1, "iff"),
 	BOX_DEFINE_S( GF_ISOM_BOX_TYPE_GRPL, grpl, "meta", "iff"),
 	FBOX_DEFINE_S( GF_ISOM_BOX_TYPE_CCST, ccst, "sample_entry", 0, "iff"),
@@ -1370,7 +1471,6 @@ static struct box_registry_entry {
 	BOX_DEFINE_S( GF_ISOM_BOX_TYPE_DIMC, dimC, "dims encs", "3gpp"),
 	BOX_DEFINE_S( GF_ISOM_BOX_TYPE_DIST, diST, "dims", "3gpp"),
 
-
 	//CENC boxes
 	FBOX_DEFINE_S( GF_ISOM_BOX_TYPE_PSSH, pssh, "moov moof meta", 0, "cenc"),
 	FBOX_DEFINE_S( GF_ISOM_BOX_TYPE_TENC, tenc, "schi", 1, "cenc"),
@@ -1528,14 +1628,26 @@ static struct box_registry_entry {
 	BOX_DEFINE_S_CHILD( GF_QT_SUBTYPE_YVYU, video_sample_entry, "stsd", "apple"),
 	BOX_DEFINE_S_CHILD( GF_QT_SUBTYPE_RGBA, video_sample_entry, "stsd", "apple"),
 	BOX_DEFINE_S_CHILD( GF_QT_SUBTYPE_ABGR, video_sample_entry, "stsd", "apple"),
-	
+	BOX_DEFINE_S_CHILD( GF_QT_SUBTYPE_LPCM, audio_sample_entry, "stsd", "apple"),
+
 	FBOX_DEFINE_S(GF_QT_BOX_TYPE_STPS, stss, "stbl", 0, "apple"),
 
+	BOX_DEFINE_S( GF_QT_BOX_TYPE_ENCD, unkn, "text_sample", "QT"),
+	BOX_DEFINE_S( GF_QT_BOX_TYPE_FRCD, empty, "text_sample", "QT"),
+
+
 	//dolby boxes
 	BOX_DEFINE_S_CHILD( GF_ISOM_BOX_TYPE_AC3, audio_sample_entry, "stsd", "dolby"),
 	BOX_DEFINE_S_CHILD( GF_ISOM_BOX_TYPE_EC3, audio_sample_entry, "stsd", "dolby"),
 	BOX_DEFINE_S( GF_ISOM_BOX_TYPE_DAC3, dac3, "ac-3 wave enca", "dolby"),
-	{GF_ISOM_BOX_TYPE_DEC3, dec3_box_new, dac3_box_del, dac3_box_read, dac3_box_write, dac3_box_size, dac3_box_dump, 0, 0, 0, "ec-3 wave enca", "dolby" },
+	{GF_ISOM_BOX_TYPE_DEC3, dec3_box_new, dac3_box_del, dac3_box_read,
+#ifndef GPAC_DISABLE_ISOM_WRITE
+		dac3_box_write, dac3_box_size,
+#endif
+#ifndef GPAC_DISABLE_ISOM_DUMP
+		dac3_box_dump,
+#endif
+		0, 0, 0, "ec-3 wave enca", "dolby" },
 	BOX_DEFINE_S(GF_ISOM_BOX_TYPE_DVCC, dvcC, "dvav dva1 dvhe dvh1 dav1 avc1 avc2 avc3 avc4 hev1 hvc1 av01 encv resv", "DolbyVision"),
 	BOX_DEFINE_S(GF_ISOM_BOX_TYPE_DVVC, dvvC, "dvav dva1 dvhe dvh1 dav1 avc1 avc2 avc3 avc4 hev1 hvc1 av01 encv resv", "DolbyVision"),
 	BOX_DEFINE_S_CHILD(GF_ISOM_BOX_TYPE_DVHE, video_sample_entry, "stsd", "DolbyVision"),
@@ -1576,8 +1688,10 @@ static struct box_registry_entry {
 	BOX_DEFINE_S(GF_ISOM_BOX_UUID_TENC, piff_tenc, "schi", "smooth"),
 	BOX_DEFINE_S(GF_ISOM_BOX_UUID_PSEC, piff_psec, "trak traf", "smooth"),
 	BOX_DEFINE_S(GF_ISOM_BOX_UUID_PSSH, piff_pssh, "moov moof", "smooth"),
+#ifndef GPAC_DISABLE_ISOM_FRAGMENTS
 	BOX_DEFINE_S(GF_ISOM_BOX_UUID_TFXD, tfxd, "traf", "smooth"),
 	BOX_DEFINE_S(GF_ISOM_BOX_UUID_TFRF, tfrf, "traf", "smooth"),
+#endif
 	BOX_DEFINE_S(GF_ISOM_BOX_UUID_MSSM, uuid, "file", "smooth"),
 	BOX_DEFINE_S(GF_ISOM_BOX_UUID_TFRF, uuid, "traf", "smooth"),
 
@@ -1620,7 +1734,18 @@ static struct box_registry_entry {
 	BOX_DEFINE_S(GF_4CC('G','M','C','W'), unkn, "stsd", "GPAC"),
 	BOX_DEFINE_S(GF_4CC('G','M','C','C'), unkn, "GMCW", "GPAC"),
 
+	// DTS in ISOBMFF boxes
+	BOX_DEFINE_S_CHILD(GF_ISOM_BOX_TYPE_DTSC, audio_sample_entry, "stsd", "DTS"),
+	BOX_DEFINE_S_CHILD(GF_ISOM_BOX_TYPE_DTSE, audio_sample_entry, "stsd", "DTS"),
+	BOX_DEFINE_S_CHILD(GF_ISOM_BOX_TYPE_DTSH, audio_sample_entry, "stsd", "DTS"),
+	BOX_DEFINE_S_CHILD(GF_ISOM_BOX_TYPE_DTSL, audio_sample_entry, "stsd", "DTS"),
+	BOX_DEFINE_S_CHILD(GF_ISOM_BOX_TYPE_DTSX, audio_sample_entry, "stsd", "DTS"),
+	BOX_DEFINE_S_CHILD(GF_ISOM_BOX_TYPE_DTSY, audio_sample_entry, "stsd", "DTS"),
+	BOX_DEFINE_S(GF_ISOM_BOX_TYPE_DDTS, ddts, "audio_sample_entry", "DTS"),
+	BOX_DEFINE_S(GF_ISOM_BOX_TYPE_UDTS, udts, "audio_sample_entry", "DTS"),
+
 	/* for now we don't parse these*/
+	BOX_DEFINE_S(GF_ISOM_SUBTYPE_UNCV, unkn, "stsd", "rawff"),
 	BOX_DEFINE_S(GF_4CC('u','n','c','C'), unkn, "video_sample_entry ipco", "rawff"),
 	BOX_DEFINE_S(GF_4CC('c','m','p','d'), unkn, "video_sample_entry ipco", "rawff"),
 	BOX_DEFINE_S(GF_4CC('c','p','a','l'), unkn, "video_sample_entry ipco", "rawff"),
@@ -1630,9 +1755,11 @@ static struct box_registry_entry {
 	BOX_DEFINE_S(GF_4CC('s','n','u','c'), unkn, "video_sample_entry ipco", "rawff"),
 	BOX_DEFINE_S(GF_4CC('s','b','p','m'), unkn, "video_sample_entry ipco", "rawff"),
 	BOX_DEFINE_S(GF_4CC('c','l','o','c'), unkn, "video_sample_entry ipco", "rawff"),
-	BOX_DEFINE_S(GF_4CC('f','p','c','k'), unkn, "video_sample_entry ipco", "rawff"),
+	BOX_DEFINE_S(GF_4CC('f','p','a','c'), unkn, "video_sample_entry ipco", "rawff"),
 	BOX_DEFINE_S(GF_4CC('d','i','s','i'), unkn, "video_sample_entry ipco", "rawff"),
 	BOX_DEFINE_S(GF_4CC('d','e','p','i'), unkn, "video_sample_entry ipco", "rawff"),
+	BOX_DEFINE_S(GF_4CC('i','l','c','p'), unkn, "ipco", "rawff"),
+
 
 /*
 	GF_ISOM_BOX_TYPE_CBMP	= GF_4CC( 'c', 'b', 'm', 'p' ),
@@ -1674,12 +1801,14 @@ static u32 get_box_reg_idx(u32 boxCode, u32 parent_type, u32 start_from)
 
 	for (i=start_from; i<count; i++) {
 		u32 start_par_from;
+		char p4cc[GF_4CC_MSIZE];
+
 		if (box_registry[i].box_4cc != boxCode)
 			continue;
 
 		if (!parent_type)
 			return i;
-		if (strstr(box_registry[i].parents_4cc, gf_4cc_to_str(parent_type)) != NULL)
+		if (strstr(box_registry[i].parents_4cc, gf_4cc_to_str_safe(parent_type, p4cc) ) != NULL)
 			return i;
 		if (strstr(box_registry[i].parents_4cc, "*") != NULL)
 			return i;
@@ -1706,9 +1835,10 @@ static u32 get_box_reg_idx(u32 boxCode, u32 parent_type, u32 start_from)
 	return 0;
 }
 
-GF_Box *gf_isom_box_new_ex(u32 boxType, u32 parentType, Bool skip_logs, Bool is_root_box)
+GF_Box *gf_isom_box_new_ex(u32 boxType, u32 parentType, Bool skip_logs, Bool is_root_box, Bool is_uuid)
 {
 	GF_Box *a;
+	const char *opt;
 	s32 idx = get_box_reg_idx(boxType, parentType, 0);
 	if (idx==0) {
 #ifndef GPAC_DISABLE_LOG
@@ -1727,12 +1857,18 @@ GF_Box *gf_isom_box_new_ex(u32 boxType, u32 parentType, Bool skip_logs, Bool is_
 			default:
 				if (boxType==GF_ISOM_BOX_TYPE_GDAT) break;
 
+				opt = gf_opts_get_key("core", "boxdir");
+				if (opt) {
+					char szPath[GF_MAX_PATH], szType[GF_4CC_MSIZE];
+					snprintf(szPath, GF_MAX_PATH-1, "%s/%s.js", opt, gf_4cc_to_str_safe(boxType, szType) );
+					if (gf_file_exists(szPath))
+						break;
+				}
+
 				if (is_root_box) {
 					GF_LOG(GF_LOG_INFO, GF_LOG_CONTAINER, ("[iso file] Unknown top-level box type %s\n", gf_4cc_to_str(boxType)));
 				} else if (parentType) {
-					char szName[GF_4CC_MSIZE];
-					strcpy(szName, gf_4cc_to_str(parentType));
-					GF_LOG(GF_LOG_INFO, GF_LOG_CONTAINER, ("[iso file] Unknown box type %s in parent %s\n", gf_4cc_to_str(boxType), szName));
+					GF_LOG(GF_LOG_INFO, GF_LOG_CONTAINER, ("[iso file] Unknown box type %s in parent %s\n", gf_4cc_to_str(boxType), gf_4cc_to_str(parentType) ));
 				} else {
 					GF_LOG(GF_LOG_INFO, GF_LOG_CONTAINER, ("[iso file] Unknown box type %s\n", gf_4cc_to_str(boxType)));
 				}
@@ -1740,13 +1876,14 @@ GF_Box *gf_isom_box_new_ex(u32 boxType, u32 parentType, Bool skip_logs, Bool is_
 			}
 		}
 #endif
-        if (boxType==GF_ISOM_BOX_TYPE_UUID) {
+        if (is_uuid || (boxType==GF_ISOM_BOX_TYPE_UUID)) {
             a = uuid_box_new();
             if (a) a->registry = &box_registry[1];
         } else {
             a = unkn_box_new();
             if (a) {
             	((GF_UnknownBox *)a)->original_4cc = boxType;
+				((GF_UnknownBox *)a)->parent_4cc = parentType;
             	a->registry = &box_registry[0];
 			}
         }
@@ -1775,7 +1912,7 @@ GF_Box *gf_isom_box_new_ex(u32 boxType, u32 parentType, Bool skip_logs, Bool is_
 GF_EXPORT
 GF_Box *gf_isom_box_new(u32 boxType)
 {
-	return gf_isom_box_new_ex(boxType, 0, 0, GF_FALSE);
+	return gf_isom_box_new_ex(boxType, 0, 0, GF_FALSE, GF_FALSE);
 }
 
 GF_Err gf_isom_box_array_read(GF_Box *parent, GF_BitStream *bs)
@@ -1784,6 +1921,9 @@ GF_Err gf_isom_box_array_read(GF_Box *parent, GF_BitStream *bs)
 	u32 parent_type = parent->type;
 	GF_Box *a = NULL;
 	Bool skip_logs = (gf_bs_get_cookie(bs) & GF_ISOM_BS_COOKIE_NO_LOGS ) ? GF_TRUE : GF_FALSE;
+	if (parent_type == GF_ISOM_BOX_TYPE_UNKNOWN) {
+		parent_type = ((GF_UnknownBox*)parent)->original_4cc;
+	}
 
 	//we may have terminators in some QT files (4 bytes set to 0 ...)
 	while (parent->size>=8) {
@@ -1811,9 +1951,12 @@ GF_Err gf_isom_box_array_read(GF_Box *parent, GF_BitStream *bs)
 		//check container validity
 		if (parent_type && strlen(a->registry->parents_4cc)) {
 			Bool parent_OK = GF_FALSE;
-			const char *parent_code = gf_4cc_to_str(parent->type);
+			char parent_code[GF_4CC_MSIZE];
 			if (parent->type == GF_ISOM_BOX_TYPE_UNKNOWN)
-				parent_code = gf_4cc_to_str( ((GF_UnknownBox*)parent)->original_4cc );
+				gf_4cc_to_str_safe(((GF_UnknownBox*)parent)->original_4cc, parent_code);
+			else
+				gf_4cc_to_str_safe(parent->type, parent_code);
+
 			if (strstr(a->registry->parents_4cc, parent_code) != NULL) {
 				parent_OK = GF_TRUE;
 			} else if (!strcmp(a->registry->parents_4cc, "*") || strstr(a->registry->parents_4cc, "* ") || strstr(a->registry->parents_4cc, " *")) {
@@ -1918,7 +2061,7 @@ void gf_isom_check_position(GF_Box *s, GF_Box *child, u32 *pos)
 
 	s32 cur_pos = gf_list_find(s->child_boxes, child);
 
-	//happens when partially cloning boxes 
+	//happens when partially cloning boxes
 	if (cur_pos < 0) return;
 
 	if (cur_pos != (s32) *pos) {
@@ -1949,7 +2092,7 @@ GF_Err gf_isom_box_write(GF_Box *a, GF_BitStream *bs)
 	if (!a) return GF_BAD_PARAM;
 	//box has been disabled, do not write
 	if (!a->size) return GF_OK;
-	
+
 	if (a->registry->disabled) {
 		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[iso file] Box %s disabled registry, skip write\n", gf_4cc_to_str(a->type)));
 		return GF_OK;
@@ -2027,6 +2170,7 @@ static GF_Err gf_isom_full_box_read(GF_Box *ptr, GF_BitStream *bs)
 GF_EXPORT
 GF_Err gf_isom_dump_supported_box(u32 idx, FILE * trace)
 {
+#ifndef GPAC_DISABLE_ISOM_DUMP
 	u32 i;
 	u32 nb_versions=0;
 	GF_Err e;
@@ -2080,6 +2224,9 @@ GF_Err gf_isom_dump_supported_box(u32 idx, FILE * trace)
 		gf_isom_box_del(a);
 	}
 	return e;
+#else
+	return GF_NOT_SUPPORTED;
+#endif
 }
 
 GF_EXPORT
