@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005 
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / ISO Media File Format sub-project
@@ -10,75 +11,83 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
 #include <gpac/internal/isomedia_dev.h>
 
+#if !defined(GPAC_DISABLE_ISOM) && !defined(GPAC_DISABLE_ISOM_HINTING)
+
 Bool IsHintTrack(GF_TrackBox *trak)
 {
-	if (trak->Media->handler->handlerType != GF_ISOM_MEDIA_HINT) return 0;
+	if (trak->Media->handler->handlerType != GF_ISOM_MEDIA_HINT) return GF_FALSE;
 	//QT doesn't specify any InfoHeader on HintTracks
-	if (trak->Media->information->InfoHeader 
-			&& trak->Media->information->InfoHeader->type != GF_ISOM_BOX_TYPE_HMHD) 
-		return 0;
+	if (trak->Media->information->InfoHeader
+	        && (trak->Media->information->InfoHeader->type != GF_ISOM_BOX_TYPE_HMHD)
+	        && (trak->Media->information->InfoHeader->type != GF_ISOM_BOX_TYPE_NMHD)
+		)
+		return GF_FALSE;
 
-	return 1;
+	return GF_TRUE;
 }
 
 u32 GetHintFormat(GF_TrackBox *trak)
 {
 	GF_HintMediaHeaderBox *hmhd = (GF_HintMediaHeaderBox *)trak->Media->information->InfoHeader;
-	if (!hmhd->subType) {
-		GF_Box *a = (GF_Box *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->boxList, 0);
+	if (hmhd && (hmhd->type != GF_ISOM_BOX_TYPE_HMHD))
+		return 0;
+		
+	if (!hmhd || !hmhd->subType) {
+		GF_Box *a = (GF_Box *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, 0);
+		if (!hmhd) return a ? a->type : 0;
 		if (a) hmhd->subType = a->type;
+		return hmhd->subType;
 	}
 	return hmhd->subType;
 }
 
 Bool CheckHintFormat(GF_TrackBox *trak, u32 HintType)
 {
-	if (!IsHintTrack(trak)) return 0;
-	if (GetHintFormat(trak) != HintType) return 0;
-	return 1;
+	if (!IsHintTrack(trak)) return GF_FALSE;
+	if (GetHintFormat(trak) != HintType) return GF_FALSE;
+	return GF_TRUE;
 }
 
-#ifndef GPAC_READ_ONLY
+#ifndef GPAC_DISABLE_ISOM_WRITE
 
 GF_Err AdjustHintInfo(GF_HintSampleEntryBox *entry, u32 HintSampleNumber)
 {
 	u32 offset, count, i, size;
-	GF_HintPacket *pck;
 	GF_Err e;
 
 	offset = gf_isom_hint_sample_size(entry->hint_sample) - entry->hint_sample->dataLength;
 	count = gf_list_count(entry->hint_sample->packetTable);
-	
+
 	for (i=0; i<count; i++) {
-		pck = (GF_HintPacket *)gf_list_get(entry->hint_sample->packetTable, i);
+		GF_HintPacket *pck = (GF_HintPacket *)gf_list_get(entry->hint_sample->packetTable, i);
 		if (offset && entry->hint_sample->dataLength) {
 			//adjust any offset in this packet
-			e = gf_isom_hint_pck_offset(entry->hint_sample->HintType, pck, offset, HintSampleNumber);
+			e = gf_isom_hint_pck_offset(pck, offset, HintSampleNumber);
 			if (e) return e;
 		}
 		//adjust the max packet size for this sample entry...
-		size = gf_isom_hint_pck_length(entry->hint_sample->HintType, pck);
+		size = gf_isom_hint_pck_length(pck);
 		if (entry->MaxPacketSize < size) entry->MaxPacketSize = size;
 	}
 	return GF_OK;
 }
 
-
-GF_Err gf_isom_setup_hint_track(GF_ISOFile *movie, u32 trackNumber, u32 HintType)
+GF_EXPORT
+GF_Err gf_isom_setup_hint_track(GF_ISOFile *movie, u32 trackNumber, GF_ISOHintFormat HintType)
 {
 	GF_Err e;
 	GF_TrackBox *trak;
@@ -112,14 +121,13 @@ GF_Err gf_isom_setup_hint_track(GF_ISOFile *movie, u32 trackNumber, u32 HintType
 	//store the HintTrack format for later use...
 	hmhd->subType = HintType;
 
-	
+
 	//hint tracks always have a tref and everything ...
 	if (!trak->References) {
-		if (!trak->References) {
-			tref = (GF_TrackReferenceBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_TREF);
-			e = trak_AddBox((GF_Box*)trak, (GF_Box *)tref);
-			if (e) return e;
-		}
+		tref = (GF_TrackReferenceBox *) gf_isom_box_new_parent(&trak->child_boxes, GF_ISOM_BOX_TYPE_TREF);
+		if (!tref) return GF_OUT_OF_MEM;
+		e = trak_on_child_box((GF_Box*)trak, (GF_Box *)tref, GF_FALSE);
+		if (e) return e;
 	}
 	tref = trak->References;
 
@@ -130,43 +138,45 @@ GF_Err gf_isom_setup_hint_track(GF_ISOFile *movie, u32 trackNumber, u32 HintType
 	if (dpnd) return GF_BAD_PARAM;
 
 	//create our dep
-	dpnd = (GF_TrackReferenceTypeBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_REFT);
+	dpnd = (GF_TrackReferenceTypeBox *) gf_isom_box_new_parent(&tref->child_boxes, GF_ISOM_BOX_TYPE_REFT);
+	if (!dpnd) return GF_OUT_OF_MEM;
 	dpnd->reference_type = GF_ISOM_BOX_TYPE_HINT;
-	e = tref_AddBox((GF_Box*)tref, (GF_Box *) dpnd);
-	if (e) return e;
 
 	//for RTP, we need to do some UDTA-related stuff...
 	if (HintType != GF_ISOM_HINT_RTP) return GF_OK;
 
 	if (!trak->udta) {
 		//create one
-		udta = (GF_UserDataBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_UDTA);
-		e = trak_AddBox((GF_Box*)trak, (GF_Box *) udta);
+		udta = (GF_UserDataBox *) gf_isom_box_new_parent(&trak->child_boxes, GF_ISOM_BOX_TYPE_UDTA);
+		if (!udta) return GF_OUT_OF_MEM;
+		e = trak_on_child_box((GF_Box*)trak, (GF_Box *) udta, GF_FALSE);
 		if (e) return e;
 	}
 	udta = trak->udta;
 
 	//HNTI
-	e = udta_AddBox(udta, gf_isom_box_new(GF_ISOM_BOX_TYPE_HNTI));
+	e = udta_on_child_box((GF_Box *)udta, gf_isom_box_new(GF_ISOM_BOX_TYPE_HNTI), GF_FALSE);
 	if (e) return e;
 
-/*
-	//NAME
-	e = udta_AddBox(udta, gf_isom_box_new(GF_ISOM_BOX_TYPE_NAME));
-	if (e) return e;
-	//HINF
-	return udta_AddBox(udta, gf_isom_box_new(GF_ISOM_BOX_TYPE_HINF));
-*/
+	/*
+		//NAME
+		e = udta_on_child_box((GF_Box *)udta, gf_isom_box_new(GF_ISOM_BOX_TYPE_NAME));
+		if (e) return e;
+		//HINF
+		return udta_on_child_box((GF_Box *)udta, gf_isom_box_new(GF_ISOM_BOX_TYPE_HINF));
+	*/
 	return GF_OK;
 }
 
 //to use with internally supported protocols
+GF_EXPORT
 GF_Err gf_isom_new_hint_description(GF_ISOFile *the_file, u32 trackNumber, s32 HintTrackVersion, s32 LastCompatibleVersion, u8 Rely, u32 *HintDescriptionIndex)
 {
 	GF_Err e;
 	u32 drefIndex;
 	GF_TrackBox *trak;
 	GF_HintSampleEntryBox *hdesc;
+	GF_SampleDescriptionBox *stsd;
 	GF_RelyHintBox *relyA;
 
 	e = CanAccessMovie(the_file, GF_ISOM_OPEN_WRITE);
@@ -176,21 +186,20 @@ GF_Err gf_isom_new_hint_description(GF_ISOFile *the_file, u32 trackNumber, s32 H
 	*HintDescriptionIndex = 0;
 	if (!trak || !IsHintTrack(trak)) return GF_BAD_PARAM;
 
+	stsd = trak->Media->information->sampleTable->SampleDescription;
 	//OK, create a new HintSampleDesc
-	hdesc = (GF_HintSampleEntryBox *) gf_isom_box_new(GetHintFormat(trak));
+	hdesc = (GF_HintSampleEntryBox *) gf_isom_box_new_parent(&stsd->child_boxes, GetHintFormat(trak));
+	if (!hdesc) return GF_OUT_OF_MEM;
 
 	if (HintTrackVersion > 0) hdesc->HintTrackVersion = HintTrackVersion;
 	if (LastCompatibleVersion > 0) hdesc->LastCompatibleVersion = LastCompatibleVersion;
 
 	//create a data reference - WE ONLY DEAL WITH SELF-CONTAINED HINT TRACKS
-	e = Media_CreateDataRef(trak->Media->information->dataInformation->dref, NULL, NULL, &drefIndex);
+	e = Media_CreateDataRef(the_file, trak->Media->information->dataInformation->dref, NULL, NULL, &drefIndex);
 	if (e) return e;
 	hdesc->dataReferenceIndex = drefIndex;
 
-	//add the entry to our table...
-	e = stsd_AddBox(trak->Media->information->sampleTable->SampleDescription, (GF_Box *) hdesc);
-	if (e) return e;
-	*HintDescriptionIndex = gf_list_count(trak->Media->information->sampleTable->SampleDescription->boxList);
+	*HintDescriptionIndex = gf_list_count(stsd->child_boxes);
 
 	//RTP needs a default timeScale... use the media one.
 	if (CheckHintFormat(trak, GF_ISOM_HINT_RTP)) {
@@ -200,13 +209,14 @@ GF_Err gf_isom_new_hint_description(GF_ISOFile *the_file, u32 trackNumber, s32 H
 	if (!Rely) return GF_OK;
 
 	//we need a rely box (common to all protocols)
-	relyA = (GF_RelyHintBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_RELY);
+	relyA = (GF_RelyHintBox *) gf_isom_box_new_parent(&hdesc->child_boxes, GF_ISOM_BOX_TYPE_RELY);
+	if (!relyA) return GF_OUT_OF_MEM;
 	if (Rely == 1) {
-		relyA->prefered = 1;
+		relyA->preferred = 1;
 	} else {
 		relyA->required = 1;
 	}
-	return gf_list_add(hdesc->HintDataTable, relyA);
+	return GF_OK;
 }
 
 
@@ -215,6 +225,7 @@ GF_Err gf_isom_new_hint_description(GF_ISOFile *the_file, u32 trackNumber, s32 H
 *******************************************************************/
 
 //sets the RTP TimeScale
+GF_EXPORT
 GF_Err gf_isom_rtp_set_timescale(GF_ISOFile *the_file, u32 trackNumber, u32 HintDescriptionIndex, u32 TimeScale)
 {
 	GF_TrackBox *trak;
@@ -226,24 +237,28 @@ GF_Err gf_isom_rtp_set_timescale(GF_ISOFile *the_file, u32 trackNumber, u32 Hint
 	if (!trak || !CheckHintFormat(trak, GF_ISOM_HINT_RTP)) return GF_BAD_PARAM;
 
 	//OK, create a new HintSampleDesc
-	hdesc = (GF_HintSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->boxList, HintDescriptionIndex - 1);
-	count = gf_list_count(hdesc->HintDataTable);
+	hdesc = (GF_HintSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, HintDescriptionIndex - 1);
+	if (!hdesc) return GF_BAD_PARAM;
+
+	count = gf_list_count(hdesc->child_boxes);
 
 	for (i=0; i< count; i++) {
-		ent = (GF_TSHintEntryBox *)gf_list_get(hdesc->HintDataTable, i);
-		if (ent->type == GF_ISOM_BOX_TYPE_TIMS) {			
+		ent = (GF_TSHintEntryBox *)gf_list_get(hdesc->child_boxes, i);
+		if (ent->type == GF_ISOM_BOX_TYPE_TIMS) {
 			ent->timeScale = TimeScale;
 			return GF_OK;
 		}
 	}
 	//we have to create a new entry...
-	ent = (GF_TSHintEntryBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_TIMS);
+	ent = (GF_TSHintEntryBox *) gf_isom_box_new_parent(&hdesc->child_boxes, GF_ISOM_BOX_TYPE_TIMS);
+	if (!ent) return GF_OUT_OF_MEM;
 	ent->timeScale = TimeScale;
-	return gf_list_add(hdesc->HintDataTable, ent);
+	return GF_OK;
 }
 
 //sets the RTP TimeOffset that the server will add to the packets
 //if not set, the server adds a random offset
+GF_EXPORT
 GF_Err gf_isom_rtp_set_time_offset(GF_ISOFile *the_file, u32 trackNumber, u32 HintDescriptionIndex, u32 TimeOffset)
 {
 	GF_TrackBox *trak;
@@ -255,24 +270,28 @@ GF_Err gf_isom_rtp_set_time_offset(GF_ISOFile *the_file, u32 trackNumber, u32 Hi
 	if (!trak || !CheckHintFormat(trak, GF_ISOM_HINT_RTP)) return GF_BAD_PARAM;
 
 	//OK, create a new HintSampleDesc
-	hdesc = (GF_HintSampleEntryBox *) gf_list_get(trak->Media->information->sampleTable->SampleDescription->boxList, HintDescriptionIndex - 1);
-	count = gf_list_count(hdesc->HintDataTable);
+	hdesc = (GF_HintSampleEntryBox *) gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, HintDescriptionIndex - 1);
+	if (!hdesc) return GF_BAD_PARAM;
+	count = gf_list_count(hdesc->child_boxes);
 
 	for (i=0; i< count; i++) {
-		ent = (GF_TimeOffHintEntryBox *)gf_list_get(hdesc->HintDataTable, i);
-		if (ent->type == GF_ISOM_BOX_TYPE_TSRO) {			
+		ent = (GF_TimeOffHintEntryBox *)gf_list_get(hdesc->child_boxes, i);
+		if (ent->type == GF_ISOM_BOX_TYPE_TSRO) {
 			ent->TimeOffset = TimeOffset;
 			return GF_OK;
 		}
 	}
 	//we have to create a new entry...
-	ent = (GF_TimeOffHintEntryBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_TSRO);
+	ent = (GF_TimeOffHintEntryBox *) gf_isom_box_new_parent(&hdesc->child_boxes, GF_ISOM_BOX_TYPE_TSRO);
+	if (!ent) return GF_OUT_OF_MEM;
 	ent->TimeOffset = TimeOffset;
-	return gf_list_add(hdesc->HintDataTable, ent);
+
+	return GF_OK;
 }
 
 //sets the RTP SequenceNumber Offset that the server will add to the packets
 //if not set, the server adds a random offset
+GF_EXPORT
 GF_Err gf_isom_rtp_set_time_sequence_offset(GF_ISOFile *the_file, u32 trackNumber, u32 HintDescriptionIndex, u32 SequenceNumberOffset)
 {
 	GF_TrackBox *trak;
@@ -284,24 +303,27 @@ GF_Err gf_isom_rtp_set_time_sequence_offset(GF_ISOFile *the_file, u32 trackNumbe
 	if (!trak || !CheckHintFormat(trak, GF_ISOM_HINT_RTP)) return GF_BAD_PARAM;
 
 	//OK, create a new HintSampleDesc
-	hdesc = (GF_HintSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->boxList, HintDescriptionIndex - 1);
-	count = gf_list_count(hdesc->HintDataTable);
+	hdesc = (GF_HintSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, HintDescriptionIndex - 1);
+	if (!hdesc) return GF_BAD_PARAM;
+	count = gf_list_count(hdesc->child_boxes);
 
 	for (i=0; i< count; i++) {
-		ent = (GF_SeqOffHintEntryBox *)gf_list_get(hdesc->HintDataTable, i);
-		if (ent->type == GF_ISOM_BOX_TYPE_SNRO) {			
+		ent = (GF_SeqOffHintEntryBox *)gf_list_get(hdesc->child_boxes, i);
+		if (ent->type == GF_ISOM_BOX_TYPE_SNRO) {
 			ent->SeqOffset = SequenceNumberOffset;
 			return GF_OK;
 		}
 	}
 	//we have to create a new entry...
-	ent = (GF_SeqOffHintEntryBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_SNRO);
+	ent = (GF_SeqOffHintEntryBox *) gf_isom_box_new_parent(&hdesc->child_boxes, GF_ISOM_BOX_TYPE_SNRO);
+	if (!ent) return GF_OUT_OF_MEM;
 	ent->SeqOffset = SequenceNumberOffset;
-	return gf_list_add(hdesc->HintDataTable, ent);
+	return GF_OK;
 }
 
 //Starts a new sample for the hint track. A sample is just a collection of packets
 //the transmissionTime is indicated in the media timeScale of the hint track
+GF_EXPORT
 GF_Err gf_isom_begin_hint_sample(GF_ISOFile *the_file, u32 trackNumber, u32 HintDescriptionIndex, u32 TransmissionTime)
 {
 	GF_TrackBox *trak;
@@ -340,8 +362,9 @@ GF_Err gf_isom_begin_hint_sample(GF_ISOFile *the_file, u32 trackNumber, u32 Hint
 }
 
 //stores the hint sample in the file
-//set IsRandomAccessPoint if you want to indicate that this is a random access point 
+//set IsRandomAccessPoint if you want to indicate that this is a random access point
 //in the stream
+GF_EXPORT
 GF_Err gf_isom_end_hint_sample(GF_ISOFile *the_file, u32 trackNumber, u8 IsRandomAccessPoint)
 {
 	GF_TrackBox *trak;
@@ -361,8 +384,8 @@ GF_Err gf_isom_end_hint_sample(GF_ISOFile *the_file, u32 trackNumber, u8 IsRando
 	//first of all, we need to adjust the offset for data referenced IN THIS hint sample
 	//and get some PckSize
 	e = AdjustHintInfo(entry, trak->Media->information->sampleTable->SampleSize->sampleCount + 1);
-	if (e) return e;	
-	
+	if (e) return e;
+
 	//ok, let's write the sample
 	bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
 	e = gf_isom_hint_sample_write(entry->hint_sample, bs);
@@ -378,7 +401,7 @@ GF_Err gf_isom_end_hint_sample(GF_ISOFile *the_file, u32 trackNumber, u8 IsRando
 	gf_bs_get_content(bs, &samp->data, &samp->dataLength);
 	gf_bs_del(bs);
 
-	//finally add the sample 
+	//finally add the sample
 	e = gf_isom_add_sample(the_file, trackNumber, trak->Media->information->sampleTable->currentEntryIndex, samp);
 	gf_isom_sample_del(&samp);
 
@@ -390,6 +413,7 @@ GF_Err gf_isom_end_hint_sample(GF_ISOFile *the_file, u32 trackNumber, u8 IsRando
 
 
 //adds a blank chunk of data in the sample that is skipped while streaming
+GF_EXPORT
 GF_Err gf_isom_hint_blank_data(GF_ISOFile *the_file, u32 trackNumber, u8 AtBegin)
 {
 	GF_TrackBox *trak;
@@ -410,13 +434,14 @@ GF_Err gf_isom_hint_blank_data(GF_ISOFile *the_file, u32 trackNumber, u8 AtBegin
 	pck = (GF_HintPacket *)gf_list_get(entry->hint_sample->packetTable, count - 1);
 
 	dte = (GF_EmptyDTE *) NewDTE(0);
-	return gf_isom_hint_pck_add_dte(entry->hint_sample->HintType, pck, (GF_GenericDTE *)dte, AtBegin);
+	return gf_isom_hint_pck_add_dte(pck, (GF_GenericDTE *)dte, AtBegin);
 }
 
 
-//adds a chunk of data (max 14 bytes) in the packet that is directly copied 
+//adds a chunk of data (max 14 bytes) in the packet that is directly copied
 //while streaming
-GF_Err gf_isom_hint_direct_data(GF_ISOFile *the_file, u32 trackNumber, char *data, u32 dataLength, u8 AtBegin)
+GF_EXPORT
+GF_Err gf_isom_hint_direct_data(GF_ISOFile *the_file, u32 trackNumber, u8 *data, u32 dataLength, u8 AtBegin)
 {
 	GF_TrackBox *trak;
 	GF_HintSampleEntryBox *entry;
@@ -440,10 +465,11 @@ GF_Err gf_isom_hint_direct_data(GF_ISOFile *the_file, u32 trackNumber, char *dat
 	dte = (GF_ImmediateDTE *) NewDTE(1);
 	memcpy(dte->data, data + offset, dataLength);
 	dte->dataLength = dataLength;
-	return gf_isom_hint_pck_add_dte(entry->hint_sample->HintType, pck, (GF_GenericDTE *)dte, AtBegin);
+	return gf_isom_hint_pck_add_dte(pck, (GF_GenericDTE *)dte, AtBegin);
 }
 
-GF_Err gf_isom_hint_sample_data(GF_ISOFile *the_file, u32 trackNumber, u32 SourceTrackID, u32 SampleNumber, u16 DataLength, u32 offsetInSample, char *extra_data, u8 AtBegin)
+GF_EXPORT
+GF_Err gf_isom_hint_sample_data(GF_ISOFile *the_file, u32 trackNumber, GF_ISOTrackID SourceTrackID, u32 SampleNumber, u16 DataLength, u32 offsetInSample, u8 *extra_data, u8 AtBegin)
 {
 	GF_TrackBox *trak;
 	GF_HintSampleEntryBox *entry;
@@ -473,7 +499,7 @@ GF_Err gf_isom_hint_sample_data(GF_ISOFile *the_file, u32 trackNumber, u32 Sourc
 
 	//we're getting data from another track
 	if (SourceTrackID != trak->Header->trackID) {
-		//get (or set) the track reference index 
+		//get (or set) the track reference index
 		e = Track_FindRef(trak, GF_ISOM_REF_HINT, &hint);
 		if (e) return e;
 		e = reftype_AddRefTrack(hint, SourceTrackID, &refIndex);
@@ -493,13 +519,13 @@ GF_Err gf_isom_hint_sample_data(GF_ISOFile *the_file, u32 trackNumber, u32 Sourc
 		if (!SampleNumber || (SampleNumber == trak->Media->information->sampleTable->SampleSize->sampleCount + 1)) {
 			//we adding some stuff in the current sample ...
 			dte->byteOffset += entry->hint_sample->dataLength;
-			entry->hint_sample->AdditionalData = (char*)realloc(entry->hint_sample->AdditionalData, sizeof(char) * (entry->hint_sample->dataLength + DataLength));
+			entry->hint_sample->AdditionalData = (char*)gf_realloc(entry->hint_sample->AdditionalData, sizeof(char) * (entry->hint_sample->dataLength + DataLength));
 			if (AtBegin) {
 				if (entry->hint_sample->dataLength)
 					memmove(entry->hint_sample->AdditionalData + DataLength, entry->hint_sample->AdditionalData, entry->hint_sample->dataLength);
 				memcpy(entry->hint_sample->AdditionalData, extra_data, DataLength);
 				/*offset existing DTE*/
-				gf_isom_hint_pck_offset(entry->hint_sample->HintType, pck, DataLength, SampleNumber);
+				gf_isom_hint_pck_offset(pck, DataLength, SampleNumber);
 			} else {
 				memcpy(entry->hint_sample->AdditionalData + entry->hint_sample->dataLength, extra_data, DataLength);
 			}
@@ -509,10 +535,11 @@ GF_Err gf_isom_hint_sample_data(GF_ISOFile *the_file, u32 trackNumber, u32 Sourc
 		}
 	}
 	//OK, add the entry
-	return gf_isom_hint_pck_add_dte(entry->hint_sample->HintType, pck, (GF_GenericDTE *)dte, AtBegin);
+	return gf_isom_hint_pck_add_dte(pck, (GF_GenericDTE *)dte, AtBegin);
 }
 
-GF_Err gf_isom_hint_sample_description_data(GF_ISOFile *the_file, u32 trackNumber, u32 SourceTrackID, u32 StreamDescriptionIndex, u16 DataLength, u32 offsetInDescription, u8 AtBegin)
+GF_EXPORT
+GF_Err gf_isom_hint_sample_description_data(GF_ISOFile *the_file, u32 trackNumber, GF_ISOTrackID SourceTrackID, u32 StreamDescriptionIndex, u16 DataLength, u32 offsetInDescription, u8 AtBegin)
 {
 	GF_TrackBox *trak;
 	GF_HintSampleEntryBox *entry;
@@ -540,7 +567,7 @@ GF_Err gf_isom_hint_sample_description_data(GF_ISOFile *the_file, u32 trackNumbe
 	if (SourceTrackID == trak->Header->trackID) {
 		dte->trackRefIndex = (s8) -1;
 	} else {
-		//get (or set) the track reference index 
+		//get (or set) the track reference index
 		e = Track_FindRef(trak, GF_ISOM_REF_HINT, &hint);
 		if (e) return e;
 		e = reftype_AddRefTrack(hint, SourceTrackID, &refIndex);
@@ -548,19 +575,16 @@ GF_Err gf_isom_hint_sample_description_data(GF_ISOFile *the_file, u32 trackNumbe
 		//WARNING: IN QT, MUST BE 0-based !!!
 		dte->trackRefIndex = (u8) (refIndex - 1);
 	}
-	return gf_isom_hint_pck_add_dte(entry->hint_sample->HintType, pck, (GF_GenericDTE *)dte, AtBegin);
+	return gf_isom_hint_pck_add_dte(pck, (GF_GenericDTE *)dte, AtBegin);
 }
 
-#endif //GPAC_READ_ONLY
-
-#ifndef GPAC_READ_ONLY
-
+GF_EXPORT
 GF_Err gf_isom_rtp_packet_set_flags(GF_ISOFile *the_file, u32 trackNumber,
-						   u8 PackingBit, 
-						   u8 eXtensionBit, 
-						   u8 MarkerBit, 
-						   u8 disposable_packet, 
-						   u8 IsRepeatedPacket)
+                                    u8 PackingBit,
+                                    u8 eXtensionBit,
+                                    u8 MarkerBit,
+                                    u8 disposable_packet,
+                                    u8 IsRepeatedPacket)
 {
 	GF_TrackBox *trak;
 	GF_HintSampleEntryBox *entry;
@@ -587,15 +611,16 @@ GF_Err gf_isom_rtp_packet_set_flags(GF_ISOFile *the_file, u32 trackNumber,
 	return GF_OK;
 }
 
-GF_Err gf_isom_rtp_packet_begin(GF_ISOFile *the_file, u32 trackNumber, 
-						   s32 relativeTime, 
-						   u8 PackingBit, 
-						   u8 eXtensionBit, 
-						   u8 MarkerBit, 
-						   u8 PayloadType, 
-						   u8 B_frame, 
-						   u8 IsRepeatedPacket, 
-						   u16 SequenceNumber)
+GF_EXPORT
+GF_Err gf_isom_rtp_packet_begin(GF_ISOFile *the_file, u32 trackNumber,
+                                s32 relativeTime,
+                                u8 PackingBit,
+                                u8 eXtensionBit,
+                                u8 MarkerBit,
+                                u8 PayloadType,
+                                u8 B_frame,
+                                u8 IsRepeatedPacket,
+                                u16 SequenceNumber)
 {
 	GF_TrackBox *trak;
 	GF_HintSampleEntryBox *entry;
@@ -610,7 +635,7 @@ GF_Err gf_isom_rtp_packet_begin(GF_ISOFile *the_file, u32 trackNumber,
 	if (e) return e;
 	if (!entry->hint_sample) return GF_BAD_PARAM;
 
-	pck = (GF_RTPPacket *) gf_isom_hint_pck_new(entry->hint_sample->HintType);
+	pck = (GF_RTPPacket *) gf_isom_hint_pck_new(entry->type);
 
 	pck->P_bit = PackingBit ? 1 : 0;
 	pck->X_bit = eXtensionBit ? 1 : 0;
@@ -624,9 +649,10 @@ GF_Err gf_isom_rtp_packet_begin(GF_ISOFile *the_file, u32 trackNumber,
 }
 
 
-//set the time offset of this packet. This enables packets to be placed in the hint track 
-//in decoding order, but have their presentation time-stamp in the transmitted 
+//set the time offset of this packet. This enables packets to be placed in the hint track
+//in decoding order, but have their presentation time-stamp in the transmitted
 //packet be in a different order. Typically used for MPEG video with B-frames
+GF_EXPORT
 GF_Err gf_isom_rtp_packet_set_offset(GF_ISOFile *the_file, u32 trackNumber, s32 timeOffset)
 {
 	GF_RTPOBox *rtpo;
@@ -656,6 +682,7 @@ GF_Err gf_isom_rtp_packet_set_offset(GF_ISOFile *the_file, u32 trackNumber, s32
 	}
 	//not found, add it
 	rtpo = (GF_RTPOBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_RTPO);
+	if (!rtpo) return GF_OUT_OF_MEM;
 	rtpo->timeOffset = timeOffset;
 
 	return gf_list_add(pck->TLV, rtpo);
@@ -683,34 +710,36 @@ static void AddSDPLine(GF_List *list, char *sdp_text, Bool is_movie_sdp)
 
 static void ReorderSDP(char *sdp_text, Bool is_movie_sdp)
 {
-	char *cur, b;
+	char *cur;
 	GF_List *lines = gf_list_new();
 	cur = sdp_text;
 	while (cur) {
+		char b;
 		char *st = strstr(cur, "\r\n");
-		assert(st);
+		if (!st) break;
 		st += 2;
 		if (!st[0]) {
-			AddSDPLine(lines, strdup(cur), is_movie_sdp);
+			AddSDPLine(lines, gf_strdup(cur), is_movie_sdp);
 			break;
 		}
 		b = st[0];
 		st[0] = 0;
-		AddSDPLine(lines, strdup(cur), is_movie_sdp);
+		AddSDPLine(lines, gf_strdup(cur), is_movie_sdp);
 		st[0] = b;
 		cur = st;
 	}
 	strcpy(sdp_text, "");
 	while (gf_list_count(lines)) {
-		char *cur = (char *)gf_list_get(lines, 0);
+		cur = (char *)gf_list_get(lines, 0);
 		gf_list_rem(lines, 0);
 		strcat(sdp_text, cur);
-		free(cur);
+		gf_free(cur);
 	}
 	gf_list_del(lines);
 }
 
 //add an SDP line to the SDP container at the track level (media-specific SDP info)
+GF_EXPORT
 GF_Err gf_isom_sdp_add_track_line(GF_ISOFile *the_file, u32 trackNumber, const char *text)
 {
 	GF_TrackBox *trak;
@@ -730,32 +759,37 @@ GF_Err gf_isom_sdp_add_track_line(GF_ISOFile *the_file, u32 trackNumber, const c
 	if (!map) return GF_ISOM_INVALID_FILE;
 
 	//we should have only one HNTI in the UDTA
-	if (gf_list_count(map->boxList) != 1) return GF_ISOM_INVALID_FILE;
+	if (gf_list_count(map->boxes) != 1) return GF_ISOM_INVALID_FILE;
 
-	hnti = (GF_HintTrackInfoBox *)gf_list_get(map->boxList, 0);
+	hnti = (GF_HintTrackInfoBox *)gf_list_get(map->boxes, 0);
 	if (!hnti->SDP) {
-		e = hnti_AddBox(hnti, gf_isom_box_new(GF_ISOM_BOX_TYPE_SDP));
+		e = hnti_on_child_box((GF_Box*)hnti, gf_isom_box_new_parent(&hnti->child_boxes, GF_ISOM_BOX_TYPE_SDP), GF_FALSE);
 		if (e) return e;
 	}
 	sdp = (GF_SDPBox *) hnti->SDP;
 
 	if (!sdp->sdpText) {
-		sdp->sdpText = (char *)malloc(sizeof(char) * (strlen(text) + 3));
+		sdp->sdpText = (char *)gf_malloc(sizeof(char) * (strlen(text) + 3));
+		if (!sdp->sdpText) return GF_OUT_OF_MEM;
+
 		strcpy(sdp->sdpText, text);
 		strcat(sdp->sdpText, "\r\n");
 		return GF_OK;
 	}
-	buf = (char *)malloc(sizeof(char) * (strlen(sdp->sdpText) + strlen(text) + 3));
+	buf = (char *)gf_malloc(sizeof(char) * (strlen(sdp->sdpText) + strlen(text) + 3));
+	if (!buf) return GF_OUT_OF_MEM;
+
 	strcpy(buf, sdp->sdpText);
 	strcat(buf, text);
 	strcat(buf, "\r\n");
-	free(sdp->sdpText);
-	ReorderSDP(buf, 0);
-	sdp->sdpText = buf;	
+	gf_free(sdp->sdpText);
+	ReorderSDP(buf, GF_FALSE);
+	sdp->sdpText = buf;
 	return GF_OK;
 }
 
 //remove all SDP info at the track level
+GF_EXPORT
 GF_Err gf_isom_sdp_clean_track(GF_ISOFile *the_file, u32 trackNumber)
 {
 	GF_TrackBox *trak;
@@ -772,12 +806,12 @@ GF_Err gf_isom_sdp_clean_track(GF_ISOFile *the_file, u32 trackNumber)
 	if (!map) return GF_ISOM_INVALID_FILE;
 
 	//we should have only one HNTI in the UDTA
-	if (gf_list_count(map->boxList) != 1) return GF_ISOM_INVALID_FILE;
+	if (gf_list_count(map->boxes) != 1) return GF_ISOM_INVALID_FILE;
 
-	hnti = (GF_HintTrackInfoBox *)gf_list_get(map->boxList, 0);
+	hnti = (GF_HintTrackInfoBox *)gf_list_get(map->boxes, 0);
 	if (!hnti->SDP) return GF_OK;
 	//and free the SDP
-	free(((GF_SDPBox *)hnti->SDP)->sdpText);
+	gf_free(((GF_SDPBox *)hnti->SDP)->sdpText);
 	((GF_SDPBox *)hnti->SDP)->sdpText = NULL;
 	return GF_OK;
 }
@@ -785,6 +819,7 @@ GF_Err gf_isom_sdp_clean_track(GF_ISOFile *the_file, u32 trackNumber)
 
 //add an SDP line to the SDP container at the movie level (presentation SDP info)
 //NOTE: the \r\n end of line for SDP is automatically inserted
+GF_EXPORT
 GF_Err gf_isom_sdp_add_line(GF_ISOFile *movie, const char *text)
 {
 	GF_UserDataMap *map;
@@ -797,55 +832,58 @@ GF_Err gf_isom_sdp_add_line(GF_ISOFile *movie, const char *text)
 
 	//check if we have a udta ...
 	if (!movie->moov->udta) {
-		e = moov_AddBox((GF_Box*)movie->moov, gf_isom_box_new(GF_ISOM_BOX_TYPE_UDTA));
+		e = moov_on_child_box((GF_Box*)movie->moov, gf_isom_box_new_parent(&movie->moov->child_boxes, GF_ISOM_BOX_TYPE_UDTA), GF_FALSE);
 		if (e) return e;
 	}
 	//find a hnti in the udta
 	map = udta_getEntry(movie->moov->udta, GF_ISOM_BOX_TYPE_HNTI, NULL);
 	if (!map) {
-		e = udta_AddBox(movie->moov->udta, gf_isom_box_new(GF_ISOM_BOX_TYPE_HNTI));
+		e = udta_on_child_box((GF_Box *)movie->moov->udta, gf_isom_box_new(GF_ISOM_BOX_TYPE_HNTI), GF_FALSE);
 		if (e) return e;
 		map = udta_getEntry(movie->moov->udta, GF_ISOM_BOX_TYPE_HNTI, NULL);
 	}
 
 	//there should be one and only one hnti
-	if (!gf_list_count(map->boxList) ) {
-		e = udta_AddBox(movie->moov->udta, gf_isom_box_new(GF_ISOM_BOX_TYPE_HNTI));
+	if (!gf_list_count(map->boxes) ) {
+		e = udta_on_child_box((GF_Box *)movie->moov->udta, gf_isom_box_new(GF_ISOM_BOX_TYPE_HNTI), GF_FALSE);
 		if (e) return e;
 	}
-	else if (gf_list_count(map->boxList) < 1) return GF_ISOM_INVALID_FILE;
+	else if (gf_list_count(map->boxes) < 1) return GF_ISOM_INVALID_FILE;
 
-	hnti = (GF_HintTrackInfoBox *)gf_list_get(map->boxList, 0);
+	hnti = (GF_HintTrackInfoBox *)gf_list_get(map->boxes, 0);
 
 	if (!hnti->SDP) {
-		//we have to create it by hand, as we have a duplication of box type 
-		//(GF_RTPSampleEntryBox and GF_RTPBox have the same type...)
-		rtp = (GF_RTPBox *) malloc(sizeof(GF_RTPBox));
-		rtp->subType = GF_ISOM_BOX_TYPE_SDP;
-		rtp->type = GF_ISOM_BOX_TYPE_RTP;
-		rtp->sdpText = NULL;
-		hnti_AddBox(hnti, (GF_Box *)rtp);
+		GF_Box *a = gf_isom_box_new_ex(GF_ISOM_BOX_TYPE_RTP, GF_ISOM_BOX_TYPE_HNTI, 0, GF_FALSE, GF_FALSE);
+		if (!a) return GF_OUT_OF_MEM;
+		hnti_on_child_box((GF_Box*)hnti, a, GF_FALSE);
+		if (!hnti->child_boxes) hnti->child_boxes = gf_list_new();
+		gf_list_add(hnti->child_boxes, a);
 	}
 	rtp = (GF_RTPBox *) hnti->SDP;
 
 	if (!rtp->sdpText) {
-		rtp->sdpText = (char*)malloc(sizeof(char) * (strlen(text) + 3));
+		rtp->sdpText = (char*)gf_malloc(sizeof(char) * (strlen(text) + 3));
+		if (!rtp->sdpText) return GF_OUT_OF_MEM;
+
 		strcpy(rtp->sdpText, text);
 		strcat(rtp->sdpText, "\r\n");
 		return GF_OK;
 	}
-	buf = (char*)malloc(sizeof(char) * (strlen(rtp->sdpText) + strlen(text) + 3));
+	buf = (char*)gf_malloc(sizeof(char) * (strlen(rtp->sdpText) + strlen(text) + 3));
+	if (!buf) return GF_OUT_OF_MEM;
+	
 	strcpy(buf, rtp->sdpText);
 	strcat(buf, text);
 	strcat(buf, "\r\n");
-	free(rtp->sdpText);
-	ReorderSDP(buf, 1);
-	rtp->sdpText = buf;	
+	gf_free(rtp->sdpText);
+	ReorderSDP(buf, GF_TRUE);
+	rtp->sdpText = buf;
 	return GF_OK;
 }
 
 
 //remove all SDP info at the movie level
+GF_EXPORT
 GF_Err gf_isom_sdp_clean(GF_ISOFile *movie)
 {
 	GF_UserDataMap *map;
@@ -859,16 +897,16 @@ GF_Err gf_isom_sdp_clean(GF_ISOFile *movie)
 	if (!map) return GF_OK;
 
 	//there should be one and only one hnti
-	if (gf_list_count(map->boxList) != 1) return GF_ISOM_INVALID_FILE;
-	hnti = (GF_HintTrackInfoBox *)gf_list_get(map->boxList, 0);
+	if (gf_list_count(map->boxes) != 1) return GF_ISOM_INVALID_FILE;
+	hnti = (GF_HintTrackInfoBox *)gf_list_get(map->boxes, 0);
 
 	//remove and destroy the entry
-	gf_list_rem(map->boxList, 0);
+	gf_list_rem(map->boxes, 0);
 	gf_isom_box_del((GF_Box *)hnti);
 	return GF_OK;
 }
 
-#endif //GPAC_READ_ONLY
+#endif /*GPAC_DISABLE_ISOM_WRITE*/
 
 GF_EXPORT
 GF_Err gf_isom_sdp_get(GF_ISOFile *movie, const char **sdp, u32 *length)
@@ -887,13 +925,13 @@ GF_Err gf_isom_sdp_get(GF_ISOFile *movie, const char **sdp, u32 *length)
 	if (!map) return GF_OK;
 
 	//there should be one and only one hnti
-	if (gf_list_count(map->boxList) != 1) return GF_ISOM_INVALID_FILE;
-	hnti = (GF_HintTrackInfoBox *)gf_list_get(map->boxList, 0);
+	if (gf_list_count(map->boxes) != 1) return GF_ISOM_INVALID_FILE;
+	hnti = (GF_HintTrackInfoBox *)gf_list_get(map->boxes, 0);
 
 	if (!hnti->SDP) return GF_OK;
 	rtp = (GF_RTPBox *) hnti->SDP;
 
-	*length = strlen(rtp->sdpText);
+	*length = (u32) strlen(rtp->sdpText);
 	*sdp = rtp->sdpText;
 	return GF_OK;
 }
@@ -917,13 +955,13 @@ GF_Err gf_isom_sdp_track_get(GF_ISOFile *the_file, u32 trackNumber, const char *
 	if (!map) return GF_ISOM_INVALID_FILE;
 
 	//we should have only one HNTI in the UDTA
-	if (gf_list_count(map->boxList) != 1) return GF_ISOM_INVALID_FILE;
+	if (gf_list_count(map->boxes) != 1) return GF_ISOM_INVALID_FILE;
 
-	hnti = (GF_HintTrackInfoBox *)gf_list_get(map->boxList, 0);
+	hnti = (GF_HintTrackInfoBox *)gf_list_get(map->boxes, 0);
 	if (!hnti->SDP) return GF_OK;
 	sdpa = (GF_SDPBox *) hnti->SDP;
 
-	*length = strlen(sdpa->sdpText);
+	*length = (u32) strlen(sdpa->sdpText);
 	*sdp = sdpa->sdpText;
 	return GF_OK;
 }
@@ -941,15 +979,15 @@ u32 gf_isom_get_payt_count(GF_ISOFile *the_file, u32 trackNumber)
 	trak = gf_isom_get_track_from_file(the_file, trackNumber);
 	if (!trak) return 0;
 
-	if (!CheckHintFormat(trak, GF_4CC('r', 't', 'p', ' '))) return 0;
+	if (!CheckHintFormat(trak, GF_ISOM_HINT_RTP)) return 0;
 	map = udta_getEntry(trak->udta, GF_ISOM_BOX_TYPE_HINF, NULL);
 	if (!map) return 0;
-	if (gf_list_count(map->boxList) != 1) return 0;
+	if (gf_list_count(map->boxes) != 1) return 0;
 
-	hinf = (GF_HintInfoBox *)gf_list_get(map->boxList, 0);
+	hinf = (GF_HintInfoBox *)gf_list_get(map->boxes, 0);
 	count = 0;
 	i = 0;
-	while ((payt = gf_list_enum(hinf->boxList, &i))) {
+	while ((payt = (GF_PAYTBox*)gf_list_enum(hinf->child_boxes, &i))) {
 		if (payt->type == GF_ISOM_BOX_TYPE_PAYT) count++;
 	}
 	return count;
@@ -967,15 +1005,15 @@ const char *gf_isom_get_payt_info(GF_ISOFile *the_file, u32 trackNumber, u32 ind
 	trak = gf_isom_get_track_from_file(the_file, trackNumber);
 	if (!trak || !index) return NULL;
 
-	if (!CheckHintFormat(trak, GF_4CC('r', 't', 'p', ' '))) return NULL;
+	if (!CheckHintFormat(trak, GF_ISOM_HINT_RTP)) return NULL;
 	map = udta_getEntry(trak->udta, GF_ISOM_BOX_TYPE_HINF, NULL);
 	if (!map) return NULL;
-	if (gf_list_count(map->boxList) != 1) return NULL;
+	if (gf_list_count(map->boxes) != 1) return NULL;
 
-	hinf = (GF_HintInfoBox *)gf_list_get(map->boxList, 0);
+	hinf = (GF_HintInfoBox *)gf_list_get(map->boxes, 0);
 	count = 0;
 	i = 0;
-	while ((payt = gf_list_enum(hinf->boxList, &i))) {
+	while ((payt = (GF_PAYTBox*)gf_list_enum(hinf->child_boxes, &i))) {
 		if (payt->type == GF_ISOM_BOX_TYPE_PAYT) {
 			count++;
 			if (count == index) {
@@ -987,3 +1025,5 @@ const char *gf_isom_get_payt_info(GF_ISOFile *the_file, u32 trackNumber, u32 ind
 	return NULL;
 }
 
+#endif /*!defined(GPAC_DISABLE_ISOM) && !defined(GPAC_DISABLE_ISOM_HINTING)*/
+
