@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2019
  *					All rights reserved
  *
  *  This file is part of GPAC / ISO Media File Format sub-project
@@ -10,28 +11,30 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
 #include <gpac/internal/isomedia_dev.h>
 
-// Rewrite the good dependancies when an OD AU is extracted from the file
+#ifndef GPAC_DISABLE_ISOM
+
+// Rewrite the good dependencies when an OD AU is extracted from the file
 GF_Err Media_RewriteODFrame(GF_MediaBox *mdia, GF_ISOSample *sample)
 {
 	GF_Err e;
 	GF_ODCodec *ODdecode;
 	GF_ODCodec *ODencode;
 	GF_ODCom *com;
-	
+
 	//the commands we proceed
 	GF_ESDUpdate *esdU, *esdU2;
 	GF_ESDRemove *esdR, *esdR2;
@@ -52,7 +55,7 @@ GF_Err Media_RewriteODFrame(GF_MediaBox *mdia, GF_ISOSample *sample)
 	e = Track_FindRef(mdia->mediaTrack, GF_ISOM_BOX_TYPE_MPOD, &mpod);
 	if (e) return e;
 	//no references, nothing to do...
-	if (!mpod) return GF_OK;
+	if (!mpod || !mpod->trackIDs) return GF_OK;
 
 	ODdecode = gf_odf_codec_new();
 	if (!ODdecode) return GF_OUT_OF_MEM;
@@ -89,12 +92,12 @@ GF_Err Media_RewriteODFrame(GF_MediaBox *mdia, GF_ISOSample *sample)
 				}
 				e = gf_odf_desc_copy(desc, (GF_Descriptor **)&isom_od);
 				if (e) goto err_exit;
-					
+
 				//create our OD...
 				if (desc->tag == GF_ODF_ISOM_IOD_TAG) {
-					od = (GF_ObjectDescriptor *) malloc(sizeof(GF_InitialObjectDescriptor));
+					od = (GF_ObjectDescriptor *) gf_malloc(sizeof(GF_InitialObjectDescriptor));
 				} else {
-					od = (GF_ObjectDescriptor *) malloc(sizeof(GF_ObjectDescriptor));
+					od = (GF_ObjectDescriptor *) gf_malloc(sizeof(GF_ObjectDescriptor));
 				}
 				if (!od) {
 					e = GF_OUT_OF_MEM;
@@ -112,7 +115,7 @@ GF_Err Media_RewriteODFrame(GF_MediaBox *mdia, GF_ISOSample *sample)
 				isom_od->IPMP_Descriptors = NULL;
 				od->OCIDescriptors = isom_od->OCIDescriptors;
 				isom_od->OCIDescriptors = NULL;
-				
+
 				//init as IOD
 				if (isom_od->tag == GF_ODF_ISOM_IOD_TAG) {
 					((GF_InitialObjectDescriptor *)od)->audio_profileAndLevel = ((GF_IsomInitialObjectDescriptor *)isom_od)->audio_profileAndLevel;
@@ -124,12 +127,13 @@ GF_Err Media_RewriteODFrame(GF_MediaBox *mdia, GF_ISOSample *sample)
 					((GF_InitialObjectDescriptor *)od)->IPMPToolList = ((GF_IsomInitialObjectDescriptor *)isom_od)->IPMPToolList;
 					((GF_IsomInitialObjectDescriptor *)isom_od)->IPMPToolList = NULL;
 				}
-				
+
 				//then rewrite the ESDesc
 				j=0;
-				while ((ref = (GF_ES_ID_Ref*)gf_list_enum(isom_od->ES_ID_RefDescriptors, &j))){
+				while ((ref = (GF_ES_ID_Ref*)gf_list_enum(isom_od->ES_ID_RefDescriptors, &j))) {
+					if (!mpod->trackIDs || !ref->trackRef || (ref->trackRef>mpod->trackIDCount)) continue;
 					//if the ref index is not valid, skip this desc...
-					if (!mpod->trackIDs || gf_isom_get_track_from_id(mdia->mediaTrack->moov, mpod->trackIDs[ref->trackRef - 1]) == NULL) continue;
+					if (gf_isom_get_track_from_id(mdia->mediaTrack->moov, mpod->trackIDs[ref->trackRef - 1]) == NULL) continue;
 					//OK, get the esd
 					e = GetESDForTime(mdia->mediaTrack->moov, mpod->trackIDs[ref->trackRef - 1], sample->DTS, &esd);
 					if (!e) e = gf_odf_desc_add_desc((GF_Descriptor *) od, (GF_Descriptor *) esd);
@@ -157,6 +161,7 @@ GF_Err Media_RewriteODFrame(GF_MediaBox *mdia, GF_ISOSample *sample)
 			esdU2->ODID = esdU->ODID;
 			i=0;
 			while ((ref = (GF_ES_ID_Ref*)gf_list_enum(esdU->ESDescriptors, &i))) {
+				if (!mpod->trackIDs || !ref->trackRef || (ref->trackRef>mpod->trackIDCount)) continue;
 				//if the ref index is not valid, skip this desc...
 				if (gf_isom_get_track_from_id(mdia->mediaTrack->moov, mpod->trackIDs[ref->trackRef - 1]) == NULL) continue;
 				//OK, get the esd
@@ -176,7 +181,7 @@ GF_Err Media_RewriteODFrame(GF_MediaBox *mdia, GF_ISOSample *sample)
 			esdR2->ODID = esdR->ODID;
 			esdR2->NbESDs = esdR->NbESDs;
 			//alloc our stuff
-			esdR2->ES_ID = (unsigned short*)malloc(sizeof(u32) * esdR->NbESDs);
+			esdR2->ES_ID = (unsigned short*)gf_malloc(sizeof(u32) * esdR->NbESDs);
 			if (!esdR2->ES_ID) {
 				e = GF_OUT_OF_MEM;
 				goto err_exit;
@@ -184,6 +189,7 @@ GF_Err Media_RewriteODFrame(GF_MediaBox *mdia, GF_ISOSample *sample)
 			skipped = 0;
 			//get the ES_ID in the mpod indicated in the ES_ID[]
 			for (i = 0; i < esdR->NbESDs; i++) {
+				if (!mpod->trackIDs || !esdR->ES_ID[i] || (esdR->ES_ID[i]>mpod->trackIDCount)) continue;
 				//if the ref index is not valid, remove this desc...
 				if (gf_isom_get_track_from_id(mdia->mediaTrack->moov, mpod->trackIDs[esdR->ES_ID[i] - 1]) == NULL) {
 					skipped ++;
@@ -192,10 +198,10 @@ GF_Err Media_RewriteODFrame(GF_MediaBox *mdia, GF_ISOSample *sample)
 					esdR2->ES_ID[i - skipped] = mpod->trackIDs[esdR->ES_ID[i] - 1];
 				}
 			}
-			//realloc...
+			//gf_realloc...
 			if (skipped && (skipped != esdR2->NbESDs) ) {
 				esdR2->NbESDs -= skipped;
-				esdR2->ES_ID = (unsigned short*)realloc(esdR2->ES_ID, sizeof(u32) * esdR2->NbESDs);
+				esdR2->ES_ID = (unsigned short*)gf_realloc(esdR2->ES_ID, sizeof(u32) * esdR2->NbESDs);
 			}
 			gf_odf_com_del((GF_ODCom **)&esdR);
 			gf_odf_codec_add_com(ODencode, (GF_ODCom *)esdR2);
@@ -211,7 +217,7 @@ GF_Err Media_RewriteODFrame(GF_MediaBox *mdia, GF_ISOSample *sample)
 	if (e) goto err_exit;
 
 	//and set the buffer in the sample
-	free(sample->data);
+	gf_free(sample->data);
 	sample->data = NULL;
 	sample->dataLength = 0;
 	e = gf_odf_codec_get_au(ODencode, &sample->data, &sample->dataLength);
@@ -223,8 +229,8 @@ err_exit:
 }
 
 
-// Update the dependancies when an OD AU is inserted in the file
-GF_Err Media_ParseODFrame(GF_MediaBox *mdia, GF_ISOSample *sample, GF_ISOSample **od_samp)
+// Update the dependencies when an OD AU is inserted in the file
+GF_Err Media_ParseODFrame(GF_MediaBox *mdia, const GF_ISOSample *sample, GF_ISOSample **od_samp)
 {
 	GF_TrackReferenceBox *tref;
 	GF_TrackReferenceTypeBox *mpod;
@@ -251,18 +257,18 @@ GF_Err Media_ParseODFrame(GF_MediaBox *mdia, GF_ISOSample *sample, GF_ISOSample
 	//First find the references, and create them if none
 	tref = mdia->mediaTrack->References;
 	if (!tref) {
-		tref = (GF_TrackReferenceBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_TREF);
-		e = trak_AddBox((GF_Box*)mdia->mediaTrack, (GF_Box *) tref);
+		tref = (GF_TrackReferenceBox *) gf_isom_box_new_parent(&mdia->mediaTrack->child_boxes, GF_ISOM_BOX_TYPE_TREF);
+		if (!tref) return GF_OUT_OF_MEM;
+		e = trak_on_child_box((GF_Box*)mdia->mediaTrack, (GF_Box *) tref, GF_FALSE);
 		if (e) return e;
 	}
 	//then find the OD reference, and create it if none
 	e = Track_FindRef(mdia->mediaTrack, GF_ISOM_BOX_TYPE_MPOD, &mpod);
 	if (e) return e;
 	if (!mpod) {
-		mpod = (GF_TrackReferenceTypeBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_REFT);
+		mpod = (GF_TrackReferenceTypeBox *) gf_isom_box_new_parent(&tref->child_boxes, GF_ISOM_BOX_TYPE_REFT);
+		if (!mpod) return GF_OUT_OF_MEM;
 		mpod->reference_type = GF_ISOM_BOX_TYPE_MPOD;
-		e = tref_AddBox((GF_Box*)tref, (GF_Box *)mpod);
-		if (e) return e;
 	}
 
 	//OK, create our codecs
@@ -303,10 +309,12 @@ GF_Err Media_ParseODFrame(GF_MediaBox *mdia, GF_ISOSample *sample, GF_ISOSample
 				e = gf_odf_desc_copy(desc, (GF_Descriptor **)&od);
 				if (e) goto err_exit;
 				if (desc->tag == GF_ODF_OD_TAG) {
-					isom_od = (GF_IsomObjectDescriptor *) malloc(sizeof(GF_IsomObjectDescriptor));
+					isom_od = (GF_IsomObjectDescriptor *) gf_malloc(sizeof(GF_IsomObjectDescriptor));
+					if (!isom_od) return GF_OUT_OF_MEM;
 					isom_od->tag = GF_ODF_ISOM_OD_TAG;
 				} else {
-					isom_od = (GF_IsomObjectDescriptor *) malloc(sizeof(GF_IsomInitialObjectDescriptor));
+					isom_od = (GF_IsomObjectDescriptor *) gf_malloc(sizeof(GF_IsomInitialObjectDescriptor));
+					if (!isom_od) return GF_OUT_OF_MEM;
 					isom_od->tag = GF_ODF_ISOM_IOD_TAG;
 					//copy PL
 					((GF_IsomInitialObjectDescriptor *)isom_od)->inlineProfileFlag = ((GF_InitialObjectDescriptor *)od)->inlineProfileFlag;
@@ -321,7 +329,7 @@ GF_Err Media_ParseODFrame(GF_MediaBox *mdia, GF_ISOSample *sample, GF_ISOSample
 				//in OD stream only ref desc are accepted
 				isom_od->ES_ID_RefDescriptors = gf_list_new();
 				isom_od->ES_ID_IncDescriptors = NULL;
-	
+
 				//TO DO: check that a given sampleDescription exists
 				isom_od->extensionDescriptors = od->extensionDescriptors;
 				od->extensionDescriptors = NULL;
@@ -336,9 +344,15 @@ GF_Err Media_ParseODFrame(GF_MediaBox *mdia, GF_ISOSample *sample, GF_ISOSample
 				j=0;
 				while ((esd = (GF_ESD*)gf_list_enum(od->ESDescriptors, &j))) {
 					ref = (GF_ES_ID_Ref *) gf_odf_desc_new(GF_ODF_ESD_REF_TAG);
+					if (!esd->ESID) {
+						GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[ISOM] Missing ESID on ESD, cannot add track reference in OD frame"));
+						e = GF_BAD_PARAM;
+						goto err_exit;
+					}
 					//1 to 1 mapping trackID and ESID. Add this track to MPOD
 					//if track does not exist, this will be remove while reading the OD stream
 					e = reftype_AddRefTrack(mpod, esd->ESID, &ref->trackRef);
+					if (e) goto err_exit;
 					e = gf_odf_desc_add_desc((GF_Descriptor *)isom_od, (GF_Descriptor *)ref);
 					if (e) goto err_exit;
 				}
@@ -363,6 +377,7 @@ GF_Err Media_ParseODFrame(GF_MediaBox *mdia, GF_ISOSample *sample, GF_ISOSample
 				ref = (GF_ES_ID_Ref *) gf_odf_desc_new(GF_ODF_ESD_REF_TAG);
 				//1 to 1 mapping trackID and ESID
 				e = reftype_AddRefTrack(mpod, esd->ESID, &ref->trackRef);
+				if (e) goto err_exit;
 				e = gf_list_add(esdU2->ESDescriptors, ref);
 				if (e) goto err_exit;
 			}
@@ -380,7 +395,7 @@ GF_Err Media_ParseODFrame(GF_MediaBox *mdia, GF_ISOSample *sample, GF_ISOSample
 			esdR2->NbESDs = esdR->NbESDs;
 			if (esdR->NbESDs) {
 				//alloc our stuff
-				esdR2->ES_ID = (unsigned short*)malloc(sizeof(u32) * esdR->NbESDs);
+				esdR2->ES_ID = (unsigned short*)gf_malloc(sizeof(u32) * esdR->NbESDs);
 				if (!esdR2->ES_ID) {
 					e = GF_OUT_OF_MEM;
 					goto err_exit;
@@ -426,7 +441,7 @@ err_exit:
 
 
 
-// Rewrite the good dependancies when an OD AU is extracted from the file
+// Rewrite the good dependencies when an OD AU is extracted from the file
 static u32 Media_FindOD_ID(GF_MediaBox *mdia, GF_ISOSample *sample, u32 track_id)
 {
 	GF_Err e;
@@ -468,12 +483,13 @@ static u32 Media_FindOD_ID(GF_MediaBox *mdia, GF_ISOSample *sample, u32 track_id
 			switch (desc->tag) {
 			case GF_ODF_OD_TAG:
 			case GF_ODF_IOD_TAG:
-				esd_list = ((GF_ObjectDescriptor *)desc)->ESDescriptors; break;
+				esd_list = ((GF_ObjectDescriptor *)desc)->ESDescriptors;
+				break;
 			default:
 				continue;
-			}	
+			}
 			j=0;
-			while ((esd = (GF_ESD*)gf_list_enum( esd_list, &j))){
+			while ((esd = (GF_ESD*)gf_list_enum( esd_list, &j))) {
 				if (esd->ESID==track_id) {
 					the_od_id = ((GF_IsomObjectDescriptor*)desc)->objectDescriptorID;
 					break;
@@ -487,22 +503,27 @@ static u32 Media_FindOD_ID(GF_MediaBox *mdia, GF_ISOSample *sample, u32 track_id
 
 err_exit:
 	gf_odf_codec_del(ODdecode);
-	if (e) return 0;
-	return the_od_id;
+	return the_od_id; //still 0 if error, no need to check for e
 }
 
 
 GF_EXPORT
-u32 gf_isom_find_od_for_track(GF_ISOFile *file, u32 track)
+u32 gf_isom_find_od_id_for_track(GF_ISOFile *file, u32 track)
 {
 	u32 i, j, di, the_od_id;
 	GF_TrackBox *od_tk;
 	GF_TrackBox *tk = gf_isom_get_track_from_file(file, track);
 	if (!tk) return 0;
 
-	the_od_id = 0;
 	i=0;
 	while ( (od_tk = (GF_TrackBox*)gf_list_enum(file->moov->trackList, &i))) {
+		if (!od_tk || !od_tk->Media || !od_tk->Media->handler  || !od_tk->Media->information ||
+			!od_tk->Media->information->sampleTable || !od_tk->Media->information->sampleTable->SampleSize ) {
+
+			continue;
+		}
+
+
 		if (od_tk->Media->handler->handlerType != GF_ISOM_MEDIA_OD) continue;
 
 		for (j=0; j<od_tk->Media->information->sampleTable->SampleSize->sampleCount; j++) {
@@ -514,3 +535,5 @@ u32 gf_isom_find_od_for_track(GF_ISOFile *file, u32 track)
 	}
 	return 0;
 }
+
+#endif /*GPAC_DISABLE_ISOM*/
