@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2021
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / ISO Media File Format sub-project
@@ -35,8 +35,6 @@ gf_realloc low, which greatly impacts performances for large files*/
 		a = new_a;\
 	}
 
-#ifndef GPAC_DISABLE_ISOM_WRITE
-
 
 #define CHECK_PACK(_e) \
 	if (!nb_pack) nb_pack = 1; \
@@ -45,6 +43,8 @@ gf_realloc low, which greatly impacts performances for large files*/
 		return _e; \
 	}
 
+#ifndef GPAC_DISABLE_ISOM_WRITE
+
 
 //adds a DTS in the table and get the sample number of this new sample
 //we could return an error if a sample with the same DTS already exists
@@ -273,7 +273,7 @@ GF_Err stbl_AddCTS(GF_SampleTableBox *stbl, u32 sampleNumber, s32 offset)
 		if (offset<0) ctts->version=1;
 		if (ABS(offset) >= ctts->max_cts_delta) {
 			ctts->max_cts_delta = ABS(offset);
-			ctts->sample_num_max_cts_delta = ctts->w_LastSampleNumber;
+			//ctts->sample_num_max_cts_delta = ctts->w_LastSampleNumber;
 		}
 		return GF_OK;
 	}
@@ -289,7 +289,7 @@ GF_Err stbl_AddCTS(GF_SampleTableBox *stbl, u32 sampleNumber, s32 offset)
 		if (e) return e;
 		if (ABS(offset) >= ctts->max_cts_delta) {
 			ctts->max_cts_delta = ABS(offset);
-			ctts->sample_num_max_cts_delta = ctts->w_LastSampleNumber;
+			//ctts->sample_num_max_cts_delta = ctts->w_LastSampleNumber;
 		}
 		return GF_OK;
 	}
@@ -310,7 +310,7 @@ GF_Err stbl_AddCTS(GF_SampleTableBox *stbl, u32 sampleNumber, s32 offset)
 				sampNum ++;
 				if (ABS(offset) >= ctts->max_cts_delta) {
 					ctts->max_cts_delta = ABS(offset);
-					ctts->sample_num_max_cts_delta = sampNum;
+					//ctts->sample_num_max_cts_delta = sampNum;
 				}
 			}
 			CTSs[sampNum] = ctts->entries[i].decodingOffset;
@@ -372,6 +372,7 @@ GF_Err stbl_repackCTS(GF_CompositionOffsetBox *ctts)
 	/*note we don't realloc*/
 	return GF_OK;
 }
+#endif // GPAC_DISABLE_ISOM_WRITE
 
 GF_Err stbl_unpackCTS(GF_SampleTableBox *stbl)
 {
@@ -417,6 +418,9 @@ GF_Err stbl_unpackCTS(GF_SampleTableBox *stbl)
 	return GF_OK;
 }
 
+
+#ifndef GPAC_DISABLE_ISOM_WRITE
+
 //add size
 GF_Err stbl_AddSize(GF_SampleSizeBox *stsz, u32 sampleNumber, u32 size, u32 nb_pack)
 {
@@ -673,6 +677,8 @@ GF_Err stbl_AddDependencyType(GF_SampleTableBox *stbl, u32 sampleNumber, u32 isL
 }
 #endif
 
+#endif //GPAC_DISABLE_ISOM_WRITE
+
 GF_Err stbl_AppendDependencyType(GF_SampleTableBox *stbl, u32 isLeading, u32 dependsOn, u32 dependedOn, u32 redundant)
 {
 	GF_SampleDependencyTypeBox *sdtp;
@@ -700,6 +706,8 @@ GF_Err stbl_AppendDependencyType(GF_SampleTableBox *stbl, u32 isLeading, u32 dep
 	return GF_OK;
 }
 
+#ifndef GPAC_DISABLE_ISOM_WRITE
+
 //this function is always called in INCREASING order of shadow sample numbers
 GF_Err stbl_AddShadow(GF_ShadowSyncBox *stsh, u32 sampleNumber, u32 shadowNumber)
 {
@@ -914,7 +922,7 @@ GF_Err stbl_AddChunkOffset(GF_MediaBox *mdia, u32 sampleNumber, u32 StreamDescIn
 			gf_free(co64->offsets);
 			co64->offsets = newLarge;
 			co64->nb_entries++;
-			co64->alloc_size++;
+			co64->alloc_size = co64->nb_entries;
 		}
 	}
 
@@ -969,7 +977,7 @@ GF_Err stbl_SetSampleCTS(GF_SampleTableBox *stbl, u32 sampleNumber, s32 offset)
 {
 	GF_CompositionOffsetBox *ctts = stbl->CompositionOffset;
 
-	assert(ctts->unpack_mode);
+	gf_assert(ctts->unpack_mode);
 
 	//if we're setting the CTS of a sample we've skipped...
 	if (ctts->w_LastSampleNumber < sampleNumber) {
@@ -1081,7 +1089,9 @@ GF_Err stbl_SetSyncShadow(GF_ShadowSyncBox *stsh, u32 sampleNumber, u32 syncSamp
 		return gf_list_insert(stsh->entries, ent, i);
 	}
 }
+#endif
 
+#if !defined(GPAC_DISABLE_ISOM_WRITE) || !defined(GPAC_DISABLE_ISOM_FRAGMENTS)
 
 //always called before removing the sample from SampleSize
 GF_Err stbl_RemoveDTS(GF_SampleTableBox *stbl, u32 sampleNumber, u32 nb_samples, u32 LastAUDefDuration)
@@ -1098,6 +1108,8 @@ GF_Err stbl_RemoveDTS(GF_SampleTableBox *stbl, u32 sampleNumber, u32 nb_samples,
 		stts->nb_entries = 0;
 		stts->r_FirstSampleInEntry = stts->r_currentEntryIndex = 0;
 		stts->r_CurrentDTS = 0;
+		if (nb_samples>1)
+			stts->cumulated_start_dts += stts->entries[0].sampleDelta;
 		return GF_OK;
 	}
 	//we're removing the last sample
@@ -1105,10 +1117,12 @@ GF_Err stbl_RemoveDTS(GF_SampleTableBox *stbl, u32 sampleNumber, u32 nb_samples,
 		ent = &stts->entries[stts->nb_entries-1];
 		ent->sampleCount--;
 		if (!ent->sampleCount) stts->nb_entries--;
+		if (nb_samples>1)
+			stts->cumulated_start_dts += ent->sampleDelta;
 	} else {
 		u64 *DTSs, curDTS;
 		u32 i, j, k, sampNum;
-		u32 tot_samples, nb_written=0;
+		u32 tot_samples;
 		//unpack the DTSs...
 		DTSs = (u64*)gf_malloc(sizeof(u64) * (stbl->SampleSize->sampleCount - 1));
 		if (!DTSs) return GF_OUT_OF_MEM;
@@ -1128,9 +1142,12 @@ GF_Err stbl_RemoveDTS(GF_SampleTableBox *stbl, u32 sampleNumber, u32 nb_samples,
 					} else {
 						DTSs[sampNum-k] = curDTS;
 					}
-				} else if (sampNum >= nb_samples) {
-					DTSs[sampNum - nb_samples] = curDTS;
-					nb_written++;
+				} else {
+					if (sampNum >= nb_samples) {
+						DTSs[sampNum - nb_samples] = curDTS;
+					} else if (sampNum + 1 == nb_samples) {
+						stts->cumulated_start_dts += curDTS+ent->sampleDelta;
+					}
 				}
 				curDTS += ent->sampleDelta;
 				sampNum ++;
@@ -1138,8 +1155,7 @@ GF_Err stbl_RemoveDTS(GF_SampleTableBox *stbl, u32 sampleNumber, u32 nb_samples,
 		}
 
 		if (nb_samples>1) {
-			assert(sampNum == stbl->SampleSize->sampleCount);
-			assert(nb_written + nb_samples == stbl->SampleSize->sampleCount);
+			gf_assert(sampNum == stbl->SampleSize->sampleCount);
 		}
 		j=0;
 
@@ -1155,7 +1171,12 @@ GF_Err stbl_RemoveDTS(GF_SampleTableBox *stbl, u32 sampleNumber, u32 nb_samples,
 			if (stbl->SampleSize->sampleCount == 2) {
 				stts->entries[0].sampleDelta = LastAUDefDuration;
 			} else {
-				stts->entries[0].sampleDelta = (u32) DTSs[1] /*- DTSs[0]*/;
+				if (tot_samples>1) {
+					stts->entries[0].sampleDelta = (u32) (DTSs[1] - DTSs[0]);
+				} else {
+					//special case if we remove all but one sample, compute delta based on last DTS
+					stts->entries[0].sampleDelta = (u32) (curDTS - DTSs[0]);
+				}
 			}
 		} else {
 			sampNum = 0;
@@ -1180,14 +1201,14 @@ GF_Err stbl_RemoveDTS(GF_SampleTableBox *stbl, u32 sampleNumber, u32 nb_samples,
 				j++;
 				stts->entries[j].sampleCount = 1;
 				stts->entries[j].sampleDelta = (u32) (DTSs[i+1] - DTSs[i]);
-				assert(stts->entries[j].sampleDelta);
+				gf_assert(stts->entries[j].sampleDelta);
 				sampNum ++;
 			}
 		}
 		stts->w_LastDTS = tot_samples ? DTSs[tot_samples - 1] : 0;
 		gf_free(DTSs);
-		assert(sampNum == tot_samples);
-		assert(sampNum + nb_samples == stbl->SampleSize->sampleCount);
+		gf_assert(sampNum == tot_samples);
+		gf_assert(sampNum + nb_samples == stbl->SampleSize->sampleCount);
 	}
 
 	//reset write the cache to the end
@@ -1205,7 +1226,7 @@ GF_Err stbl_RemoveCTS(GF_SampleTableBox *stbl, u32 sampleNumber, u32 nb_samples)
 	GF_CompositionOffsetBox *ctts = stbl->CompositionOffset;
 	if (!ctts) return GF_OK;
 
-	assert(ctts->unpack_mode);
+	gf_assert(ctts->unpack_mode);
 	if ((nb_samples>1) && (sampleNumber>1)) return GF_BAD_PARAM;
 	ctts->max_cts_delta = 0;
 
@@ -1222,7 +1243,7 @@ GF_Err stbl_RemoveCTS(GF_SampleTableBox *stbl, u32 sampleNumber, u32 nb_samples)
 	if (sampleNumber > ctts->w_LastSampleNumber) return GF_OK;
 
 	if (nb_samples==1) {
-		assert(ctts->nb_entries);
+		gf_assert(ctts->nb_entries);
 		memmove(&ctts->entries[sampleNumber-1], &ctts->entries[sampleNumber], sizeof(GF_DttsEntry)* (ctts->nb_entries-sampleNumber) );
 		ctts->nb_entries--;
 	} else {
@@ -1230,7 +1251,7 @@ GF_Err stbl_RemoveCTS(GF_SampleTableBox *stbl, u32 sampleNumber, u32 nb_samples)
 		ctts->nb_entries -= nb_samples;
 	}
 	ctts->w_LastSampleNumber -= nb_samples;
-	assert(ctts->w_LastSampleNumber >= ctts->nb_entries);
+	gf_assert(ctts->w_LastSampleNumber >= ctts->nb_entries);
 
 	return GF_OK;
 }
@@ -1304,10 +1325,10 @@ GF_Err stbl_RemoveChunk(GF_SampleTableBox *stbl, u32 sampleNumber, u32 nb_sample
 
 		//update the firstchunk info
 		for (i=sampleNumber-1; i < stsc->nb_entries; i++) {
-			assert(stsc->entries[i].firstChunk >= 1);
+			gf_assert(stsc->entries[i].firstChunk >= 1);
 			stsc->entries[i].firstChunk -= 1;
 			if (stsc->entries[i].nextChunk) {
-				assert(stsc->entries[i].nextChunk >= 1);
+				gf_assert(stsc->entries[i].nextChunk >= 1);
 				stsc->entries[i].nextChunk -= 1;
 			}
 		}
@@ -1339,7 +1360,8 @@ GF_Err stbl_RemoveChunk(GF_SampleTableBox *stbl, u32 sampleNumber, u32 nb_sample
 			stco->alloc_size = 0;
 			return GF_OK;
 		}
-		assert(stco->nb_entries - nb_samples == stbl->SampleSize->sampleCount);
+		if (stco->nb_entries - nb_samples != stbl->SampleSize->sampleCount)
+			return GF_ISOM_INVALID_FILE;
 		if (nb_samples==1) {
 			memmove(&stco->offsets[sampleNumber-1], &stco->offsets[sampleNumber], sizeof(u32) * (stco->nb_entries - sampleNumber) );
 		} else {
@@ -1356,7 +1378,8 @@ GF_Err stbl_RemoveChunk(GF_SampleTableBox *stbl, u32 sampleNumber, u32 nb_sample
 			return GF_OK;
 		}
 
-		assert(co64->nb_entries - nb_samples == stbl->SampleSize->sampleCount);
+		if (co64->nb_entries - nb_samples != stbl->SampleSize->sampleCount)
+			return GF_ISOM_INVALID_FILE;
 		if (nb_samples==1) {
 			memmove(&co64->offsets[sampleNumber-1], &co64->offsets[sampleNumber], sizeof(u64) * (co64->nb_entries - sampleNumber) );
 		} else {
@@ -1377,7 +1400,13 @@ GF_Err stbl_RemoveRAP(GF_SampleTableBox *stbl, u32 sampleNumber)
 
 	//we remove the only one around...
 	if (stss->nb_entries == 1) {
-		if (stss->sampleNumbers[0] != sampleNumber) return GF_OK;
+		if (stss->sampleNumbers[0] != sampleNumber) {
+			if (sampleNumber < stss->sampleNumbers[0]) {
+				if (!stss->sampleNumbers[0]) return GF_ISOM_INVALID_FILE;
+				stss->sampleNumbers[0]--;
+			}
+			return GF_OK;
+		}
 		//free our numbers but don't delete (all samples are NON-sync
 		gf_free(stss->sampleNumbers);
 		stss->sampleNumbers = NULL;
@@ -1391,16 +1420,46 @@ GF_Err stbl_RemoveRAP(GF_SampleTableBox *stbl, u32 sampleNumber)
 		if (sampleNumber == stss->sampleNumbers[i]) {
 			memmove(&stss->sampleNumbers[i], &stss->sampleNumbers[i+1], sizeof(u32)* (stss->nb_entries-i-1) );
 			stss->nb_entries--;
+			i--;
 		}
 
-		if (sampleNumber < stss->sampleNumbers[i]) {
-			assert(stss->sampleNumbers[i]);
+		else if (sampleNumber < stss->sampleNumbers[i]) {
+			if (!stss->sampleNumbers[i]) return GF_ISOM_INVALID_FILE;
 			stss->sampleNumbers[i]--;
 		}
 	}
 	return GF_OK;
 }
 
+GF_Err stbl_RemoveRAPs(GF_SampleTableBox *stbl, u32 nb_samples)
+{
+	u32 i;
+
+	GF_SyncSampleBox *stss = stbl->SyncSample;
+	if (!stss) return GF_OK;
+
+	for (i=0; i<stss->nb_entries; i++) {
+		if (stss->sampleNumbers[i] <= nb_samples) {
+			memmove(&stss->sampleNumbers[i], &stss->sampleNumbers[i+1], sizeof(u32)* (stss->nb_entries-i-1) );
+			stss->nb_entries--;
+			i--;
+			continue;
+		}
+		stss->sampleNumbers[i] -= nb_samples;
+	}
+
+	if (!stss->nb_entries) {
+		//free our numbers but don't delete (all samples are NON-sync
+		gf_free(stss->sampleNumbers);
+		stss->sampleNumbers = NULL;
+		stss->r_LastSampleIndex = stss->r_LastSyncSample = 0;
+		stss->alloc_size = stss->nb_entries = 0;
+		return GF_OK;
+	}
+
+	return GF_OK;
+}
+
 GF_Err stbl_RemoveRedundant(GF_SampleTableBox *stbl, u32 SampleNumber, u32 nb_samples)
 {
 	u32 i;
@@ -1435,9 +1494,11 @@ GF_Err stbl_RemoveShadow(GF_SampleTableBox *stbl, u32 sampleNumber)
 	//shadows for 1 sample...
 	i=0;
 	while ((ent = (GF_StshEntry *)gf_list_enum(stsh->entries, &i))) {
-		if (ent->shadowedSampleNumber == sampleNumber) {
+		if (ent->shadowedSampleNumber <= sampleNumber) {
 			i--;
 			gf_list_rem(stsh->entries, i);
+		} else {
+			ent->shadowedSampleNumber--;
 		}
 	}
 	//reset the cache
@@ -1445,7 +1506,9 @@ GF_Err stbl_RemoveShadow(GF_SampleTableBox *stbl, u32 sampleNumber)
 	stsh->r_LastFoundSample = 0;
 	return GF_OK;
 }
+#endif
 
+#ifndef GPAC_DISABLE_ISOM_WRITE
 
 GF_Err stbl_SetPaddingBits(GF_SampleTableBox *stbl, u32 SampleNumber, u8 bits)
 {
@@ -1482,6 +1545,8 @@ GF_Err stbl_SetPaddingBits(GF_SampleTableBox *stbl, u32 SampleNumber, u8 bits)
 	return GF_OK;
 }
 
+#endif // GPAC_DISABLE_ISOM_WRITE
+
 GF_Err stbl_RemovePaddingBits(GF_SampleTableBox *stbl, u32 SampleNumber)
 {
 	u8 *p;
@@ -1517,7 +1582,7 @@ GF_Err stbl_RemovePaddingBits(GF_SampleTableBox *stbl, u32 SampleNumber)
 
 GF_Err stbl_RemoveSubSample(GF_SampleTableBox *stbl, u32 SampleNumber)
 {
-	u32 i, count, j, subs_count, prev_sample, delta=0;
+	u32 i, count, j, subs_count, prev_sample;
 
 	if (! stbl->sub_samples) return GF_OK;
 	subs_count = gf_list_count(stbl->sub_samples);
@@ -1525,12 +1590,19 @@ GF_Err stbl_RemoveSubSample(GF_SampleTableBox *stbl, u32 SampleNumber)
 		GF_SubSampleInformationBox *subs = gf_list_get(stbl->sub_samples, j);
 		if (! subs->Samples) continue;
 
+
 		prev_sample = 0;
 		count = gf_list_count(subs->Samples);
 		for (i=0; i<count; i++) {
 			GF_SubSampleInfoEntry *e = gf_list_get(subs->Samples, i);
 			prev_sample += e->sample_delta;
-			if (prev_sample==SampleNumber) {
+			//convert to sample num
+			e->sample_delta = prev_sample;
+		}
+		for (i=0; i<count; i++) {
+			GF_SubSampleInfoEntry *e = gf_list_get(subs->Samples, i);
+			//remove
+			if (e->sample_delta<=SampleNumber) {
 				gf_list_rem(subs->Samples, i);
 				while (gf_list_count(e->SubSamples)) {
 					GF_SubSampleEntry *pSubSamp = (GF_SubSampleEntry*) gf_list_get(e->SubSamples, 0);
@@ -1541,10 +1613,19 @@ GF_Err stbl_RemoveSubSample(GF_SampleTableBox *stbl, u32 SampleNumber)
 				gf_free(e);
 				i--;
 				count--;
-				delta=1;
 				continue;
+			} else {
+				e->sample_delta--;
 			}
-			e->sample_delta+=delta;
+
+		}
+		//convert back to delta
+		GF_SubSampleInfoEntry *prev_e = gf_list_get(subs->Samples, 0);
+		if (!prev_e) return GF_OK;
+		for (i=1; i<count; i++) {
+			GF_SubSampleInfoEntry *e = gf_list_get(subs->Samples, i);
+			e->sample_delta = e->sample_delta - prev_e->sample_delta;
+			prev_e = e;
 		}
 	}
 	return GF_OK;
@@ -1581,6 +1662,8 @@ GF_Err stbl_RemoveSampleGroup(GF_SampleTableBox *stbl, u32 SampleNumber)
 	return GF_OK;
 }
 
+#ifndef GPAC_DISABLE_ISOM_WRITE
+
 GF_Err stbl_SampleSizeAppend(GF_SampleSizeBox *stsz, u32 data_size)
 {
 	u32 i;
@@ -1730,6 +1813,7 @@ GF_Err stbl_AppendChunk(GF_SampleTableBox *stbl, u64 offset)
 	if (!co64->offsets) return GF_OUT_OF_MEM;
 	co64->offsets[co64->nb_entries] = offset;
 	co64->alloc_size = co64->nb_entries;
+        co64->nb_entries += 1;
 	return GF_OK;
 }
 
@@ -1805,7 +1889,7 @@ GF_Err stbl_AppendRAP(GF_SampleTableBox *stbl, u8 isRap)
 	return GF_OK;
 }
 
-GF_Err stbl_AppendTrafMap(GF_SampleTableBox *stbl, Bool is_seg_start, u64 seg_start_offset, u64 frag_start_offset, u8 *moof_template, u32 moof_template_size, u64 sidx_start, u64 sidx_end, u32 nb_pack_samples)
+GF_Err stbl_AppendTrafMap(GF_ISOFile *mov, GF_SampleTableBox *stbl, Bool is_seg_start, u64 seg_start_offset, u64 frag_start_offset, u64 tfdt, u8 *moof_template, u32 moof_template_size, u64 sidx_start, u64 sidx_end, u32 nb_pack_samples)
 {
 	GF_TrafToSampleMap *tmap;
 	GF_TrafMapEntry *tmap_ent;
@@ -1836,13 +1920,14 @@ GF_Err stbl_AppendTrafMap(GF_SampleTableBox *stbl, Bool is_seg_start, u64 seg_st
 	memset(tmap_ent, 0, sizeof(GF_TrafMapEntry));
 	tmap_ent->sample_num = stbl->SampleSize->sampleCount;
 	if (nb_pack_samples)
-		tmap_ent->sample_num -= tmap_ent->sample_num-1;
+		tmap_ent->sample_num -= nb_pack_samples-1;
 
 	tmap_ent->moof_template = moof_template;
 	tmap_ent->moof_template_size = moof_template_size;
 	tmap_ent->moof_start = frag_start_offset;
 	tmap_ent->sidx_start = sidx_start;
 	tmap_ent->sidx_end = sidx_end;
+	tmap_ent->first_dts = tfdt;
 	if (is_seg_start)
 		tmap_ent->seg_start_plus_one = 1 + seg_start_offset;
 
@@ -1871,9 +1956,16 @@ GF_Err stbl_AppendCTSOffset(GF_SampleTableBox *stbl, s32 offset)
 		if (!stbl->CompositionOffset) return GF_OUT_OF_MEM;
 	}
 	ctts = stbl->CompositionOffset;
+#ifndef GPAC_DISABLE_ISOM_WRITE
 	ctts->w_LastSampleNumber ++;
+#endif
 
-	if (!ctts->unpack_mode && ctts->nb_entries && (ctts->entries[ctts->nb_entries-1].decodingOffset == offset) ) {
+	if (
+#ifndef GPAC_DISABLE_ISOM_WRITE
+		!ctts->unpack_mode &&
+#endif
+		ctts->nb_entries && (ctts->entries[ctts->nb_entries-1].decodingOffset == offset)
+	) {
 		ctts->entries[ctts->nb_entries-1].sampleCount++;
 		return GF_OK;
 	}
@@ -1888,9 +1980,11 @@ GF_Err stbl_AppendCTSOffset(GF_SampleTableBox *stbl, s32 offset)
 	ctts->nb_entries++;
 	if (offset<0) ctts->version=1;
 
-	if (ABS(offset) >= ctts->max_cts_delta) {
+	if(offset == GF_INT_MIN) {
+		ctts->max_cts_delta = GF_INT_MAX;
+	} else if (ABS(offset) > ctts->max_cts_delta) {
 		ctts->max_cts_delta = ABS(offset);
-		ctts->sample_num_max_cts_delta = ctts->w_LastSampleNumber;
+		//ctts->sample_num_max_cts_delta = ctts->w_LastSampleNumber;
 	}
 
 	return GF_OK;
