@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2022
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / MPEG-4 ObjectDescriptor sub-project
@@ -726,6 +726,7 @@ GF_TextSampleDescriptor *gf_odf_tx3g_read(u8 *dsi, u32 dsi_size)
 	GF_TextSampleDescriptor *txtc = (GF_TextSampleDescriptor *) gf_odf_new_tx3g();
 	GF_BitStream *bs = gf_bs_new(dsi, dsi_size, GF_BITSTREAM_READ);
 
+	txtc->displayFlags = gf_bs_read_int(bs, 32);
 	txtc->horiz_justif = gf_bs_read_int(bs, 8);
 	txtc->vert_justif  = gf_bs_read_int(bs, 8);
 	txtc->back_color = gpp_read_rgba(bs);
@@ -757,6 +758,7 @@ GF_Err gf_odf_tx3g_write(GF_TextSampleDescriptor *a, u8 **outData, u32 *outSize)
 	void gpp_write_style(GF_BitStream *bs, GF_StyleRecord *rec);
 	GF_BitStream *bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
 
+	gf_bs_write_u32(bs, a->displayFlags);
 	gf_bs_write_u8(bs, a->horiz_justif);
 	gf_bs_write_u8(bs, a->vert_justif);
 	gpp_write_rgba(bs, a->back_color);
@@ -1395,12 +1397,12 @@ GF_VVCConfig *gf_odf_vvc_cfg_read_bs(GF_BitStream *bs)
 			gf_list_add(cfg->param_array, ar);
 			break;
 		default:
-			GF_LOG(GF_LOG_WARNING, GF_LOG_CODING, ("[VVC] Invalid NALU type in vvcC - ignoring\n", ar->nalus));
+			GF_LOG(GF_LOG_WARNING, GF_LOG_CODING, ("[VVC] Invalid NALU type %d in vvcC - ignoring\n", ar->type));
 			gf_free(ar);
 			break;
 		}
 
-		if ((ar->type != GF_VVC_NALU_DEC_PARAM) && (ar->type != GF_VVC_NALU_OPI))
+		if (!valid || ((ar->type != GF_VVC_NALU_DEC_PARAM) && (ar->type != GF_VVC_NALU_OPI)))
 			nalucount = gf_bs_read_int(bs, 16);
 		else
 			nalucount = 1;
@@ -1479,8 +1481,10 @@ GF_EXPORT
 GF_Err gf_odf_av1_cfg_write_bs(GF_AV1Config *cfg, GF_BitStream *bs)
 {
 	u32 i = 0;
-	gf_bs_write_int(bs, cfg->marker, 1); assert(cfg->marker == 1);
-	gf_bs_write_int(bs, cfg->version, 7); assert(cfg->version == 1);
+	gf_bs_write_int(bs, cfg->marker, 1);
+	gf_assert(cfg->marker == 1);
+	gf_bs_write_int(bs, cfg->version, 7);
+	gf_assert(cfg->version == 1);
 	gf_bs_write_int(bs, cfg->seq_profile, 3);
 	gf_bs_write_int(bs, cfg->seq_level_idx_0, 5);
 	gf_bs_write_int(bs, cfg->seq_tier_0, 1);
@@ -1644,11 +1648,6 @@ GF_AV1Config *gf_odf_av1_cfg_read_bs_size(GF_BitStream *bs, u32 size)
 	cfg->chroma_sample_position = gf_bs_read_int(bs, 2);
 
 	reserved = gf_bs_read_int(bs, 3);
-	if (reserved != 0 || cfg->marker != 1 || cfg->version != 1) {
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_CODING, ("[AV1] wrong avcC reserved %d / marker %d / version %d expecting 0 1 1\n", reserved, cfg->marker, cfg->version));
-		gf_odf_av1_cfg_del(cfg);
-		return NULL;
-	}
 	cfg->initial_presentation_delay_present = gf_bs_read_int(bs, 1);
 	if (cfg->initial_presentation_delay_present) {
 		cfg->initial_presentation_delay_minus_one = gf_bs_read_int(bs, 4);
@@ -1658,6 +1657,13 @@ GF_AV1Config *gf_odf_av1_cfg_read_bs_size(GF_BitStream *bs, u32 size)
 	}
 	size -= 4;
 
+	if (reserved != 0 || cfg->marker != 1 || cfg->version != 1) {
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_CODING, ("[AV1] wrong av1C reserved %d / marker %d / version %d expecting 0 1 1\n", reserved, cfg->marker, cfg->version));
+		gf_odf_av1_cfg_del(cfg);
+		return NULL;
+	}
+
+
 	while (size) {
 		u64 pos, obu_size;
 		ObuType obu_type;
@@ -1669,7 +1675,7 @@ GF_AV1Config *gf_odf_av1_cfg_read_bs_size(GF_BitStream *bs, u32 size)
 			GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[AV1] could not parse AV1 OBU at position "LLU". Leaving parsing.\n", pos));
 			break;
 		}
-		assert(obu_size == gf_bs_get_position(bs) - pos);
+		gf_assert(obu_size == gf_bs_get_position(bs) - pos);
 		GF_LOG(GF_LOG_DEBUG, GF_LOG_CODING, ("[AV1] parsed AV1 OBU type=%u size="LLU" at position "LLU".\n", obu_type, obu_size, pos));
 
 		if (!av1_is_obu_header(obu_type)) {
