@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2012
  *					All rights reserved
  *
  *  This file is part of GPAC / MPEG-4 ObjectDescriptor sub-project
@@ -10,20 +11,61 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
 #include <gpac/internal/odf_dev.h>
 
+#ifndef GPAC_MINIMAL_ODF
+
+/************************************************************
+		QoSQualifiers Functions
+************************************************************/
+GF_QoS_Default *NewQoS(u8 tag);
+
+GF_EXPORT
+GF_QoS_Default *gf_odf_qos_new(u8 tag)
+{
+	GF_QoS_Default *qos = NewQoS(tag);
+	return qos;
+}
+
+GF_EXPORT
+GF_Err gf_odf_qos_del(GF_QoS_Default **qos)
+{
+	if (*qos) gf_odf_delete_qos_qual(*qos);
+	*qos = NULL;
+	return GF_OK;
+}
+
+
+//same function, but for QoS, as a Qualifier IS NOT a descriptor
+GF_EXPORT
+GF_Err gf_odf_qos_add_qualif(GF_QoS_Descriptor *desc, GF_QoS_Default *qualif)
+{
+	u32 i;
+	GF_QoS_Default *def;
+
+	if (desc->tag != GF_ODF_QOS_TAG) return GF_BAD_PARAM;
+	if (desc->predefined) return GF_ODF_FORBIDDEN_DESCRIPTOR;
+
+	i=0;
+	while ((def = (GF_QoS_Default *)gf_list_enum(desc->QoS_Qualifiers, &i))) {
+		//if same Qualifier, not allowed...
+		if (def->tag == qualif->tag) return GF_ODF_FORBIDDEN_DESCRIPTOR;
+	}
+	return gf_list_add(desc->QoS_Qualifiers, qualif);
+}
+
 void gf_odf_delete_qos_qual(GF_QoS_Default *qos)
 {
 	switch (qos->tag) {
@@ -34,13 +76,13 @@ void gf_odf_delete_qos_qual(GF_QoS_Default *qos)
 	case QoSMaxAUSizeTag:
 	case QoSAvgAUSizeTag:
 	case QoSMaxAURateTag:
-		free(qos);
+		gf_free(qos);
 		return;
 
 	default:
-		if ( ((GF_QoS_Private *)qos)->DataLength)
-			free(((GF_QoS_Private *)qos)->Data);
-		free( (GF_QoS_Private *) qos);
+		if ( ((GF_QoS_Private *)qos)->Data)
+			gf_free(((GF_QoS_Private *)qos)->Data);
+		gf_free( (GF_QoS_Private *) qos);
 		return;
 	}
 }
@@ -76,7 +118,7 @@ GF_Err gf_odf_write_qos_qual(GF_BitStream *bs, GF_QoS_Default *qos)
 {
 	GF_Err e;
 	if (!bs || !qos) return GF_BAD_PARAM;
-	
+
 	e = gf_odf_size_qos_qual(qos);
 	if (e) return e;
 	e = gf_odf_write_base_descriptor(bs, qos->tag, qos->size);
@@ -140,51 +182,59 @@ GF_Err gf_odf_parse_qos(GF_BitStream *bs, GF_QoS_Default **qos_qual, u32 *qual_s
 	do {
 		val = gf_bs_read_int(bs, 8);
 		sizeHeader++;
+		if (sizeHeader > 5) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[ODF] Descriptor size on more than 4 bytes\n"));
+			return GF_ODF_INVALID_DESCRIPTOR;
+		}
 		qos_size <<= 7;
 		qos_size |= val & 0x7F;
 	} while ( val & 0x80 );
+	if (gf_bs_available(bs) < qos_size) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[ODF] Not enough bytes (%d) to read descriptor (size=%d)\n", gf_bs_available(bs), qos_size));
+		return GF_ODF_INVALID_DESCRIPTOR;
+	}
 	bytesParsed += sizeHeader;
 
 	//Payload
 	switch (tag) {
 	case QoSMaxDelayTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_MaxDelay));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_MaxDelay));
 		((GF_QoS_MaxDelay *)newQoS)->MaxDelay = gf_bs_read_int(bs, 32);
 		bytesParsed += 4;
 		break;
 
 	case QoSPrefMaxDelayTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_PrefMaxDelay));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_PrefMaxDelay));
 		((GF_QoS_PrefMaxDelay *)newQoS)->PrefMaxDelay = gf_bs_read_int(bs, 32);
 		bytesParsed += 4;
 		break;
 
 	case QoSLossProbTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_LossProb));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_LossProb));
 		((GF_QoS_LossProb *)newQoS)->LossProb = gf_bs_read_float(bs);
 		bytesParsed += 4;
 		break;
 
 	case QoSMaxGapLossTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_MaxGapLoss));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_MaxGapLoss));
 		((GF_QoS_MaxGapLoss *)newQoS)->MaxGapLoss = gf_bs_read_int(bs, 32);
 		bytesParsed += 4;
 		break;
 
 	case QoSMaxAUSizeTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_MaxAUSize));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_MaxAUSize));
 		((GF_QoS_MaxAUSize *)newQoS)->MaxAUSize = gf_bs_read_int(bs, 32);
 		bytesParsed += 4;
 		break;
 
 	case QoSAvgAUSizeTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_AvgAUSize));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_AvgAUSize));
 		((GF_QoS_AvgAUSize *)newQoS)->AvgAUSize = gf_bs_read_int(bs, 32);
 		bytesParsed += 4;
 		break;
 
 	case QoSMaxAURateTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_MaxAURate));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_MaxAURate));
 		((GF_QoS_MaxAURate *)newQoS)->MaxAURate = gf_bs_read_int(bs, 32);
 		bytesParsed += 4;
 		break;
@@ -195,10 +245,14 @@ GF_Err gf_odf_parse_qos(GF_BitStream *bs, GF_QoS_Default **qos_qual, u32 *qual_s
 
 	default:
 		//we defined the private qos...
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_Private));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_Private));
+		((GF_QoS_Private *)newQoS)->Data = NULL;
 		((GF_QoS_Private *)newQoS)->DataLength = qos_size;
-		gf_bs_read_data(bs, ((GF_QoS_Private *)newQoS)->Data, ((GF_QoS_Private *)newQoS)->DataLength);
-		bytesParsed += ((GF_QoS_Private *)newQoS)->DataLength;
+		if (qos_size > 0) {
+			((GF_QoS_Private *)newQoS)->Data = (char *) gf_malloc( qos_size );
+			gf_bs_read_data(bs, ((GF_QoS_Private *)newQoS)->Data, ((GF_QoS_Private *)newQoS)->DataLength);
+			bytesParsed += ((GF_QoS_Private *)newQoS)->DataLength;
+		}
 		break;
 	}
 	newQoS->size = qos_size;
@@ -219,43 +273,43 @@ GF_QoS_Default *NewQoS(u8 tag)
 
 	switch (tag) {
 	case QoSMaxDelayTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_MaxDelay));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_MaxDelay));
 		((GF_QoS_MaxDelay *)newQoS)->MaxDelay = 0;
 		((GF_QoS_MaxDelay *)newQoS)->size = 4;
 		break;
 
 	case QoSPrefMaxDelayTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_PrefMaxDelay));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_PrefMaxDelay));
 		((GF_QoS_PrefMaxDelay *)newQoS)->PrefMaxDelay = 0;
 		((GF_QoS_PrefMaxDelay *)newQoS)->size = 4;
 		break;
 
 	case QoSLossProbTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_LossProb));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_LossProb));
 		((GF_QoS_LossProb *)newQoS)->LossProb = 0;
 		((GF_QoS_LossProb *)newQoS)->size = 4;
 		break;
 
 	case QoSMaxGapLossTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_MaxGapLoss));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_MaxGapLoss));
 		((GF_QoS_MaxGapLoss *)newQoS)->MaxGapLoss = 0;
 		((GF_QoS_MaxGapLoss *)newQoS)->size = 4;
 		break;
 
 	case QoSMaxAUSizeTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_MaxAUSize));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_MaxAUSize));
 		((GF_QoS_MaxAUSize *)newQoS)->MaxAUSize = 0;
 		((GF_QoS_MaxAUSize *)newQoS)->size = 0;
 		break;
 
 	case QoSAvgAUSizeTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_AvgAUSize));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_AvgAUSize));
 		((GF_QoS_AvgAUSize *)newQoS)->AvgAUSize = 0;
 		((GF_QoS_AvgAUSize *)newQoS)->size = 4;
 		break;
 
 	case QoSMaxAURateTag:
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_MaxAURate));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_MaxAURate));
 		((GF_QoS_MaxAURate *)newQoS)->MaxAURate = 0;
 		((GF_QoS_MaxAURate *)newQoS)->size = 4;
 		break;
@@ -266,7 +320,7 @@ GF_QoS_Default *NewQoS(u8 tag)
 
 	default:
 		//we defined the private qos...
-		newQoS = (GF_QoS_Default *) malloc(sizeof(GF_QoS_Private));
+		newQoS = (GF_QoS_Default *) gf_malloc(sizeof(GF_QoS_Private));
 		((GF_QoS_Private *)newQoS)->DataLength = 0;
 		((GF_QoS_Private *)newQoS)->Data = NULL;
 		break;
@@ -280,7 +334,7 @@ GF_QoS_Default *NewQoS(u8 tag)
 //
 GF_Descriptor *gf_odf_new_qos()
 {
-	GF_QoS_Descriptor *newDesc = (GF_QoS_Descriptor *) malloc(sizeof(GF_QoS_Descriptor));
+	GF_QoS_Descriptor *newDesc = (GF_QoS_Descriptor *) gf_malloc(sizeof(GF_QoS_Descriptor));
 	if (!newDesc) return NULL;
 	newDesc->QoS_Qualifiers = gf_list_new();
 	newDesc->predefined = 0;
@@ -289,7 +343,7 @@ GF_Descriptor *gf_odf_new_qos()
 }
 
 //
-//	Desctructor
+//	Destructor
 //
 GF_Err gf_odf_del_qos(GF_QoS_Descriptor *qos)
 {
@@ -301,6 +355,7 @@ GF_Err gf_odf_del_qos(GF_QoS_Descriptor *qos)
 		gf_list_rem(qos->QoS_Qualifiers, 0);
 	}
 	gf_list_del(qos->QoS_Qualifiers);
+	gf_free(qos);
 	return GF_OK;
 }
 
@@ -319,13 +374,14 @@ GF_Err gf_odf_read_qos(GF_BitStream *bs, GF_QoS_Descriptor *qos, u32 DescSize)
 	nbBytes += 1;
 
 	if (qos->predefined) {
-		if (nbBytes != DescSize) return GF_ODF_INVALID_DESCRIPTOR;		
+		if (nbBytes != DescSize) return GF_ODF_INVALID_DESCRIPTOR;
 		return GF_OK;
 	}
 
 	while (nbBytes < DescSize) {
 		tmp = NULL;
 		e = gf_odf_parse_qos(bs, &tmp, &tmp_size);
+		if (e) return e;
 		if (!tmp) return GF_ODF_INVALID_DESCRIPTOR;
 		e = gf_list_add(qos->QoS_Qualifiers, tmp);
 		if (e) return e;
@@ -348,9 +404,9 @@ GF_Err gf_odf_size_qos(GF_QoS_Descriptor *qos, u32 *outSize)
 	GF_QoS_Default *tmp;
 
 	if (!qos) return GF_BAD_PARAM;
-	
+
 	*outSize = 1;
-	
+
 	i=0;
 	while ((tmp = (GF_QoS_Default *)gf_list_enum(qos->QoS_Qualifiers, &i))) {
 		e = gf_odf_size_qos_qual(tmp);
@@ -367,7 +423,6 @@ GF_Err gf_odf_write_qos(GF_BitStream *bs, GF_QoS_Descriptor *qos)
 {
 	GF_Err e;
 	u32 size, i;
-	GF_QoS_Default *tmp;
 	if (!qos) return GF_BAD_PARAM;
 
 	e = gf_odf_size_descriptor((GF_Descriptor *)qos, &size);
@@ -378,6 +433,7 @@ GF_Err gf_odf_write_qos(GF_BitStream *bs, GF_QoS_Descriptor *qos)
 	gf_bs_write_int(bs, qos->predefined, 8);
 
 	if (! qos->predefined) {
+		GF_QoS_Default *tmp;
 		i=0;
 		while ((tmp = (GF_QoS_Default *)gf_list_enum(qos->QoS_Qualifiers, &i))) {
 			e = gf_odf_write_qos_qual(bs, tmp);
@@ -388,3 +444,4 @@ GF_Err gf_odf_write_qos(GF_BitStream *bs, GF_QoS_Descriptor *qos)
 }
 
 
+#endif /*GPAC_MINIMAL_ODF*/
