@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Management sub-project
@@ -10,15 +11,15 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 #include <gpac/nodes_mpeg4.h>
@@ -27,7 +28,10 @@
 #include <gpac/internal/swf_dev.h>
 #include <gpac/internal/scenegraph_dev.h>
 
-#ifndef GPAC_READ_ONLY
+
+#ifndef GPAC_DISABLE_VRML
+
+#ifndef GPAC_DISABLE_SWF_IMPORT
 
 #define SWF_TEXT_SCALE				(1/1024.0f)
 
@@ -39,7 +43,6 @@ typedef struct
 
 static GF_Err s2b_insert_symbol(SWFReader *read, GF_Node *n)
 {
-	GF_Command *com;
 	GF_CommandField *f;
 
 	if (read->flags & GF_SM_SWF_STATIC_DICT) {
@@ -47,7 +50,7 @@ static GF_Err s2b_insert_symbol(SWFReader *read, GF_Node *n)
 		gf_node_list_add_child(&par->choice, n);
 		gf_node_register((GF_Node *)n, (GF_Node *)par);
 	} else {
-		com = gf_sg_command_new(read->load->scene_graph, GF_SG_INDEXED_INSERT);
+		GF_Command *com = gf_sg_command_new(read->load->scene_graph, GF_SG_INDEXED_INSERT);
 		com->node = gf_sg_find_node_by_name(read->load->scene_graph, "DICTIONARY");
 		gf_node_register(com->node, NULL);
 		f = gf_sg_command_field_new(com);
@@ -161,12 +164,10 @@ static Bool s2b_same_color(SFColor c1, SFColor c2)
 
 static GF_Node *s2b_get_appearance(SWFReader *read, GF_Node *parent, u32 fill_col, Fixed line_width, u32 l_col)
 {
-	char szDEF[1024];
 	u32 ID, i;
 	SFColor fc, lc;
 	Fixed fill_transp, line_transp;
 	M_Appearance *app;
-	M_Material2D *mat;
 
 	fc = s2b_get_color(fill_col);
 	fill_transp = FIX_ONE - s2b_get_alpha(fill_col);
@@ -177,7 +178,7 @@ static GF_Node *s2b_get_appearance(SWFReader *read, GF_Node *parent, u32 fill_co
 
 	i=0;
 	while ((app = (M_Appearance*)gf_list_enum(read->apps, &i))) {
-		mat = (M_Material2D *)app->material;
+		M_Material2D *mat = (M_Material2D *)app->material;
 		if (!line_width) {
 			if (mat->lineProps || !mat->filled) continue;
 		} else {
@@ -227,6 +228,7 @@ static GF_Node *s2b_get_appearance(SWFReader *read, GF_Node *parent, u32 fill_co
 	gf_node_register((GF_Node *)app, parent);
 
 	if (read->load->swf_import_flags & GF_SM_SWF_REUSE_APPEARANCE) {
+		char szDEF[1024];
 		sprintf(szDEF, "FILLAPP_%d", gf_list_count(read->apps));
 		read->load->ctx->max_node_id++;
 		ID = read->load->ctx->max_node_id;
@@ -306,7 +308,7 @@ static GF_Node *s2b_get_gradient(SWFReader *read, GF_Node *parent, SWFShape *sha
 		}
 		/*and remove material !!*/
 		((M_Material2D *)app->material)->filled = 0;
-		((M_Material2D *)app->material)->lineProps = s2b_new_node(read, TAG_MPEG4_LineProperties);;
+		((M_Material2D *)app->material)->lineProps = s2b_new_node(read, TAG_MPEG4_LineProperties);
 		((M_LineProperties *)((M_Material2D *)app->material)->lineProps)->width = 0;
 		gf_node_register(((M_Material2D *)app->material)->lineProps, app->material);
 	}
@@ -468,7 +470,7 @@ static GF_Node *s2b_shape_to_curve2d(SWFReader *read, SWFShape *shape, SWFShapeR
 	}
 
 
-	assert(srec->path->nbType);
+	gf_assert(srec->path->nbType);
 
 	pt_idx = 0;
 	for (i=0; i<srec->path->nbType; i++) {
@@ -582,7 +584,7 @@ static void s2b_merge_curve2d(M_Curve2D *s, M_Curve2D *tomerge)
 		gf_sg_vrml_mf_append(&dest->point, GF_SG_VRML_MFVEC2F, &ptr);
 		*((SFVec2f *)ptr) = pt;
 	}
-	
+
 	i = 0;
 	if (tomerge->type.vals[0] == 0) i=1;
 	pt_idx = 1;
@@ -631,11 +633,10 @@ static void s2b_merge_curve2d(M_Curve2D *s, M_Curve2D *tomerge)
 
 static void s2b_insert_shape(M_OrderedGroup *og, M_Shape *n, Bool is_proto)
 {
-	M_Shape *prev;
 	GF_ChildNodeItem *l = og->children;
 	if (!is_proto) {
 		while (l) {
-			prev = (M_Shape*)l->node;
+			M_Shape *prev = (M_Shape*)l->node;
 			if (prev->appearance == n->appearance) {
 				s2b_merge_curve2d( (M_Curve2D *)prev->geometry, (M_Curve2D *)n->geometry);
 				gf_node_register((GF_Node *)n, NULL);
@@ -652,7 +653,7 @@ static void s2b_insert_shape(M_OrderedGroup *og, M_Shape *n, Bool is_proto)
 static void s2b_insert_rec_in_coord(M_Coordinate2D *c, SWFShapeRec *srec)
 {
 	u32 i, j;
-	srec->path->idx = malloc(sizeof(u32)*srec->path->nbPts);
+	srec->path->idx = gf_malloc(sizeof(u32)*srec->path->nbPts);
 
 	for (i=0; i<srec->path->nbPts; i++) {
 		for (j=0; j<c->point.count; j++) {
@@ -662,10 +663,10 @@ static void s2b_insert_rec_in_coord(M_Coordinate2D *c, SWFShapeRec *srec)
 		}
 		if (j==c->point.count) {
 			c->point.count++;
-			c->point.vals = realloc(c->point.vals, sizeof(SFVec2f)*c->point.count);
+			c->point.vals = gf_realloc(c->point.vals, sizeof(SFVec2f)*c->point.count);
 			c->point.vals[j] = srec->path->pts[i];
 		}
-		srec->path->idx[i] = j; 
+		srec->path->idx[i] = j;
 	}
 }
 
@@ -684,12 +685,12 @@ static GF_Err swf_bifs_define_shape(SWFReader *read, SWFShape *shape, SWFFont *p
 	/*we need a grouping node*/
 	if (!read->cur_shape) {
 
-		/*empty shape - for fonts, not a mistake, that's likelly space char*/
+		/*empty shape - for fonts, not a mistake, that's likely space char*/
 		if (!shape) {
-			if (!parent_font) 
+			if (!parent_font)
 				return GF_OK;
 			n = s2b_new_node(read, TAG_MPEG4_Shape);
-		} 
+		}
 		/*direct match, no top group*/
 		else if (last_sub_shape && (gf_list_count(shape->fill_left) + gf_list_count(shape->lines)<=1)) {
 			Bool is_fill = 1;
@@ -847,6 +848,7 @@ static GF_Err swf_bifs_define_text(SWFReader *read, SWFText *text)
 		gf_node_register((GF_Node *) par, (GF_Node *)tr);
 
 		if (use_text) {
+			u32 _len;
 			u16 *str_w, *widestr;
 			char *str;
 			void *ptr;
@@ -861,38 +863,39 @@ static GF_Err swf_bifs_define_text(SWFReader *read, SWFText *text)
 			if (ft->fontName) {
 				gf_sg_vrml_mf_reset(&f->family, GF_SG_VRML_MFSTRING);
 				gf_sg_vrml_mf_append(&f->family, GF_SG_VRML_MFSTRING, &ptr);
-				((SFString*)ptr)->buffer = strdup(ft->fontName);
+				((SFString*)ptr)->buffer = gf_strdup(ft->fontName);
 			}
 			gf_sg_vrml_mf_reset(&f->justify, GF_SG_VRML_MFSTRING);
 			gf_sg_vrml_mf_append(&f->justify, GF_SG_VRML_MFSTRING, &ptr);
-			((SFString*)ptr)->buffer = strdup("BEGIN");
+			((SFString*)ptr)->buffer = gf_strdup("BEGIN");
 
-			if (f->style.buffer) free(f->style.buffer);
-			if (ft->is_italic && ft->is_bold) f->style.buffer = strdup("BOLDITALIC");
-			else if (ft->is_bold) f->style.buffer = strdup("BOLD");
-			else if (ft->is_italic) f->style.buffer = strdup("ITALIC");
-			else f->style.buffer = strdup("PLAIN");
+			if (f->style.buffer) gf_free(f->style.buffer);
+			if (ft->is_italic && ft->is_bold) f->style.buffer = gf_strdup("BOLDITALIC");
+			else if (ft->is_bold) f->style.buffer = gf_strdup("BOLD");
+			else if (ft->is_italic) f->style.buffer = gf_strdup("ITALIC");
+			else f->style.buffer = gf_strdup("PLAIN");
 
 			/*convert to UTF-8*/
-			str_w = (u16*)malloc(sizeof(u16) * (gr->nbGlyphs+1));
+			str_w = (u16*)gf_malloc(sizeof(u16) * (gr->nbGlyphs+1));
 			for (j=0; j<gr->nbGlyphs; j++) str_w[j] = ft->glyph_codes[gr->indexes[j]];
 			str_w[j] = 0;
-			str = (char*)malloc(sizeof(char) * (gr->nbGlyphs+2));
+			str = (char*)gf_malloc(sizeof(char) * (gr->nbGlyphs+2));
 			widestr = str_w;
-			j = gf_utf8_wcstombs(str, sizeof(u8) * (gr->nbGlyphs+1), (const unsigned short **) &widestr);
-			if (j != (u32) -1) {
+			_len = gf_utf8_wcstombs(str, sizeof(u8) * (gr->nbGlyphs+1), (const unsigned short **) &widestr);
+			if (_len != GF_UTF8_FAIL) {
+				j = _len;
 				str[j] = 0;
 				gf_sg_vrml_mf_reset(&t->string, GF_SG_VRML_MFSTRING);
 				gf_sg_vrml_mf_append(&t->string, GF_SG_VRML_MFSTRING, &ptr);
-				((SFString*)ptr)->buffer = (char*)malloc(sizeof(char) * (j+1));
+				((SFString*)ptr)->buffer = (char*)gf_malloc(sizeof(char) * (j+1));
 				memcpy(((SFString*)ptr)->buffer, str, sizeof(char) * (j+1));
 			}
 
-			free(str);
-			free(str_w);
+			gf_free(str);
+			gf_free(str_w);
 
 			gl = (M_Shape *) s2b_new_node(read, TAG_MPEG4_Shape);
-			gl->appearance = s2b_get_appearance(read, (GF_Node *) gl, gr->col, 0, 0);				
+			gl->appearance = s2b_get_appearance(read, (GF_Node *) gl, gr->col, 0, 0);
 			gl->geometry = (GF_Node *) t;
 			gf_node_register(gl->geometry, (GF_Node *) gl);
 			gf_node_insert_child((GF_Node *) par, (GF_Node *)gl, -1);
@@ -911,7 +914,7 @@ static GF_Err swf_bifs_define_text(SWFReader *read, SWFText *text)
 					dx += gr->dx[j];
 					continue;
 				}
-				assert((gf_node_get_tag(gl->geometry)==TAG_MPEG4_Curve2D) || (gf_node_get_tag(gl->geometry)==TAG_MPEG4_XCurve2D));
+				gf_assert((gf_node_get_tag(gl->geometry)==TAG_MPEG4_Curve2D) || (gf_node_get_tag(gl->geometry)==TAG_MPEG4_XCurve2D));
 
 				gl_par = (M_Transform2D *) s2b_new_node(read, TAG_MPEG4_Transform2D);
 				gl->appearance = s2b_get_appearance(read, (GF_Node *) gl, gr->col, 0, 0);
@@ -956,14 +959,14 @@ static void swf_ntext(void *sax_cbck, const char *content, Bool is_cdata)
 {
 	u32 len;
 	SWFFlatText *t;
-	if (!content || is_cdata) return;
+	if (!content || is_cdata || !sax_cbck) return;
 	t = (SWFFlatText *)sax_cbck;
-	len = strlen(content);
+	len = (u32) strlen(content);
 	if (!len) return;
-	t->final = realloc(t->final, sizeof(char)*(t->len+len+1));
+	t->final = gf_realloc(t->final, sizeof(char)*(t->len+len+1));
 	t->final [t->len] = 0;
 	strcat(t->final, content);
-	t->len = strlen(t->final)+1;
+	t->len = (u32) strlen(t->final)+1;
 }
 
 
@@ -1004,25 +1007,25 @@ static GF_Err swf_bifs_define_edit_text(SWFReader *read, SWFEditText *text)
 	gf_sg_vrml_mf_append(&f->justify, GF_SG_VRML_MFSTRING, (void**)&ptr);
 	switch (text->align) {
 	case 0:
-		((SFString*)ptr)->buffer = strdup("BEGIN"); 
+		((SFString*)ptr)->buffer = gf_strdup("BEGIN");
 		break;
 	case 1:
-		((SFString*)ptr)->buffer = strdup("END"); 
+		((SFString*)ptr)->buffer = gf_strdup("END");
 		break;
 	case 3:
-		((SFString*)ptr)->buffer = strdup("JUSTIFY"); 
+		((SFString*)ptr)->buffer = gf_strdup("JUSTIFY");
 		break;
 	default:
-		((SFString*)ptr)->buffer = strdup("MIDDLE"); 
+		((SFString*)ptr)->buffer = gf_strdup("MIDDLE");
 		break;
 	}
 
 	strcpy(styles, "");
 	if (!text->read_only) strcat(styles, "EDITABLE");
 	if (text->password) strcat(styles, "PASSWORD");
-	
-	if (f->style.buffer) free(f->style.buffer);
-	f->style.buffer = strdup(styles);
+
+	if (f->style.buffer) gf_free(f->style.buffer);
+	f->style.buffer = gf_strdup(styles);
 
 	if (text->init_value) {
 		gf_sg_vrml_mf_reset(&t->string, GF_SG_VRML_MFSTRING);
@@ -1039,35 +1042,35 @@ static GF_Err swf_bifs_define_edit_text(SWFReader *read, SWFEditText *text)
 			gf_xml_sax_del(xml);
 
 			if (flat.final) {
-				((SFString*)ptr)->buffer = strdup(flat.final);
-				free(flat.final);
+				((SFString*)ptr)->buffer = gf_strdup(flat.final);
+				gf_free(flat.final);
 			}
 		} else {
-			((SFString*)ptr)->buffer = strdup(text->init_value);
+			((SFString*)ptr)->buffer = gf_strdup(text->init_value);
 		}
 	}
 
 
 	txt = (M_Shape *) s2b_new_node(read, TAG_MPEG4_Shape);
-	txt->appearance = s2b_get_appearance(read, (GF_Node *) txt, text->color, 0, 0);				
+	txt->appearance = s2b_get_appearance(read, (GF_Node *) txt, text->color, 0, 0);
 	txt->geometry = (GF_Node *) t;
 	gf_node_register(txt->geometry, (GF_Node *) txt);
 
-	if (layout) {		
+	if (layout) {
 		gf_sg_vrml_mf_reset(&layout->justify, GF_SG_VRML_MFSTRING);
 		gf_sg_vrml_mf_append(&layout->justify, GF_SG_VRML_MFSTRING, NULL);
 		switch (text->align) {
 		case 0:
-			layout->justify.vals[0] = strdup("BEGIN"); 
+			layout->justify.vals[0] = gf_strdup("BEGIN");
 			break;
 		case 1:
-			layout->justify.vals[0] = strdup("END"); 
+			layout->justify.vals[0] = gf_strdup("END");
 			break;
 		case 3:
-			layout->justify.vals[0] = strdup("JUSTIFY"); 
+			layout->justify.vals[0] = gf_strdup("JUSTIFY");
 			break;
 		default:
-			layout->justify.vals[0] = strdup("MIDDLE"); 
+			layout->justify.vals[0] = gf_strdup("MIDDLE");
 			break;
 		}
 		if (text->multiline || text->word_wrap) layout->wrap = 1;
@@ -1080,7 +1083,7 @@ static GF_Err swf_bifs_define_edit_text(SWFReader *read, SWFEditText *text)
 	} else {
 		gf_node_insert_child((GF_Node *) tr, (GF_Node *)txt, -1);
 		gf_node_register((GF_Node *) txt, (GF_Node *) tr);
-	} 
+	}
 	if (tr) {
 		char szDEF[1024];
 		u32 ID;
@@ -1097,14 +1100,13 @@ static GF_Err swf_bifs_define_edit_text(SWFReader *read, SWFEditText *text)
 /*called upon end of sprite or clip*/
 static void swf_bifs_end_of_clip(SWFReader *read)
 {
+#if 0
 	char szDEF[1024];
 	u32 i;
 	GF_AUContext *au;
 	GF_Command *com;
 	GF_CommandField *f;
 	GF_Node *empty;
-	
-	return;
 
 	empty = gf_sg_find_node_by_name(read->load->scene_graph, "Shape0");
 
@@ -1126,6 +1128,8 @@ static void swf_bifs_end_of_clip(SWFReader *read)
 
 		gf_list_insert(au->commands, com, i);
 	}
+#endif
+
 }
 
 static Bool swf_bifs_allocate_depth(SWFReader *read, u32 depth)
@@ -1157,7 +1161,7 @@ static GF_Err swf_init_od(SWFReader *read, Bool root_only)
 		esd = (GF_ESD *) gf_odf_desc_esd_new(0);
 		if (!esd) return GF_OUT_OF_MEM;
 		esd->decoderConfig->streamType = GF_STREAM_SCENE;
-		esd->decoderConfig->objectTypeIndication = 1;
+		esd->decoderConfig->objectTypeIndication = GF_CODECID_BIFS;
 		esd->slConfig->timestampResolution = read->bifs_es->timeScale;
 		esd->ESID = 1;
 		gf_list_add(read->load->ctx->root_od->ESDescriptors, esd);
@@ -1173,13 +1177,13 @@ static GF_Err swf_init_od(SWFReader *read, Bool root_only)
 	if (root_only) return GF_OK;
 
 	if (read->od_es) return GF_OK;
-	read->od_es = gf_sm_stream_new(read->load->ctx, 2, 1, 1);
+	read->od_es = gf_sm_stream_new(read->load->ctx, 2, GF_STREAM_OD, GF_CODECID_OD_V1);
 	if (!read->od_es) return GF_OUT_OF_MEM;
 
 	esd = (GF_ESD *) gf_odf_desc_esd_new(0);
 	if (!esd) return GF_OUT_OF_MEM;
 	esd->decoderConfig->streamType = GF_STREAM_OD;
-	esd->decoderConfig->objectTypeIndication = 1;
+	esd->decoderConfig->objectTypeIndication = GF_CODECID_BIFS;
 	esd->slConfig->timestampResolution = read->od_es->timeScale = read->bifs_es->timeScale;
 	esd->ESID = 2;
 	esd->OCRESID = 1;
@@ -1247,13 +1251,13 @@ static GF_Err swf_bifs_define_sprite(SWFReader *read, u32 nb_frames)
 	/*always depends on main scene*/
 	esd->dependsOnESID = 1;
 	esd->decoderConfig->streamType = GF_STREAM_SCENE;
-	esd->decoderConfig->objectTypeIndication = 1;
+	esd->decoderConfig->objectTypeIndication = GF_CODECID_BIFS;
 	esd->slConfig->timestampResolution = read->bifs_es->timeScale;
 	gf_odf_desc_del((GF_Descriptor *) esd->decoderConfig->decoderSpecificInfo);
 	esd->decoderConfig->decoderSpecificInfo = NULL;
 	gf_list_add(od->ESDescriptors, esd);
 
-	/*by default insert OD at begining*/
+	/*by default insert OD at beginning*/
 	e = swf_insert_od(read, 0, od);
 	if (e) {
 		gf_odf_desc_del((GF_Descriptor *) od);
@@ -1294,7 +1298,7 @@ static GF_Err swf_bifs_define_sprite(SWFReader *read, u32 nb_frames)
 	ID = read->load->ctx->max_node_id;
 	gf_node_set_id(n, ID, szDEF);
 	par = gf_sg_find_node_by_name(read->load->scene_graph, "DICTIONARY");
-	assert(par);
+	gf_assert(par);
 	gf_node_list_add_child(&((M_Switch *)par)->choice, n);
 	gf_node_register(n, par);
 	par = gf_sg_find_node_by_name(read->load->scene_graph, "Shape0");
@@ -1305,9 +1309,9 @@ static GF_Err swf_bifs_define_sprite(SWFReader *read, u32 nb_frames)
 	prev_sc = read->bifs_es;
 	prev_au = read->bifs_au;
 	/*create new BIFS stream*/
-	read->bifs_es = gf_sm_stream_new(read->load->ctx, esd->ESID, GF_STREAM_SCENE, 1);
+	read->bifs_es = gf_sm_stream_new(read->load->ctx, esd->ESID, GF_STREAM_SCENE, GF_CODECID_BIFS);
 	read->bifs_es->timeScale = prev_sc->timeScale;
-	read->bifs_es->dump_time_offset = prev_sc->dump_time_offset + prev_au->timing;
+	read->bifs_es->imp_exp_time = prev_sc->imp_exp_time + prev_au->timing;
 
 	/*create first AU*/
 	read->bifs_au = gf_sm_stream_au_new(read->bifs_es, 0, 0, 1);
@@ -1363,14 +1367,13 @@ static GF_Err swf_bifs_setup_sound(SWFReader *read, SWFSound *snd, Bool soundstr
 		esd->OCRESID = esd->ESID;
 	} else {
 		/*soundstream runs on movie/sprite timeline*/
-		esd->OCRESID = read->bifs_es->ESID;
-		esd->OCRESID = esd->ESID;
+		esd->OCRESID = read->bifs_es ? read->bifs_es->ESID : esd->ESID;
 	}
 	gf_list_add(od->ESDescriptors, esd);
 
 	/*setup mux info*/
 	mux = (GF_MuxInfo*)gf_odf_desc_new(GF_ODF_MUXINFO_TAG);
-	mux->file_name = strdup(snd->szFileName);
+	mux->file_name = gf_strdup(snd->szFileName);
 //	mux->startTime = snd->frame_delay_ms;
 	mux->startTime = 0;
 	/*MP3 in, destroy file once done*/
@@ -1378,7 +1381,7 @@ static GF_Err swf_bifs_setup_sound(SWFReader *read, SWFSound *snd, Bool soundstr
 	gf_list_add(esd->extensionDescriptors, mux);
 
 
-	/*by default insert OD at begining*/
+	/*by default insert OD at beginning*/
 	e = swf_insert_od(read, 0, od);
 	if (e) {
 		gf_odf_desc_del((GF_Descriptor *) od);
@@ -1411,11 +1414,10 @@ static GF_Err swf_bifs_setup_sound(SWFReader *read, SWFSound *snd, Bool soundstr
 		/*if sprite always have the media active but controled by its mediaControl*/
 		if (read->current_sprite_id) {
 			((M_AudioClip*)n)->startTime = 0;
-		} 
+		}
 		/*otherwise start the media at the first soundstream block*/
 		else {
 			((M_AudioClip*)n)->startTime = snd->frame_delay_ms/1000.0;
-			((M_AudioClip*)n)->startTime = 0;
 		}
 
 		sprintf(szDEF, "CLIP%d_SND", read->current_sprite_id);
@@ -1447,7 +1449,7 @@ static GF_Err swf_bifs_setup_image(SWFReader *read, u32 ID, char *fileName)
 	GF_Node *n, *par;
 	GF_FieldInfo info;
 	char szDEF[100];
-	
+
 	e = swf_init_od(read, 0);
 	if (e) return e;
 
@@ -1464,13 +1466,13 @@ static GF_Err swf_bifs_setup_image(SWFReader *read, u32 ID, char *fileName)
 	/*setup mux info*/
 	mux = (GF_MuxInfo*)gf_odf_desc_new(GF_ODF_MUXINFO_TAG);
 
-	mux->file_name = strdup(fileName);
+	mux->file_name = gf_strdup(fileName);
 	/*destroy file once done*/
 	//mux->delete_file = 1;
 	gf_list_add(esd->extensionDescriptors, mux);
 
 
-	/*by default insert OD at begining*/
+	/*by default insert OD at beginning*/
 	e = swf_insert_od(read, 0, od);
 	if (e) {
 		gf_odf_desc_del((GF_Descriptor *) od);
@@ -1557,8 +1559,8 @@ static void s2b_set_field(SWFReader *read, GF_List *dst, GF_Node *n, char *field
 
 		if (type==GF_SG_VRML_SFSTRING) {
 			if (((SFString*)f->field_ptr)->buffer)
-				free(((SFString*)f->field_ptr)->buffer);
-			((SFString*)f->field_ptr)->buffer = strdup( (char *) val);
+				gf_free(((SFString*)f->field_ptr)->buffer);
+			((SFString*)f->field_ptr)->buffer = gf_strdup( (char *) val);
 		} else {
 			gf_sg_vrml_field_copy(f->field_ptr, val, type);
 		}
@@ -1573,7 +1575,7 @@ static void s2b_set_field(SWFReader *read, GF_List *dst, GF_Node *n, char *field
 	f = gf_sg_command_field_new(com);
 	f->field_ptr = gf_sg_vrml_field_pointer_new(type);
 	if (type==GF_SG_VRML_SFSTRING) {
-		((SFString*)f->field_ptr)->buffer = strdup( (char *) val);
+		((SFString*)f->field_ptr)->buffer = gf_strdup( (char *) val);
 	} else {
 		gf_sg_vrml_field_copy(f->field_ptr, val, type);
 	}
@@ -1581,7 +1583,7 @@ static void s2b_set_field(SWFReader *read, GF_List *dst, GF_Node *n, char *field
 	f->pos = pos;
 	f->fieldIndex = info.fieldIndex;
 
-	if (insert) 
+	if (insert)
 		gf_list_insert(dst, com, 0);
 	else
 		gf_list_add(dst, com);
@@ -1684,7 +1686,7 @@ static GF_Err swf_bifs_place_obj(SWFReader *read, u32 depth, u32 ID, u32 prev_id
 	f->pos = depth;
 	f->fieldIndex = 2;	/*children index*/
 	f->new_node = par;
-	gf_node_register(f->new_node, com->node);
+	gf_node_register(f->new_node, NULL);
 	gf_list_add(read->bifs_au->commands, com);
 
 	if (ID==prev_id) return GF_OK;
@@ -1729,7 +1731,7 @@ static GF_Err swf_bifs_remove_obj(SWFReader *read, u32 depth, u32 ID)
 	f->pos = depth;
 	f->fieldIndex = 2;	/*children index*/
 	f->new_node = gf_sg_find_node_by_name(read->load->scene_graph, "Shape0");
-	gf_node_register(f->new_node, com->node);
+	gf_node_register(f->new_node, NULL);
 	gf_list_add(read->bifs_au->commands, com);
 	/*check if this is a sprite*/
 	if (ID)
@@ -1742,8 +1744,10 @@ static GF_Err swf_bifs_show_frame(SWFReader *read)
 	u32 ts;
 	Bool is_rap;
 
-	/*hack to allow for empty BIFS AU to be encoded in order to keep the frame-rate (this reduces MP4 table size...)*/
-	if (0 && !gf_list_count(read->bifs_au->commands)) {
+	/*hack to allow for empty BIFS AU to be encoded in order to keep the frame-rate (this reduces MP4 table size...)
+	todo: make this an option - commented for now in both master and filters*/
+#if 0
+	if (!gf_list_count(read->bifs_au->commands)) {
 		GF_Command *com;
 		GF_CommandField *f;
 		com = gf_sg_command_new(read->load->scene_graph, GF_SG_FIELD_REPLACE);
@@ -1757,6 +1761,7 @@ static GF_Err swf_bifs_show_frame(SWFReader *read)
 		*((SFInt32 *)f->field_ptr) = -1;
 		gf_list_add(read->bifs_au->commands, com);
 	}
+#endif
 
 	/*create a new AU for next frame*/
 	ts = (read->current_frame + 1) * 100;
@@ -1767,7 +1772,7 @@ static GF_Err swf_bifs_show_frame(SWFReader *read)
 	*/
 	is_rap = (read->current_sprite_id || (read->flags & GF_SM_SWF_SPLIT_TIMELINE)) ? 1 : 0;
 #else
-	/*"The contents of the second frame are the cumulative effect of performing all the control tag operations from 
+	/*"The contents of the second frame are the cumulative effect of performing all the control tag operations from
 	the beginning of the file to the second ShowFrame tag, and so on"
 	using RAP=0 forces reprocessing of previous frames when seeking/jumping*/
 	is_rap = 0;
@@ -1782,10 +1787,10 @@ static GF_Err swf_bifs_show_frame(SWFReader *read)
 			read->bifs_dict_au = gf_sm_stream_au_new(read->bifs_dict_es, ts, 0, 0);
 		}
 		/*if wait_frame is specified, aggregate all dictionary commands until frame is reached*/
-		if (read->wait_frame && read->wait_frame<=read->current_frame) 
+		if (read->wait_frame && read->wait_frame<=read->current_frame)
 			read->wait_frame = 0;
 	}
-	
+
 	return GF_OK;
 }
 
@@ -1793,14 +1798,14 @@ static GF_Err swf_bifs_show_frame(SWFReader *read)
 static GF_Node *s2b_button_add_child(SWFReader *read, GF_Node *button, u32 tag, char *def_name, s32 pos)
 {
 	GF_Node *n = s2b_new_node(read, tag);
-	
+
 	if (def_name) {
 		u32 ID;
 		read->load->ctx->max_node_id++;
 		ID = read->load->ctx->max_node_id;
 		gf_node_set_id((GF_Node *)n, ID, def_name);
 	}
-	
+
 	gf_node_insert_child((GF_Node *)button, (GF_Node *)n, pos);
 	gf_node_register((GF_Node *) n, (GF_Node *) button);
 	return n;
@@ -1844,7 +1849,7 @@ static GF_Err swf_bifs_define_button(SWFReader *read, SWF_Button *btn)
 	gf_node_set_id((GF_Node *)btn_root, ID, szName);
 
 	n = s2b_button_add_child(read, btn_root, TAG_MPEG4_ColorTransform, NULL, -1);
-	((M_ColorTransform*)n)->maa = ((M_ColorTransform*)n)->mab = ((M_ColorTransform*)n)->mar = ((M_ColorTransform*)n)->mag = ((M_ColorTransform*)n)->ta = 0; 
+	((M_ColorTransform*)n)->maa = ((M_ColorTransform*)n)->mab = ((M_ColorTransform*)n)->mar = ((M_ColorTransform*)n)->mag = ((M_ColorTransform*)n)->ta = 0;
 
 	/*locate hit buttons and add them to the color transform*/
 	for (i=0; i<btn->count; i++) {
@@ -1911,7 +1916,7 @@ static GF_Err swf_bifs_define_button(SWFReader *read, SWF_Button *btn)
 		}
 		if (character) {
 			SFInt32 choice = 0;
-			GF_Node *n = s2b_wrap_node(read, character, &br->mx, &br->cmx);
+			n = s2b_wrap_node(read, character, &br->mx, &br->cmx);
 
 			sprintf(szName, "BTN%d_R%d", btn->ID, i+1);
 			button = (M_Switch *) s2b_button_add_child(read, btn_root, TAG_MPEG4_Switch, szName, pos);
@@ -1926,7 +1931,7 @@ static GF_Err swf_bifs_define_button(SWFReader *read, SWF_Button *btn)
 				if (sprite_ctrl) {
 					S2BBtnRec *btnrec;
 					if (!read->buttons) read->buttons = gf_list_new();
-					btnrec = malloc(sizeof(S2BBtnRec));
+					btnrec = gf_malloc(sizeof(S2BBtnRec));
 					btnrec->btn_id = btn->ID;
 					btnrec->sprite_up_id = br->character_id;
 					gf_list_add(read->buttons, btnrec);
@@ -1963,20 +1968,22 @@ static GF_Err swf_bifs_define_button(SWFReader *read, SWF_Button *btn)
 	return GF_OK;
 }
 
-static void swf_bifs_finalize(SWFReader *read)
+static void swf_bifs_finalize(SWFReader *read, Bool is_destroy)
 {
 	u32 i, count;
-
-	swf_bifs_end_of_clip(read);	
+	if (!is_destroy)
+		swf_bifs_end_of_clip(read);
 
 	while (gf_list_count(read->buttons)) {
 		S2BBtnRec *btnrec = gf_list_get(read->buttons, 0);
 		gf_list_rem(read->buttons, 0);
-		free(btnrec);
+		gf_free(btnrec);
 	}
+	gf_list_del(read->buttons);
+	read->buttons = NULL;
 
 	count = gf_list_count(read->fonts);
-	for (i=0;i<count; i++) {
+	for (i=0; i<count; i++) {
 		SWFFont *ft = (SWFFont *)gf_list_get(read->fonts, i);
 		while (gf_list_count(ft->glyphs)) {
 			GF_Node *gl = (GF_Node *)gf_list_get(ft->glyphs, 0);
@@ -1991,8 +1998,6 @@ Bool swf_bifs_action(SWFReader *read, SWFAction *act)
 	GF_List *dst;
 	MFURL url;
 	SFURL sfurl;
-	MFString str;
-	SFString sfstr;
 	Bool bval;
 	GF_Node *n;
 	Double time;
@@ -2020,14 +2025,11 @@ Bool swf_bifs_action(SWFReader *read, SWFAction *act)
 		break;
 	case GF_SWF_AS3_GET_URL:
 		n = gf_sg_find_node_by_name(read->load->scene_graph, "MOVIE_URL");
-		sfurl.OD_ID = 0; sfurl.url = act->url;
-		url.count = 1; url.vals = &sfurl;
+		sfurl.OD_ID = 0;
+		sfurl.url = act->url;
+		url.count = 1;
+		url.vals = &sfurl;
 		s2b_set_field(read, dst, n, "url", -1, GF_SG_VRML_MFURL, &url, 0);
-
-		sfstr.buffer = act->target;
-		str.count = 1; str.vals = &act->target;
-		s2b_set_field(read, dst, n, "parameter", -1, GF_SG_VRML_MFSTRING, &url, 0);
-		
 		bval = 1;
 		s2b_set_field(read, dst, n, "activate", -1, GF_SG_VRML_SFBOOL, &bval, 0);
 		break;
@@ -2073,8 +2075,16 @@ GF_Err swf_to_bifs_init(SWFReader *read)
 	read->action = swf_bifs_action;
 	read->finalize = swf_bifs_finalize;
 
+#ifdef GPAC_ENABLE_COVERAGE
+	if (gf_sys_is_cov_mode()) {
+		swf_nstart(NULL, NULL, NULL, NULL, 0);
+		swf_nend(NULL, NULL, NULL);
+		swf_ntext(NULL, NULL, GF_FALSE);
+	}
+#endif
+
 	/*create BIFS stream*/
-	read->bifs_es = gf_sm_stream_new(read->load->ctx, 1, GF_STREAM_SCENE, 0x01);
+	read->bifs_es = gf_sm_stream_new(read->load->ctx, 1, GF_STREAM_SCENE, GF_CODECID_BIFS);
 	read->bifs_es->timeScale = read->frame_rate*100;
 
 	read->bifs_au = gf_sm_stream_au_new(read->bifs_es, 0, 0.0, 1);
@@ -2085,7 +2095,6 @@ GF_Err swf_to_bifs_init(SWFReader *read)
 	read->load->ctx->is_pixel_metrics = 1;
 
 	gf_list_add(read->bifs_au->commands, com);
-	read->load->scene_graph = read->load->scene_graph;
 
 	/*create base tree*/
 	com->node = read->root = s2b_new_node(read, TAG_MPEG4_OrderedGroup);
@@ -2095,13 +2104,17 @@ GF_Err swf_to_bifs_init(SWFReader *read)
 	n = s2b_new_node(read, TAG_MPEG4_WorldInfo);
 	gf_node_insert_child(read->root, n, -1);
 	gf_node_register(n, read->root);
-	((M_WorldInfo *)n)->title.buffer = strdup("GPAC SWF CONVERTION DISCLAIMER");
-	gf_sg_vrml_mf_alloc( & ((M_WorldInfo *)n)->info, GF_SG_VRML_MFSTRING, 3);
+	((M_WorldInfo *)n)->title.buffer = gf_strdup("GPAC SWF CONVERTION DISCLAIMER");
+	gf_sg_vrml_mf_alloc( & ((M_WorldInfo *)n)->info, GF_SG_VRML_MFSTRING, 2);
 
 	sprintf(szMsg, "%s file converted to MPEG-4 Systems", read->load->fileName);
-	((M_WorldInfo *)n)->info.vals[0] = strdup(szMsg);
-	((M_WorldInfo *)n)->info.vals[1] = strdup("Conversion done using GPAC version " GPAC_FULL_VERSION " - (C) 2000-2005 GPAC");
-	((M_WorldInfo *)n)->info.vals[2] = strdup("Macromedia SWF to MPEG-4 Conversion mapping released under GPL license");
+	((M_WorldInfo *)n)->info.vals[0] = gf_strdup(szMsg);
+	if (gf_sys_is_test_mode()) {
+		sprintf(szMsg, "Conversion done using GPAC");
+	} else {
+		sprintf(szMsg, "Conversion done using GPAC version %s - %s", gf_gpac_version(), gf_gpac_copyright() );
+	}
+	((M_WorldInfo *)n)->info.vals[1] = gf_strdup(szMsg);
 
 	/*background*/
 	n = s2b_new_node(read, TAG_MPEG4_Background2D);
@@ -2111,13 +2124,13 @@ GF_Err swf_to_bifs_init(SWFReader *read)
 	gf_node_set_id(n, 1, "BACKGROUND");
 	gf_node_insert_child(read->root, n, -1);
 	gf_node_register(n, read->root);
-	
+
 	/*movie anchor*/
 	n = s2b_new_node(read, TAG_MPEG4_Anchor);
 	gf_node_set_id(n, 2, "MOVIE_URL");
 	gf_node_insert_child(read->root, n, -1);
 	gf_node_register(n, read->root);
-	
+
 	/*dictionary*/
 	n = s2b_new_node(read, TAG_MPEG4_Switch);
 	gf_node_set_id(n, 3, "DICTIONARY");
@@ -2153,17 +2166,16 @@ GF_Err swf_to_bifs_init(SWFReader *read)
 	/*setup IndexedCurve2D proto*/
 	if (read->flags & GF_SM_SWF_USE_IC2D) {
 		GF_ProtoFieldInterface *pfield;
-		GF_FieldInfo info;
 		SFURL *url;
 		Fixed ftMin, ftMax;
 		GF_Proto *proto = gf_sg_proto_new(read->load->scene_graph, 1, "IndexedCurve2D", 0);
 		if (read->load->ctx) read->load->ctx->max_proto_id = 1;
 		gf_sg_vrml_mf_reset(&proto->ExternProto, GF_SG_VRML_MFURL);
 		gf_sg_vrml_mf_append(&proto->ExternProto, GF_SG_VRML_MFURL, (void **) &url);
-		url->url = strdup("urn:inet:gpac:builtin:IndexedCurve2D");
+		url->url = gf_strdup("urn:inet:gpac:builtin:IndexedCurve2D");
 
 		gf_sg_proto_field_new(proto, GF_SG_VRML_SFNODE, GF_SG_EVENT_EXPOSED_FIELD, "coord");
-		
+
 		pfield = gf_sg_proto_field_new(proto, GF_SG_VRML_SFFLOAT, GF_SG_EVENT_EXPOSED_FIELD, "fineness");
 		gf_sg_proto_field_get_field(pfield, &info);
 		*((SFFloat*)info.far_ptr) = FIX_ONE/2;
@@ -2172,7 +2184,7 @@ GF_Err swf_to_bifs_init(SWFReader *read)
 		ftMin = 0;
 		ftMax = 15*FIX_ONE;
 		gf_bifs_proto_field_set_aq_info(pfield, 13, 1, GF_SG_VRML_SFINT32, &ftMin, &ftMax, 4);
-		
+
 		pfield = gf_sg_proto_field_new(proto, GF_SG_VRML_MFINT32, GF_SG_EVENT_EXPOSED_FIELD, "coordIndex");
 		ftMin = 0;
 		ftMax = FIX_MAX;
@@ -2201,7 +2213,7 @@ GF_Err swf_to_bifs_init(SWFReader *read)
 	esd->ESID = esd->OCRESID = 3;
 	esd->dependsOnESID = 1;
 	esd->decoderConfig->streamType = GF_STREAM_SCENE;
-	esd->decoderConfig->objectTypeIndication = 1;
+	esd->decoderConfig->objectTypeIndication = GF_CODECID_BIFS;
 	esd->slConfig->timestampResolution = read->bifs_es->timeScale;
 	gf_odf_desc_del((GF_Descriptor *) esd->decoderConfig->decoderSpecificInfo);
 	esd->decoderConfig->decoderSpecificInfo = NULL;
@@ -2214,7 +2226,7 @@ GF_Err swf_to_bifs_init(SWFReader *read)
 
 	/*setup a new BIFS context*/
 	prev_sc = read->bifs_es;
-	read->bifs_es = gf_sm_stream_new(read->load->ctx, esd->ESID, GF_STREAM_SCENE, 1);
+	read->bifs_es = gf_sm_stream_new(read->load->ctx, esd->ESID, GF_STREAM_SCENE, GF_CODECID_BIFS);
 	read->bifs_es->timeScale = prev_sc->timeScale;
 	/*create first AU*/
 	read->bifs_au = gf_sm_stream_au_new(read->bifs_es, 0, 0, 1);
@@ -2248,5 +2260,6 @@ GF_Err swf_to_bifs_init(SWFReader *read)
 	return GF_OK;
 }
 
-#endif
+#endif /*GPAC_DISABLE_SWF_IMPORT*/
 
+#endif /*GPAC_DISABLE_VRML*/
