@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2012
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Graph sub-project
@@ -10,15 +11,15 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
@@ -29,6 +30,8 @@
 
 #include <gpac/internal/laser_dev.h>
 
+
+GF_EXPORT
 GF_Command *gf_sg_command_new(GF_SceneGraph *graph, u32 tag)
 {
 	GF_Command *ptr;
@@ -40,56 +43,35 @@ GF_Command *gf_sg_command_new(GF_SceneGraph *graph, u32 tag)
 	if (tag < GF_SG_LAST_BIFS_COMMAND) ptr->new_proto_list = gf_list_new();
 	return ptr;
 }
-static void SG_CheckNodeUnregister(GF_Command *com)
-{
-	NodeIDedItem *reg_node;
-	switch (com->tag) {
-	case GF_SG_SCENE_REPLACE:
-	case GF_SG_LSR_NEW_SCENE:
-	case GF_SG_LSR_REFRESH_SCENE:
-		gf_node_unregister(com->node, NULL);
-		break;
-	default:
-		/*check if node is in registry - do NOT check the node's internals, because it may have been destroyed
-		when cyclic references are found (only happens with conditional replacing self/other conditional buffer)
-		note that we're sure the node has an ID since this is not a replace scene*/
-		reg_node = com->in_scene->id_node;
-		while (reg_node) {
-			if (reg_node->node == com->node) {
-				gf_node_unregister(com->node, NULL);
-				return;
-			}
-			reg_node = reg_node->next;
-		}
-		break;
-	}
-}
 
 GF_EXPORT
 void gf_sg_command_del(GF_Command *com)
 {
+#ifndef GPAC_DISABLE_VRML
 	u32 i;
 	GF_Proto *proto;
+#endif
 	if (!com) return;
 
 	if (com->tag < GF_SG_LAST_BIFS_COMMAND) {
+#ifndef GPAC_DISABLE_VRML
 		while (gf_list_count(com->command_fields)) {
 			GF_CommandField *inf = (GF_CommandField *)gf_list_get(com->command_fields, 0);
 			gf_list_rem(com->command_fields, 0);
 
 			switch (inf->fieldType) {
 			case GF_SG_VRML_SFNODE:
-				if (inf->new_node) gf_node_unregister(inf->new_node, com->node);
+				if (inf->new_node) gf_node_try_destroy(com->in_scene, inf->new_node, NULL);
 				break;
 			case GF_SG_VRML_MFNODE:
 				if (inf->field_ptr) {
-					GF_ChildNodeItem *cur, *child;
+					GF_ChildNodeItem *child;
 					child = inf->node_list;
 					while (child) {
-						cur = child;
-						gf_node_unregister(child->node, com->node);
+						GF_ChildNodeItem *cur = child;
+						gf_node_try_destroy(com->in_scene, child->node, NULL);
 						child = child->next;
-						free(cur);
+						gf_free(cur);
 					}
 				}
 				break;
@@ -97,89 +79,115 @@ void gf_sg_command_del(GF_Command *com)
 				if (inf->field_ptr) gf_sg_vrml_field_pointer_del(inf->field_ptr, inf->fieldType);
 				break;
 			}
-			free(inf);
+			gf_free(inf);
 		}
+#endif
 	} else {
 #ifndef GPAC_DISABLE_SVG
 		while (gf_list_count(com->command_fields)) {
 			GF_CommandField *inf = (GF_CommandField *)gf_list_get(com->command_fields, 0);
 			gf_list_rem(com->command_fields, 0);
 
-			if (inf->new_node) gf_node_unregister(inf->new_node, com->node);
+			if (inf->new_node)
+				gf_node_try_destroy(com->in_scene, inf->new_node, NULL);
 			else if (inf->node_list) {
-				gf_node_unregister_children(com->node, inf->node_list);
+				GF_ChildNodeItem *child;
+				child = inf->node_list;
+				while (child) {
+					GF_ChildNodeItem *cur = child;
+					gf_node_try_destroy(com->in_scene, child->node, NULL);
+					child = child->next;
+					gf_free(cur);
+				}
 			} else if (inf->field_ptr) {
 				gf_svg_delete_attribute_value(inf->fieldType, inf->field_ptr, com->in_scene);
 			}
-			free(inf);
+			gf_free(inf);
 		}
 #endif
 	}
 	gf_list_del(com->command_fields);
 
+#ifndef GPAC_DISABLE_VRML
 	i=0;
 	while ((proto = (GF_Proto*)gf_list_enum(com->new_proto_list, &i))) {
 		gf_sg_proto_del(proto);
 	}
 	gf_list_del(com->new_proto_list);
+#endif
 
 	if (com->node) {
-		if (!com->in_scene) gf_node_unregister(com->node, NULL);
-		else SG_CheckNodeUnregister(com);
+		gf_node_try_destroy(com->in_scene, com->node, NULL);
 	}
 
-	if (com->del_proto_list) free(com->del_proto_list);
-	if (com->def_name) free(com->def_name);
+	if (com->del_proto_list) gf_free(com->del_proto_list);
+	if (com->def_name) gf_free(com->def_name);
 	if (com->scripts_to_load) gf_list_del(com->scripts_to_load);
-	free(com);
+	if (com->unres_name) gf_free(com->unres_name);
+	gf_free(com);
 }
 
 static void SG_CheckFieldChange(GF_Node *node, GF_FieldInfo *field)
 {
 	/*and propagate eventIn if any*/
 	if (field->on_event_in) {
-		field->on_event_in(node);
-	} else if ((field->eventType==GF_SG_EVENT_IN) && (gf_node_get_tag(node) == TAG_MPEG4_Script)) {
+		field->on_event_in(node, NULL);
+	}
+#ifndef GPAC_DISABLE_VRML
+	else if ((field->eventType==GF_SG_EVENT_IN) && (gf_node_get_tag(node) == TAG_MPEG4_Script)) {
 		gf_sg_script_event_in(node, field);
-	} else {
+	}
+	else {
 		/*Notify eventOut in all cases to handle protos*/
 		gf_node_event_out(node, field->fieldIndex);
 	}
+#endif
 	/*signal node modif*/
 	gf_node_changed(node, field);
 }
 
+#ifndef GPAC_DISABLE_SVG
 static void gf_node_unregister_children_deactivate(GF_Node *container, GF_ChildNodeItem *child)
 {
-	GF_ChildNodeItem *cur;
 	while (child) {
-		gf_node_unregister(child->node, container);
+		GF_ChildNodeItem *cur;
 		gf_node_deactivate(child->node);
+		gf_node_unregister(child->node, container);
 		cur = child;
 		child = child->next;
-		free(cur);
+		gf_free(cur);
 	}
 }
+#endif
 
 
 GF_EXPORT
 GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_offset)
 {
 	GF_Err e;
+#if !defined(GPAC_DISABLE_VRML) || !defined(GPAC_DISABLE_SVG)
 	GF_CommandField *inf;
+	GF_Node *node;
+#endif
+
+#ifndef GPAC_DISABLE_VRML
 	GF_FieldInfo field;
-	GF_Node *def, *node;
 	void *slot_ptr;
+	GF_Node *def;
+#endif
 
 	if (!com || !graph) return GF_BAD_PARAM;
 
+	if (com->never_apply) return GF_OK;
+
 	e = GF_OK;
 	switch (com->tag) {
+#ifndef GPAC_DISABLE_VRML
 	case GF_SG_SCENE_REPLACE:
 		/*unregister root*/
 		gf_node_unregister(graph->RootNode, NULL);
 		/*remove all protos and routes*/
-		while (gf_list_count(graph->routes_to_activate)) 
+		while (gf_list_count(graph->routes_to_activate))
 			gf_list_rem(graph->routes_to_activate, 0);
 
 		if (!com->aggregated) {
@@ -199,6 +207,9 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		/*DO NOT TOUCH node registry*/
 		/*DO NOT TOUCH UNREGISTERED PROTOS*/
 
+		/*if no protos (previously aggregated command) create proto list*/
+		if (!graph->protos) graph->protos = gf_list_new();
+
 		/*move all protos in graph*/
 		while (gf_list_count(com->new_proto_list)) {
 			GF_Proto *p = (GF_Proto*)gf_list_get(com->new_proto_list, 0);
@@ -215,43 +226,8 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		if (!gf_list_count(com->command_fields)) return GF_OK;
 		inf = (GF_CommandField*)gf_list_get(com->command_fields, 0);
 		e = gf_node_replace(com->node, inf->new_node, 0);
-		if (inf->new_node) gf_node_register(inf->new_node, NULL);
-		break;
-
-	case GF_SG_FIELD_REPLACE_OP:
-	{
-		void *src_ptr;
-		u32 src_type;
-		u32 j=0;
-		GF_FieldInfo src;
-		def = gf_sg_find_node(com->in_scene, com->fromNodeID);
-		if (!def) return GF_SG_UNKNOWN_NODE;
-		e = gf_node_get_field(def, com->fromFieldIndex, &src);
-		if (e) return e;
-
-		src_type = src.fieldType;
-		src_ptr = src.far_ptr;
-		if (com->send_event_x>=0) {
-			src_type = gf_sg_vrml_get_sf_type(src.fieldType);
-			e = gf_sg_vrml_mf_get_item(src.far_ptr, src.fieldType, &src_ptr, com->send_event_x);
-			if (e) return e;
-		}
-
-		while ((inf = (GF_CommandField*)gf_list_enum(com->command_fields, &j))) {
-			e = gf_node_get_field(com->node, inf->fieldIndex, &field);
-			if (e) return e;
-			if (inf->pos>=0) {
-				void *slot_ptr;
-				e = gf_sg_vrml_mf_get_item(field.far_ptr, field.fieldType, & slot_ptr, inf->pos);
-				if (e) return e;
-				gf_sg_vrml_field_copy(slot_ptr, src_ptr, src_type);
-			} else {
-				gf_sg_vrml_field_copy(field.far_ptr, src_ptr, src_type);
-			}
-		}
-
-		SG_CheckFieldChange(com->node, &field);
-	}
+		//TOCHECK - this is commented as registering shouldn't happen (already done at command creation) and creates mem leak
+//		if (inf->new_node) gf_node_register(inf->new_node, NULL);
 		break;
 
 	case GF_SG_MULTIPLE_REPLACE:
@@ -280,7 +256,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 				list = * ((GF_ChildNodeItem **) inf->field_ptr);
 				prev=NULL;
 				while (list) {
-					cur = malloc(sizeof(GF_ChildNodeItem));
+					cur = gf_malloc(sizeof(GF_ChildNodeItem));
 					cur->next = NULL;
 					cur->node = list->node;
 					if (prev) {
@@ -301,26 +277,28 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 				SFCommandBuffer *cb_src = (SFCommandBuffer *)inf->field_ptr;
 
 				/*reset dest*/
+				if (!cb_dst->commandList) cb_dst->commandList = gf_list_new();
+
 				while (gf_list_count(cb_dst->commandList)) {
 					GF_Command *sub_com = (GF_Command *)gf_list_get(cb_dst->commandList, 0);
 					gf_sg_command_del(sub_com);
 					gf_list_rem(cb_dst->commandList, 0);
 				}
 				if (cb_dst->buffer) {
-					free(cb_dst->buffer);
+					gf_free(cb_dst->buffer);
 					cb_dst->buffer = NULL;
 				}
 
 				/*clone command list*/
 				sg = gf_node_get_graph(com->node);
 				count = gf_list_count(cb_src->commandList);
-				for (i=0; i<count;i++) {
+				for (i=0; i<count; i++) {
 					GF_Command *sub_com = (GF_Command *)gf_list_get(cb_src->commandList, i);
-					GF_Command *new_com = gf_sg_command_clone(sub_com, sg, 0);
+					GF_Command *new_com = gf_sg_vrml_command_clone(sub_com, sg, 0);
 					gf_list_add(cb_dst->commandList, new_com);
 				}
 			}
-				break;
+			break;
 
 			default:
 				/*this is a regular field, reset it and clone - we cannot switch pointers since the
@@ -330,7 +308,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 				}
 				if (e) return e;
 				gf_sg_vrml_field_copy(field.far_ptr, inf->field_ptr, field.fieldType);
-				
+
 				if ((field.fieldType==GF_SG_VRML_SFTIME) && !strstr(field.name, "media"))
 					*(SFTime *)field.far_ptr = *(SFTime *)field.far_ptr + time_offset;
 				break;
@@ -357,6 +335,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 			}
 			/*erase the field item*/
 			else {
+				if (!field.far_ptr) return GF_NON_COMPLIANT_BITSTREAM;
 				if ((inf->pos < 0) || ((u32) inf->pos >= ((GenMFField *) field.far_ptr)->count) ) {
 					inf->pos = ((GenMFField *)field.far_ptr)->count - 1;
 					/*may happen with text and default value*/
@@ -393,7 +372,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		gf_sg_route_set_id(r, com->RouteID);
 		if (name) {
 			gf_sg_route_set_name(r, name);
-			free(name);
+			gf_free(name);
 		}
 		break;
 	}
@@ -420,10 +399,11 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		if (gf_sg_vrml_get_sf_type(field.fieldType) == GF_SG_VRML_SFNODE) {
 			e = gf_node_replace_child(com->node, (GF_ChildNodeItem **) field.far_ptr, inf->pos, NULL);
 		} else {
+			if (!field.far_ptr) return GF_NON_COMPLIANT_BITSTREAM;
 			if ((inf->pos < 0) || ((u32) inf->pos >= ((GenMFField *) field.far_ptr)->count) ) {
 				inf->pos = ((GenMFField *)field.far_ptr)->count - 1;
 			}
-			/*this is a regular MFField, just remove the item (realloc)*/
+			/*this is a regular MFField, just remove the item (gf_realloc)*/
 			e = gf_sg_vrml_mf_remove(field.far_ptr, field.fieldType, inf->pos);
 		}
 		/*deletion -> node has changed*/
@@ -436,9 +416,11 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		inf = (GF_CommandField*)gf_list_get(com->command_fields, 0);
 
 		e = gf_node_insert_child(com->node, inf->new_node, inf->pos);
-		if (!e) gf_node_register(inf->new_node, com->node);
-		if (!e) gf_node_event_out(com->node, inf->fieldIndex);
-		if (!e) gf_node_changed(com->node, NULL);
+		if (!e) e = gf_node_register(inf->new_node, com->node);
+		if (!e) {
+			gf_node_event_out(com->node, inf->fieldIndex);
+			gf_node_changed(com->node, NULL);
+		}
 		break;
 	}
 	case GF_SG_ROUTE_INSERT:
@@ -451,7 +433,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		if (com->RouteID) gf_sg_route_set_id(r, com->RouteID);
 		if (com->def_name) {
 			gf_sg_route_set_name(r, com->def_name);
-			free(com->def_name);
+			gf_free(com->def_name);
 			com->def_name = NULL;
 		}
 		break;
@@ -497,16 +479,16 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		}
 		return GF_OK;
 	case GF_SG_PROTO_DELETE:
-		{
-			u32 i;
-			for (i=0; i<com->del_proto_list_size; i++) {
-				/*note this will check for unregistered protos, but since IDs are unique we are sure we will 
-				not destroy an unregistered one*/
-				GF_Proto *proto = gf_sg_find_proto(graph, com->del_proto_list[i], NULL);
-				if (proto) gf_sg_proto_del(proto);
-			}
+	{
+		u32 i;
+		for (i=0; i<com->del_proto_list_size; i++) {
+			/*note this will check for unregistered protos, but since IDs are unique we are sure we will
+			not destroy an unregistered one*/
+			GF_Proto *proto = gf_sg_find_proto(graph, com->del_proto_list[i], NULL);
+			if (proto) gf_sg_proto_del(proto);
 		}
-		return GF_OK;
+	}
+	return GF_OK;
 	case GF_SG_PROTO_DELETE_ALL:
 		/*destroy all proto*/
 		while (gf_list_count(graph->protos)) {
@@ -517,10 +499,144 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		}
 		/*DO NOT TOUCH UNREGISTERED PROTOS*/
 		return GF_OK;
+	case GF_SG_XREPLACE:
+	{
+		s32 pos = -2;
+		GF_Node *target = NULL;
+		GF_ChildNodeItem *list, *cur;
+		GF_FieldInfo value;
+		inf = (GF_CommandField*)gf_list_get(com->command_fields, 0);
+		if (!inf) return GF_SG_UNKNOWN_NODE;
+
+		e = gf_node_get_field(com->node, inf->fieldIndex, &field);
+		if (e) return e;
+		target = com->node;
+
+		if (!gf_sg_vrml_is_sf_field(field.fieldType)) {
+			GF_FieldInfo idxField;
+			if ((inf->pos != -2) || com->toNodeID) {
+				if (com->toNodeID) {
+					GF_Node *idxNode = gf_sg_find_node(graph, com->toNodeID);
+					if (!idxNode) return GF_SG_UNKNOWN_NODE;
+
+					if (gf_node_get_field(idxNode, com->toFieldIndex, &idxField) != GF_OK) return GF_OK;
+					pos = 0;
+					switch (idxField.fieldType) {
+					case GF_SG_VRML_SFBOOL:
+						if (*(SFBool*)idxField.far_ptr) pos = 1;
+						break;
+					case GF_SG_VRML_SFINT32:
+						if (*(SFInt32*)idxField.far_ptr >=0) pos = *(SFInt32*)idxField.far_ptr;
+						break;
+					case GF_SG_VRML_SFFLOAT:
+						if ( (*(SFFloat *)idxField.far_ptr) >=0) pos = (s32) floor( FIX2FLT(*(SFFloat*)idxField.far_ptr) );
+						break;
+					case GF_SG_VRML_SFTIME:
+						if ( (*(SFTime *)idxField.far_ptr) >=0) pos = (s32) floor( (*(SFTime *)idxField.far_ptr) );
+						break;
+					}
+				} else {
+					pos = inf->pos;
+				}
+			}
+		}
+		/*override target node*/
+		if ((field.fieldType==GF_SG_VRML_MFNODE) && (pos>=-1) && com->ChildNodeTag) {
+			target = gf_node_list_get_child(*(GF_ChildNodeItem **)field.far_ptr, pos);
+			if (!target) return GF_SG_UNKNOWN_NODE;
+			if (gf_node_get_field(target, com->child_field, &field) != GF_OK) return GF_SG_UNKNOWN_NODE;
+			pos=-2;
+		}
+
+		if (com->fromNodeID) {
+			GF_Node *fromNode = gf_sg_find_node(graph, com->fromNodeID);
+			if (!fromNode) return GF_SG_UNKNOWN_NODE;
+			e = gf_node_get_field(fromNode, com->fromFieldIndex, &value);
+			if (e) return e;
+		} else {
+			value.far_ptr = inf->field_ptr;
+			value.fieldType = inf->fieldType;
+		}
+		/*indexed replacement*/
+		if (pos>=-1) {
+			/*if MFNode remove the child and set new node*/
+			if (field.fieldType == GF_SG_VRML_MFNODE) {
+				GF_Node *nn = *(GF_Node**)value.far_ptr;
+				gf_node_replace_child(target, (GF_ChildNodeItem**) field.far_ptr, pos, nn);
+				if (nn) gf_node_register(nn, NULL);
+			}
+			/*erase the field item*/
+			else {
+				u32 sftype;
+				if (!field.far_ptr) return GF_NON_COMPLIANT_BITSTREAM;
+				if ((pos < 0) || ((u32) pos >= ((GenMFField *) field.far_ptr)->count) ) {
+					pos = ((GenMFField *)field.far_ptr)->count - 1;
+					/*may happen with text and default value*/
+					if (pos < 0) {
+						pos = 0;
+						gf_sg_vrml_mf_alloc(field.far_ptr, field.fieldType, 1);
+					}
+				}
+				e = gf_sg_vrml_mf_get_item(field.far_ptr, field.fieldType, & slot_ptr, pos);
+				if (e) return e;
+				sftype = gf_sg_vrml_get_sf_type(field.fieldType);
+				gf_sg_vrml_field_copy(slot_ptr, value.far_ptr, sftype);
+				/*note we don't add time offset, since there's no MFTime*/
+			}
+		} else {
+			GF_ChildNodeItem *prev;
+			switch (field.fieldType) {
+			case GF_SG_VRML_SFNODE:
+			{
+				node = *((GF_Node **) field.far_ptr);
+				e = gf_node_unregister(node, target);
+				*((GF_Node **) field.far_ptr) = *((GF_Node **) value.far_ptr) ;
+				if (!e) gf_node_register(*(GF_Node **) value.far_ptr, target);
+				break;
+			}
+			case GF_SG_VRML_MFNODE:
+				gf_node_unregister_children(target, * ((GF_ChildNodeItem **) field.far_ptr));
+				* ((GF_ChildNodeItem **) field.far_ptr) = NULL;
+
+				list = * ((GF_ChildNodeItem **) value.far_ptr);
+				prev=NULL;
+				while (list) {
+					cur = gf_malloc(sizeof(GF_ChildNodeItem));
+					cur->next = NULL;
+					cur->node = list->node;
+					if (prev) {
+						prev->next = cur;
+					} else {
+						* ((GF_ChildNodeItem **) field.far_ptr) = cur;
+					}
+					gf_node_register(list->node, target);
+					prev = cur;
+					list = list->next;
+				}
+				break;
+
+			default:
+				if (!gf_sg_vrml_is_sf_field(field.fieldType)) {
+					e = gf_sg_vrml_mf_reset(field.far_ptr, field.fieldType);
+				}
+				if (e) return e;
+				gf_sg_vrml_field_clone(field.far_ptr, value.far_ptr, field.fieldType, graph);
+
+				if ((field.fieldType==GF_SG_VRML_SFTIME) && !strstr(field.name, "media"))
+					*(SFTime *)field.far_ptr = *(SFTime *)field.far_ptr + time_offset;
+				break;
+			}
+		}
+		SG_CheckFieldChange(target, &field);
+	}
+	return GF_OK;
 	/*only used by BIFS*/
 	case GF_SG_GLOBAL_QUANTIZER:
 		return GF_OK;
 
+#endif /*GPAC_DISABLE_VRML*/
+
+
 #ifndef GPAC_DISABLE_SVG
 	/*laser commands*/
 	case GF_SG_LSR_NEW_SCENE:
@@ -548,7 +664,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		inf = (GF_CommandField*)gf_list_get(com->command_fields, 0);
 		if (!com->node || !inf) return GF_NON_COMPLIANT_BITSTREAM;
 		if (inf->new_node) {
-			if (inf->pos<0) 
+			if (inf->pos<0)
 				gf_node_list_add_child(& ((SVG_Element *)com->node)->children, inf->new_node);
 			else
 				gf_node_list_insert_child(& ((SVG_Element *)com->node)->children, inf->new_node, inf->pos);
@@ -596,7 +712,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 			prev = NULL;
 			child = inf->node_list;
 			while (child) {
-				cur = (GF_ChildNodeItem*)malloc(sizeof(GF_ChildNodeItem));
+				cur = (GF_ChildNodeItem*)gf_malloc(sizeof(GF_ChildNodeItem));
 				cur->next = NULL;
 				cur->node = child->node;
 				gf_node_register(child->node, com->node);
@@ -619,7 +735,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 				GF_Matrix2D *dest;
 				gf_node_get_field_by_name(com->node, "transform", &a);
 				dest = (GF_Matrix2D*)a.far_ptr;
-				
+
 				if (com->tag==GF_SG_LSR_REPLACE) {
 					if (gf_mx2d_decompose(dest, &scale, &rotate, &translate)) {
 						gf_mx2d_init(*dest);
@@ -637,25 +753,29 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 					else if (inf->fieldType==SVG_TRANSFORM_TRANSLATE) gf_mx2d_add_translation(dest, pt->x, pt->y);
 					else if (inf->fieldType == SVG_TRANSFORM_ROTATE) gf_mx2d_add_rotation(dest, 0, 0, ((SVG_Point_Angle*)inf->field_ptr)->angle);
 				}
+				/*signal node modif*/
+				gf_node_changed(com->node, &a);
 			} else {
 				if ((inf->fieldIndex==(u32) -1) && (inf->fieldType==DOM_String_datatype)) {
 					char *str = *(SVG_String*)inf->field_ptr;
 
 					if (com->tag == GF_SG_LSR_REPLACE) {
-						GF_DOMText *t = ((SVG_Element*)com->node)->children ? (GF_DOMText*) ((SVG_Element*)com->node)->children->node :NULL; 
+						GF_DOMText *t = ((SVG_Element*)com->node)->children ? (GF_DOMText*) ((SVG_Element*)com->node)->children->node :NULL;
 						if (t && (t->sgprivate->tag==TAG_DOMText)) {
-							if (t->textContent) free(t->textContent);
+							if (t->textContent) gf_free(t->textContent);
 							t->textContent = NULL;
-							if (str) t->textContent = strdup(str);
+							if (str) t->textContent = gf_strdup(str);
 						}
 					} else {
-						if (str) gf_dom_add_text_node(com->node, strdup(str));
+						if (str) gf_dom_add_text_node(com->node, gf_strdup(str));
 					}
+					/*signal node modif*/
+					gf_node_changed(com->node, NULL);
 				}
-				else if ((inf->fieldIndex==TAG_LSR_ATT_scale) 
-					|| (inf->fieldIndex==TAG_LSR_ATT_translation)
-					|| (inf->fieldIndex==TAG_LSR_ATT_rotation)
-				) {
+				else if ((inf->fieldIndex==TAG_LSR_ATT_scale)
+				         || (inf->fieldIndex==TAG_LSR_ATT_translation)
+				         || (inf->fieldIndex==TAG_LSR_ATT_rotation)
+				        ) {
 					SVG_Transform *mx;
 					gf_node_get_attribute_by_tag(com->node, TAG_SVG_ATT_transform, 1, 0, &a);
 					mx = a.far_ptr;
@@ -677,6 +797,8 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 						if (inf->fieldIndex==TAG_LSR_ATT_translation) gf_mx2d_add_translation(&mx->mat, ((GF_Point2D*)inf->field_ptr)->x, ((GF_Point2D*)inf->field_ptr)->y);
 						if (inf->fieldIndex==TAG_LSR_ATT_rotation) gf_mx2d_add_rotation(&mx->mat, 0, 0, ((SVG_Point_Angle*)inf->field_ptr)->angle);
 					}
+					/*signal node modif*/
+					gf_node_changed(com->node, &a);
 				}
 				else if (gf_node_get_attribute_by_tag(com->node, inf->fieldIndex, 1, 0, &a) == GF_OK) {
 					b = a;
@@ -686,12 +808,13 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 					} else {
 						gf_svg_attributes_add(&a, &b, &a, 0);
 					}
+					if (a.fieldType==XMLRI_datatype) {
+						gf_node_dirty_set(com->node, GF_SG_SVG_XLINK_HREF_DIRTY, 0);
+					}
+					/*signal node modif*/
+					gf_node_changed(com->node, &a);
 				}
-				b = a;
-				b.far_ptr = inf->field_ptr;
 			}
-			/*signal node modif*/
-			gf_node_changed(com->node, &a);
 		} else if (com->fromNodeID) {
 			GF_FieldInfo a, b;
 			GF_Node *fromNode = gf_sg_find_node(graph, com->fromNodeID);
@@ -699,17 +822,17 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 			if (gf_node_get_field(fromNode, com->fromFieldIndex, &b) != GF_OK) return GF_NON_COMPLIANT_BITSTREAM;
 
 			if ((inf->fieldIndex==(u32) -1) && (inf->fieldType==DOM_String_datatype)) {
-				char *str = *(SVG_String*)inf->field_ptr;
+				char *str = inf->field_ptr ? *(SVG_String*)inf->field_ptr : NULL;
 
 				if (com->tag == GF_SG_LSR_REPLACE) {
-					GF_DOMText *t = ((SVG_Element*)com->node)->children ? (GF_DOMText*) ((SVG_Element*)com->node)->children->node :NULL; 
+					GF_DOMText *t = ((SVG_Element*)com->node)->children ? (GF_DOMText*) ((SVG_Element*)com->node)->children->node :NULL;
 					if (t && (t->sgprivate->tag==TAG_DOMText)) {
-						if (t->textContent) free(t->textContent);
+						if (t->textContent) gf_free(t->textContent);
 						t->textContent = NULL;
-						if (str) t->textContent = strdup(str);
+						if (str) t->textContent = gf_strdup(str);
 					}
 				} else {
-					if (str) gf_dom_add_text_node(com->node, strdup(str));
+					if (str) gf_dom_add_text_node(com->node, gf_strdup(str));
 				}
 			} else {
 				gf_node_get_field(com->node, inf->fieldIndex, &a);
@@ -742,7 +865,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		evt.clientY = com->send_event_y;
 		gf_dom_event_fire(com->node, &evt);
 	}
-		break;
+	break;
 #endif
 
 	default:
@@ -750,6 +873,7 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 	}
 	if (e) return e;
 
+#ifndef GPAC_DISABLE_VRML
 	if (com->scripts_to_load) {
 		while (gf_list_count(com->scripts_to_load)) {
 			GF_Node *script = (GF_Node *)gf_list_get(com->scripts_to_load, 0);
@@ -759,14 +883,18 @@ GF_Err gf_sg_command_apply(GF_SceneGraph *graph, GF_Command *com, Double time_of
 		gf_list_del(com->scripts_to_load);
 		com->scripts_to_load = NULL;
 	}
+#endif
+
 	return GF_OK;
 }
 
+GF_EXPORT
 GF_CommandField *gf_sg_command_field_new(GF_Command *com)
 {
 	GF_CommandField *ptr;
 	GF_SAFEALLOC(ptr, GF_CommandField);
-	gf_list_add(com->command_fields, ptr);
+	if (ptr)
+		gf_list_add(com->command_fields, ptr);
 	return ptr;
 }
 
@@ -784,12 +912,12 @@ GF_Err gf_sg_command_apply_list(GF_SceneGraph *graph, GF_List *comList, Double t
 	return GF_OK;
 }
 
-GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph, Bool force_clone)
+#ifndef GPAC_DISABLE_VRML
+GF_Command *gf_sg_vrml_command_clone(GF_Command *com, GF_SceneGraph *inGraph, Bool force_clone)
 {
 	u32 i, count;
 	GF_Command *dest;
-	
-	if (com->tag==GF_SG_SCENE_REPLACE) return NULL;
+
 	/*FIXME - to do*/
 	if (gf_list_count(com->new_proto_list)) return NULL;
 	dest = gf_sg_command_new(inGraph, com->tag);
@@ -805,7 +933,10 @@ GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph, Bool fo
 	}
 	/*route insert, replace and delete*/
 	dest->RouteID = com->RouteID;
-	if (com->def_name) dest->def_name = strdup(com->def_name);
+	if (com->def_name) dest->def_name = gf_strdup(com->def_name);
+	//this is an union
+	//if (com->send_event_string) dest->send_event_string = gf_strdup(com->send_event_string);
+
 	dest->fromNodeID = com->fromNodeID;
 	dest->fromFieldIndex = com->fromFieldIndex;
 	dest->toNodeID = com->toNodeID;
@@ -813,12 +944,10 @@ GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph, Bool fo
 	dest->send_event_integer = com->send_event_integer;
 	dest->send_event_x = com->send_event_x;
 	dest->send_event_y = com->send_event_y;
-	if (com->send_event_string)
-		dest->send_event_string = strdup(com->send_event_string);
 
 	dest->del_proto_list_size = com->del_proto_list_size;
 	if (com->del_proto_list_size) {
-		dest->del_proto_list = (u32*)malloc(sizeof(u32) * com->del_proto_list_size);
+		dest->del_proto_list = (u32*)gf_malloc(sizeof(u32) * com->del_proto_list_size);
 		memcpy(dest->del_proto_list, com->del_proto_list, sizeof(u32) * com->del_proto_list_size);
 	}
 	count = gf_list_count(com->command_fields);
@@ -829,9 +958,11 @@ GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph, Bool fo
 		fd->fieldIndex = fo->fieldIndex;
 		fd->fieldType = fo->fieldType;
 		fd->pos = fo->pos;
+
+		/*FIXME - this can also be LASeR commands, not supported for now*/
 		if (fo->field_ptr) {
 			fd->field_ptr = gf_sg_vrml_field_pointer_new(fd->fieldType);
-			gf_sg_vrml_field_copy(fd->field_ptr, fo->field_ptr, fo->fieldType);
+			gf_sg_vrml_field_clone(fd->field_ptr, fo->field_ptr, fo->fieldType, dest->in_scene);
 		}
 
 		if (fo->new_node) {
@@ -848,7 +979,7 @@ GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph, Bool fo
 			prev = NULL;
 			child = fo->node_list;
 			while (child) {
-				cur = (GF_ChildNodeItem*) malloc(sizeof(GF_ChildNodeItem));
+				cur = (GF_ChildNodeItem*) gf_malloc(sizeof(GF_ChildNodeItem));
 				if (force_clone) {
 					cur->node = gf_node_clone(inGraph, child->node, dest->node, "", 0);
 				} else {
@@ -867,3 +998,5 @@ GF_Command *gf_sg_command_clone(GF_Command *com, GF_SceneGraph *inGraph, Bool fo
 	return dest;
 }
 
+#endif
+
