@@ -1,7 +1,8 @@
 /*
  *			GPAC - Multimedia Framework C SDK
  *
- *			Copyright (c) Jean Le Feuvre 2000-2005
+ *			Authors: Jean Le Feuvre
+ *			Copyright (c) Telecom ParisTech 2000-2023
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Graph sub-project
@@ -10,15 +11,15 @@
  *  it under the terms of the GNU Lesser General Public License as published by
  *  the Free Software Foundation; either version 2, or (at your option)
  *  any later version.
- *   
+ *
  *  GPAC is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Lesser General Public License for more details.
- *   
+ *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this library; see the file COPYING.  If not, write to
- *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. 
+ *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  */
 
@@ -28,6 +29,43 @@
 #include <gpac/nodes_x3d.h>
 
 
+GF_EXPORT
+SFRotation gf_sg_sfrotation_interpolate(SFRotation kv1, SFRotation kv2, Fixed fraction)
+{
+	SFRotation res;
+	Fixed newa, olda;
+	Bool stzero = ( ABS(kv1.q) < FIX_EPSILON) ? 1 : 0;
+	Bool endzero = ( ABS(kv2.q) < FIX_EPSILON) ? 1 : 0;
+	Fixed testa = gf_mulfix(kv1.x, kv2.x) + gf_mulfix(kv1.y, kv2.y) + gf_mulfix(kv1.y, kv2.y);
+
+	if (testa>= 0) {
+		res.x = kv1.x + gf_mulfix(fraction, kv2.x-kv1.x);
+		res.y = kv1.y + gf_mulfix(fraction, kv2.y-kv1.y);
+		res.z = kv1.z + gf_mulfix(fraction, kv2.z-kv1.z);
+		newa = kv2.q;
+	} else {
+		res.x = kv1.x + gf_mulfix(fraction, -kv2.x -kv1.x);
+		res.y = kv1.y + gf_mulfix(fraction, -kv2.y-kv1.y);
+		res.z = kv1.z + gf_mulfix(fraction, -kv2.z-kv1.z);
+		newa = -kv2.q;
+	}
+	olda = kv1.q;
+	if (stzero || endzero) {
+		res.x = stzero ? kv2.x : kv1.x;
+		res.y = stzero ? kv2.y : kv1.y;
+		res.z = stzero ? kv2.z : kv1.z;
+	}
+	res.q = olda + gf_mulfix(fraction, newa - olda);
+	if (res.q > GF_2PI) {
+		res.q -= GF_2PI;
+	} else if (res.q < GF_2PI) {
+		res.q += GF_2PI;
+	}
+	return res;
+}
+
+#ifndef GPAC_DISABLE_VRML
+
 static Fixed Interpolate(Fixed keyValue1, Fixed keyValue2, Fixed fraction)
 {
 	return gf_mulfix(keyValue2 - keyValue1, fraction) + keyValue1;
@@ -36,12 +74,12 @@ static Fixed Interpolate(Fixed keyValue1, Fixed keyValue2, Fixed fraction)
 static Fixed GetInterpolateFraction(Fixed key1, Fixed key2, Fixed fraction)
 {
 	Fixed keyDiff = key2 - key1;
-	assert((fraction >= key1) && (fraction <= key2));
+	gf_assert((fraction >= key1) && (fraction <= key2));
 	if (ABS(keyDiff) < FIX_EPSILON) return 0;
 	return gf_divfix(fraction - key1, keyDiff);
 }
 
-static void CI2D_SetFraction(GF_Node *n)
+static void CI2D_SetFraction(GF_Node *n, GF_Route *route)
 {
 	Fixed frac;
 	u32 numElemPerKey, i, j;
@@ -49,7 +87,7 @@ static void CI2D_SetFraction(GF_Node *n)
 
 	if (! _this->key.count) return;
 	if (_this->keyValue.count % _this->key.count) return;
-	
+
 	numElemPerKey = _this->keyValue.count / _this->key.count;
 	//set size
 	if (_this->value_changed.count != numElemPerKey)
@@ -71,11 +109,11 @@ static void CI2D_SetFraction(GF_Node *n)
 			frac = GetInterpolateFraction(_this->key.vals[j-1], _this->key.vals[j], _this->set_fraction);
 			for (i=0; i<numElemPerKey; i++) {
 				_this->value_changed.vals[i].x = Interpolate(_this->keyValue.vals[(j-1)*numElemPerKey + i].x,
-															_this->keyValue.vals[(j)*numElemPerKey + i].x, 
-															frac);
+				                                 _this->keyValue.vals[(j)*numElemPerKey + i].x,
+				                                 frac);
 				_this->value_changed.vals[i].y = Interpolate(_this->keyValue.vals[(j-1)*numElemPerKey + i].y,
-															_this->keyValue.vals[(j)*numElemPerKey + i].y,
-															frac);
+				                                 _this->keyValue.vals[(j)*numElemPerKey + i].y,
+				                                 frac);
 			}
 			break;
 		}
@@ -105,7 +143,7 @@ static Bool CI_SetFraction(Fixed fraction, MFVec3f *vals, MFFloat *key, MFVec3f
 
 	if (! key->count) return 0;
 	if (keyValue->count % key->count) return 0;
-	
+
 	numElemPerKey = keyValue->count / key->count;
 
 	if (vals->count != numElemPerKey) gf_sg_vrml_mf_alloc(vals, GF_SG_VRML_MFVEC3F, numElemPerKey);
@@ -125,14 +163,14 @@ static Bool CI_SetFraction(Fixed fraction, MFVec3f *vals, MFFloat *key, MFVec3f
 			frac = GetInterpolateFraction(key->vals[j-1], key->vals[j], fraction);
 			for (i=0; i<numElemPerKey; i++) {
 				vals->vals[i].x = Interpolate(keyValue->vals[(j-1)*numElemPerKey + i].x,
-															keyValue->vals[(j)*numElemPerKey + i].x,
-															frac);
+				                              keyValue->vals[(j)*numElemPerKey + i].x,
+				                              frac);
 				vals->vals[i].y = Interpolate(keyValue->vals[(j-1)*numElemPerKey + i].y,
-															keyValue->vals[(j)*numElemPerKey + i].y,
-															frac);
+				                              keyValue->vals[(j)*numElemPerKey + i].y,
+				                              frac);
 				vals->vals[i].z = Interpolate(keyValue->vals[(j-1)*numElemPerKey + i].z,
-															keyValue->vals[(j)*numElemPerKey + i].z, 
-															frac);
+				                              keyValue->vals[(j)*numElemPerKey + i].z,
+				                              frac);
 			}
 			break;
 		}
@@ -141,11 +179,11 @@ static Bool CI_SetFraction(Fixed fraction, MFVec3f *vals, MFFloat *key, MFVec3f
 }
 
 
-static void CoordInt_SetFraction(GF_Node *n)
+static void CoordInt_SetFraction(GF_Node *n, GF_Route *route)
 {
 	M_CoordinateInterpolator *_this = (M_CoordinateInterpolator *) n;
 
-	if (CI_SetFraction(_this->set_fraction, &_this->value_changed, &_this->key, &_this->keyValue)) 
+	if (CI_SetFraction(_this->set_fraction, &_this->value_changed, &_this->key, &_this->keyValue))
 		gf_node_event_out(n, 3);//"value_changed"
 }
 
@@ -156,7 +194,7 @@ Bool InitCoordinateInterpolator(M_CoordinateInterpolator *n)
 	return 1;
 }
 
-static void NormInt_SetFraction(GF_Node *n)
+static void NormInt_SetFraction(GF_Node *n, GF_Route *route)
 {
 	u32 i;
 	M_NormalInterpolator *_this = (M_NormalInterpolator *) n;
@@ -176,7 +214,7 @@ Bool InitNormalInterpolator(M_NormalInterpolator *n)
 	return 1;
 }
 
-static void ColorInt_SetFraction(GF_Node *node)
+static void ColorInt_SetFraction(GF_Node *node, GF_Route *route)
 {
 	u32 i;
 	Fixed frac;
@@ -185,7 +223,7 @@ static void ColorInt_SetFraction(GF_Node *node)
 
 	if (! _this->key.count) return;
 	if (_this->keyValue.count != _this->key.count) return;
-	
+
 	// The given fraction is less than the specified range
 	if (_this->set_fraction < _this->key.vals[0]) {
 		_this->value_changed = _this->keyValue.vals[0];
@@ -198,15 +236,15 @@ static void ColorInt_SetFraction(GF_Node *node)
 			if (_this->set_fraction >= _this->key.vals[i]) continue;
 
 			frac = GetInterpolateFraction(_this->key.vals[i-1], _this->key.vals[i], _this->set_fraction);
-			_this->value_changed.red = Interpolate(_this->keyValue.vals[i-1].red, 
-													_this->keyValue.vals[i].red, 
-													frac);
+			_this->value_changed.red = Interpolate(_this->keyValue.vals[i-1].red,
+			                                       _this->keyValue.vals[i].red,
+			                                       frac);
 			_this->value_changed.green = Interpolate(_this->keyValue.vals[i-1].green,
-													_this->keyValue.vals[i].green, 
-													frac);
+			                             _this->keyValue.vals[i].green,
+			                             frac);
 			_this->value_changed.blue = Interpolate(_this->keyValue.vals[i-1].blue,
-													_this->keyValue.vals[i].blue,
-													frac);
+			                                        _this->keyValue.vals[i].blue,
+			                                        frac);
 			break;
 		}
 	}
@@ -215,13 +253,13 @@ static void ColorInt_SetFraction(GF_Node *node)
 
 Bool InitColorInterpolator(M_ColorInterpolator *node)
 {
-	node->on_set_fraction = ColorInt_SetFraction; 
+	node->on_set_fraction = ColorInt_SetFraction;
 	if (node->keyValue.count) node->value_changed = node->keyValue.vals[0];
 	return 1;
 }
 
 
-static void PosInt2D_SetFraction(GF_Node *node)
+static void PosInt2D_SetFraction(GF_Node *node, GF_Route *route)
 {
 	M_PositionInterpolator2D *_this = (M_PositionInterpolator2D *)node;
 	u32 i;
@@ -229,7 +267,7 @@ static void PosInt2D_SetFraction(GF_Node *node)
 
 	if (! _this->key.count) return;
 	if (_this->keyValue.count != _this->key.count) return;
-	
+
 	// The given fraction is less than the specified range
 	if (_this->set_fraction < _this->key.vals[0]) {
 		_this->value_changed = _this->keyValue.vals[0];
@@ -252,12 +290,12 @@ static void PosInt2D_SetFraction(GF_Node *node)
 
 Bool InitPositionInterpolator2D(M_PositionInterpolator2D *node)
 {
-	node->on_set_fraction = PosInt2D_SetFraction; 
+	node->on_set_fraction = PosInt2D_SetFraction;
 	if (node->keyValue.count) node->value_changed = node->keyValue.vals[0];
 	return 1;
 }
 
-static void PosInt_SetFraction(GF_Node *node)
+static void PosInt_SetFraction(GF_Node *node, GF_Route *route)
 {
 	u32 i;
 	Fixed frac;
@@ -265,7 +303,7 @@ static void PosInt_SetFraction(GF_Node *node)
 
 	if (! _this->key.count) return;
 	if (_this->keyValue.count != _this->key.count) return;
-	
+
 	// The given fraction is less than the specified range
 	if (_this->set_fraction < _this->key.vals[0]) {
 		_this->value_changed = _this->keyValue.vals[0];
@@ -289,12 +327,12 @@ static void PosInt_SetFraction(GF_Node *node)
 
 Bool InitPositionInterpolator(M_PositionInterpolator *node)
 {
-	node->on_set_fraction = PosInt_SetFraction; 
+	node->on_set_fraction = PosInt_SetFraction;
 	if (node->keyValue.count) node->value_changed = node->keyValue.vals[0];
 	return 1;
 }
 
-static void ScalarInt_SetFraction(GF_Node *node)
+static void ScalarInt_SetFraction(GF_Node *node, GF_Route *route)
 {
 	M_ScalarInterpolator *_this = (M_ScalarInterpolator *)node;
 	u32 i;
@@ -302,7 +340,7 @@ static void ScalarInt_SetFraction(GF_Node *node)
 
 	if (! _this->key.count) return;
 	if (_this->keyValue.count != _this->key.count) return;
-	
+
 	// The given fraction is less than the specified range
 	if (_this->set_fraction < _this->key.vals[0]) {
 		_this->value_changed = _this->keyValue.vals[0];
@@ -323,47 +361,13 @@ static void ScalarInt_SetFraction(GF_Node *node)
 }
 Bool InitScalarInterpolator(M_ScalarInterpolator *node)
 {
-	node->on_set_fraction = ScalarInt_SetFraction; 
+	node->on_set_fraction = ScalarInt_SetFraction;
 	if (node->keyValue.count) node->value_changed = node->keyValue.vals[0];
 	return 1;
 }
 
-GF_EXPORT
-SFRotation gf_sg_sfrotation_interpolate(SFRotation kv1, SFRotation kv2, Fixed fraction)
-{
-	SFRotation res;
-	Fixed newa, olda;
-	Bool stzero = ( ABS(kv1.q) < FIX_EPSILON) ? 1 : 0;
-	Bool endzero = ( ABS(kv2.q) < FIX_EPSILON) ? 1 : 0;
-	Fixed testa = gf_mulfix(kv1.x, kv2.x) + gf_mulfix(kv1.y, kv2.y) + gf_mulfix(kv1.y, kv2.y);
 
-	if (testa>= 0) {
-		res.x = kv1.x + gf_mulfix(fraction, kv2.x-kv1.x);
-		res.y = kv1.y + gf_mulfix(fraction, kv2.y-kv1.y);
-		res.z = kv1.z + gf_mulfix(fraction, kv2.z-kv1.z);
-		newa = kv2.q;
-	} else {
-		res.x = kv1.x + gf_mulfix(fraction, -kv2.x -kv1.x);
-		res.y = kv1.y + gf_mulfix(fraction, -kv2.y-kv1.y);
-		res.z = kv1.z + gf_mulfix(fraction, -kv2.z-kv1.z);
-		newa = -kv2.q;
-	}
-	olda = kv1.q;
-	if (stzero || endzero) {
-		res.x = stzero ? kv2.x : kv1.x;
-		res.y = stzero ? kv2.y : kv1.y;
-		res.z = stzero ? kv2.z : kv1.z;
-	}
-	res.q = olda + gf_mulfix(fraction, newa - olda);
-	if (res.q > GF_2PI) { 
-		res.q -= GF_2PI;
-	} else if (res.q < GF_2PI) {
-		res.q += GF_2PI;
-	}
-	return res;
-}
-
-static void OrientInt_SetFraction(GF_Node *node)
+static void OrientInt_SetFraction(GF_Node *node, GF_Route *route)
 {
 	u32 i;
 	Fixed frac;
@@ -371,7 +375,7 @@ static void OrientInt_SetFraction(GF_Node *node)
 
 	if (! _this->key.count) return;
 	if (_this->keyValue.count != _this->key.count) return;
-	
+
 	// The given fraction is less than the specified range
 	if (_this->set_fraction < _this->key.vals[0]) {
 		_this->value_changed = _this->keyValue.vals[0];
@@ -393,12 +397,12 @@ static void OrientInt_SetFraction(GF_Node *node)
 
 Bool InitOrientationInterpolator(M_OrientationInterpolator *node)
 {
-	node->on_set_fraction = OrientInt_SetFraction; 
+	node->on_set_fraction = OrientInt_SetFraction;
 	if (node->keyValue.count) node->value_changed = node->keyValue.vals[0];
 	return 1;
 }
 
-static void CI4D_SetFraction(GF_Node *n)
+static void CI4D_SetFraction(GF_Node *n, GF_Route *route)
 {
 	Fixed frac;
 	u32 numElemPerKey, i, j;
@@ -406,7 +410,7 @@ static void CI4D_SetFraction(GF_Node *n)
 
 	if (! _this->key.count) return;
 	if (_this->keyValue.count % _this->key.count) return;
-	
+
 	numElemPerKey = _this->keyValue.count / _this->key.count;
 	//set size
 	if (_this->value_changed.count != numElemPerKey)
@@ -428,17 +432,17 @@ static void CI4D_SetFraction(GF_Node *n)
 			frac = GetInterpolateFraction(_this->key.vals[j-1], _this->key.vals[j], _this->set_fraction);
 			for (i=0; i<numElemPerKey; i++) {
 				_this->value_changed.vals[i].x = Interpolate(_this->keyValue.vals[(j-1)*numElemPerKey + i].x,
-															_this->keyValue.vals[(j)*numElemPerKey + i].x, 
-															frac);
+				                                 _this->keyValue.vals[(j)*numElemPerKey + i].x,
+				                                 frac);
 				_this->value_changed.vals[i].y = Interpolate(_this->keyValue.vals[(j-1)*numElemPerKey + i].y,
-															_this->keyValue.vals[(j)*numElemPerKey + i].y,
-															frac);
+				                                 _this->keyValue.vals[(j)*numElemPerKey + i].y,
+				                                 frac);
 				_this->value_changed.vals[i].z = Interpolate(_this->keyValue.vals[(j-1)*numElemPerKey + i].z,
-															_this->keyValue.vals[(j)*numElemPerKey + i].z,
-															frac);
+				                                 _this->keyValue.vals[(j)*numElemPerKey + i].z,
+				                                 frac);
 				_this->value_changed.vals[i].q = Interpolate(_this->keyValue.vals[(j-1)*numElemPerKey + i].q,
-															_this->keyValue.vals[(j)*numElemPerKey + i].q,
-															frac);
+				                                 _this->keyValue.vals[(j)*numElemPerKey + i].q,
+				                                 frac);
 			}
 			break;
 		}
@@ -460,7 +464,7 @@ Bool InitCoordinateInterpolator4D(M_CoordinateInterpolator4D *node)
 	return 1;
 }
 
-static void PI4D_SetFraction(GF_Node *node)
+static void PI4D_SetFraction(GF_Node *node, GF_Route *route)
 {
 	u32 i;
 	Fixed frac;
@@ -468,7 +472,7 @@ static void PI4D_SetFraction(GF_Node *node)
 
 	if (! _this->key.count) return;
 	if (_this->keyValue.count != _this->key.count) return;
-	
+
 	// The given fraction is less than the specified range
 	if (_this->set_fraction < _this->key.vals[0]) {
 		_this->value_changed = _this->keyValue.vals[0];
@@ -500,19 +504,21 @@ Bool InitPositionInterpolator4D(M_PositionInterpolator4D *node)
 
 
 
-static void BooleanFilter_setValue(GF_Node *n)
+#ifndef GPAC_DISABLE_X3D
+
+static void BooleanFilter_setValue(GF_Node *n, GF_Route *route)
 {
 	X_BooleanFilter *bf = (X_BooleanFilter *)n;
 	if (!bf->set_boolean) {
 		bf->inputFalse = 1;
-		gf_node_event_out_str(n, "inputFalse");
+		gf_node_event_out(n, 1/*"inputFalse"*/);
 	}
 	if (bf->set_boolean) {
 		bf->inputTrue = 1;
-		gf_node_event_out_str(n, "inputTrue");
+		gf_node_event_out(n, 3/*"inputTrue"*/);
 	}
 	bf->inputNegate = bf->set_boolean ? 0 : 1;
-	gf_node_event_out_str(n, "inputNegate");
+	gf_node_event_out(n, 2/*"inputNegate"*/);
 }
 
 void InitBooleanFilter(GF_Node *n)
@@ -521,13 +527,13 @@ void InitBooleanFilter(GF_Node *n)
 	bf->on_set_boolean = BooleanFilter_setValue;
 }
 
-static void BooleanSequencer_setFraction(GF_Node *n)
+static void BooleanSequencer_setFraction(GF_Node *n, GF_Route *route)
 {
 	u32 i;
 	X_BooleanSequencer *bs = (X_BooleanSequencer*)n;
 	if (! bs->key.count) return;
 	if (bs->keyValue.count != bs->key.count) return;
-	
+
 	if (bs->set_fraction < bs->key.vals[0]) {
 		bs->value_changed = bs->keyValue.vals[0];
 	} else if (bs->set_fraction >= bs->key.vals[bs->key.count-1]) {
@@ -543,7 +549,7 @@ static void BooleanSequencer_setFraction(GF_Node *n)
 	gf_node_event_out(n, 3);//"value_changed"
 }
 
-static void BooleanSequencer_setNext(GF_Node *n)
+static void BooleanSequencer_setNext(GF_Node *n, GF_Route *route)
 {
 	s32 *prev_val, val;
 	X_BooleanSequencer *bs = (X_BooleanSequencer*)n;
@@ -556,7 +562,7 @@ static void BooleanSequencer_setNext(GF_Node *n)
 	gf_node_event_out(n, 3);//"value_changed"
 }
 
-static void BooleanSequencer_setPrevious(GF_Node *n)
+static void BooleanSequencer_setPrevious(GF_Node *n, GF_Route *route)
 {
 	s32 *prev_val, val;
 	X_BooleanSequencer *bs = (X_BooleanSequencer*)n;
@@ -574,7 +580,7 @@ static void DestroyBooleanSequencer(GF_Node *n, void *eff, Bool is_destroy)
 {
 	if (is_destroy) {
 		s32 *st = (s32 *) gf_node_get_private(n);
-		free(st);
+		gf_free(st);
 	}
 }
 void InitBooleanSequencer(GF_Node *n)
@@ -583,17 +589,17 @@ void InitBooleanSequencer(GF_Node *n)
 	bs->on_next = BooleanSequencer_setNext;
 	bs->on_previous = BooleanSequencer_setPrevious;
 	bs->on_set_fraction = BooleanSequencer_setFraction;
-	n->sgprivate->UserPrivate = malloc(sizeof(s32));
+	n->sgprivate->UserPrivate = gf_malloc(sizeof(s32));
 	*(s32 *)n->sgprivate->UserPrivate = 0;
 	n->sgprivate->UserCallback = DestroyBooleanSequencer;
 }
 
-static void BooleanToggle_setValue(GF_Node *n)
+static void BooleanToggle_setValue(GF_Node *n, GF_Route *route)
 {
 	X_BooleanToggle *bt = (X_BooleanToggle *)n;
 	if (bt->set_boolean) {
 		bt->toggle = !bt->toggle;
-		gf_node_event_out_str(n, "toggle");
+		gf_node_event_out(n, 1/*"toggle"*/);
 	}
 }
 void InitBooleanToggle(GF_Node *n)
@@ -602,11 +608,11 @@ void InitBooleanToggle(GF_Node *n)
 	bt->on_set_boolean = BooleanToggle_setValue;
 }
 
-static void BooleanTrigger_setTime(GF_Node *n)
+static void BooleanTrigger_setTime(GF_Node *n, GF_Route *route)
 {
 	X_BooleanTrigger *bt = (X_BooleanTrigger *)n;
 	bt->triggerTrue = 1;
-	gf_node_event_out_str(n, "triggerTrue");
+	gf_node_event_out(n, 1/*"triggerTrue"*/);
 }
 void InitBooleanTrigger(GF_Node *n)
 {
@@ -614,13 +620,13 @@ void InitBooleanTrigger(GF_Node *n)
 	bt->on_set_triggerTime = BooleanTrigger_setTime;
 }
 
-static void IntegerSequencer_setFraction(GF_Node *n)
+static void IntegerSequencer_setFraction(GF_Node *n, GF_Route *route)
 {
 	u32 i;
 	X_IntegerSequencer *is = (X_IntegerSequencer *)n;
 	if (! is->key.count) return;
 	if (is->keyValue.count != is->key.count) return;
-	
+
 	if (is->set_fraction < is->key.vals[0]) {
 		is->value_changed = is->keyValue.vals[0];
 	} else if (is->set_fraction >= is->key.vals[is->key.count-1]) {
@@ -636,7 +642,7 @@ static void IntegerSequencer_setFraction(GF_Node *n)
 	gf_node_event_out(n, 3);//"value_changed"
 }
 
-static void IntegerSequencer_setNext(GF_Node *n)
+static void IntegerSequencer_setNext(GF_Node *n, GF_Route *route)
 {
 	s32 *prev_val, val;
 	X_IntegerSequencer *is = (X_IntegerSequencer*)n;
@@ -649,7 +655,7 @@ static void IntegerSequencer_setNext(GF_Node *n)
 	gf_node_event_out(n, 3);//"value_changed"
 }
 
-static void IntegerSequencer_setPrevious(GF_Node *n)
+static void IntegerSequencer_setPrevious(GF_Node *n, GF_Route *route)
 {
 	s32 *prev_val, val;
 	X_IntegerSequencer *is = (X_IntegerSequencer *)n;
@@ -667,7 +673,7 @@ static void DestroyIntegerSequencer(GF_Node *n, void *eff, Bool is_destroy)
 {
 	if (is_destroy) {
 		s32 *st = (s32 *)gf_node_get_private(n);
-		free(st);
+		gf_free(st);
 	}
 }
 void InitIntegerSequencer(GF_Node *n)
@@ -676,17 +682,17 @@ void InitIntegerSequencer(GF_Node *n)
 	bs->on_next = IntegerSequencer_setNext;
 	bs->on_previous = IntegerSequencer_setPrevious;
 	bs->on_set_fraction = IntegerSequencer_setFraction;
-	n->sgprivate->UserPrivate = malloc(sizeof(s32));
+	n->sgprivate->UserPrivate = gf_malloc(sizeof(s32));
 	*(s32 *)n->sgprivate->UserPrivate = 0;
 	n->sgprivate->UserCallback = DestroyIntegerSequencer;
 }
 
-static void IntegerTrigger_setTrigger(GF_Node *n)
+static void IntegerTrigger_setTrigger(GF_Node *n, GF_Route *route)
 {
 	X_IntegerTrigger *it = (X_IntegerTrigger *)n;
 	if (it->set_boolean) {
 		it->triggerValue = it->integerKey;
-		gf_node_event_out_str(n, "triggerValue");
+		gf_node_event_out(n, 2/*"triggerValue"*/);
 	}
 }
 void InitIntegerTrigger(GF_Node *n)
@@ -695,11 +701,11 @@ void InitIntegerTrigger(GF_Node *n)
 	it->on_set_boolean = IntegerTrigger_setTrigger;
 }
 
-static void TimeTrigger_setTrigger(GF_Node *n)
+static void TimeTrigger_setTrigger(GF_Node *n, GF_Route *route)
 {
 	X_TimeTrigger *tt = (X_TimeTrigger *)n;
 	tt->triggerTime = gf_node_get_scene_time(n);
-	gf_node_event_out_str(n, "triggerTime");
+	gf_node_event_out(n, 1/*"triggerTime"*/);
 }
 void InitTimeTrigger(GF_Node *n)
 {
@@ -707,3 +713,7 @@ void InitTimeTrigger(GF_Node *n)
 	tt->on_set_boolean = TimeTrigger_setTrigger;
 }
 
+#endif /*GPAC_DISABLE_X3D*/
+
+
+#endif /*GPAC_DISABLE_VRML*/
